#! /Library/Frameworks/Python.framework/Versions/Current/bin python

import sys
import os
import ir
import time
import VolumeToXML

path = os.path.abspath(sys.argv[1]);
print path;

TimingOutput = 'Timing.txt';

MrcToMosaicStart = 0;
MrcToTifStart = 0;
TranslateStart = 0;
RefineStart = 0;
AssembleStart = 0;
StosBruteStart = 0;
StosGridStart = 0;

#Put all .mrc files in thier own directory, doesn't work for multiple volumes
#ir.CleanTempFiles(path);
ir.MRCMove(path); 

#Extract stage-position meta data from .mrc files
MrcToMosaicStart = time.time();
ir.MRCsToMosaic(path);
MrcToMosaicEnd = time.time();

#Extract images from .mrc files
MrcToTifStart = time.time();
ir.MRCsTo8BitPNG(path);
#ir.MRCstToClahePNGs(path); 
MrcToTifEnd = time.time();

#Construct any image pyramids which are unbuilt
BuildPyramidStart = time.time();
ir.BuildPyramids(path);
BuildPyramidEnd = time.time(); 

#Run translate to get the approximate tile position
#This needs to be ir-fft if stage position is missing
#TODO: ir-fft script hasn't been used in months
TranslateStart = time.time(); 
ir.Translate(path,4,"8-bit");
TranslateEnd = time.time();

#Refine the tile placement to make seemless edges in the mosaic
RefineStart = time.time(); 
ir.Refine(path,8,"8-bit");
RefineEnd = time.time(); 

#Output image files for each section at the specified downsample levels
AssembleStart = time.time();
ir.Assemble(path,[16,32],"8-bit");
AssembleEnd = time.time();

#Generate crude translation/rotation alignment of sections, use the
#largest downsample value passed to assemble
StosBruteStart = time.time();
ir.StosBrute(path,32);
StosBruteEnd = time.time();

#Refine the brute results.  Starting with highest downsample value
#and decreasing until desired quality is reached. 
StosGridStart = time.time();
ir.StosGrid(path,[16,32]);
StosGridEnd = time.time(); 

#Display elasped time
TFile = open(TimingOutput, 'w+');

tdelta = MrcToMosaicEnd - MrcToMosaicStart; 
s = 'Mrc to Mosaic Time : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = MrcToTifEnd - MrcToTifStart; 
s = '\nMrc to TIF Time    : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = BuildPyramidEnd - BuildPyramidStart; 
s = '\nBuildPyramids Time    : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = TranslateEnd - TranslateStart; 
s = '\nTranslate Time     : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = RefineEnd - RefineStart; 
s = '\nRefine Time        : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = AssembleEnd - AssembleStart; 
s = '\nAssemble Time      : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = StosBruteEnd - StosBruteStart; 
s = '\nStos Brute Time    : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

tdelta = StosGridEnd - StosGridStart; 
s = '\nStos Grid Time     : %2d days %s' % (tdelta / (60 * 60 * 24), time.strftime('%H:%M:%S', time.gmtime(tdelta)))
print s;
TFile.write(s);

TFile.close();

#Construct a descriptive XML for our viewer
VolumeToXML.CreateXML(path);


