import sys
import os
import subprocess
import time
import glob
import ir

tabLevel = 0;

#Creates an XML File describing a given volume
def CreateXML(path):
    tabLevel = 0; 
    StartingPath = os.getcwd(); 
    os.chdir(path); 
    path = os.path.abspath(sys.argv[1]);
    print path;

    #Create our XML File
    basename = os.path.basename(path);
    xmlFile = open("Volume.xml","w");

    WriteXML(xmlFile,"<?xml version=\"1.0\"?>");
    
    StosCount = 0;
    SectionCount = 0;

    #Find all the stos files:
    stosfiles = glob.glob(os.path.join(path, "*.stos"));
    StosCount = len(stosfiles); 

    #Count all the sections
    SectionDirs = os.listdir(path);
    for path in SectionDirs:
        #Skip if it contains a .
        if path.find('.') > -1:
            continue;
        
        (SectionNumber, Downsample) = ir.GetSectionInfo(path);
        if(SectionNumber < 0):
            continue;

        SectionCount = SectionCount + 1;

    
    WriteXML(xmlFile,"<Volume name=\"" + basename + "\" " + 
             " num_stos=\"" + str(StosCount) + "\" " +
             " num_sections=\"" + str(SectionCount) + "\">");
    

    for sfile in stosfiles:
        AddStosTransform(xmlFile, sfile);

    for path in SectionDirs:

        #Skip if it contains a .
        if path.find('.') > -1:
            continue;
        
        (SectionNumber, Downsample) = ir.GetSectionInfo(path);
        if(SectionNumber < 0):
            continue;

        print path; 
        AddSection(xmlFile, path); 

    WriteXML(xmlFile, "</Volume>");
    xmlFile.close();

    os.chdir(StartingPath); 

#Write one XML entry into the file
def WriteXML(xmlFile, string):
    global tabLevel; 
    if(string.find("</") > 0):
        tabLevel -= 1; 
        
    for i in range(0,tabLevel):
        xmlFile.write('\t');

    xmlFile.write(string + '\n');

    tabLevel += 1; 

    if(string.find("/>") > 0):
        tabLevel -= 1; 

def AddStosTransform(xmlFile, stosFile):
    basename = os.path.basename(stosFile);
    if(basename.startswith("Temp")):
        return;
    
    parts = os.path.splitext(basename);
    basefilename = parts[0];
    [mappedSection, controlSection, stosType, downsample] = ir.GetStosInfo(basefilename);
    if(stosType != "brute"):
        WriteXML(xmlFile, "<stos mappedSection=\"" + str(mappedSection) + "\" controlSection=\"" + str(controlSection) + "\" pixelSpacing=\"" + str(downsample) + "\" type=\"" + stosType + "\" path=\"" + basename + "\"/>");

def AddSection(xmlFile, path):
    (SectionNumber, Downsample) = ir.GetSectionInfo(path);
    sectionPath = os.path.basename(path);

    WriteXML(xmlFile, "<Section number=\"" + str(SectionNumber) + "\" path=\"" + sectionPath + "\">");

    #Find all the mosaic files:
    mosaicfiles = glob.glob(os.path.join(path, "*.mosaic"));

    for mfile in mosaicfiles:
        AddMosaicTransform(xmlFile, mfile);

    #Find all the image types we have
    dirs = os.listdir(path);
    for d in dirs:
        if(d.find('.') > -1):
            continue;

        AddImageType(xmlFile, os.path.join(path,d));

    WriteXML(xmlFile, "</Section>");

def AddMosaicTransform(xmlFile, mosaicFile):
    mfile = os.path.basename(mosaicFile); 
    WriteXML(xmlFile, "<transform name=\"" + mfile + "\" path=\"" + mfile + "\"/>");

#If the directory name contains an image pyramid add it to the XML file
def AddImageType(xmlFile, path):

    downsampleDict = dict(); 
    dirs = os.listdir(path);
    for d in dirs:
        downsample = 0; 
        try:
            downsample = int(d);
        except:
            continue;

        if(downsample > 0):
            downsampleDict[downsample] = d;

    downsampleLevels = downsampleDict.keys();
    downsampleLevels.sort(); 
    
    if(len(downsampleLevels) > 0):
        pyramidName = os.path.basename(path);
        minLevel = downsampleLevels[0]; 
        maxLevel = downsampleLevels[len(downsampleLevels)-1];
        WriteXML(xmlFile, "<Pyramid name=\"" + pyramidName + "\" path=\"" + pyramidName + "\">");

        for level in downsampleLevels:
            WriteXML(xmlFile, "<Level value=\"" + str(level) + "\" path=\"" + downsampleDict[level] + "\"/>"); 
        
        WriteXML(xmlFile, "</Pyramid>"); 
        
        
