import sys
import os
import glob
import shutil
import subprocess
import time
import struct
#import Image
import string
import math
import random
import string
import stat
import thread
import threading

#Tries to determine the number of processors
def GetNumProcs():
    try:
        numProcs = int(os.environ["NUMBER_OF_PROCESSORS"]);
    except:
        numProcs = 4;

    return numProcs;

def GetToolPath():
    try:
        path = os.environ["IR-TOOLS"];
    except:
        path= "/Users/Shared/ir-tools"

    return path;

def ClaheFFTMosaic():
    return "FFT-Clahe.mosaic";

def FFTMosaic():
    return "FFT.mosaic";

def TranslateMosaic():
    return "translate.mosaic";

def RefineMosaic():
    return "grid.mosaic";

def BlobCmd():
    return "ir-blob -sh 1 -r 8 -max 3 -median 1";

def ClaheCmd():
    return "ir-clahe -sh 1 -bins 4096 -remap 0 255 -slope 5 -window 8 8";

def FFTCmd():
    return "ir-fft -sh 8 -sp 1 -py 3 -ol 0.05 0.3 -clahe 6 ";

def TranslateCmd():
    return "ir-refine-translate -threads " + str(GetNumProcs()) + "  -sh 1 ";

def RefineCmd():
    return "ir-refine-grid -it 4 -cell 96 -mesh 8 8 -threads " + str(GetNumProcs()) + " ";

def AssembleCmd():
    return "ir-assemble -feathering blend -remap";

def MultipleIntensityAverageCmd():
    return "MultipleIntensityAverage";

def StosBruteCmd():
#    return "ir-stos-brute -sh 1 -clahe 2 -refine -cubic -regularize ";
    return "ir-stos-brute -sh 1 ";

def StosGridCmd():
    return "ir-stos-grid -sh 1 -fft 2 0.25 -grid_spacing 192 -neighborhood 128 -it 4 ";

def GetSectionInfo(fileName):
    fileName = os.path.basename(fileName); 
    SectionNumber = -1;
    Downsample = 1; 
    parts = fileName.split("_");
    try:
        SectionNumber = int(parts[0]);
    except:
        SectionNumber = -1; 

    #If we don't have a valid downsample value we assume 1
    try:
        DownsampleStrings = parts[2].split("."); 
        Downsample = int(DownsampleStrings[0]);
    except:
        Downsample = 1;
        
    return [SectionNumber,Downsample];

def GetStosInfo(baseName):
    parts = baseName.split("_");
    try:
        sections = parts[0].split('-');
        mappedSection = int(sections[0]);
        controlSection = int(sections[1]);

    except:
        mappedSection = -1; 
        controlSection = -1;

    try:
        stosType = parts[1];
    except:
        stosType = "";

    try:
        Downsample = int(parts[2]);
    except:
        Downsample = 1;

    return [mappedSection, controlSection, stosType, Downsample];

TileFormat = "png";
SectionFormat = '04d'; 
DownsampleFormat = '%03d';
ImageTypes = ['8-bit','16-bit', 'Clahe', 'Blob', 'MultIntAvg'];
# I hate to hard-code this, but right now it is the easiest way
EMImageSize = [4080,4080];
LMImageSize = [1388,1036]; 
PyramidLevels = [1,2,4,8,16,32,64];

#Return the name of the unimproved mosaic file for a directory
def MosaicName(path):
    outname = prefix(path);
    MosaicFileName = outname + '.mosaic';
    return MosaicFileName;

#Recursively walk each subdirectory from the path and add it to the list if its name does not match the exclude list
def RecurseSubdirectories(Path, ExcludeWords=["clahe","mbproj","8-bit","16-bit","Blob"]):
    Dirlist = list();
    Dirlist.append(Path);
    
    #Recursively list the subdirectories
    dirs = os.listdir(Path)

    for d in dirs:
        #Skip if it contains a .
        if d.find('.') > -1:
            continue;

        #Skip if it contains words from the exclude list
        name = os.path.basename(d); 
        name = string.lower(name);
        if (name in ExcludeWords):
            continue; 

        #Add directory tree to list
        fullpath = os.path.join(Path,d);
        if os.path.isdir(os.path.join(Path,d)):
            Dirlist = Dirlist + RecurseSubdirectories(fullpath, ExcludeWords); 
            
    return Dirlist;

#Removes all .mosaic, .stos, .png, and .tif files with zero size. 
def CleanTempFiles(Path, TargetExtensions = ['png','tif','stos','mosaic']):
    FileList = [];
    
    #Recursively list the subdirectories
    dirs = os.listdir(Path)

    for d in dirs:
        fullpath = os.path.join(Path,d);
        #Skip if it doesn't have an extension
        if d.find('.') < 0:
            if os.path.isdir(fullpath):
                CleanTempFiles(fullpath); 
        
        [path,extension] = os.path.splitext(d); 

        #Skip if it does not have extension from the exclude list
        extension = extension[1:];
        extension= string.lower(extension);
        if extension in TargetExtensions:
            #Check the size, delete file if it has zero size
            filesize = os.path.getsize(fullpath);
            if(filesize <= 0):
                print "Deleting " + fullpath + " size: " + str(filesize);
                os.remove(fullpath); 
                break;
            
    return;
        
        
    


#Move the .MRC files in the directory into subdirectories with the same name
#TODO: Make this recursive and skip moving mrc's if they already live in a dir
#named after the file
def MRCMove(Path):
    
     if os.path.exists(Path) == False:
        print "No valid path provided as first argument"
        return;

     ext = "*.mrc";
     print ext;

     print os.path.join(Path, ext)

     #List all of the .mrc files in the path
     files = glob.glob(os.path.join(Path, ext));

     #We expect .mrc files to be named ####_string.mrc
     # #### is a section number
     # string is anything the users chooses to add

     print files;
     for f in files:
         filename = os.path.basename(f);
         
         dirname = prefix(filename);
         parts = dirname.split('_');

         sectionNumberString = parts[0]; 
         
         #Convert the filename to a number, then pad it to make the dir name
         if sectionNumberString.isdigit():
             sectionNumber = int(sectionNumberString);
             dirname = "%04d" % sectionNumber;
             
         fulldirname = os.path.join(Path, dirname); 
         print dirname;      
         try:
            os.mkdir(fulldirname);
         except OSError, (errno, strerror):
            if errno==17:
                print "Already exists: " + dirname; 
                pass

         #Try to move the file to the new directory
         print f;
         print os.path.join(fulldirname, filename); 
         shutil.move(f, os.path.join(fulldirname, filename));   
          
#Clahes the specified directory and all subdirectories
#SrcPath - Path to load images from
#DestPath - Path to save images to
#SrcExt - extension of files to load
#DestExt - extension to use when saving
def Clahe(SrcPath, DestPath, SrcExt, DestExt):

    Procs = []; 
    numProcs = GetNumProcs();

    print 
    print "Executing clahe on: " + SrcPath + " *." + SrcExt;
    print "Clahe output to: " + DestPath  + " *." + DestExt;
    print
    
    ImageExtension = '*.' + SrcExt;
       
    #List all of the .tif files in the path
    files = glob.glob(os.path.join(SrcPath, ImageExtension));
    print "Files: " + ImageExtension;
    
    #Create a clahe directory if there are files
    if(len(files) > 0):
        try:
            os.mkdir(DestPath);
        except OSError, (errno, strerror):
            if errno==17:
                print "Already exists: " + DestPath; 
                pass

    #create a clahe version of each file
    for f in files:
        filename = os.path.basename(f);
        (ImageName, ext) = os.path.splitext(filename); 
        clahename = os.path.join(DestPath, ImageName) + "." + DestExt;
        print "Clahe: " + filename + " -> " + clahename; 
        if os.path.exists(clahename):
            print "Already exists: " + clahename;
        else:
            #create a clahe version of each file
            cmd=ClaheCmd() + ' -load ' + f + ' -save ' + clahename;
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            print "Created PID: "+ str(NewP.pid)

            Procs.append(NewP);
        
        while(len(Procs) >= numProcs):
            #Wait 10 seconds and check for openings again
            time.sleep(0.5);
            TestProcList = Procs
            for proc in TestProcList:
                if proc.poll() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(10);
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"

#Returns number of time blob command was called
#TODO: This really should be made into a generic function that can run any filter over the pyramid, not just Blob
def BlobAll(Path, SrcType="16-bit", DestType="blob", SrcExt="png", DestExt="png"):
    count = 0; 

    #Recursively list the subdirectories
    dirs = os.listdir(Path)

    #If our patch contains both the source and target image types then run the filter
    if(SrcType in dirs and DestType in dirs):
        count += Blob(os.path.join(Path, SrcType), os.path.join(Path, DestType), SrcExt, DestExt); 

    else:
        for d in dirs:
            fullpath = os.path.join(Path,d);
            #Skip if it doesn't have an extension
            if d.find('.') < 0:
                if os.path.isdir(fullpath):
                    count += BlobAll(fullpath, Cmd, SrcType, DestType, SrcExt, DestExt);
    return count;
    
    
#Blobs the specified directory and all subdirectories
#SrcPath - Path to load images from, "16-bit" is typical 
#DestPath - Path to save images to, "blob" is typical
#SrcExt - extension of files to load
#DestExt - extension to use when saving
def Blob(SrcPath, DestPath, SrcExt, DestExt):

    CmdCount = 0; #Number of times a command was executed
    Procs = []; 
    numProcs = GetNumProcs();

    print 
    print "Executing Blob on: " + SrcPath + " *." + SrcExt;
    print "Blob output to: " + DestPath  + " *." + DestExt;
    print
    
    ImageExtension = '*.' + SrcExt;
       
    #List all of the .tif files in the path
    files = glob.glob(os.path.join(SrcPath, ImageExtension));
    print "Files: " + ImageExtension;
    
    #Create a clahe directory if there are files
    if(len(files) > 0):
        try:
            os.mkdir(DestPath);
        except OSError, (errno, strerror):
            if errno==17:
                print "Already exists: " + DestPath; 
                pass

    #create a blob version of each file
    for f in files:
        filename = os.path.basename(f);
        (ImageName, ext) = os.path.splitext(filename); 
        clahename = os.path.join(DestPath, ImageName) + "." + DestExt;
        print "Blob: " + filename + " -> " + clahename; 
        if os.path.exists(clahename):
            print "Already exists: " + clahename;
        else:
            #Create a placeholder file so no other computer tries to scoop us
            tempFile = open(clahename,"w");
            tempFile.close();
            
            #create a clahe version of each file
            cmd=BlobCmd() + ' -load ' + f + ' -save ' + clahename;
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            print "Created PID: "+ str(NewP.pid)
            CmdCount += 1; 

            Procs.append(NewP);
        
        while(len(Procs) >= numProcs):
            #Wait 10 seconds and check for openings again
            time.sleep(2);
            TestProcList = Procs
            for proc in TestProcList:
                if proc.poll() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(5);
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"

    return CmdCount;

#Clahes the specified directory and all subdirectories
#SrcPath - Path to load images from
#DestPath - Path to save images to
#SrcExt - extension of files to load
#DestExt - extension to use when saving
def MultipleIntensityAverage(SrcPath, DestPath, SrcExt, DestExt, MeanIntensity, StdDeviation):

    Procs = []; 
    numProcs = GetNumProcs();

    print 
    print "Executing MultIntAvg on: " + SrcPath + " *." + SrcExt;
    print "Clahe output to: " + DestPath  + " *." + DestExt;
    print
    
    ImageExtension = '*.' + SrcExt;
       
    #List all of the .tif files in the path
    files = glob.glob(os.path.join(SrcPath, ImageExtension));
    print "Files: " + ImageExtension;
    
    #Create a clahe directory if there are files
    if(len(files) > 0):
        try:
            os.mkdir(DestPath);
        except OSError, (errno, strerror):
            if errno==17:
                print "Already exists: " + DestPath; 
                pass

    #create a clahe version of each file
    for f in files:
        filename = os.path.basename(f);
        (ImageName, ext) = os.path.splitext(filename); 
        outname = os.path.join(DestPath, ImageName) + "." + DestExt;
        print "MultIntAverage: " + filename + " -> " + clahename; 
        if os.path.exists(outname):
            print "Already exists: " + outname;
        else:
            #create a clahe version of each file
            cmd=MultipleIntensityAverageCmd() + ' ' + str(MeanIntensity) + ' ' + str(StdDeviation) + ' ' + DestPath + ' ' + f;
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            print "Created PID: "+ str(NewP.pid)

            Procs.append(NewP);
        
        while(len(Procs) >= numProcs):
            #Wait 10 seconds and check for openings again
            time.sleep(0.5);
            TestProcList = Procs
            for proc in TestProcList:
                if proc.poll() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(10);
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"

#Translate the _Supertile.mosaic using ir-fft
def FFT(RootPath, ImageExt):
    #Get all of the directories we should consider running ir-fft on.
    CmdCount = 0;
    
    DirList = RecurseSubdirectories(RootPath);

    #Process list
    Procs = [];
    numProcs = GetNumProcs();
    
    #Check each directory for running ir-FFT on
    for Path in DirList:
        FFTResults = os.path.join(Path, ClaheFFTMosaic());

        if os.path.exists(FFTResults) == False:
            
            #Add clahe to path if we want to use clahe images for ir-fft
            #clahepath = os.path.join(path, 'clahe');
            clahepath = path;

            files = glob.glob(os.path.join(clahepath, ImageExt));
            
            cmd=FFTCmd() + '-save ' + FFTResults + ' -data ';
            for i,v in enumerate(files):
                cmd = cmd + v + ' '
            
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            print "Created PID: "+ str(NewP.pid)

            CmdCount += 1; 
            Procs.append(NewP);

        while(len(Procs) >= numProcs):
           #Wait 10 seconds and check for openings again
           time.sleep(10)
           TestProcList = Procs
           for proc in TestProcList:
                if proc.poll() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(10)
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"

    #Remove references to clahe-based ffts from mosaic files
    for Path in DirList:
        ClaheFFTResults = os.path.join(Path, ClaheFFTMosaic());
        FFTResults = os.path.join(Path, FFTMosaic()); 
        Declahe(ClaheFFTResults, FFTResults);

    return CmdCount; 

#Translate the _Supertile.mosaic using ir-refine-translate
#Downsample indicates which level of pyramid to use, zero indicates
#no pyramid is used
def Translate(RootPath, Downsample = 0, TargetFormat = "8-bit"):
    print "Translate";
    CmdCount = 0; #Number of times a command was executed
    StartingPath = os.getcwd();
    os.chdir(RootPath);
    
    #Get all of the directories we should consider running ir-refine-translate on. 
    DirList = RecurseSubdirectories(RootPath);

    #Determine name of pyramid level
    DownSampleDirectory = DownsampleFormat % Downsample;
    
    #Check each directory for running ir-translate on
    for Path in DirList:
        ParentDir = os.path.dirname(Path);
        TargetDir = os.path.basename(Path);
        SectionName = TargetDir;
        
        #Check for the presence of a supertile .mosaic file
        InputFile = os.path.join(Path, SectionName + "_Supertile.mosaic");
        
        if(os.path.exists(InputFile) == False):
            continue;
            
        #Figure out the output directory
        OutputFile = os.path.join(Path, TranslateMosaic());
        InputImageDir = Path;
        TranslateArgs = ''; 
        
        if(Downsample != 0):
            InputImageDir = os.path.join(Path, TargetFormat);
            InputImageDir = os.path.join(InputImageDir, DownsampleFormat % Downsample);
            TranslateArgs = ' -sp ' + str(Downsample); #EM args
            print "Loading from: " + InputImageDir; 
        
        if(os.path.exists(InputImageDir) == False):
            print "Desired input image format does not exist: " + InputImageDir; 
        
        os.chdir(InputImageDir);
        
        if os.path.exists(OutputFile) == False:
            cmd=TranslateCmd() + '-save ' + OutputFile + ' -load ' + InputFile + TranslateArgs;
            print cmd;
            CmdCount += 1; 
            NewP = subprocess.call(cmd + " && exit", shell=True);
        else:
            print 'Translate output exists:\n' + InputFile + " " + OutputFile; 
            continue; 
            
 
            #Once complete we propogate the transalte.mosaic to all other directories in the volume
      
    os.chdir(StartingPath);
    return CmdCount;

def Refine(RootPath, Downsample = 0, TargetFormat = "8-bit"):
    print "Refine grid";
    CmdCount = 0; #Number of times a command was executed
    StartingPath = os.getcwd();
    os.chdir(RootPath);
    
    #Get all of the directories we should consider running ir-fft on. 
    DirList = RecurseSubdirectories(RootPath);

    #Determine name of pyramid level
    DownSampleDirectory = DownsampleFormat % Downsample;

#    print DirList;
    
    #Check each directory for running ir-FFT on
    for Path in DirList:
        ParentDir = os.path.dirname(Path);
        TargetDir = os.path.basename(Path);
        SectionName = TargetDir;
        
        #Check for the presence of a supertile .mosaic file
        InputFile = os.path.join(Path, TranslateMosaic());
        
        if(os.path.exists(InputFile) == False):
            continue;
            
        #Figure out the output directory
        OutputFile = os.path.join(Path, RefineMosaic());
        InputImageDir = Path;
        
        if(Downsample != 0):
            InputImageDir = os.path.join(Path, TargetFormat);
            InputImageDir = os.path.join(InputImageDir, DownsampleFormat % Downsample);
        
        if(os.path.exists(InputImageDir) == False):
            print "Desired input image format does not exist: " + InputImageDir; 
        
        os.chdir(InputImageDir);
               
        if os.path.exists(OutputFile) == False:

            #The new pyramid scheme requires -sh 1 because the pixel spacing is overridden by forcing
            #pixel spacing with -sp (number)
            if(Downsample == 0):
                cmd=RefineCmd() + '-sh 2' + str(Downsample) + ' -save ' + OutputFile + ' -load ' + InputFile;
            else:
                cmd=RefineCmd() + '-sh 1 -sp ' + str(Downsample) + ' -save ' + OutputFile + ' -load ' + InputFile;                
            
            print cmd;
            NewP = subprocess.call(cmd + " && exit", shell=True);
            CmdCount += 1; 
        else:
            print 'Refine output exists:\n' + InputFile + " " + OutputFile;
            
    os.chdir(StartingPath);
    return CmdCount; 
    
def Assemble(RootPath, DownsampleList = [0], TargetFormat = "8-bit"):
    print "Assemble";
    CmdCount = 0; 
    StartingPath = os.getcwd();
    os.chdir(RootPath);

    if(DownsampleList.__class__ != list):
        print "Assemble requires downsample argument to be a list";
        return;
    
    #Get all of the directories we should consider running assemble on. 
    DirList = RecurseSubdirectories(RootPath);
    
    #Process list
    Procs = [];
    numProcs = GetNumProcs();
    
    #Check each directory for running assemble on
    #We just run ir-assemble on largest mosaic and then use the PIL to downsample from that
    DownsampleList.sort();

    Downsample = DownsampleList[0];
    #Determine name of pyramid level
    DownSampleDirectory = os.path.join(TargetFormat, DownsampleFormat % Downsample);
    print "Assembling largest image using downsample " + str(Downsample) + " in directory " + str(DownSampleDirectory);

    ImagesToDownsample = list(); 
    
    for Path in DirList:
        ParentDir = os.path.dirname(Path);
        TargetDir = os.path.basename(Path);
        SectionName = TargetDir;
        
        #Check for the presence of a supertile .mosaic file
        InputFile = os.path.join(Path, RefineMosaic());
        
        if(os.path.exists(InputFile) == False):
            continue;
            
        #Figure out the output directory
        OutputFile = os.path.join(ParentDir, SectionName + '_mosaic_' + str(Downsample) + '.png');
        MaskOutputFile = os.path.join(ParentDir, SectionName + '_mask_' + str(Downsample) + '.png');
        InputImageDir = Path;
        
        if(Downsample != 0):
            InputImageDir = os.path.join(Path, TargetFormat);
            InputImageDir = os.path.join(InputImageDir, DownsampleFormat % Downsample);
        
        if(os.path.exists(InputImageDir) == False):
            print "Desired input image format does not exist: " + InputImageDir; 
        
        os.chdir(InputImageDir);

        print "Input: " + InputFile;
        print "Output: " + OutputFile;
        ImagesToDownsample.append(OutputFile);
        
        if os.path.exists(OutputFile) == False and os.path.exists(InputFile):
           
            if(Downsample == 0):
                cmd=AssembleCmd() + ' -sh 1 -save ' + OutputFile + ' -load ' + InputFile + ' -mask ' + MaskOutputFile;
            else:
                cmd=AssembleCmd() + ' -sh 1 -sp ' + str(Downsample) + ' -save ' + OutputFile + ' -load ' + InputFile + ' -mask ' + MaskOutputFile;                          
            
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            print "Created PID: "+ str(NewP.pid)
            CmdCount += 1; 
            Procs.append(NewP);

        while(len(Procs) >= numProcs):
           #Wait 10 seconds and check for openings again
           time.sleep(10)
           TestProcList = Procs
           for proc in TestProcList:
                if proc.poll() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(10)
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"


    if(len(DownsampleList) == 1):
        os.chdir(StartingPath);
        return; 

    #Go back and downsample results with PIL
    for Downsample in DownsampleList[1:]:
        for imageFile in ImagesToDownsample:
            Path = os.path.dirname(imageFile);
            fileName = os.path.basename(imageFile);
            [SectionNumber,SectionDownsample] = GetSectionInfo(fileName);
            
            os.chdir(Path);
            DestinationDir = Path;
             
            #print "Image Name: " + fileName; 
            #print "Image Path: " + Path; 
            #print "Section Name: " + str(SectionNumber); 
            #print "Section Downsample: " + str(SectionDownsample); 
            
            OriginalDownsample = SectionDownsample;
            InputFile = os.path.join(DestinationDir, ('%' + SectionFormat) % SectionNumber + '_mosaic_' + str(OriginalDownsample) + '.png');
            InputMask = os.path.join(DestinationDir, ('%' + SectionFormat) % SectionNumber + '_mask_' + str(OriginalDownsample) + '.png');
            
            print "Open image: " + InputFile;      
            
            OutputFile = os.path.join(DestinationDir, ('%' + SectionFormat) % SectionNumber + '_mosaic_' + str(Downsample) + '.png');
            OutputMask = os.path.join(DestinationDir, ('%' + SectionFormat) % SectionNumber + '_mask_' + str(Downsample) + '.png');
            if( os.path.exists(OutputFile)):
                InputFile = OutputFile; 
                OriginalDownsample = Downsample;
                continue;
            
#            im = Image.open(InputFile);

            scale = Downsample / OriginalDownsample;
#            newWidth = int(math.ceil(im.size[0] / scale));
#            newHeight =  int(math.ceil(im.size[1] / scale));
#            resized = im.resize([newWidth, newHeight], Image.ANTIALIAS);
#            print str(Downsample) + " resize: " + str(newWidth) + "," + str(newHeight) + " " + OutputFile; 
#            resized.save(OutputFile);

            cmd = "ir-shrink -sh " + str(scale) + " -load " + InputFile + " -save " + OutputFile;
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            Procs.append(NewP)

            cmd = "ir-shrink -sh " + str(scale) + " -load " + InputMask + " -save " + OutputMask;
            print cmd;
            NewP = subprocess.Popen(cmd + " && exit", shell=True);
            Procs.append(NewP)
            
            CmdCount += 1; 
            
#            del(im);
            
            InputFile = OutputFile; 
            OriginalDownsample = Downsample;

            while(len(Procs) >= numProcs):
               #Wait 1 seconds and check for openings again
               time.sleep(0.5)
               TestProcList = Procs
               for proc in TestProcList:
                    if proc.poll() != None:
                        Procs.remove(proc)
                        print "PID " + str(proc.pid) + " Terminated"

        #Keep waiting until all processes are finished
        while(len(Procs) > 0):
            #Wait 1 seconds and check for openings again
            time.sleep(1)
            TestProcList = Procs
            for proc in TestProcList:
                if proc.wait() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"
                
    os.chdir(StartingPath);
    return CmdCount; 
   
#Read a tab-delimited text file that has the columns
#Mapped Control Flip    Note
#Return a dictionary with mapped numbers as keys and all column entries as a list
def ReadorCreateStosMap(path, Downsample):
    StartingPath = os.getcwd();
    os.chdir(path);
    
    StosMapPath = os.path.join(path,"stosmap.txt"); 
    
    if(os.path.exists(StosMapPath) == False):
        print "Creating stosmap.txt";
        return CreateStosMap(path, Downsample); 
    
    print "Found existing stosmap.txt";
        
    fStosMap = open(StosMapPath,"r");
    
    StosEntries = fStosMap.readlines(); 
    #First entry is headers
    StosEntries = StosEntries[1:]; 
    
    EntryDict = dict(); 
    
    for entry in StosEntries:
        parts = entry.split("\t");
        #Sometimes I put an extra line in the file by accident or habit
        if(len(parts) < 2):
            continue;
        
        print parts; 
        MappedSection = int(parts[0]);
        ControlSection = int(parts[1]);
        
        Args = ""; 
        if len(parts) > 2:
	        Args = parts[2];
	    	        
	    #Add entry to dictionary
        EntryDict[MappedSection] = [MappedSection, ControlSection, Args];
   
    os.chdir(StartingPath);
    return EntryDict; 
    
def CreateStosMap(path, Downsample):
    StartingPath = os.getcwd();
    os.chdir(path);
    
    StosMapPath = os.path.join(path,"stosmap.txt"); 
    
    #Input files have the downsample level appended to the name
    ImageExtension = '*mosaic_' + str(Downsample) + '.png';

    
    files = glob.glob(os.path.join(path, ImageExtension));
    files.sort();

    #Can't run stos without two slices
    if(len(files) < 2):
        return dict();
        
    fStosMap = open(StosMapPath,"w");
    fStosMap.write("Mapped\tControl\tArgs\tNote\n"); 
        
    try:
        EntryDict = dict(); 
        for i in range(1, len(files)):
            #Create an entry for these files
            [ControlSection,ControlSectionDownsample] = GetSectionInfo(os.path.basename(files[i-1]));
            [MappedSection,MappedSectionDownsample] = GetSectionInfo(os.path.basename(files[i]));
           
            line = ("%(mapped)" + SectionFormat + "\t%(control)" + SectionFormat + "\n") % {'mapped' : MappedSection, 'control' : ControlSection};
            print(line); 
            fStosMap.write(line); 
            EntryDict[MappedSection] = [MappedSection, ControlSection, ""];
            
           
        fStosMap.close(); 
    except Exception, e:
        os.remove(StosMapPath);
        raise e;
        
    os.chdir(StartingPath);
    return EntryDict;
    
    
def StosBrute(RootPath, Downsample):
    print "stos-brute";
    CmdCount = 0;
    
    if(Downsample.__class__ != int):
        print "StosGrid requires downsample argument to be a list";
        return 0;
    
    #Get all of the directories we should consider running assemble on. 
    DirList = RecurseSubdirectories(RootPath);

    #Process list
    Procs = [];
    numProcs = GetNumProcs();

    #Input files have the downsample level appended to the name
    ImageExtension = '*mosaic_' + str(Downsample) + '.png';
    
    #Check each directory for running stos-brute on
    for Path in DirList:
        os.chdir(Path);
        #Find all mosaic PNG's in the directory
        files = glob.glob(os.path.join(Path, ImageExtension));
        files.sort();

        #Can't run stos without two slices
        if(len(files) < 2):
            continue;
            
        #This either creates a default map where each section is mapped to the section with the next lowest number, or we load a customized 
        #map that the user has edited/created
        stosmap = ReadorCreateStosMap(Path, Downsample);
        
        mappedSections = stosmap.keys();
        mappedSections.sort(); 

        for iMapped in mappedSections:
            
            Entry = stosmap[iMapped]; 
            args = Entry[2]; 
            iControl = Entry[1]; 
            
            ControlMosaicName = ('%' + SectionFormat) % iControl + '_mosaic_' + str(Downsample) + '.png';
            MappedMosaicName = ('%' + SectionFormat) % iMapped + '_mosaic_' + str(Downsample) + '.png';
            ControlMaskName = ('%' + SectionFormat) % iControl + '_mask_' + str(Downsample) + '.png';
            MappedMaskName = ('%' + SectionFormat) % iMapped + '_mask_' + str(Downsample) + '.png';
            
            
            #Mosaics are named ####_mosaic_##.png
            #The first numbers are the section.  The second number s
            #TODO: Extract section number for use in .stos file
#            fileNameParts = part(
            #Example output file '0002-0001_brute32.stos'
            OutputFile = ("%(mapped)" + SectionFormat + "-%(control)" + SectionFormat + "_brute_" + str(Downsample) + ".stos") % {'mapped' : iMapped, 'control' : iControl};
            
            if os.path.exists(OutputFile) == False:
                cmd=StosBruteCmd() + ' -save ' + OutputFile + ' -load ' + ControlMosaicName + " " + MappedMosaicName + "  -mask " + ControlMaskName + ' ' + MappedMaskName + " " + args; 
                print cmd;
                NewP = subprocess.Popen(cmd + " && exit", shell=True);
                CmdCount += 1; 
                print "Created PID: "+ str(NewP.pid)
                Procs.append(NewP);

            while(len(Procs) >= numProcs):
               #Wait 10 seconds and check for openings again
               time.sleep(10)
               TestProcList = Procs
               for proc in TestProcList:
                    if proc.poll() != None:
                        Procs.remove(proc)
                        print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(10)
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"
    os.chdir(RootPath);

    return CmdCount;

#Stos grid is run multiple times as it walks up the downsample levels
def StosGrid(RootPath, DownsampleList):
    print "stos-grid: " + RootPath + " Downsample List: " + str(DownsampleList);
    CmdCount = 0;
    if(DownsampleList.__class__ != list):
        print "StosGrid requires downsample argument to be a list";
        return CmdCount;
    
    #Get all of the directories we should consider running assemble on. 
    DirList = RecurseSubdirectories(RootPath);
    
    numProcs = GetNumProcs();

    #Ensure downsample list goes from high to low
    DownsampleList.sort();
    DownsampleList.reverse();

    FilesToDelete = list();
    
    #Run stos grid on each level of downsample
    for iDownsample in range(0,len(DownsampleList)):
        Downsample = DownsampleList[iDownsample]; 

        ImageExtension = '*mosaic_' + str(Downsample) + '.png';
        
        #Check each directory for running stos-grid on
        for Path in DirList:
            os.chdir(Path);
            
            #Find all mosaic PNG's in the directory
            files = glob.glob(os.path.join(Path, ImageExtension));
            files.sort();

            #Can't run stos without two slices
            if(len(files) < 2):
                continue; 
                
            stosmap = ReadorCreateStosMap(Path, Downsample);
        
            mappedSections = stosmap.keys();
            mappedSections.sort(); 

            for iMapped in mappedSections:

                print "Examining section #: " + str(iMapped); 
                Entry = stosmap[iMapped]; 
                args = Entry[2]; 
                iControl = Entry[1]; 

                #If it is first iteration we base ir-stos-grid on ir-stos-brute version.  After that we use previous ir-stos-grid transform
                if(iDownsample == 0):
                    InputFile = ("%(mapped)" + SectionFormat + "-%(control)" + SectionFormat + "_brute_" + str(Downsample) + ".stos") % {'mapped' : iMapped, 'control' : iControl};
                else:
                    InputFile = ("%(mapped)" + SectionFormat + "-%(control)" + SectionFormat + "_grid_" + str(DownsampleList[iDownsample-1]) + ".stos") % {'mapped' : iMapped, 'control' : iControl};
                    
                OutputFile = ("%(mapped)" + SectionFormat + "-%(control)" + SectionFormat + "_grid_" + str(Downsample) + ".stos") % {'mapped' : iMapped, 'control' : iControl};

                print "Input: " + InputFile;
                print "Output: " + OutputFile; 
                
                if os.path.exists(OutputFile) == False:
                    cmd=StosGridCmd() + ' -threads ' + str(numProcs) + ' -save ' + OutputFile + ' -load ' + InputFile + " " + args; 
                    print cmd;
                    subprocess.call(cmd + " && exit", shell=True);
                    CmdCount += 1; 

        #TODO Map the transform to the next downsample level and increase the refinement
        #Check each directory for running stos-grid on
        
        for iDownsample in range(1,len(DownsampleList)):
            LastDownsample = DownsampleList[iDownsample-1];
            Downsample = DownsampleList[iDownsample]; 
            for Path in DirList:
                os.chdir(Path);

                gridExtension = "*grid_" + str(LastDownsample) + ".stos";
                #Find all mosaic PNG's in the directory
                files = glob.glob(os.path.join(Path, gridExtension));
                files.sort();
                
                #If there's nothing to refine then continue
                if(len(files) < 1):
                    continue; 

                for f in files:
                    path = os.path.dirname(f); 
                    baseName = os.path.basename(f);
                   
                    #Skip any temporary files we find
                    if(baseName.startswith('Temp')):
                        continue; 
                        
                    newFileName = "Temp_" + baseName;

                    [mappedSection, controlSection, stosType, fileDownsample] = GetStosInfo(baseName);

                    #Change the spacing in the original stos-grid file and store in a temporary file
                    newFile = os.path.join(path, newFileName); 
                    ChangeStosGridPixelSpacing(f, newFile, LastDownsample, Downsample);

                    #rerun stos-grid on a higher-resolution version of the image
                    InputFile = newFile;
                    OutputFile = ("%(mapped)" + SectionFormat + "-%(control)" + SectionFormat + "_grid_" + str(Downsample) + ".stos") % {'mapped' :mappedSection, 'control' : controlSection};

                    print "Input: " + InputFile;
                    print "Output: " + OutputFile;

                    if os.path.exists(OutputFile) == False:
                        cmd=StosGridCmd() + ' -threads ' + str(numProcs) +  ' -save ' + OutputFile + ' -load ' + InputFile; 
                        print cmd;
                        subprocess.call(cmd + " && exit", shell=True);
                        CmdCount += 1; 

                    #Delete the temporary transform file
                    print "Deleting: " + InputFile;
                    os.remove(InputFile);
    
    os.chdir(RootPath);
    return CmdCount;

def ChangeStosGridPixelSpacing(filename, outfileName, oldspacing, newspacing):
    print "ChangeStosGridPixelSpacing: " + filename + " from " + str(oldspacing) + " to " + str(newspacing);
    scale = oldspacing / newspacing;

    InFile = open(filename, "r");
    OutFile = open(outfileName, "w");

    InLines = InFile.readlines();
    InFile.close();
    
    OutLines = list(); 

    #Update the filenames which are the first two lines of the file
    controlImageName = os.path.basename(InLines[0]);
    controlImagePath = os.path.dirname(InLines[0]);
    warpImageName = os.path.basename(InLines[1]);
    warpImagePath = os.path.dirname(InLines[1]);

    [ControlSectionNumber, ControlDownsample] = GetSectionInfo(controlImageName);
    [WarpSectionNumber, WarpDownsample] = GetSectionInfo(warpImageName);

    print newspacing;

    #Reference the larger mosaic files
    NewImageNameTemplate = ("%(section)" + SectionFormat + "_mosaic_" + str(newspacing) + ".png\n");
    controlNewImageName = NewImageNameTemplate % {'section' : ControlSectionNumber};
    warpNewImageName = NewImageNameTemplate % {'section' : WarpSectionNumber};

    NewMaskNameTemplate = ("%(section)" + SectionFormat + "_mask_" + str(newspacing) + ".png\n");
    controlNewMaskName = NewMaskNameTemplate % {'section' : ControlSectionNumber};
    warpNewMaskName = NewMaskNameTemplate % {'section' : WarpSectionNumber};

    OutLines.append(os.path.join(controlImagePath, controlNewImageName));
    OutLines.append(os.path.join(warpImagePath, warpNewImageName ));

    #Copy over old values from original file
    OutLines.append(InLines[2]);    
    OutLines.append(InLines[3]);

    #Scale sizes of images
    controlDims = InLines[4].split();
    warpDims = InLines[5].split();

    print controlDims; 

    controlNewDims = "%(l)e %(b)e %(r)d %(t)d\n" % {'l': float(controlDims[0]), 'b' : float(controlDims[1]), 'r' : int(controlDims[2]) * scale, 't' : int(controlDims[3]) * scale}; 
    warpNewDims = "%(l)e %(b)e %(r)d %(t)d\n" % {'l': float(warpDims[0]), 'b' : float(warpDims[1]), 'r' : int(warpDims[2]) * scale, 't' : int(warpDims[3]) * scale}; 

    OutLines.append(controlNewDims);
    OutLines.append(warpNewDims);

    #Adjust the grid points
    parts = InLines[6].split();
    outString = "";

    #Multiple all vp parts by the scalar if they are numbers
    for i in range(0,len(parts)):
        part = parts[i];

        #Pass through the transform type, the 'vp', and the number of points
        if(i < 3):
            outString = outString + part + " ";
            continue; 
            
        #run until we hit the fixed parameters
        if(part == "fp"):
            iFP = i;
            break; 

        #scale point coordinatesparts
        val = float(part) * scale;

        outString = outString + "%e " % val;

    for i in range (iFP,len(parts)):
        part = parts[i];

        if(i < iFP + 7):
            outString = outString + part + " ";
        else:
            #Last two fixed parameters are dimensions of image, multiply them
            val = float(part) * scale;
            outString = outString + "%e " % val;

    OutLines.append(outString + '\n');
    OutFile.writelines(OutLines);

    #Find the mask entries, if any and change mask downsample
    OutFile.write('two_user_supplied_masks:\n');
    OutFile.write(controlNewMaskName);
    OutFile.write(warpNewMaskName); 
    OutFile.close(); 
    
    print(OutLines); 
    


#Convert all PMG's in Rootpath and subdirectories to mosaic files
def ConvertAllPMG(RootPath, TileOverlap = 128):
    #Get all of the directories we should consider running PMGToMosaic on. 
    DirList = RecurseSubdirectories(RootPath);
    for Path in DirList:
        FFTResults = os.path.join(Path, TranslateMosaic());
        PMGToMosaic(Path, FFTResults, TileOverlap);
        
#Converts a PMG
#Tileoverlap is the amount of overlap in pixels between tiles.  If the default on the scope is
#changed this value should be changed as well.
def PMGToMosaic(path, OutputFileName=FFTMosaic(), TileOverlap=128):     
    filenames = glob.glob(os.path.join(path, '*.pmg'));

    zero= ' 0.000000000000e+00';
    one = ' 1.000000000000e+00';

    TileWidth = LMImageSize[0];
    TileHeight = LMImageSize[1];

    #Create pyramid directories if we find a .pmg
    if(len(filenames) > 0):
        CreatePyramidDirectories(path); 

    for filename in filenames:
        #Create a dictionary to store tile position
        Tiles = dict();
        
        iExt = filename.rfind('.');
        #OutFilename = filename[0:iExt] + '.mosaic'
        OutFilename =  os.path.join(path,OutputFileName);
        print "Filename: " + filename;
        OutFilename = filename[0:iExt] + "_supertile.mosaic";
        print "PMG to: " + OutFilename; 
        OutFile = open(OutFilename, 'w'); 
        SourceFile = open(filename, 'r');
        Data = SourceFile.read();

        Data = Data.replace('\r\n', '\n'); 

        Tuple = Data.partition('VISPIECES');
        Tuple = Tuple[2].partition('\n'); 
        NumPieces = int(Tuple[0]);

        Tuple = Data.partition('VISSCALE');
        Tuple = Tuple[2].partition('\n'); 
        ScaleFactor = 1/float(Tuple[0]);
        
        Tuple = Data.partition('IMAGEREDUCTION');
        Tuple = Tuple[2].partition('\n'); 
        ReductionFactor = 1/float(Tuple[0]);

        Data = Tuple[2]; 

        print("Num Tiles: " + str(NumPieces));
        print("Scale    : " + str(ScaleFactor));
        print("Reduction: " + str(ReductionFactor));
        print("Overlap  : " + str(TileOverlap));
        
        Tuple = Data.partition('PIECE');
        Data = Tuple[2];

        bFirstTile = True;

        #Default to Syncroscan defaults for current camera
        
        TileWidthMinusOverlap = 1388 - TileOverlap;
        TileHeightMinusOverlap = 1036 - TileOverlap;
        
        while Data:
            Tuple = Data.partition('ENDPIECE');
            #Remaining unprocessed file goes into data
            Data = Tuple[2]
            Entry = Tuple[0];
            #Find filename
            Tuple = Entry.partition('PATH'); 
            Tuple = Tuple[2].partition('<');
            Tuple = Tuple[2].partition('>');
            TileFilename = Tuple[0];
            #Find Position
            Tuple = Entry.partition('CORNER')
            Tuple = Tuple[2].partition('\n'); 
            Position = Tuple[0].split();
     #       print Position;
            X = float(Position[0]);
            Y = float(Position[1]);

            #Find Height & Width of tiles if it is our first tile
#            if(bFirstTile):
#                bFirstTile = False; 
#                Tuple = Entry.partition('CORNER')
#                Tuple = Tuple[2].partition('CORNER')
#                Tuple = Tuple[2].partition('CORNER')
#                Tuple = Tuple[2].partition('\n'); 
#                Position = Tuple[0].split();
            
#                TileWidth = float(Position[0]) * ScaleFactor * ReductionFactor;
#                TileHeight = float(Position[1]) * ScaleFactor * ReductionFactor;
#                print("Tile Width/Height: " + str(TileWidth) + "," + str(TileHeight)); 
            
            #Convert Position into pixel units using Reduction and Scale factors
            X = X * ScaleFactor * ReductionFactor;
            Y = Y * ScaleFactor * ReductionFactor;

            #Syncroscan lays out tiles in grids, so find the grid coordinates of this tile
            iX = round(X /  TileWidthMinusOverlap);
            iY = round(Y /  TileHeightMinusOverlap);

            print("Name,iX,iY: " + TileFilename + " " + str((iX,iY))); 

            #Add tile to dictionary
            #Using the indicies will replace any existing tiles in that location.
            #Syncroscan adds the tiles to the file in order, so the older, blurry tiles will
            #be replaced.
            
            Tiles[(iY,iX)] = TileFilename,X,Y;

            #Prime for next iteration
            Tuple = Data.partition('PIECE');
            Data = Tuple[2];
                   
        #Write the .mosaic file
        #Write standard .mosiac file header
        OutFile.write('number_of_images: ' + str(len(Tiles)) + '\n');
        OutFile.write('pixel_spacing: 1\n');
        OutFile.write('use_std_mask: 0\n'); 

        Keys = Tiles.keys();
        Keys.sort();
        for Key in Keys:
            #Convert Position into pixel units using Reduction and Scale factors
            Value = Tiles[Key];
            TileFilename = Value[0];
            X = Value[1];
            Y = Value[2]; 
            print(TileFilename + ' ' + str(X) + ' ' + str(Y));
            basename = os.path.basename(TileFilename); 
            (basefile,ext) =  os.path.splitext(basename); 
           # OutFile.write('image: ' + os.path.join(path, TileFilename) + ' LegendrePolynomialTransform_double_2_2_1 vp 6 1 0 1 1 1 0 fp 4 ' + str(X) + ' ' + str(Y) + ' 1 1\n');
            OutFile.write('image: ' + basefile + '.' + TileFormat + ' LegendrePolynomialTransform_double_2_2_1 vp 6 1 0 1 1 1 0 fp 4 ' + str(X) + ' ' + str(Y) + ' ' + str(TileWidth/2) + ' ' + str(TileHeight/2) + '\n'); 
           
        #Write 
        OutFile.close();

        ConvertImages(path,os.path.join(path,"8-bit" + os.path.sep + "001"), False, "bmp", "png"); 

        return Keys; 




#When we use circular capture we know that the center of the image is on some feature that is interesting
#tissue.  So I implemented this to take the std. dev. of only the center image in the .mrc file as an optimization
def MRCCenterIntensityRange(MrcPath):
    HeaderSize = 1024; 
    MrcFile = open(MrcPath, 'rb');
    
    #The mrc file header is always 1024, mostly empty space
    Header = MrcFile.read(HeaderSize);
    
    (EndianStamp,) = struct.unpack('I', Header[0xD4:0xD8]);
    if (EndianStamp == 17):
        Endian = '>'; #Big-Endian
    elif(EndianStamp == 68):
        Endian = '<'; #Little-Endian

    (MinPixelValue, ) = struct.unpack(Endian + 'f', Header[0x4C:0x50]);
    (MaxPixelValue, ) = struct.unpack(Endian + 'f', Header[0x50:0x54]);
    (MeanPixelValue, ) = struct.unpack(Endian + 'f', Header[0x54:0x58]);

    (PixelType, ) = struct.unpack(Endian + 'I', Header[0x0C:0x10]);

    #Calculate pixel size in bytes
    if(PixelType == 0):
        PixelSize = 1;
        PixelType = 'B'; 
    elif(PixelType == 1):
        PixelSize = 2;
        PixelType = 'h'; 
    elif(PixelType == 2):
        PixelSize = 2;
        PixelType = 'f';
    elif(PixelType == 6):
        PixelSize = 2;
        PixelType = 'H'; 
    else:
        print "ERROR: Unrecognized pixel format";
        PixelSize = 1;
        PixelType = 'B'; 
    
    #Determine the standard deviation by taking a small subsample of the pixels
    (NumSections,) = struct.unpack(Endian + 'I', Header[0x08:0x0C]);
    (TileHeaderSize,) = struct.unpack(Endian + 'H', Header[0x80:0x82]);
    (NumCols,) = struct.unpack(Endian + 'I', Header[0x00:0x04]);
    (NumRows,) = struct.unpack(Endian + 'I', Header[0x04:0x08]);

    ImageStart = HeaderSize + (NumSections * TileHeaderSize);

    TargetSection = math.floor(NumSections/2);

    print "Targeting section " + str(TargetSection) + " for contrast.";


 
    NumPixels = long(NumCols) * long(NumRows); # We only sample the tile near the center
    NumSamples = math.sqrt(NumPixels); #Number of pixels to sample from image
    print "Num Pixels: " + str(NumPixels)  + " Num Samples: " + str(NumSamples);

    random.seed();

    Offsets = list(); 
    for i in range(0, long(NumSamples)):
        Offsets.append(ImageStart + random.randint(0, NumPixels)); 

    Offsets.sort();

    StdDevScalar = float(1 / (NumSamples - 1));
    print "Scalar: " +str(StdDevScalar); 
    Total = float();

    ProgressReadout = 0.00; #I use this to indicate how far into calculating the mean we are

    print "PixelSize: " + str(PixelSize) + " PixelType: " + PixelType;
    print "NumSamples: " + str(NumSamples);

    FileCache = [];
    FileCacheOffset = 0;

    TargetSectionOffset = long(NumCols) * long(NumRows) * long(TargetSection) * long(PixelSize);

    IValues = []; 

#   Functional Unoptimized implementation
    for Offset in Offsets:
        MrcFile.seek(TargetSectionOffset + (Offset * PixelSize));
        FileData = MrcFile.read(PixelSize);

        LoopLimit = 10000;
        iLoopCount =0; 
        while len(FileData) < PixelSize:
            iLoopCount += 1;
            FileData = FileData + MrcFile.read(PixelSize - len(FileData));
            if len(FileData) < PixelSize:
                print 're-read';
            if iLoopCount > LoopLimit:
                print 'Exceed max read retries, aborting read';
                return; 

#        print 'Offset: ' + str(Offset) + " FileData: " + str(len(FileData));
        (Intensity,) = struct.unpack(Endian + PixelType, FileData);

        IValues.append(long(Intensity));

        Percent = (Offset-ImageStart) / float(NumPixels);
        if(Percent >= ProgressReadout):
            print '%02d'%(Percent*100) + '%';
            ProgressReadout = ProgressReadout + .01;

    MeanPixelValue = sum(IValues) / float(NumSamples);

    for I in IValues:
        Temp = float(I) - MeanPixelValue;
        Temp = Temp * Temp
        Total = Total + (Temp * StdDevScalar);
        
    StdDevPixelValue = math.sqrt(Total); 

    MrcFile.close();

    return (MinPixelValue, MaxPixelValue, MeanPixelValue, StdDevPixelValue);

#Get range of pixel intensity values from MRC File
def MRCIntensityRange(MrcPath):
    HeaderSize = 1024; 
    MrcFile = open(MrcPath, 'rb');
    
    #The mrc file header is always 1024, mostly empty space
    Header = MrcFile.read(HeaderSize);
    
    (EndianStamp,) = struct.unpack('I', Header[0xD4:0xD8]);
    if (EndianStamp == 17):
        Endian = '>'; #Big-Endian
    elif(EndianStamp == 68):
        Endian = '<'; #Little-Endian

    (MinPixelValue, ) = struct.unpack(Endian + 'f', Header[0x4C:0x50]);
    (MaxPixelValue, ) = struct.unpack(Endian + 'f', Header[0x50:0x54]);
    (MeanPixelValue, ) = struct.unpack(Endian + 'f', Header[0x54:0x58]);

    (PixelType, ) = struct.unpack(Endian + 'I', Header[0x0C:0x10]);

    #Calculate pixel size in bytes
    if(PixelType == 0):
        PixelSize = 1;
        PixelType = 'B'; 
    elif(PixelType == 1):
        PixelSize = 2;
        PixelType = 'h'; 
    elif(PixelType == 2):
        PixelSize = 2;
        PixelType = 'f';
    elif(PixelType == 6):
        PixelSize = 2;
        PixelType = 'H'; 
    else:
        print "ERROR: Unrecognized pixel format";
        PixelSize = 1;
        PixelType = 'B'; 
    
    #Determine the standard deviation by taking a small subsample of the pixels
    (NumSections,) = struct.unpack(Endian + 'I', Header[0x08:0x0C]);
    (TileHeaderSize,) = struct.unpack(Endian + 'H', Header[0x80:0x82]);
    (NumCols,) = struct.unpack(Endian + 'I', Header[0x00:0x04]);
    (NumRows,) = struct.unpack(Endian + 'I', Header[0x04:0x08]);

    ImageStart = HeaderSize + (NumSections * TileHeaderSize);
 
    NumPixels = long(NumCols) * long(NumRows) * long(NumSections);
    NumSamples = math.sqrt(NumPixels); #Number of pixels to sample from image
    print "Num Pixels: " + str(NumPixels)  + " Num Samples: " + str(NumSamples);

    random.seed();


    Offsets = list(); 
    for i in range(0, long(NumSamples)):
        Offsets.append(ImageStart + random.randint(0, NumPixels)); 

    Offsets.sort();

    StdDevScalar = float(1 / (NumSamples - 1));
    print "Scalar: " +str(StdDevScalar); 
    Total = float();

    ProgressReadout = 0.00; #I use this to indicate how far into calculating the mean we are

    print "PixelSize: " + str(PixelSize) + " PixelType: " + PixelType;
    print "NumSamples: " + str(NumSamples);

    FileCache = [];
    FileCacheOffset = 0; 

#   Functional Unoptimized implementation
    for Offset in Offsets:
        MrcFile.seek(Offset * PixelSize);
        FileData = MrcFile.read(PixelSize);
      
        while len(FileData) < PixelSize:
            FileData = FileData + MrcFile.read(PixelSize - len(FileData));
            if len(FileData) < PixelSize:
                print 're-read';

#        print 'Offset: ' + str(Offset) + " FileData: " + str(len(FileData));
        (Intensity,) = struct.unpack(Endian + PixelType, FileData);           

        Temp = float(Intensity - MeanPixelValue);
        Temp = Temp * Temp
        Total = Total + (Temp * StdDevScalar);

        Percent = (Offset-ImageStart) / float(NumPixels);
        if(Percent >= ProgressReadout):
            print '%02d'%(Percent*100) + '%';
            ProgressReadout = ProgressReadout + .01;
        
    StdDevPixelValue = math.sqrt(Total); 

    MrcFile.close();

    return (MinPixelValue, MaxPixelValue, MeanPixelValue, StdDevPixelValue);

#Returns [XSize,YSize, NumSections] from MRC File
def MRCSizeAndSections(MrcPath):
    HeaderSize = 1024; 
    MrcFile = open(MrcPath, 'rb');
    
    #The mrc file header is always 1024, mostly empty space
    Header = MrcFile.read(HeaderSize);
    
    (EndianStamp,) = struct.unpack('I', Header[0xD4:0xD8]);
    if (EndianStamp == 17):
        Endian = '>'; #Big-Endian
    elif(EndianStamp == 68):
        Endian = '<'; #Little-Endian

    (SizeX,) = struct.unpack(Endian + 'I', Header[0x00:0x04]);
    (SizeY,) = struct.unpack(Endian + 'I', Header[0x04:0x08]);
    (NumSections,) = struct.unpack(Endian + 'I', Header[0x08:0x0C]);

    MrcFile.close();

    return [SizeX,SizeY,NumSections];

def CroppingOffsets():
    #Crop 8 pixels off both the X & Y borders
    return (8,8); 

#Generate the basename of a file without the extension
def prefix(Filename):
	 return os.path.splitext(os.path.basename(Filename))[0];

#Generate the tif name for a given tile in a mrc file
def PredictTileName(TilePrefix, i ):
    return TilePrefix + '.' + str(i).zfill(3);
    
#Converts every mrc file in a directory into one mosaic based on stage position
def MRCsToMosaic(dirpath):

    (CroppingXOffset, CroppingYOffset) = CroppingOffsets(); 

    #MRC Header info:
    #http://bio3d.colorado.edu/imod/doc/mrc_format.txt
    #For each tile in the MRC the 
    def ExtractCoordFromMrc(MrcPath):
        print MrcPath;

        HeaderSize = 1024; 
        MrcFile = open(MrcPath, 'rb');

        #Get the prefix of the path
        TileNamePrefix = prefix(MrcPath); 
     
        #The mrc file header is always 1024, mostly empty space
        Header = MrcFile.read(HeaderSize);

        print('Header Length:' + str(len(Header)));

        (EndianStamp,) = struct.unpack('I', Header[0xD4:0xD8]);
        if (EndianStamp == 17):
            Endian = '>'; #Big-Endian
            print 'Big Endian';
        elif(EndianStamp == 68):
            Endian = '<'; #Little-Endian
            print 'Little Endian';
        else:
            Endian = '<'; #Little-Endian
            print 'ENDIAN Value incorrect, defaulting to Little Endian'; 


        (ImgCols,) = struct.unpack(Endian + 'I', Header[0x00:0x04]);
        (ImgRows,) = struct.unpack(Endian + 'I', Header[0x04:0x08]);

        (mx,) = struct.unpack(Endian + 'I', Header[0x1C:0x20]);
        (my,) = struct.unpack(Endian + 'I', Header[0x20:0x24]);
        (mz,) = struct.unpack(Endian + 'I', Header[0x24:0x28]);

        (xlen,) = struct.unpack(Endian + 'f', Header[0x28: 0x2C]);
        (ylen,) = struct.unpack(Endian + 'f', Header[0x2C: 0x30]);
        (zlen,) = struct.unpack(Endian + 'f', Header[0x30: 0x34]); 

        (NumSections,) = struct.unpack(Endian + 'I', Header[0x08:0x0C]);
        (TileHeaderSize,) = struct.unpack(Endian + 'H', Header[0x80:0x82]);
        (TileHeaderFlags,) = struct.unpack(Endian + 'H', Header[0x82:0x84]);

        (MinPixelValue, ) = struct.unpack(Endian + 'f', Header[0x4C:0x50]);
        (MaxPixelValue, ) = struct.unpack(Endian + 'f', Header[0x50:0x54]);
        (MeanPixelValue, ) = struct.unpack(Endian + 'f', Header[0x54:0x58]);


        
        print "Number of Image Columns: " + str(ImgCols);
        print "Number of Image Rows   : " + str(ImgRows); 
        print "Number of Tiles  : " + str(NumSections);
        print;
        print "Intensity Min: " + str(MinPixelValue);
        print "Intensity Max: " + str(MaxPixelValue);
        print "Intensity Mean: " + str(MeanPixelValue); 
        print;
        print "Grid Size X: " + str(mx);
        print "Grid Size Y: " + str(my);
        print "Grid Size Z: " + str(mz);
        print "Cell Size X: " + str(xlen);
        print "Cell Size Y: " + str(ylen);
        print "Cell Size Z: " + str(zlen); 
        
    #    print TileHeaderSize;
    #    print TileHeaderFlags;
        bHasPixelPosition = True; 
        if (TileHeaderFlags & 0x02) == 0: #Piece coordinates
            print "Pixel coordinates not present in header";
            bHasPixelPosition = False;
            
        if (TileHeaderFlags & 0x04) == 0:
            print "Stage Position Not Present in Header";
            return dict(); 

        #Determine the offset in the tile header of the stage position    
        StagePositionOffset = 0;
        PixelPositionOffset = 0; 
        if(TileHeaderFlags & 0x01): #tilt angle
            StagePositionOffset += 2;
            PixelPositionOffset += 2; 
        if(TileHeaderFlags & 0x02): #piece coordinates
            StagePositionOffset += 6;

        PositionMap = dict();

        print "Num Sections: " + str(NumSections);

        #Used to roughly translate stage space into pixel space
        PixelStartX = 0.0;
        PixelStartX = 0.0;
        StageStartX = 0.0;
        StageStartY = 0;.0 

        PixelEndX = 0.0;
        PixelEndY = 0.0;
        StageEndX = 0.0;
        StageEndY = 0.0;

        AbsoluteMaxStageX = 0.0; #This is used to center supermosaics
        AbsoluteMaxStageY = 0.0;

        #The supermontage combines montages, but montages may only have one or two tiles if the most
        #of the montage was outside the bounding polygon.  So I find a montage with most of its members
        #and use that to calculate the scalar
        MontageTileCount = dict();
        for i in range(NumSections):
            MrcFile.seek(HeaderSize + (i * TileHeaderSize));
            TileHeader = MrcFile.read(TileHeaderSize);
            while len(TileHeader) < TileHeaderSize:
                print 're-read';
                TileHeader = TileHeader + MrcFile.read(TileHeaderSize - len(TileHeader));

            Z = 0; 
            if(bHasPixelPosition):
                (Z,) = struct.unpack(Endian + 'H', TileHeader[PixelPositionOffset+4:PixelPositionOffset+6]);
                

            count = 1; 
            if(MontageTileCount.has_key(Z)):
                count = MontageTileCount[Z] + 1;

            MontageTileCount[Z] = count;


        #Figure out which Z level has the highest count
        counts = MontageTileCount.values();
        highCount = max(counts);

        print MontageTileCount;

        TargetZLevel = 0; 

        for i in range(len(counts)):
            if(counts[i] == highCount):
                TargetZLevel = i;
                break;

        print "Target Z Level: " + str(TargetZLevel);

        #Set to True when we record the first tile of the target montage
        InitializedStart = False;

        PixelX = 0.0;
        PixelY = 0.0;
        PixelZ = 0.0; 
        
        
        #read each extended header and add to dictionary
        for i in range(NumSections):
            MrcFile.seek(HeaderSize + (i * TileHeaderSize));
            
            TileHeader = MrcFile.read(TileHeaderSize);
            
            while len(TileHeader) < TileHeaderSize:
                print 're-read';
                TileHeader = TileHeader + MrcFile.read(TileHeaderSize - len(TileHeader));
                    

            #Read the information contained in the header

            #We have to make an estimated mapping of stage space to pixel space.  The MRC may actually be a stack of
            #seperate montages.  In this case we need to make our estimate using only one layer of the stack.  This
            #approach fails horribly if each layer only has one tile, but we should never be doing that.

            #Read tiles position in space
            if(bHasPixelPosition):
                (PixelX,) = struct.unpack(Endian + 'H', TileHeader[PixelPositionOffset:PixelPositionOffset+2]);
                (PixelY,) = struct.unpack(Endian + 'H', TileHeader[PixelPositionOffset+2:PixelPositionOffset+4]);
                (PixelZ,) = struct.unpack(Endian + 'H', TileHeader[PixelPositionOffset+4:PixelPositionOffset+6]);
                
            (StageX,) = struct.unpack(Endian + 'h', TileHeader[StagePositionOffset:StagePositionOffset+2]);
            (StageY,) = struct.unpack(Endian + 'h', TileHeader[StagePositionOffset+2:StagePositionOffset+4]);

            StageX = float(StageX) / 25.0;
            StageY = float(StageY) / 25.0;

            #Temp: Read the actual pixel position
            if PixelZ == TargetZLevel and InitializedStart == False:
                InitializedStart = True;
                PixelStartX = PixelX;
                PixelStartY = PixelY;
                PixelStartZ = PixelZ;
                StageStartX = StageX;
                StageStartY = StageY;
                PixelEndX = PixelStartX;
                PixelEndY = PixelStartY;
                StageEndX = StageStartX;
                StageEndY = StageStartY;
                AbsoluteMaxStageX = StageStartX;
                AbsoluteMaxStageX = StageStartY;

            #Make sure tile is on the same z level as the first measurement
            #print "PixelZ: " + str(PixelZ); 

            #Find the end of the first montage so we can map stage->pixel space
            if PixelZ == TargetZLevel:
                if(StageX > StageEndX):
                    StageEndX = StageX;
                    PixelEndX = PixelX;
                if(StageY > StageEndY):
                    StageEndY = StageY;
                    PixelEndY = PixelY;

            #Find absolute stage limits so we can center the mosaic on 0,0
            if(AbsoluteMaxStageX < StageX):
                AbsoluteMaxStageX = StageX;
            if(AbsoluteMaxStageY < StageY):
                AbsoluteMaxStageY = StageY;


            #Determine the name of the .tif that will be generated for this tile
            PredictedTileName = PredictTileName(TileNamePrefix, i) + '.' + TileFormat; 

            PositionMap[PredictedTileName] = (StageX, StageY);

        if(StageEndX == StageStartX or bHasPixelPosition == False):
            XScalar = 429.733727811; #Values from 5000x magnification
        else: 
            XScalar = (PixelEndX - PixelStartX) / (StageEndX - StageStartX);

        if(StageEndY == StageStartY  or bHasPixelPosition == False):
            YScalar = -448.302469136;
        else:
            YScalar = -(PixelEndY - PixelStartY) / (StageEndY - StageStartY);
      
        print "X/Y Stage Start: " + str(StageStartX) + "\t" + str(StageStartY);
        print "X/Y Stage End:   " + str(StageEndX) + "\t" + str(StageEndY);
        print "Max X/Y Stage:   " + str(AbsoluteMaxStageX) + "\t" + str(AbsoluteMaxStageY);
        print "X/Y Pixel Start: " + str(PixelStartX) + "\t" + str(PixelStartY);
        print "X/Y Pixel End:   " + str(PixelEndX) + "\t" + str(PixelEndY);
        print "X Scale Factor:  " + str(XScalar);
        print "Y Scale Factor:  " + str(YScalar);


        #Center the tiles around 0,0 - this breaks with multiple MRC's
#        XCenterOffset = (AbsoluteMaxStageX - StageStartX) / 2;
#        YCenterOffset = (AbsoluteMaxStageY - StageStartY) / 2;

#        print "X Centering Offset: "+ str(XCenterOffset);
#        print "Y Centering Offset: " +str(YCenterOffset);
        
        #Calculate the scalar value for stage position to pixel space and adjust values
        keys = PositionMap.keys();
        keys.sort();
        for key in keys:

            Coord = PositionMap[key];

            print "Tile: " + key + " Stage X,Y: " + str(Coord);
            
            XPos = (float(Coord[0] ) * XScalar) + CroppingXOffset;
            YPos = (float(Coord[1] ) * YScalar) + CroppingYOffset;
            PositionMap[key] = (XPos, YPos);
    #    print str(PositionMap);
        MrcFile.close(); 
        return PositionMap

    #Creates a .mosaic file in the specified directory containing the specified
    #Dictionary (Entries) at the specified Downsample rate
    #Setting Flip to True will invert all X coordinates
    def WriteMosaicFile(OutfilePath, Entries, Flip = False, Downsample = 1):

        OutFile = open(OutfilePath, 'w+');

        #TODO - find min/max values and center image in .mosaic file
        minX = sys.maxint;
        minY = sys.maxint;
        maxX = -sys.maxint - 1;
        maxY = -sys.maxint - 1;

        keys = Entries.keys();
        keys.sort();
        
        for key in keys:
            Coord = Entries[key];
            if(Coord[0] < minX):
                minX = Coord[0];
            if(Coord[0] > maxX):
                maxX = Coord[0];
            if(Coord[1] < minY):
                minY = Coord[1];
            if(Coord[1] > maxY):
                maxY = Coord[1];
                

        CenterXOffset = (maxX - minX) / 2;
        CenterYOffset = (maxY - minY) / 2;

   #     print 'MinX:\t' + str(minX) + '\tMaxX:\t' + str(maxX);
   #     print 'MinY:\t' + str(minY) + '\tMaxY:\t' + str(maxY); 
        
        #Write standard .mosiac file header
        OutFile.write('number_of_images: ' + str(len(Entries)) + '\n');
        OutFile.write('pixel_spacing: ' + str(Downsample) + '\n');
        OutFile.write('use_std_mask: 0\n');

    #    print "Keys: " + str(Entries.keys());
        keys = Entries.keys();
        keys.sort();
        
        for key in keys:
            Coord = Entries[key];
            Coord = (Coord[0] - (minX + CenterXOffset), Coord[1] - (minY + CenterYOffset));
            X = Coord[0];
            Y = Coord[1];
            if Flip:
                X = -X;
            outstr = 'image: ' + key + ' LegendrePolynomialTransform_double_2_2_1 vp 6 1 0 1 1 1 0 fp 4 ' + str(X) + ' ' + str(Y) + ' ' + str(EMImageSize[0] / 2) + ' ' + str(EMImageSize[1] / 2);
            OutFile.write(outstr + '\n');
  #          print outstr;
            
        OutFile.close();

    #MAIN    
    path = dirpath;

    DirList = RecurseSubdirectories(path);

    print DirList;
    
    #Check each directory for .MRC's
    for path in DirList:

        filenames = glob.glob(os.path.join(path, '*.mrc'));
        if(len(filenames) == 0):
            continue; 
        
        SectionName = os.path.basename(path);
        SectionParts = SectionName.split('_'); 
        SectionNumber = int(SectionParts[0]); 
        ParentDir = os.path.dirname(path);
        
        FlipList = GetFlipList(ParentDir);

        StartingPath = os.getcwd();
        os.chdir(path);

        CreatePyramidDirectories(path); 

        #Create a supertile mosaic file for all .mrc's:
        SupertileName = SectionName + "_Supertile.mosaic";
        if os.path.exists(SupertileName) == False:

            #Create a dictionary that maps predicted filename to coordinates
            Entries = dict();

            #Create one .mosaic for each .mrc and one supertile .mosaic combining all of them
            for filename in filenames:
                if os.path.exists(MosaicName(filename)) == False:
                    NewEntries = ExtractCoordFromMrc(filename);
                    MosiacFilePath = os.path.join(path, MosaicName(filename)); 
                    #WriteMosaicFile(MosiacFilePath, NewEntries, False, 1);
                    Entries.update(NewEntries);

            
            print "Supertile Mosaic: " + SupertileName;
            Flip = SectionNumber in FlipList;
            if Flip:
                    print "FLIPPING SECTION " + str(SectionNumber);
            WriteMosaicFile(SupertileName, Entries, Flip);

            #Don't need these since ir-tools feature -sp option to override spacing
            #WriteMosaicFile(os.path.join(pyPath, SectionName + "_Supertile.mosaic"), Entries, False, level);
            #print "Supertile Mosaic: " + SectionName + "_Supertile.mosaic";

        os.chdir(StartingPath);
        
    os.chdir(dirpath);

def CreatePyramidDirectories(path):
    
    for IType in ImageTypes:
            pyImageTypePath = os.path.join(path, IType);

            try:
                os.mkdir(pyImageTypePath);
#                print 'Created ' + pyImageTypePath; 
            except OSError, (errno, strerror):
                if errno==17:
#                    print "Already exists: " + pyImageTypePath; 
                    pass


            #Create a directory for each level of the pyramid
            for level in PyramidLevels:
                pyPath = os.path.join(pyImageTypePath, '%03d' % level);
                
                #Create the directory if we need to
                try:
                    os.mkdir(pyPath);
                except OSError, (errno, strerror):
                    if errno==17:
#                       print 'Created Pyramid Level: ' + pyPath;
                        pass


#Extract 16-bit TIF's from MRC files.  Execute clahe on MRC files
def MRCsTo16BitPNGs(dirpath):
    (CroppingXOffset, CroppingYOffset) = CroppingOffsets();

    DirList = RecurseSubdirectories(dirpath);

    ImageType = "MultIntAvg"; 

    print DirList;
    
    #Check each directory for running ir-FFT on
    for path in DirList:
        
        StartingPath = os.getcwd();
        os.chdir(path);

        Flip = False; 
        FlipList = GetFlipList(os.path.dirname(path)); 

        filenames = glob.glob(os.path.join(path, '*.mrc'));

        for filename in filenames:
            print "CWD: " + os.getcwd(); 
            ImgFiles = list();
            SectionName = os.path.basename(path);
            parts = SectionName.split("_"); 
            sectionNumber = int(parts[0]);
            
            [XSize,YSize,NumSections] = MRCSizeAndSections(filename);

            #There is an 8-pixel border of garbage around each tile.  Crop these once and replace the original MRC file if we need to. 
            if(XSize== 4096 and YSize ==4096):
                croppedMRCName = "cropped" + prefix(filename) + ".mrc"; 
               
                #Crop border around images
                cmd = "newstack -input " + filename + " -output " + croppedMRCName + " -mode 1 -size " + str(EMImageSize[0]) + ',' + str(EMImageSize[1]);
                print cmd;
                subprocess.call(cmd + " && exit", shell=True);

                #replace the original MRC
                os.remove(filename);
                shutil.move(croppedMRCName, filename);
                
            #Check if the last section already exists as a TIF, if it does we will skip conversion
            iFinal = NumSections-1;
            TileNamePrefix = prefix(filename);
            FinalPngName = "16-bit" + os.path.sep + "001" + os.path.sep + PredictTileName(TileNamePrefix, iFinal) + ".png";
            FinalTifName = "16-bit" + os.path.sep + "001" + os.path.sep + PredictTileName(TileNamePrefix, iFinal) + ".tif";
            FinalPyramidPng = ImageType + os.path.sep + "002" + os.path.sep + PredictTileName(TileNamePrefix, iFinal) + ".png";
            bBuildPyramid = False;

            #write down all of the tif's we've created
            for i in range(NumSections):
                ImgFiles.append(os.path.join( PredictTileName(TileNamePrefix, i)));

            if os.path.exists(FinalPyramidPng) == False:
                print str(FinalPyramidPng) + " does not exist, building pyramid"; 
                bBuildPyramid = True;

            ImageOutputPath = path + os.path.sep + "16-bit" + os.path.sep + "001";
            
                   
            #Determine if we need to extract tif files
            if os.path.exists(FinalTifName) or os.path.exists(FinalPngName):
                print str(filename) + " already converted to TIFs"
            else:
                print str(FinalTifName) + " does not exist"
                bBuildPyramid = True; 
    
                #Create tif's from 16-bit MRC
                cmd = "mrc2tif " + filename + " " + ImageOutputPath + os.path.sep + prefix(filename); 
                print cmd;
                subprocess.call(cmd + " && exit", shell=True);

                #Flip tif's if needed and convert to png's
#                Flip = sectionNumber in FlipList;
#                ConvertImages(ImageOutputPath,ImageOutputPath, Flip);

            #Flip images if needed, at least put them in the right place
            if(os.path.exists(FinalPngName) == False):
                Flip = sectionNumber in FlipList;
                ConvertImages(ImageOutputPath,ImageOutputPath, Flip); 

            print str(os.path.exists(FinalTifName)) + " " + str(os.path.exists(FinalPngName));

            ImageClahePath = path + os.path.sep + ImageType + os.path.sep + "001";

            #Create Blobs from 16-bit png files into blob PNG's
#            Blob(ImageOutputPath, ImageClahePath, "png", "png");
#            (MinPixelValue, MaxPixelValue, MeanPixelValue, StdDevPixelValue) = MRCIntensityRange(filename);
#            MultipleIntensityAverage(ImageOutputPath, ImageClahePath, "tif", "png", MeanPixelValue, StdDevPixelValue); 
        
        os.chdir(StartingPath);
    os.chdir(dirpath);

#Extract 8-bit TIF's from all MRC files in path.  Does not convert if the last tif in MRC already exists.
def MRCsTo8BitPNG(dirpath):
    DarkPercentThreashold = 0.02; #Remove tiles below this percentage of dark pixels
    
    (CroppingXOffset, CroppingYOffset) = CroppingOffsets();

    DirList = RecurseSubdirectories(dirpath);

    print DirList;
    
    #Check each directory for running ir-FFT on
    for path in DirList:
        
        StartingPath = os.getcwd();
        os.chdir(path);

        Flip = False; 
        FlipList = GetFlipList(os.path.dirname(path)); 

        filenames = glob.glob(os.path.join(path, '*.mrc'));

        for filename in filenames:
            print "CWD: " + os.getcwd(); 
            ImgFiles = list();
            SectionName = os.path.basename(path);
            parts = SectionName.split("_"); 
            sectionNumber = int(parts[0]);
            
            [XSize,YSize,NumSections] = MRCSizeAndSections(filename);

            #Check if the last section already exists as a TIF, if it does we will skip conversion
            iFinal = NumSections-1;
            TileNamePrefix = prefix(filename);
            FinalTifName = "8-bit" + os.path.sep + "001" + os.path.sep + PredictTileName(TileNamePrefix, iFinal) + ".tif";
            FinalPngName = "8-bit" + os.path.sep + "001" + os.path.sep + PredictTileName(TileNamePrefix, iFinal) + ".png";
            FinalPyramidPng = "8-bit" + os.path.sep + "002" + os.path.sep + PredictTileName(TileNamePrefix, iFinal) + ".png";
            tifExtractionPath = "8-bit" + os.path.sep + "001"
            bBuildPyramid = False;

            #write down all of the tif's we've created
            for i in range(NumSections):
                ImgFiles.append(os.path.join( PredictTileName(TileNamePrefix, i)));

            if os.path.exists(FinalPyramidPng) == False:
                print str(FinalPyramidPng) + " does not exist, building pyramid"; 
                bBuildPyramid = True; 

            if os.path.exists(FinalTifName) or os.path.exists(FinalPngName):
                print str(filename) + " already extracted tiles from MRC"
            else:
                print str(FinalPngName) + " does not exist. Extracting images from MRC"
                bBuildPyramid = True; 

                #Create tif's in same directory as .
        #        try:
        #            os.mkdir(TileNamePrefix);
        #        except:
        #            pass
                
                #Convert MRC to 8-bit MRC
                (MinPixelValue, MaxPixelValue, MeanPixelValue, StdDevPixelValue) = MRCIntensityRange(filename);
                print "Min: " + str(MinPixelValue) + " Max: " + str(MaxPixelValue) + " Mean: " + str(MeanPixelValue) + " Std: " + str(StdDevPixelValue);

                if(MaxPixelValue == 32767):
                    MaxPixelValue = MeanPixelValue + (MeanPixelValue - MinPixelValue); 
                    
                byteMRCName = "byte" + prefix(filename) + ".mrc"; 
                MinPixelValue = MeanPixelValue - (2.5 * StdDevPixelValue);
                MaxPixelValue = MeanPixelValue + (2.5 * StdDevPixelValue); 
                mult = 255/(float(MaxPixelValue) - float(MinPixelValue));
                add = -MinPixelValue * mult;
                #Scale the new stack to 8-bit depth and crop border around images
                cmd = "newstack -input " + filename + " -output " + byteMRCName + " -mode 0 -multadd " + str(mult) + "," + str(add) + " -size " + str(EMImageSize[0]) + ',' + str(EMImageSize[1]);
                print cmd;
                subprocess.call(cmd + " && exit", shell=True);

        #        os.chdir(TileNamePrefix); 

                #Create tif's from 8-bit MRC
                
                cmd = "mrc2tif " + byteMRCName + " " + tifExtractionPath + os.path.sep + prefix(filename); 
                print cmd;
                subprocess.call(cmd + " && exit", shell=True);
                
                #Remove temp 8-bit MRC
                os.remove(byteMRCName);

            #Flip images if needed, at least put them in the right place
            if(os.path.exists(FinalPngName) == False):
                Flip = sectionNumber in FlipList;
                ConvertImages(tifExtractionPath,tifExtractionPath, Flip); 
        
        os.chdir(StartingPath);
    os.chdir(dirpath);
    
def GetFlipList(path):
    FlippedSections = list();
    
    flipFileName = os.path.join(path, 'FlipList.txt');
    if os.path.exists(flipFileName) == False:
        return FlippedSections; 
	
    flipFile = open(flipFileName, 'r');
    lines = flipFile.readlines();
    flipFile.close(); 
    
    for line in lines:
        sectionNumber = int(line);
        FlippedSections.append(sectionNumber);
	
    return FlippedSections;

#Convert all images in the specified path to the new extension type, flipping the images if necessary
def ConvertImages(SourcePath, TargetPath, Flip = False, SourceExt ='tif', TargetExt='png', bDeleteOriginal = True):
    files = glob.glob(SourcePath + os.path.sep + "*." + SourceExt);

    Procs = [];
    numProcs = GetNumProcs() * 1.25; #ir-flip spends about half the time loading from disk...
                                  #doubling the number of procs should keep the CPU busy

    originalFileName = "";
    targetFileName = ""; 

    for f in files:
        f = os.path.basename(f);
        [filename, ext] = os.path.splitext(f);
        print filename;

        originalFileName = SourcePath + os.path.sep + filename + '.' + SourceExt; #We put the original images in the 001 directory
        targetFileName = TargetPath + os.path.sep + filename + '.' + TargetExt; #What we want the file to be named when we're done
        
        print originalFileName + ' -> ' + targetFileName;

        if(os.path.exists(targetFileName)):
            continue; 

        #Find out if we need to flip the image
        if Flip:
            #cmd = "ir-flip -flipx -load " + originalFileName + " -save " + targetFileName;
            #Using image magick to work around 16-bit .png ITK bug
            cmd = "convert " + originalFileName + " -type Grayscale -flop " + targetFileName; 
            print cmd; 
            proc = subprocess.Popen(cmd + " && exit", shell=True);
            Procs.append(proc); 
        elif(originalFileName != targetFileName):
            #I'm calling ir-flip, but really I'm just using it as an image converter by omitting -flip flags
            #cmd = "ir-flip -load " + originalFileName + " -save " + targetFileName;
            cmd = "convert " + originalFileName + " -type Grayscale " + targetFileName; 
            print cmd; 
            proc = subprocess.Popen(cmd + " && exit", shell=True);
            Procs.append(proc); 
        else:
            #Nothing to do, source and target names match and no flipping required, skip everything
            return;

        #A slight pause to offset process launch
        #time.sleep(0.75); 

        while(len(Procs) >= numProcs):
            #Wait 10 seconds and check for openings again
            time.sleep(0.1);
            TestProcList = Procs
            for proc in TestProcList:
                if proc.poll() != None:
                    Procs.remove(proc)
                    print "PID " + str(proc.pid) + " Terminated"

    #Keep waiting until all processes are finished
    while(len(Procs) > 0):
        #Wait 10 seconds and check for openings again
        time.sleep(0.5);
        TestProcList = Procs
        for proc in TestProcList:
            if proc.wait() != None:
                Procs.remove(proc)
                print "PID " + str(proc.pid) + " Terminated"

    if bDeleteOriginal and (originalFileName != targetFileName):
        for f in files:
            filename = os.path.basename(f);
            originalFileName = SourcePath + os.path.sep + filename; #We put the original images in the 001 directory
            print "Deleting: " + originalFileName; 
            os.remove(originalFileName);

#Build the image pyramid for the specified path.  We expect the "001" level of the pyramid to be pre-populated
def BuildPyramids(path, Extension = 'png'):

    CmdCount = 0; 
    #TODO: Once we have an ir-shrink that doesn't remap, then include 16-bit images in processing by removing it
    #from the exclude list passed to RecurseSubdirectories
    dirs = RecurseSubdirectories(path, ["mbproj","16-bit","001","002","004","008","016","032","064","128"]);

    Procs = [];
    numProcs = GetNumProcs();
    
    for PyramidPath in dirs:

        #Create a pyramid if we find a path that contains images
        if(PyramidPath in ImageTypes == False):
            continue;

        print "Checking path for unbuilt pyramids: " + os.path.join(path, PyramidPath); 
        
        for i in range(1,len(PyramidLevels)):
            upLevel = PyramidLevels[i-1]; 
            thisLevel = PyramidLevels[i];
            
            SourceFiles = glob.glob(PyramidPath + os.path.sep + '%03d' % upLevel + os.path.sep + "*." + Extension);
            #Simply a speedup so we aren't constantly hitting the server with exist requests for populated directories
            DestFiles = glob.glob(PyramidPath + os.path.sep + '%03d' % thisLevel + os.path.sep + "*." + Extension);
            for f in SourceFiles:
                filename = os.path.basename(f);
                
                outputFile = os.path.join(PyramidPath + os.path.sep + '%03d' % thisLevel, filename);
                inputFile = os.path.join(PyramidPath + os.path.sep + '%03d' % upLevel, filename);
                               
   #             print "In: " + inputFile;
#                print "Out: " + outputFile;

                if(outputFile in DestFiles):
                    continue;

                #Don't process if the input is temp file
                if(os.path.getsize(inputFile) <= 0):
                    continue;
                
                if(os.path.exists(outputFile)):
                    continue;
                
                #Try to keep other machines from working on the same file
                tempFile = open(outputFile,"w");
                tempFile.close();
                
                shrinkFactor = thisLevel / upLevel; 

                cmd = "ir-shrink -sh " + str(shrinkFactor) + " -load " + inputFile + " -save " + outputFile;
                print cmd;
                proc = subprocess.Popen(cmd + " && exit", shell=True);
                CmdCount += 1; 
                Procs.append(proc);

                while(len(Procs) >= numProcs * 1.5):
                    #Wait 10 seconds and check for openings again
                    time.sleep(0.1);
                    TestProcList = Procs
                    for proc in TestProcList:
                        if proc.poll() != None:
                            Procs.remove(proc)
                            print "PID " + str(proc.pid) + " Terminated"

            #Keep waiting until all processes are finished
            while(len(Procs) > 0):
                #Wait 10 seconds and check for openings again
                time.sleep(1);
                TestProcList = Procs
                for proc in TestProcList:
                    if proc.wait() != None:
                        Procs.remove(proc)
                        print "PID " + str(proc.pid) + " Terminated"
        
    return CmdCount;

##def CreateImagePyramid(path, filename, pyramidLevels, Flip = False, StartingImageExtension = "tif"):
##
##    originalFileName = path + os.path.sep + "001" + os.path.sep + filename + '.' + StartingImageExtension; #We now put the original images in the "down-sampled
##                                                                 #by 1" directory
##    targetFileName = path + os.path.sep + "001" + os.path.sep + filename + ".png"; #What we want the file to be named when we're done
##    tempFileName = path + os.path.sep + "001" + os.path.sep + "C" + filename  + '.' + TileFormat;
##
##    print targetFileName;
##
##    #Open the tif file from mrc2tif and save it in a new format in case we are switching to png or something else
##    #After we are done we will delete the original and rename the temp file in case we didn't switch formats and the
##    #names match
##    im = Image.open(originalFileName);
##    
##    if Flip:
##        im = im.transpose(Image.FLIP_LEFT_RIGHT);
##
##    if Flip or StartingImageExtension != "png": 
##        im.save(tempFileName);
##
##    #Create thumbnails of the originals and move them to the appropriate directory
##    #BUG: This assumes pyramid levels are a power of two
##    for level in pyramidLevels:
##        if(level > 1):
##            pyramidFileName = filename + '.' + TileFormat;
##            saveDir = os.path.join(path + os.path.sep + '%03d' % level, pyramidFileName);
##
##            #As we divide the image in two I want the larger image if we get a fraction
##            newWidth = int(math.ceil(im.size[0]/2.0));
##            newHeight = int(math.ceil(im.size[1]/2.0));
##            
##            resized = im.resize([newWidth, newHeight], Image.ANTIALIAS);
##                
##  #          print str(level) + " resize: " + str(im.size) + " " + saveDir; 
##            resized.save(saveDir);
##
##            im = resized;
##
##    del(im);
##    
##    if Flip or StartingImageExtension != "png": 
##        os.remove(originalFileName);
##        shutil.move(tempFileName, targetFileName);
    
#Searches dirpath and subdirectories, any MRC's found that do not live in subdirectories are moved to subdirectories
def MRCsToFolders(dirpath):
    print "MRCsToFolders";
    
#Takes the specified image file and removes "clahe" from path so original images are used to construct mosaic
def Declahe(MosaicFile, OutputFile):

    #read Input and substitue clahe files with the actual image data
    try:
        MFile = open(MosaicFile, 'r');
        FixedFile = open(OutputFile, 'w+');

        Data = MFile.read();
        MFile.close();

        FixedData = Data.replace(os.path.sep + 'clahe', '');

        FixedFile.write(FixedData);
        FixedFile.close();
    except:
        print "No clahe .mosaic found: " + MosaicFile; 
        pass;
