/*  MosaicBuilder
    Copyright (C) 2007 Spaltenstein Natural Image

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#import <Cocoa/Cocoa.h>
@class MBTileWindowController;
@class MBMosaicWindowController;
@class MBTransformation;
#define kMBUTTypeMBProj @"MosaicBuilderProjectFile"
#define kMBTileSources @"titleSources"

#define kMBTiffExtension @"tiff"

#define kMBTransformationLinearType @"transformationLinearType"

#define kMBSubList @"subList"
#define kMBName @"name"
#define kMBID @"id"
#define kMBContrast @"contrast"
#define kMBBrightness  @"brightness"
#define kMBHistogram  @"histogram"
// #define kMBCGImage @"cgImage"
// #define kMBCGImageSource @"cgImageSource"
#define kMBNSImageBitmapImageRep @"nsBitmapImageRep"
#define kMBNSModifiedImageBitmapImageRep @"nsModifiedBitmapImageRep"
#define kMBTileIDs @"tileIDs"
#define kMBTileList @"tileList"
#define kMBLayerList @"layerList"
#define kMBNextTileID @"nextTileID"
#define kMBTransformations @"transformations"
#define kMBNextTransformationID @"nextTransformationID"
#define kMBTileListType @"tileListType"
#define kMBTileListFolderType @"tileListFolderType"
#define kMBTileListTileType @"tileListTileType"
#define kMBRootName @"rootName"

#define kMBLayerID @"layerID"
#define kMBLayerName @"layerName"
#define kMBDefaultLayerName @"Layer 1"
#define kMBLayerItemType @"layerItemType"
#define kMBIsLayerShowing @"isLayerShowing"
#define kMBIsLayerOpaque @"isLayerOpaque"
#define kMBLayerItemLayerType @"layerItemLayerType"
#define kMBLayerItemTransformationType @"layerItemTransformationType"
#define kMBLayerTransformationList @"layerTransformationList"
#define kMBLayerTransformationDictionary @"layerTransformationDictionary"
#define kMBTransformationSourceTile @"transformationSourceTile"
#define kMBTransformationType @"transformationType"
#define kMBTransformationName @"transformationName"
#define kMBTransformationData @"transformationData"
#define kMBTransformationID @"transformationID"

#define kMBLayerColor @"layerColor"
#define kMBColorType @"colorType"
#define kMBBlackColor @"blackColor"
#define kMBBlueRedColor @"blueRedColor"
#define kMBBlueGreenColor @"blueGreenColor"
#define kMBRedGreenColor @"redGreenColor"
#define kMBCustomColor @"customColor"
#define kMBMixedColor @"mixedColor"
#define kMBRedComponent @"redComponent"
#define kMBGreenComponent @"greenComponent"
#define kMBBlueComponent @"blueComponent"

#define kMBTileOutlineViewData @"tileOutlineViewData"
#define kMBMosaicOutlineViewData @"mosaicOutlineViewData"
#define kMBTileWindowFrame @"tileWindowFrame"
#define kMBMosaicWindowFrame @"mosaicWindowFrame"
#define kMBTileViewFrame @"tileViewFrame"
#define kMBMosaicViewFrame @"mosaicViewFrame"
#define kMBMosaicViewingRect @"mosaicViewingRect"

#define kMBTransformationOperation @"transformationOperation"

#define kMBTileDragDataType @"tileDragDataType"
#define kMBTransformationDragDataType @"transformationDragDataType"
#define kMBLayerDragDataType @"layerDragDataType"

#define kMBTileListChangedNotification @"tileListChangedNotification"
#define kMBLayerListChangedNotification @"layerListChangedNotification"
#define kMBActiveTileViewInfoChangedNotification @"activeTileViewChangedNotification"
#define kMBActiveLayerChangedNotification @"activeLayerChangedNotification"
#define kMBShouldSelectItemNotification @"shouldSelectItemNotification"
// kMBShouldSelectLayerNotification can also be used for selecting transformations
#define kMBShouldSelectLayerNotification @"shouldSelectLayerNotification"
#define kMBSelectTransformationNotification @"selectTransformationNotification"
#define kMBGLContextIsReleasingNotification @"glcontextIsReleasingNotification"
#define kMBGLContextKey @"glcontextKey"
#define kMBBrightnessContrastChangeNotification @"brightnessContrastNotification"
#define kMBTileKey @"tileKey"
#define kMBItemKey @"itemKey"
#define kMBLayerKey @"layerKey"
#define kMBTransformationKey @"transformationKey"

#define kMBName @"name"
#define kMBShow @"show"
#define kMBOpaque @"opaque"
#define kMBColor @"color"

#define kMBLZWCompressCards @"LZWCompressCards"

typedef enum {
	kMBBlackColorTag = 0,
	kMBBlueRedColorTag,
	kMBBlueGreenColorTag,
	kMBRedGreenColorTag,
	kMBCustomColorTag,
	kMBMixedColorTag
} kMBColorTags;

@interface MBDocument : NSDocument
{
	IBOutlet NSView*		saveAccessoryView;

//	NSDictionary*	tileSources; // dictionary where keys are layernames and values are
//													//dictionaries were keys are layers.tilenames excluding the .tif extension
	
	// stuff saved
	NSMutableArray*			tileIDs; // list of the the IDs of the tiles in this document
	NSMutableDictionary*	tileList; // array with dictionaries containing: type(kMBTileListType) (tile or folder), and name(kMBName), if folder contains an array like this one at key kMBSubList
									// a tile contains keys kMBName, kMBID, kMBContrast (float), kMBBrightness (float);
	long					nextTileID;	
	long					nextTransformationID;	// layers use transformationIDs
	
//	NSMutableDictionary*	transformations; // keys are the transformation IDs, Objects are dictionaries which contain whatever the transformation needs
																		// to save to restore itself
	NSMutableArray*			layerList; // a list of the layers, the layers are dictionaries with kMBLayerItemType(set to kMBLayerItemLayerType),
									// kMBLayerID, kMBLayerName, kMBIsLayerShowing, kMBLayerColor, kMBLayerTransformationList which is an array of dictionaries which have:
											// kMBTransformationID, kMBTransformationName, kMBTransformationType, kMBTransformationSourceTile,
											// kMBIsLayerShowing, kMBLayerColor, kMBIsLayerOpaque, kMBLayerItemType (set to kMBLayerItemTransformationType),
											
											// kMBLayerColor is a dictionary containing keys kMBColorType (set to one of kMBBlackColor, kMBBlueRedColor,
											// kMBBlueGreenColor, kMBRedGreenColor,of kMBCustomColor), if it is kMBCustomColor, then kMBRedComponent,
											// kMBGreenComponent and kMBBlueComponent are used for the color
	// stuff saved but with some massaging
	NSMutableDictionary*	transformationObjects; // key is the transformation IDs, objects are the transformation objects, which subclass the MBTransformation
	
	// stuff not saved
//	NSMutableDictionary*	tileViewInfo; // arr of dictionaries that have, kMBID, do I even use this!?!?
	NSDictionary*			activeTileViewInfo;	
	NSMutableDictionary*	activeLayer; // this is the layer that is loaded
	
	// these are loaded in a lazy manner, use bitmapImageRepForID: and histogramForID: to access these
	NSMutableDictionary*	tileImages; // key is the tile IDs, the objects are dictionaries with kMBNSImageBitmapImageRep, kMBNSModifiedImageBitmapImageRep, kMBHistogram (NSArray of 256 Longs), kMBID

	// saved UI
	NSData*					tileOutlineViewData;
	NSData*					mosaicOutlineViewData;

	NSString*				tileWindowFrame;
	NSString*				mosaicWindowFrame;
	
	NSString*				tileViewFrame;
	NSString*				mosaicViewFrame;
	
	NSString*				mosaicViewingRect;
}

- (IBAction) exportCards: (id) sender;

- (NSMutableDictionary*) tileList;
- (NSMutableArray*) layerList;
- (NSMutableDictionary*) transformationObjects;
- (NSDictionary*) activeTileViewInfo;
- (NSDictionary*) tileImages;

- (NSData*) tileOutlineViewData;
- (void) setTileOutlineViewData:(NSData*) data;
- (NSData*) mosaicOutlineViewData;
- (void) setMosaicOutlineViewData:(NSData*) data;
- (NSString*) tileWindowFrame;
- (void) setTileWindowFrame:(NSString*) frame;
- (NSString*) mosaicWindowFrame;
- (void) setMosaicWindowFrame:(NSString*) frame;
- (NSString*) tileViewFrame;
- (void) setTileViewFrame:(NSString*) frame;
- (NSString*) mosaicViewFrame;
- (void) setMosaicViewFrame:(NSString*) frame;
- (NSString*) mosaicViewingRect;
- (void) setMosaicViewingRect:(NSString*) rect;

- (void) archiveMosaicWindow:(MBMosaicWindowController*) controller;
- (void) archiveTileWindow:(MBTileWindowController*) controller;
- (void) archiveControllers;

- (void) unarchiveMosaicWindow:(MBMosaicWindowController*) controller;
- (void) unarchiveTileWindow:(MBTileWindowController*) controller;

- (void) touchDocument;

+ (id) makeMutable:(id)object;

- (MBTileWindowController*) tileWindowController;
- (MBMosaicWindowController*) mosiacWindowController;

- (void) setActiveTileViewInfo: (NSDictionary*) tile;
- (NSMutableDictionary*) activeLayer;
- (void) setActiveLayer:(NSDictionary*) layer;

- (NSMutableDictionary*) transformationDictForID:(long)theID;
- (NSMutableDictionary*) layerDictForID:(long) theID;
- (NSMutableDictionary*) tileDictForID:(long) theID inTileList:(NSArray*)tileArray;
- (NSMutableDictionary*) tileDictForID:(long) theID;
- (NSMutableDictionary*) parentOfTile:(NSDictionary*) tile index:(long*)theIndex;
- (NSMutableDictionary*) layerForTransformation:(NSDictionary*)transformation index:(long*)theIndex;

- (NSBitmapImageRep*) bitmapImageRepWithBrightness:(float) brightness contrast:(float) contrast fromBitmapImageRep:(NSBitmapImageRep*) originalImage;

// load and unload tileImageDictionary
- (void) loadTileImageDictionaryForID:(long) theID;

// use this method to access the imageRep, it will handle the lazy loading of the image
- (NSBitmapImageRep*) bitmapImageRepForID:(long) theID;
- (NSBitmapImageRep*) modifiedBitmapImageRepForID:(long) theID;
- (void) setModifiedBitmapImageRep:(NSBitmapImageRep*) modifiedImage forID:(long) theID;
- (NSArray*) histogramForID:(long) theID;

- (void) setLayerName: (NSString*) name ofLayer:(NSMutableDictionary*) layer; // works for transformations too
- (void) setTransformationName: (NSString*) name ofTransformation:(NSMutableDictionary*) transformation;
- (void) setTileName: (NSString*) name ofTile:(NSMutableDictionary*) tileDict;

// if the two parents are the same before and after, make sure that the 
- (void) moveTransformation:(NSDictionary*) transDict toLayer:(NSMutableDictionary*)layerDict atIndex:(long) index;
- (void) moveLayer:(NSDictionary*) layerDict toIndex:(long) index;
- (void) moveTile:(NSDictionary*) tileDict toParent:(NSMutableDictionary*)parentDict atIndex:(long) index;


//undoable opperations
- (NSMutableDictionary*) makeNewTransformationWithTile:(NSDictionary*)tileDict inLayer:(NSMutableDictionary*)layerDict atIndex:(long)index;
- (NSMutableDictionary*) makeNewTransformationWithPlist:(NSDictionary*)plist inLayer:(NSDictionary*)layerDict atIndex:(long)index;
- (void) removeTransformation:(NSDictionary*)transformationDict;

- (void) addLayer:(NSDictionary*)layerDict atIndex:(long)index;
- (void) removeLayer:(NSDictionary*)layerDict;

- (void) performTrasformationOperation:(NSDictionary*) operation;
- (void) performTrasformationOperation:(NSDictionary*) operation onTransformation:(MBTransformation*) transformation;

- (void) makeLayerOpaque:(NSMutableDictionary*)layerDict;
- (void) makeLayerNotOpaque:(NSMutableDictionary*)layerDict;

- (void) setBrightness:(float) brightness contrast:(float) contrast ofTile:(NSMutableDictionary*)tileDict;

// accepts an NSArray of dictionaries, each dict has a kMBItemKey, holding the dict, and a kMBIsLayerOpaque key
- (void) setOpaqueStatus:(NSArray*) statusArray;

- (IBAction) importTiles: (id) sender;
- (IBAction) newLayer: (id) sender;

- (void) deleteTile:(NSMutableDictionary*) tileDict;


//- (IBAction) deleteLayer: (id) sender;
- (IBAction) showTileWindow: (id) sender;
- (IBAction) showMosaicWindow: (id) sender;

- (BOOL) importTilesFromURLs: (NSArray*) URLs error:(NSError **)outError;
- (BOOL) loadTileFromURL: (NSURL*) URL withID:(long)theID error:(NSError **)outError;
- (BOOL) writeTileToURL:(NSURL *)tileURL tileID:(long)theID error:(NSError **)outError;
- (BOOL) buildPackageStructureToURL: (NSURL*) bundleURL error:(NSError **)outError;

- (NSArray*) writeableTransformationObjectArray;
- (BOOL) loadTransformationObjects:(NSArray*)fromArray error:(NSError **)outError;

- (void) touchDocument;
//- (void) removeWindowController:(NSWindowController *)windowController;
@end
