/*  MosaicBuilder
    Copyright (C) 2007 Spaltenstein Natural Image

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#import <Cocoa/Cocoa.h>
#import <QuartzCore/QuartzCore.h>
#import "CustomOpenGLView.h"
@class MBMosaicWindowController;
@class MBDocument;
@class MBTransformation;

@interface MBMosaicView : CustomOpenGLView {
    IBOutlet NSSlider*	zoomSlider;
	IBOutlet NSScroller*	horizontalScroller;
	IBOutlet NSScroller*	verticalScroller;
	// drawing state
//	NSCursor*	activeCursor;
	NSRect		viewingRect;
	BOOL		suppressControls;
	
// drag and dropping
	BOOL dragging;
	NSPoint draggingLocation;
	long draggedTileID;
	GLuint* draggedTextureName;
	
	BOOL trackingWasAcceptingMouseEvents;
//	NSTrackingRectTag cursorTrackingRect;
	NSBezierPath*	trackingTransformationOutline;
	NSArray*		trackingControlRects;
	NSCursor*		defaultCursor;
	BOOL			cursorTracking;
	
	NSDictionary*	modifyingTransformation;
	float			modifyingTransparency;
	
	NSPointingDeviceType	pointDeviceType;
}

//- (void)awakeFromNib; // don't know why I need this, according to the apple docs initWithFrame should get called, but it is not...

- (id)initWithFrame:(NSRect)frame;
- (id)initWithCoder:(NSCoder *)coder;

- (void)dealloc;

- (BOOL)acceptsFirstResponder;

+ (NSOpenGLPixelFormat*) pixelFormat;

// drag and drop
- (NSDragOperation) draggingEntered:(id <NSDraggingInfo>)sender;
- (NSDragOperation) draggingUpdated:(id <NSDraggingInfo>)sender;
- (void) draggingExited:(id <NSDraggingInfo>)sender;
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender;
- (BOOL)wantsPeriodicDraggingUpdates;

- (IBAction) zoomSliderChanged: (id) sender;
- (IBAction) horizontalScrollerChanged: (id) sender;
- (IBAction) verticalScrollerChanged: (id) sender;

- (void) scrollTransformationToVisible:(NSDictionary*) transDict;

- (NSMutableDictionary*) selectedTransformationDict;
- (MBTransformation*) selectedTransformation;
- (NSPoint) transformPointToView:(NSPoint) thePoint;
- (NSPoint) transformPointToTransformation:(NSPoint) thePoint;
- (NSRect) transformRectToView:(NSRect) theRect;
- (NSRect) transformRectToTransformation:(NSRect) theRect;
- (NSBezierPath*) transformBezierToView:(NSBezierPath*) theBezier;
- (NSBezierPath*) transformBezierToTransformation:(NSBezierPath*) theBezier;

- (NSDictionary*) transformationDictAtPoint:(NSPoint) thePoint;
- (NSRect) usedAreaBounds;
- (NSRect) layerBounds:(NSDictionary*) layer;

- (BOOL) suppressControls;
- (void) setSuppressControls:(BOOL) suppress;

//- (NSCursor*) activeCursor;
//- (void) setActiveCursor:(NSCursor*) cursor;

- (NSRect) viewingRect;
- (void) setViewingRect:(NSRect) theRect;
- (float) scale;

- (void) updateZoomSlider;
- (void) updateScrollers;
- (void) resetViewport;

- (void) drawLayer:(NSDictionary*) layer context:(NSOpenGLContext*) context;
- (void) layerListChanged:(NSNotification *)notification;

- (void) scrollWheel:(NSEvent *)theEvent;
- (void) rightMouseDown:(NSEvent *)theEvent;
- (void) mouseDown:(NSEvent *)theEvent;
- (void) mouseMoved:(NSEvent *)theEvent;
- (void) tabletProximity:(NSEvent *)theEvent;
//- (void) mouseEntered:(NSEvent *)theEvent;
//- (void) mouseExited:(NSEvent *)theEvent;
//- (void) discardCursorRects;
//- (void) resetCursorRects;
- (void) updateCursor:(NSPoint) cursorPos;
- (void) viewDidMoveToWindow;

- (void) resetCursorTracking;
- (void) resetCursorTrackingWithDefaultCursor:(NSCursor*) cursor;

- (void) setCursorTracking:(BOOL) tracking;

- (MBMosaicWindowController*) controller;
- (MBDocument*) document;

@end
