/*  MosaicBuilder
    Copyright (C) 2007 Spaltenstein Natural Image

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define kMBTransformationMBTransformationType @"transformationMBTransformationType"


#import <Cocoa/Cocoa.h>
@class MBDocument;

@interface MBTransformation : NSObject {

@private
	long		sourceTileID;
	MBDocument*	document;
}

+(id) transformation; // don't call this
+(id) transformationWithPList:(NSDictionary*)plist document:(MBDocument*) theDoc;
//+(id) transformationWithType:(NSString*)type;
+(id) transformationWithTransformationDict:(NSDictionary*)transDict document:(MBDocument*) theDoc;

-(id) initWithDocument:(MBDocument*) theDoc;
-(id) initWithPList:(NSDictionary*)plist document:(MBDocument*) theDoc;
//-(id) initWithType:(NSString*)type;
-(id) initWithTransformationDict:(NSDictionary*)transDict document:(MBDocument*) theDoc;


- (BOOL)isEqual:(id)anObject;

- (NSBitmapImageRep*) bitmapImage;

// methods to override
+ (NSString*) transformationType;
+ (void) finishedLoading;

-(NSDictionary*) getPList;

- (long) sourceTileID;
- (MBDocument*) document;

-(void) loadTexturesInContext:(NSOpenGLContext*) context; // transformations need to be able to draw themselves even if they are not loaded
-(void) loadLowResTexturesInContext:(NSOpenGLContext*) context; // transformations need to be able to draw themselves even if they are not loaded
-(void) unloadTexturesInContext:(NSOpenGLContext*) context;

-(NSArray*) controlRects:(float) scale; // returns an array of NSRects that represent the control points being drawn by the transformation
-(NSBezierPath*) outlinePath;
-(float) scale;

-(NSArray*) maskingPaths;
-(void) deleteMask:(long) mask;

- (void) resetToPoint:(NSPoint) thePoint;

- (void) nudge:(NSPoint) direction scale:(float)scale;

-(void) performTrasformationOperation:(NSDictionary*) operation; // operation needs kMBTransformationOperation, kMBTransformationID keys as well
																// as any other keys needed for the transformation
-(void) mouseDown:(NSPoint) thePoint modifiers:(int) modifiers scale:(float) scale; // mouse events messages are sent to the transformation if the initial mouse down took place in a controlRect
-(void) mouseUp:(NSPoint) thePoint modifiers:(int) modifiers scale:(float) scale;
-(void) mouseDragged:(NSPoint) thePoint modifiers:(int) modifiers scale:(float) scale;

-(void) startMaskRegion:(NSPoint) thePoint scale:(float) scale;
-(void) lineToMaskRegion:(NSPoint) thePoint scale:(float) scale;
-(void) endMaskRegion:(NSPoint) thePoint scale:(float) scale;


-(void) drawToGL:(NSOpenGLContext*) context withMasking:(BOOL)masking;
-(void) drawMosiacControls:(NSOpenGLContext*) context scale:(float) scale;
-(void) drawTileControls:(NSOpenGLContext*) context;


@end
