using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;
using NGVV.Controls;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace VolumeViewer
{
    class Command
    {
        protected ViewerControl control; //Control the command is listening to

        protected MouseEventArgs oldMouse = null; 

        public Command(ViewerControl ctrl)
        {
            this.control = ctrl; 
            ctrl.MouseMove += new MouseEventHandler(this.OnMouseMove);
            ctrl.MouseDown += new MouseEventHandler(this.OnMouseDown);
            ctrl.MouseWheel += new MouseEventHandler(this.OnMouseWheel);
            ctrl.KeyPress += new KeyPressEventHandler(this.OnKeyPress);
          //  ctrl.MouseHover += new MouseEventHandler(this.OnMouseHover); 
        }

        public void OnKeyPress(object sender, KeyPressEventArgs e)
        {
            
            if (e.KeyChar == '=' || e.KeyChar == '+')
            {
                string transform = Controller.volume.CurrentSection.CurrentTransform;
                Controller.volume.SectionIndex++;

                if (Controller.UseSectionSpecificTransform == false)
                    Controller.volume.CurrentSection.CurrentTransform = transform;

                Debug.WriteLine("Open Section: " + Controller.volume.CurrentSection.Number.ToString());
                control.StatusSection = Controller.volume.CurrentSection.Number; 
            }

            if (e.KeyChar == '-' || e.KeyChar == '_')
            {
                string transform = Controller.volume.CurrentSection.CurrentTransform;
                Controller.volume.SectionIndex--;
                if(Controller.UseSectionSpecificTransform == false)
                    Controller.volume.CurrentSection.CurrentTransform = transform;

                Debug.WriteLine("Open Section: " + Controller.volume.CurrentSection.Number.ToString());
                control.StatusSection = Controller.volume.CurrentSection.Number;
            }

            control.Refresh();
        }

        public void OnMouseWheel(object sender, MouseEventArgs e)
        {
            float multiplier = ((float)e.Delta / 120.0f);
            float StartDistance = control.CameraDistance;
            if(multiplier > 0)
                control.CameraDistance = control.CameraDistance * 0.86956521739130434782608695652174f;
            else
                control.CameraDistance = control.CameraDistance * 1.15f;

            this.control.Refresh(); 
        }

        public void OnMouseMove(object sender, MouseEventArgs e)
        {
            Vector3 NewPosition = control.ScreenToWorld(e.X, -e.Y);
            this.control.StatusPosition = NewPosition;

            if (oldMouse == null)
            {
                oldMouse = e;
                return; 
            }

            if (e.Button == MouseButtons.Right)
            {
                Vector3 OldPosition = control.ScreenToWorld(oldMouse.X, -oldMouse.Y);
               
                Debug.Assert(float.IsNaN(NewPosition.X) == false); 

                Vector3 delta = NewPosition - OldPosition;

                if (float.IsNaN(delta.X))
                    return; 

                control.CameraLookAt -= delta;

                this.control.Refresh();
            }
            else if (e.Button == MouseButtons.Middle)
            {
                //Figure out if the mouse went clockwise or counterclockwise relative to the center of the screen
                System.Drawing.Rectangle rect = control.ClientRectangle;
                Vector2 Center = new Vector2(((rect.Width - rect.X) / 2) + rect.X, ((rect.Height - rect.Y) / 2) + rect.Y);

                Vector2 old = new Vector2(oldMouse.X - Center.X, oldMouse.Y - Center.Y);
                Vector2 newMouse = new Vector2(e.X - Center.X,  e.Y - Center.Y);

                newMouse.Normalize();
                old.Normalize(); 

                float angle = (float)Math.Acos((double)Vector2.Dot(newMouse, old));

                if (newMouse.Y < 0)
                {
                    if (newMouse.X - old.X < 0)
                        angle = -angle; 
                }
                else if (newMouse.Y >= 0)
                {
                    if (newMouse.X - old.X > 0)
                        angle = -angle; 
                }
                angle = MathHelper.ToDegrees(angle);

            //    Debug.WriteLine("Angle changed " + angle.ToString() + " to " + control.CameraRotation.ToString()); 

                control.CameraRotation += angle;

                this.control.Refresh(); 
            }

            oldMouse = e; 
        }

        public void OnMouseDown(object sender, MouseEventArgs e)
        {
        }

        public void OnMouseHover(object sender, MouseEventArgs e)
        {
            Vector3 Position = control.ScreenToWorld(e.X, e.Y);

            //TODO: Figure out which tile the mouse is over and show a tooltip
        }

    }
}
