using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using NGVV.Controls;
using System.Diagnostics; 

namespace VolumeViewer
{
    public partial class NGVVMain : Form
    {
        Command command; 
        public NGVVMain()
        {
            InitializeComponent();
            Controller.Appwindow = this;

            //    ThreadPool.SetMaxThreads(8, 8);
            //    ThreadPool.SetMinThreads(1, 1); 

            command = new Command(this.viewerControl); 
        }
        public NGVVMain(string[] args) : this()
        {
            Debug.Write("Arguments: " + args); 
            if(args.Length > 0)
            {
                Controller.volume = new Volume(args[0]);
                Controller.volume.CurrentSection.CurrentTransform = Controller.volume.CurrentSection.Transforms.Keys[0];
                this.viewerControl.Drawables.Add(Controller.volume); 
            }
        }

        private void menuFileOpen_Click(object sender, EventArgs e)
        {
            FolderBrowserDialog browserDlg = new FolderBrowserDialog();
            browserDlg.SelectedPath = "X:\\Test\\MouseVolume";
            DialogResult result = browserDlg.ShowDialog(this);
            if (result != DialogResult.OK)
                return;

            this.viewerControl.Drawables.Clear(); 
            Controller.volume = new Volume(browserDlg.SelectedPath);
            Controller.volume.CurrentSection.CurrentTransform = Controller.volume.CurrentSection.Transforms.Keys[0];
            this.viewerControl.Drawables.Add(Controller.volume); 

            
            this.Refresh(); 
        }

        private void menuFileOpenWebVolume_Click(object sender, EventArgs e)
        {
            OpenFileDialog fileDlg = new OpenFileDialog();
            fileDlg.CheckFileExists = true;
            fileDlg.DefaultExt = ".xml"; 
            fileDlg.Title = "Locate volume XML to load";

            DialogResult result = fileDlg.ShowDialog();
            if(result != DialogResult.OK)
                return;

            Controller.volume = new Volume(fileDlg.FileName); 

            Controller.volume.CurrentSection.CurrentTransform = Controller.volume.CurrentSection.Transforms.Keys[0]; 

            this.Refresh(); 
        }


        private void viewerControl_KeyPress(object sender, KeyPressEventArgs e)
        {

        }

        /// <summary>
        /// I only have this in case Asynch file IO completes, in which case I want the screen to refresh
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void timer_Tick(object sender, EventArgs e)
        {
            this.Refresh();
        }

        private void menuSectionTransform_DropDownOpening(object sender, EventArgs e)
        {
            
        }

        private void OnTransformClick(object sender, EventArgs e)
        {
            ToolStripMenuItem menuItem = sender as ToolStripMenuItem;
            if (menuItem == null)
                return;

            Controller.volume.CurrentSection.CurrentTransform = menuItem.Text;

            this.Refresh(); 
        }

        private void OnSectionClick(object sender, EventArgs e)
        {
            ToolStripMenuItem menuItem = sender as ToolStripMenuItem;
            if (menuItem == null)
                return;

            int SectionNumber = System.Convert.ToInt32(menuItem.Text);

            string transform = Controller.volume.CurrentSection.CurrentTransform;
            Controller.volume.SectionIndex = Controller.volume.sections.Keys.IndexOf(SectionNumber); 
            Controller.volume.CurrentSection.CurrentTransform = transform;

            this.Refresh();
        }

        private void OnSectionModeClick(object sender, EventArgs e)
        {
            ToolStripMenuItem menuItem = sender as ToolStripMenuItem;
            if (menuItem == null)
                return;

            if (menuItem.Text == Controller.CurrentMode)
                return;

            Controller.CurrentMode = menuItem.Text;
            Controller.volume.CurrentSection.FreeTextures(); 

            this.Refresh();
        }

        private void menuSection_DropDownOpening(object sender, EventArgs e)
        {
            menuSectionUseSpecific.Checked = Controller.UseSectionSpecificTransform;
            menuSectionShowMesh.Checked = Controller.ShowMesh; 

            if (Controller.volume == null)
                return; 

            menuSectionTransform.DropDownItems.Clear();
            menuSectionNumbers.DropDownItems.Clear(); 

            if (Controller.volume == null)
                return;

            IList<string> transformNames = Controller.volume.CurrentSection.Transforms.Keys;

            List<ToolStripItem> items = new List<ToolStripItem>();
          
            foreach (string t in transformNames)
            {
                ToolStripMenuItem menuItem = new ToolStripMenuItem(t, null, OnTransformClick);
                if (t == Controller.volume.CurrentSection.CurrentTransform)
                    menuItem.Checked = true; 

                items.Add(menuItem); 
                //menuItem.Parent = menuSectionTransform; 
            }

            
            menuSectionTransform.DropDownItems.AddRange(items.ToArray()); 


            IList<int> sectionNumbers = Controller.volume.sections.Keys;
            items.Clear(); 

            foreach (int iSection in sectionNumbers)
            {
                string s = iSection.ToString();

                ToolStripMenuItem menuItem = new ToolStripMenuItem(s, null, OnSectionClick);
                if (iSection == Controller.volume.CurrentSection.Number)
                    menuItem.Checked = true;

                items.Add(menuItem); 
            }

            menuSectionNumbers.DropDownItems.AddRange(items.ToArray());

        }

        private void modeToolStripMenuItem_DropDownOpening(object sender, EventArgs e)
        {
            menuSectionMode.DropDownItems.Clear();

            if (Controller.volume == null)
                return; 

            if (Controller.volume.CurrentSection == null)
                return;

            List<ToolStripItem> items = new List<ToolStripItem>();

            foreach (string mode in Controller.volume.CurrentSection.Modes)
            {
                ToolStripMenuItem menuItem = new ToolStripMenuItem(mode, null, OnSectionModeClick);
                if (mode == Controller.CurrentMode)
                    menuItem.Checked = true;

                items.Add(menuItem); 
            }

            menuSectionMode.DropDownItems.AddRange(items.ToArray()); 
        }

        private void useSectionSpecificTransformsToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Controller.UseSectionSpecificTransform = !Controller.UseSectionSpecificTransform; 
        }

        private void menuSectionShowMesh_Click(object sender, EventArgs e)
        {
            Controller.ShowMesh = !Controller.ShowMesh;
        }

       

    }
}