﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading; 
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using System.Diagnostics;
using System.IO;
using System.Drawing; 

namespace VolumeViewer
{
    class TextureReader
    {
        string Filename;
        public int Downsample;
        GraphicsDevice graphicsDevice = null;
        public Texture2D Result = null;
        public ManualResetEvent DoneEvent;
        public bool FinishedReading = false; 
   
        static private bool TextureErrorReported = false; 

        public TextureReader(GraphicsDevice graphicsDevice, string filename, int downsample, ManualResetEvent DoneEvent)
        {
            this.graphicsDevice = graphicsDevice; 
            this.Filename = filename;
            this.Downsample = downsample;
            this.DoneEvent = DoneEvent; 
        }

        public void ThreadPoolCallback(Object threadContext)
        {
            
            int threadIndex = Thread.CurrentThread.ManagedThreadId;
  //          Debug.WriteLine("Thread " + threadIndex.ToString() + " reading " + this.Filename);

            TextureCreationParameters parameters = TextureCreationParameters.Default;
            parameters.Format = SurfaceFormat.Luminance8; 
            Texture2D tex = null;
            try
            {
                if (Filename.StartsWith("http"))
                {
                    System.Net.HttpWebRequest request = (System.Net.HttpWebRequest)System.Net.HttpWebRequest.CreateDefault(new Uri(Filename));
                    if(Downsample >= 64)
                        request.CachePolicy = new System.Net.Cache.RequestCachePolicy(System.Net.Cache.RequestCacheLevel.CacheIfAvailable);
                    System.Net.WebResponse response = request.GetResponse();
                    Debug.WriteLine(response.ToString());
                    System.IO.Stream TileStream = response.GetResponseStream();

                    tex = Texture2D.FromFile(graphicsDevice, TileStream, (int)response.ContentLength, parameters);

                    TileStream.Close();
                    response.Close();
                }
                else
                {
                    tex = Texture2D.FromFile(graphicsDevice, Filename, parameters);
                }
            }
            catch(Exception e)
            {
                //Print out the first error, but don't flood the output in case we simply have a section where we are
                //missing some tiles. 
                if(!TextureErrorReported)
                {
                    Debug.WriteLine("Error loading texture: " + e.ToString()); 
                    TextureErrorReported = true;
                }
            }

            Monitor.Enter(this);
            this.Result = tex;
            this.FinishedReading = true; 
            Monitor.Exit(this); 

            DoneEvent.Set(); 
        }

        public override bool Equals(object obj)
        {
            TextureReader Tobj = obj as TextureReader;
            if (Tobj == null)
                return false;

            if (Tobj.Filename != this.Filename)
                return false;

            return true;
        }

        public override int GetHashCode()
        {
            return Filename.GetHashCode(); 
        }
    }
}
