using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace Util
{
    public class TileGridTransform : GridTransform
    {
        static int ImageWidth = int.MinValue;
        static int ImageHeight = int.MinValue;
        public string filename;

        public TileGridTransform(string SectionPath, string file) : base()
        {
            ParseMosaicGridFile(SectionPath, file);
        }

        public void ParseMosaicGridFile(string SectionPath, string transform)
        {
            string[] parts = transform.Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

            string transformType = parts[2];

            switch (transformType)
            {
                case "GridTransform_double_2_2":
                    ParseGridTransform(parts);
                    break; 
                case "LegendrePolynomialTransform_double_2_2_1":
                    ParsePolyTransform(SectionPath, parts); 
                    break;
                case "TranslationTransform_double_2_2":
                    ParseTranslateTransform(SectionPath, parts); 
                    break; 
                default:
                    Debug.Assert(false, "Unexpected transform type: " + transformType);
                    break; 
            }
        }


        public void ParsePolyTransform(string SectionPath, string[] parts)
        {
            //Find the dimensions of the grid
            int iFixedParameters = 0;
            int iVariableParameters = 0;
            for (int i = 0; i < parts.Length; i++)
            {
                if (parts[i] == "vp")
                {
                    iVariableParameters = i;
                    continue;
                }

                if (parts[i] == "fp")
                {
                    iFixedParameters = i;
                    break;
                }
            }

            this.filename = System.IO.Path.GetFileName(parts[1]);

            //Figure out tile size if we haven't already
            if (TileGridTransform.ImageHeight == int.MinValue)
            {
                //We need to figure out what the texture size is
             //   TextureInformation info = Texture2D.GetTextureInformation(SectionPath + "\\8-bit\\001\\" + filename);

                TileGridTransform.ImageWidth = System.Convert.ToInt32(parts[iFixedParameters + 4]) * 2;
                TileGridTransform.ImageHeight = System.Convert.ToInt32(parts[iFixedParameters + 5]) * 2;
            }

            //The poly transform parameters dictate the center of the image
            float x = System.Convert.ToSingle(parts[iFixedParameters + 2]) - (TileGridTransform.ImageWidth /  2);
            float y = System.Convert.ToSingle(parts[iFixedParameters + 3]) - (TileGridTransform.ImageHeight / 2);

            Vector2 BotLeft = new Vector2(x, y);
            Vector2 BotRight = new Vector2(x + ImageWidth, y);
            Vector2 TopLeft = new Vector2(x, y + ImageHeight);
            Vector2 TopRight = new Vector2(x + ImageWidth, y + ImageHeight);

            Triangle controlOne = new Triangle(BotLeft, BotRight, TopLeft);

            Vector2 p1 = new Vector2(0, 0);
            Vector2 p2 = new Vector2(ImageWidth, 0);
            Vector2 p3 = new Vector2(0, ImageHeight);
            Triangle mappedOne = new Triangle(p1, p2, p3);

            mappings.Add(new MappingTriangle(controlOne, mappedOne));

            Triangle controlTwo = new Triangle(BotRight, TopRight, TopLeft);

            Vector2 p4 = new Vector2(ImageWidth, 0);
            Vector2 p5 = new Vector2(ImageWidth, ImageHeight);
            Vector2 p6 = new Vector2(0, ImageHeight);
            Triangle mappedTwo = new Triangle(p4, p5, p6);

            mappings.Add(new MappingTriangle(controlTwo, mappedTwo));
        }

        public void ParseTranslateTransform(string SectionPath, string[] parts)
        {
            //Find the dimensions of the grid
            int iFixedParameters = 0;
            int iVariableParameters = 0;
            for (int i = 0; i < parts.Length; i++)
            {
                if (parts[i] == "vp")
                {
                    iVariableParameters = i;
                    continue;
                }

                if (parts[i] == "fp")
                {
                    iFixedParameters = i;
                    break;
                }
            }

            this.filename = System.IO.Path.GetFileName(parts[1]);

            //Figure out tile size if we haven't already
            if (TileGridTransform.ImageHeight == int.MinValue)
            {
                //We need to figure out what the texture size is
                TextureInformation info = Texture2D.GetTextureInformation(SectionPath + "\\8-bit\\001\\" + filename);

                TileGridTransform.ImageWidth = info.Width;
                TileGridTransform.ImageHeight = info.Height;
            }

            float x = System.Convert.ToSingle(parts[iVariableParameters + 2]);
            float y = System.Convert.ToSingle(parts[iVariableParameters + 3]);

            //TODO: This looks really wrong and I should figure out why ir-translate rotates image
            Vector2 BotLeft = new Vector2(x + ImageWidth, y + ImageHeight);
            Vector2 BotRight = new Vector2(x , y + ImageHeight);
            Vector2 TopLeft = new Vector2(x + ImageWidth, y);
            Vector2 TopRight = new Vector2(x , y);

            Triangle controlOne = new Triangle(BotLeft, BotRight, TopLeft);

            Vector2 p1 = new Vector2(0, 0);
            Vector2 p2 = new Vector2(ImageWidth, 0);
            Vector2 p3 = new Vector2(0, ImageHeight);
            Triangle mappedOne = new Triangle(p1, p2, p3);

            mappings.Add(new MappingTriangle(controlOne, mappedOne));

            Triangle controlTwo = new Triangle(BotRight, TopRight, TopLeft);

            Vector2 p4 = new Vector2(ImageWidth, 0);
            Vector2 p5 = new Vector2(ImageWidth, ImageHeight);
            Vector2 p6 = new Vector2(0, ImageHeight);
            Triangle mappedTwo = new Triangle(p4, p5, p6);

            mappings.Add(new MappingTriangle(controlTwo, mappedTwo));
        }

        public void ParseGridTransform(string[] parts)
        {
            //Find the dimensions of the grid
            int iFixedParameters = 0;
            int iVariableParameters = 0;
            for (int i = 0; i < parts.Length; i++)
            {
                if (parts[i] == "vp")
                {
                    iVariableParameters = i;
                    continue;
                }

                if (parts[i] == "fp")
                {
                    iFixedParameters = i;
                    break;
                }
            }

            filename = System.IO.Path.GetFileName(parts[1]);
            int gridWidth = System.Convert.ToInt32(System.Convert.ToDouble(parts[iFixedParameters + 3]) + 1.0);
            int gridHeight = System.Convert.ToInt32(System.Convert.ToDouble(parts[iFixedParameters + 4]) + 1.0);

            ImageHeight = System.Convert.ToInt32(System.Convert.ToDouble(parts[iFixedParameters + 8]));
            ImageWidth = System.Convert.ToInt32(System.Convert.ToDouble(parts[iFixedParameters + 7]));

            this.MappedBounds.Width = ImageWidth;
            this.MappedBounds.Height = ImageHeight;

            int NumPts = System.Convert.ToInt32(parts[4]) / 2;
            Vector2[] Points = new Vector2[System.Convert.ToInt32(NumPts)];

 //           verticies = new VertexPositionNormalTexture[numPts];

            Single minX = Single.MaxValue;
            Single minY = Single.MaxValue;
            Single maxX = Single.MinValue;
            Single maxY = Single.MinValue;

            //Every number in the array is seperated by an empty space in the array
            for (int i = 0; i < NumPts; i++)
            {
                int iPoint = 5 + (i * 2);
                Single x = System.Convert.ToSingle(parts[iPoint]);
                Single y = System.Convert.ToSingle(parts[iPoint + 1]);

                Points[i] = new Vector2(x, y);

                //Debug.WriteLine(x.ToString() + ", " + y.ToString());
                if (x < minX)
                    minX = x;
                if (x > maxX)
                    maxX = x;
                if (y < minY)
                    minY = y;
                if (y > maxY)
                    maxY = y;
            }

            List<int> indicies = new List<int>();

            for (int y = 0; y < gridHeight - 1; y++)
            {
                for (int x = 0; x < gridWidth - 1; x++)
                {
                    int i = x + (y * gridHeight);
                    Triangle controlOne = new Triangle(Points[i], Points[i + 1], Points[i + gridWidth]);

                    Vector2 p1 = CoordinateFromGridPos(x, y, gridWidth, gridHeight);
                    Vector2 p2 = CoordinateFromGridPos(x + 1, y, gridWidth, gridHeight);
                    Vector2 p3 = CoordinateFromGridPos(x, y + 1, gridWidth, gridHeight);
                    Triangle mappedOne = new Triangle(p1, p2, p3);

                    mappings.Add(new MappingTriangle(controlOne, mappedOne));

                    Triangle controlTwo = new Triangle(Points[i + 1], Points[i + gridWidth + 1], Points[i + gridWidth]);

                    Vector2 p4 = CoordinateFromGridPos(x + 1, y, gridWidth, gridHeight);
                    Vector2 p5 = CoordinateFromGridPos(x + 1, y + 1, gridWidth, gridHeight);
                    Vector2 p6 = CoordinateFromGridPos(x, y + 1, gridWidth, gridHeight);
                    Triangle mappedTwo = new Triangle(p4, p5, p6);

                    mappings.Add(new MappingTriangle(controlTwo, mappedTwo));
                }
            }
//            this.gridIndicies = indicies;
        }

        public VertexPositionNormalTexture[] GetVerticies()
        {
            VertexPositionNormalTexture[] verticies = new VertexPositionNormalTexture[mappings.Count * 3];

            for (int i = 0; i < mappings.Count; i++)
            {
                MappingTriangle mapTri = mappings[i];
                Vector3 positionA = new Vector3(mapTri.Control.p1, 0);
                Vector3 positionB = new Vector3(mapTri.Control.p2, 0);
                Vector3 positionC = new Vector3(mapTri.Control.p3, 0);

                Vector2 textureA = new Vector2(mapTri.Mapped.p1.X / (float)ImageWidth, mapTri.Mapped.p1.Y / (float)ImageHeight);
                Vector2 textureB = new Vector2(mapTri.Mapped.p2.X / (float)ImageWidth, mapTri.Mapped.p2.Y / (float)ImageHeight);
                Vector2 textureC = new Vector2(mapTri.Mapped.p3.X / (float)ImageWidth, mapTri.Mapped.p3.Y / (float)ImageHeight);

                verticies[(i * 3)] = new VertexPositionNormalTexture(positionA, Vector3.UnitZ, textureA);
                verticies[(i * 3)+1] = new VertexPositionNormalTexture(positionB, Vector3.UnitZ, textureB);
                verticies[(i * 3)+2] = new VertexPositionNormalTexture(positionC, Vector3.UnitZ, textureC); 
            }


            return verticies;
        }

        
    }
}
