// system includes:
#include <assert.h>
#include <iostream>

// local includes:
#include "hex_dec_bin_utils.hxx"

// namespace access:
using std::cout;
using std::cerr;
using std::endl;

inline static the_text_t
empty_txt(const unsigned int & size)
{
  the_text_t txt;
  txt.fill('\0', size);
  return txt;
}


//----------------------------------------------------------------
// hex_to_dec
// 
// args: a hex number, for example: F, return 15.
// 
the_text_t hex_to_dec(const char & c)
{
  the_text_t dec("0");
  dec[0] = c;
  
  switch (c)
  {
    case '0':
    case '1':
    case '2':
    case '3':
    case '4':
    case '5':
    case '6':
    case '7':
    case '8':
    case '9':
      return dec;
      
    case 'a':
    case 'A':
      return the_text_t("10");

    case 'b':
    case 'B':
      return the_text_t("11");

    case 'c':
    case 'C':
      return the_text_t("12");

    case 'd':
    case 'D':
      return the_text_t("13");

    case 'e':
    case 'E':
      return the_text_t("14");

    case 'f':
    case 'F':
      return the_text_t("15");

    default:
      cerr << "expected hex, got \'" << c << "\', aborting..." << endl;
      assert(0);
  }
  
  return dec;
}

//----------------------------------------------------------------
// dec_to_hex
// 
// args: a decimal number, for example: "15", return "f".
//
char
dec_to_hex(const the_text_t & dec)
{
  if (dec == "0" ||
      dec == "1" ||
      dec == "2" ||
      dec == "3" ||
      dec == "4" ||
      dec == "5" ||
      dec == "6" ||
      dec == "7" ||
      dec == "8" ||
      dec == "9")
  {
    return dec[0];
  }
  
  if (dec == "10") return 'a';
  if (dec == "11") return 'b';
  if (dec == "12") return 'c';
  if (dec == "13") return 'd';
  if (dec == "14") return 'e';
  if (dec == "15") return 'f';
  
  cerr << "expected a number in the [0:15] domain, got \"" << dec
       << "\", aborting..." << endl;
  assert(0);
  
  return dec[0];
}

//----------------------------------------------------------------
// hex_to_bin
// 
// args: a hexadecimal number, for example: "2", return "0010".
//
the_text_t
hex_to_bin(const char & hex)
{
  if (hex == '0') return "0000";
  if (hex == '1') return "0001";
  if (hex == '2') return "0010";
  if (hex == '3') return "0011";
  if (hex == '4') return "0100";
  if (hex == '5') return "0101";
  if (hex == '6') return "0110";
  if (hex == '7') return "0111";
  if (hex == '8') return "1000";
  if (hex == '9') return "1001";
  if (hex == 'a' || hex == 'A') return "1010";
  if (hex == 'b' || hex == 'B') return "1011";
  if (hex == 'c' || hex == 'C') return "1100";
  if (hex == 'd' || hex == 'D') return "1101";
  if (hex == 'e' || hex == 'E') return "1110";
  if (hex == 'f' || hex == 'F') return "1111";
  
  cerr << "expected a number on the [0:f] domain, got \"" << hex
       << "\", aborting..." << endl;
  assert(0);
  
  return "2222";
}

//----------------------------------------------------------------
// bin_to_dec
// 
// args: a binary number, for example: "0010", return "2".
//
char
bin_to_hex(const the_text_t & bin)
{
  if (bin == "0000") return '0';
  if (bin == "0001") return '1';
  if (bin == "0010") return '2';
  if (bin == "0011") return '3';
  if (bin == "0100") return '4';
  if (bin == "0101") return '5';
  if (bin == "0110") return '6';
  if (bin == "0111") return '7';
  if (bin == "1000") return '8';
  if (bin == "1001") return '9';
  if (bin == "1010") return 'a';
  if (bin == "1011") return 'b';
  if (bin == "1100") return 'c';
  if (bin == "1101") return 'd';
  if (bin == "1110") return 'e';
  if (bin == "1111") return 'f';
  
  cerr << "expected a number in the [0000:1111] domain, got \"" << bin
       << "\", aborting..." << endl;
  return 'z';
}

//----------------------------------------------------------------
// bin_byte_reverse
// 
// args: a string representing one byte in binary
// example: "11101000", returns "00010111"
//
the_text_t
bin_byte_reverse(const the_text_t & byte)
{
  unsigned int len = byte.size();
  the_text_t out = empty_txt(len);
  
  for (unsigned int i = 0; i < len; i++)
  {
    out[i] = byte[len - 1 - i];
  }
  return out;
}

//----------------------------------------------------------------
// bin_byte_shift_left
// 
// args: a string representing one byte in binary,
//       and a shift-in digit (0 or 1)
// example: "11101000" "1", returns "11010001"
//
the_text_t
bin_byte_shift_left(const the_text_t & byte, const char shift_in)
{
  unsigned int len = byte.size();
  the_text_t out = empty_txt(len);
  for (unsigned int i = 1; i < len; i++)
  {
    out[i - 1] = byte[i];
  }
  out[len - 1] = shift_in;
  return out;
}

//----------------------------------------------------------------
// bin_byte_shift_right
// 
// args: a string representing one byte in binary,
//       and a shift-in digit (0 or 1)
// example: "11101000" "0", returns "01110100"
//
the_text_t
bin_byte_shift_right(const the_text_t & byte, const char shift_in)
{
  unsigned int len = byte.size();
  the_text_t out = empty_txt(len);
  for (unsigned int i = 1; i < len; i++)
  {
    out[i] = byte[i - 1];
  }
  out[0] = shift_in;
  return out;
}

//----------------------------------------------------------------
// bin_union_of_two_bits
// 
// args: two bits
// example: "0" "1", returns "1"
char
bin_union_of_two_bits(const char & a, const char & b)
{
  if (a == '1') return '1';
  if (b == '1') return '1';
  return '0';
}

//----------------------------------------------------------------
// bin_union_of_two_bytes
// 
// args: two strings representing two bytes in binary
// example: "11100001" "10001000", returns "11101001"
the_text_t
bin_union_of_two_bytes(const the_text_t & a, const the_text_t & b)
{
  unsigned int len = a.size();
  assert(len == b.size());
  
  the_text_t out = empty_txt(len);
  for (unsigned int i = 0; i < len; i++)
  {
    out[i] = bin_union_of_two_bits(a[i], b[i]);
  }
  return out;
}

//----------------------------------------------------------------
// little_endian_to_big_endian
// 
// args: little endian byte, for example: 0x02, return 0x40.
//
the_text_t
hex_little_endian_to_big_endian(const the_text_t & le_byte)
{
  assert(le_byte.size() == 4);
  char hex_a = le_byte[2];
  char hex_b = le_byte[3];

  the_text_t be_bin_a = bin_byte_reverse(hex_to_bin(hex_a));
  the_text_t be_bin_b = bin_byte_reverse(hex_to_bin(hex_b));
  
  the_text_t be_byte("0x00");
  be_byte[2] = bin_to_hex(be_bin_b);
  be_byte[3] = bin_to_hex(be_bin_a);
  
  return be_byte;
}

//----------------------------------------------------------------
// hex_byte_to_bin_byte
// 
// args: a hex byte, example: "0xF0", return "11110000"
//
the_text_t
hex_byte_to_bin_byte(const the_text_t & hex)
{
  assert(hex.size() == 4);
  
  the_text_t bin_a = hex_to_bin(hex[2]);
  the_text_t bin_b = hex_to_bin(hex[3]);
  return bin_a + bin_b;
}

//----------------------------------------------------------------
// bin_byte_to_hex_byte
// 
// args: a binary byte, example: "11110000", return "0xF0"
//
the_text_t
bin_byte_to_hex_byte(const the_text_t & bin)
{
  assert(bin.size() == 8);
  
  the_text_t bin_a = bin.extract(0, 4);
  the_text_t bin_b = bin.extract(4, 4);
  
  the_text_t hex("0x00");
  hex[2] = bin_to_hex(bin_a);
  hex[3] = bin_to_hex(bin_b);
  
  return hex;
}
