#ifndef HEX_DEC_BIN_UTILS_HXX_
#define HEX_DEC_BIN_UTILS_HXX_

// local includes:
#include "the_text.hxx"


//----------------------------------------------------------------
// hex_to_dec
// 
// args: a hex number, for example: F, return 15.
// 
extern the_text_t hex_to_dec(const char & c);

//----------------------------------------------------------------
// dec_to_hex
// 
// args: a decimal number, for example: "15", return "f".
//
extern char dec_to_hex(const the_text_t & dec);

//----------------------------------------------------------------
// hex_to_bin
// 
// args: a hexadecimal number, for example: "2", return "0010".
//
extern the_text_t hex_to_bin(const char & hex);

//----------------------------------------------------------------
// bin_to_dec
// 
// args: a binary number, for example: "0010", return "2".
//
extern char bin_to_hex(const the_text_t & bin);

//----------------------------------------------------------------
// bin_byte_reverse
// 
// args: a string representing one byte in binary
// example: "11101000", returns "00010111"
//
extern the_text_t bin_byte_reverse(const the_text_t & byte);

//----------------------------------------------------------------
// bin_byte_shift_left
// 
// args: a string representing one byte in binary,
//       and a shift-in digit (0 or 1)
// example: "11101000" "1", returns "11010001"
//
extern the_text_t
bin_byte_shift_left(const the_text_t & byte, const char shift_in = '0');

//----------------------------------------------------------------
// bin_byte_shift_right
// 
// args: a string representing one byte in binary,
//       and a shift-in digit (0 or 1)
// example: "11101000" "0", returns "01110100"
//
extern the_text_t
bin_byte_shift_right(const the_text_t & byte, const char shift_in = '0');

//----------------------------------------------------------------
// bin_union_of_two_bits
// 
// args: two bits
// example: "0" "1", returns "1"
extern char
bin_union_of_two_bits(const char & a, const char & b);

//----------------------------------------------------------------
// bin_union_of_two_bytes
// 
// args: two strings representing two bytes in binary
// example: "11100001" "10001000", returns "11101001"
extern the_text_t
bin_union_of_two_bytes(const the_text_t & a, const the_text_t & b);

//----------------------------------------------------------------
// little_endian_to_big_endian
// 
// args: little endian byte, for example: 0x02, return 0x40.
//
extern the_text_t
hex_little_endian_to_big_endian(const the_text_t & le_byte);

//----------------------------------------------------------------
// hex_byte_to_bin_byte
// 
// args: a hex byte, example: "0xF0", return "11110000"
//
extern the_text_t
hex_byte_to_bin_byte(const the_text_t & hex);

//----------------------------------------------------------------
// bin_byte_to_hex_byte
// 
// args: a binary byte, example: "11110000", return "0xF0"
//
extern the_text_t
bin_byte_to_hex_byte(const the_text_t & bin);


#endif // HEX_DEC_BIN_UTILS_HXX_
