// system includes:
#include <assert.h>
#include <iostream>

// local includes:
#include "the_text.hxx"
#include "hex_dec_bin_utils.hxx"

// namespace access:
using std::cout;
using std::cerr;
using std::endl;


//----------------------------------------------------------------
// to_upper
// 
inline the_text_t to_upper(const the_text_t & txt)
{
  the_text_t out(txt);
  out.to_upper();
  return out;
}


//----------------------------------------------------------------
// xbm_to_cxx
// 
// args: filename of an X11 bitmap file
// 
xbm_to_cxx(const the_text_t & XBM)
{
  std::vector<the_text_t> tmp;
  XBM.split(tmp, '.');
  the_text_t NAME = tmp[0];
  
  the_text_t CURSOR_ID =
    the_text_t("THE_") + to_upper(NAME) + "_CURSOR_E";
  
  WIDTH=`grep width $XBM | cut -d' ' -f3`
  if [ -z "$WIDTH" ]; then
    cout << " width attribute not found, aborting...
    exit 1
  fi
  
  HEIGHT=`grep height $XBM | cut -d' ' -f3`
  if [ -z "$HEIGHT" ]; then
    cout << " height attribute not found, aborting...
    exit 1
  fi
  
  HOT_X=`grep x_hot $XBM | cut -d' ' -f3`
  if [ -z "$HOT_X" ]; then
    HOT_X=$(($((${WIDTH} - 1)) / 2));
    cout << " hot_x not specified, will use ${HOT_X}...
  fi
  
  HOT_Y=`grep y_hot $XBM | cut -d' ' -f3`
  if [ -z "$HOT_Y" ]; then
    HOT_Y=$(($((${HEIGHT} - 1)) / 2));
    cout << " hot_y not specified, will use ${HOT_Y}...
  fi
  
  COLS=$(($((${WIDTH} - 1))/8 + 1))
  ROWS=${HEIGHT}
  
  // read the cursor bitmap:
  ROW="0"
  COL="0"
  declare ICON;
  exec 0<$XBM
  while read -r j; do
    IFS_A=${IFS}
    IFS=', '
    for i in `cout << " "${j}"`; do
      RAW_BYTE=`cout << " $i | grep '0x'`
      if [ -n "$RAW_BYTE" ]; then
	BYTE=${RAW_BYTE:0:4}
//        cout << " BYTE 1: "$BYTE"
        BE_BYTE=`little_endian_to_big_endian $BYTE`
	ICON[$((${ROW} * ${COLS} + ${COL}))]="${BE_BYTE}"
	
	// increment the array index:
	COL=$((${COL} + 1))
	if [ ${COL} = ${COLS} ]; then
	  COL="0"
	  ROW=$((${ROW} + 1))
	fi
      fi
    done
    IFS=${IFS_A}
  done
  
  // either read in a mask file, or generate one on-the-fly:
  ROW="0"
  COL="0"
  declare MASK;
  if [ -f "${NAME}_mask.xbm" ]; then
    // read the mask bitmap file:
    exec 0<"${NAME}_mask.xbm"
    while read -r j; do
      IFS_A=${IFS}
      IFS=', '
      for i in `cout << " "${j}"`; do
        RAW_BYTE=`cout << " $i | grep '0x'`
        if [ -n "$RAW_BYTE" ]; then
	  BYTE=${RAW_BYTE:0:4}
//          cout << " BYTE 2: "$BYTE"
          BE_BYTE=`little_endian_to_big_endian $BYTE`
	  MASK[$((${ROW} * ${COLS} + ${COL}))]="${BE_BYTE}"
	  
	  // increment the array index:
	  COL=$((${COL} + 1))
	  if [ ${COL} = ${COLS} ]; then
	    COL="0"
	    ROW=$((${ROW} + 1))
	  fi
        fi
      done
      IFS=${IFS_A}
    done
  else
    // generate a mask bitmap:
    cout << " generating the mask on the fly...
    declare ICON_A;
    declare ICON_B;
    declare ICON_C;
    declare ICON_D;
    
    // shift up:
    ROW="0"
    COL="0"
    while [ ! ${ROW} = ${ROWS} ]; do
      BYTE=${ICON[$((${ROW} * ${COLS} + ${COL}))]}
      if [ ${ROW} = "0" ]; then
	ICON_A[$(($((${ROWS} - 1)) * ${COLS} + ${COL}))]="0x00"
      else
	ICON_A[$(($((${ROW} - 1)) * ${COLS} + ${COL}))]=$BYTE
      fi
      
      COL=$((${COL} + 1))
      if [ ${COL} = ${COLS} ]; then
	COL="0"
	ROW=$((${ROW} + 1))
      fi
    done
    
    // shift down:
    ROW="0"
    COL="0"
    while [ ! ${ROW} = ${ROWS} ]; do
      BYTE=${ICON[$((${ROW} * ${COLS} + ${COL}))]}
      if [ ${ROW} = "$((${ROWS} - 1))" ]; then
	ICON_B[$((${COL}))]="0x00"
      else
	ICON_B[$(($((${ROW} + 1)) * ${COLS} + ${COL}))]=$BYTE
      fi
      
      COL=$((${COL} + 1))
      if [ ${COL} = ${COLS} ]; then
	COL="0"
	ROW=$((${ROW} + 1))
      fi
    done
    
    // shift left:
    ROW="0"
    COL="0"
    while [ ! ${ROW} = ${ROWS} ]; do
      INDEX=$((${ROW} * ${COLS} + ${COL}))
      BYTE=${ICON[$INDEX]}
      BIN_A=`hex_byte_to_bin_byte ${BYTE}`
      
      SHIFT_IN="0"
      if [ ! ${COL} = "$((${COLS} - 1))" ]; then
	NEXT_INDEX=$((${ROW} * ${COLS} + ${COL} + 1))
	NEXT_BYTE=${ICON[$NEXT_INDEX]}
	NEXT_BIN=`hex_byte_to_bin_byte ${NEXT_BYTE}`
	SHIFT_IN="${NEXT_BIN:0:1}"
      fi
      
      BIN_B=`bin_byte_shift_left $BIN_A $SHIFT_IN`
      BYTE=`bin_byte_to_hex_byte $BIN_B`
      ICON_C[$INDEX]=$BYTE
      
      COL=$((${COL} + 1))
      if [ ${COL} = ${COLS} ]; then
	COL="0"
	ROW=$((${ROW} + 1))
      fi
    done
    
    // shift right:
    ROW="0"
    COL="0"
    while [ ! ${ROW} = ${ROWS} ]; do
      INDEX=$((${ROW} * ${COLS} + ${COL}))
      BYTE=${ICON[$INDEX]}
      BIN_A=`hex_byte_to_bin_byte ${BYTE}`
      
      SHIFT_IN="0"
      if [ ! ${COL} = "0" ]; then
	PREV_INDEX=$((${ROW} * ${COLS} + ${COL} - 1))
	PREV_BYTE=${ICON[$PREV_INDEX]}
	PREV_BIN=`hex_byte_to_bin_byte ${PREV_BYTE}`
	SHIFT_IN="${PREV_BIN:7:1}"
      fi
      
      BIN_B=`bin_byte_shift_right $BIN_A $SHIFT_IN`
      BYTE=`bin_byte_to_hex_byte $BIN_B`
      ICON_D[$INDEX]=$BYTE
      
      COL=$((${COL} + 1))
      if [ ${COL} = ${COLS} ]; then
	COL="0"
	ROW=$((${ROW} + 1))
      fi
    done
    
    // merge them all together:
    ROW="0"
    COL="0"
    while [ ! ${ROW} = ${ROWS} ]; do
      INDEX=$((${ROW} * ${COLS} + ${COL}));
      BIN_A=`hex_byte_to_bin_byte ${ICON[$INDEX]}`
      BIN_B=`hex_byte_to_bin_byte ${ICON_A[$INDEX]}`
      BIN_C=`hex_byte_to_bin_byte ${ICON_B[$INDEX]}`
      BIN_D=`hex_byte_to_bin_byte ${ICON_C[$INDEX]}`
      BIN_E=`hex_byte_to_bin_byte ${ICON_D[$INDEX]}`
      
      BIN_AB=`bin_union_of_two_bytes $BIN_A $BIN_B`
      BIN_ABC=`bin_union_of_two_bytes $BIN_AB $BIN_C`
      BIN_ABCD=`bin_union_of_two_bytes $BIN_ABC $BIN_D`
      BIN_ABCDE=`bin_union_of_two_bytes $BIN_ABCD $BIN_E`
      
      BYTE=`bin_byte_to_hex_byte $BIN_ABCDE`
      MASK[$INDEX]="${BYTE}"
      
      COL=$((${COL} + 1))
      if [ ${COL} = ${COLS} ]; then
	COL="0"
	ROW=$((${ROW} + 1))
      fi
    done
  fi
  
  // generate the source code:
  printf "    case %s:\n" "$CURSOR_ID" >> the_cursor.cxx
  printf "    {\n" >> the_cursor.cxx
  printf "      const byte_t icon[] =\n      {\n\t" >> the_cursor.cxx
  ROW="0"
  COL="0"
  while [ ! ${ROW} = ${ROWS} ]; do
    INDEX=$((${ROW} * ${COLS} + ${COL}));
    printf "%s" "${ICON[$INDEX]}" >> the_cursor.cxx
    COL=$((${COL} + 1))
    if [ ${COL} = ${COLS} ]; then
      while [ ! ${COL} = 4 ]; do
	printf "%s" ", 0x00" >> the_cursor.cxx
	COL=$((${COL} + 1))
      done
      COL="0"
      ROW=$((${ROW} + 1))
      if [ ${ROW} = ${ROWS} ]; then break; fi
      printf ",\n\t" >> the_cursor.cxx
    else
      printf ", " >> the_cursor.cxx
    fi
  done
  printf "\n      };\n\n" >> the_cursor.cxx
  
  printf "      const byte_t mask[] =\n      {\n\t" >> the_cursor.cxx
  ROW="0"
  COL="0"
  while [ ! ${ROW} = ${ROWS} ]; do
    INDEX=$((${ROW} * ${COLS} + ${COL}));
    printf "%s" "${MASK[$INDEX]}" >> the_cursor.cxx
    COL=$((${COL} + 1))
    if [ ${COL} = ${COLS} ]; then
      while [ ! ${COL} = 4 ]; do
	printf "%s" ", 0x00" >> the_cursor.cxx
	COL=$((${COL} + 1))
      done
      COL="0"
      ROW=$((${ROW} + 1))
      if [ ${ROW} = ${ROWS} ]; then break; fi
      printf ",\n\t" >> the_cursor.cxx
    else
      printf ", " >> the_cursor.cxx
    fi
  done
  printf "\n      };\n\n" >> the_cursor.cxx

  printf "      widget.setCursor(QCursor(QBitmap::fromData(QSize(%s, %s), icon, QImage::Format_Mono),\n" "32" "$HEIGHT" >> the_cursor.cxx
  printf "                               QBitmap::fromData(QSize(%s, %s), mask, QImage::Format_Mono), " "32" "$HEIGHT" >> the_cursor.cxx
  printf "%s, %s));\n" "$HOT_X" "$HOT_Y" >> the_cursor.cxx
  printf "    }\n    break;\n\n" >> the_cursor.cxx
}

//----------------------------------------------------------------
// xbm_to_hxx
// 
// args: a filename of an X11 bitmap file, and the number of the file.
// 
xbm_to_hxx()
{
  XBM=$1
  NUM=$2
  NAME=`cout << " $XBM | cut -d'.' -f1`
  NAME=`str_to_upper "${NAME}"`
  CURSOR_ID="THE_${NAME}_CURSOR_E"
  
  if [ ! ${NUM} = "0" ]; then
    printf ",\n  " >> the_cursor.hxx
  fi
  
  printf "%s" $CURSOR_ID >> the_cursor.hxx
}

//----------------------------------------------------------------
// gen_hxx_head
// 
gen_hxx_head()
{
  printf "// NOTE: \n"
  printf "//       DO NOT EDIT THIS FILE, IT IS AUTOGENERATED.\n"
  printf "//       ALL CHANGES WILL BE LOST UPON REGENERATION.\n\n"
  printf "//ifndef THE_CURSOR_HXX_\n"
  printf "//define THE_CURSOR_HXX_\n\n\n"
  printf "// forward declarations:\n"
  printf "class QWidget;\n"
  printf "\n"
  printf "\n"
  printf "//----------------------------------------------------------------\n"
  printf "// the_cursor_id_t\n"
  printf "//\n"
  printf "typedef enum\n"
  printf "{\n  "
}

//----------------------------------------------------------------
// gen_hxx_tail
// 
gen_hxx_tail()
{
  printf "\n} the_cursor_id_t;\n\n"
  printf "//----------------------------------------------------------------\n"
  printf "// the_cursor_change\n"
  printf "// \n"
  printf "// global function for changing a widgets cursor:\n"
  printf "// \n"
  printf "extern void\n"
  printf "the_cursor_change(QWidget & widget, the_cursor_id_t cursor);\n\n\n"
  printf "//endif // THE_CURSOR_HXX_\n"
}

//----------------------------------------------------------------
// gen_cxx_head
// 
gen_cxx_head()
{
  printf "// NOTE: \n"
  printf "//       DO NOT EDIT THIS FILE, IT IS AUTOGENERATED.\n"
  printf "//       ALL CHANGES WILL BE LOST UPON REGENERATION.\n\n"
  printf "// QT includes:\n"
  printf "//include <QWidget>\n"
  printf "//include <QCursor>\n"
  printf "//include <QBitmap>\n"
  printf "\n"
  printf "// system includes:\n"
  printf "//include <assert.h>\n"
  printf "\n"
  printf "// local includes:\n"
  printf "//include \"the_cursor.hxx\"\n"
  printf "\n"
  printf "\n"
  printf "//----------------------------------------------------------------\n"
  printf "// the_cursor_change\n"
  printf "// \n"
  printf "void\n"
  printf "the_cursor_change(QWidget & widget, the_cursor_id_t cursor)\n"
  printf "{\n"
  printf "  typedef unsigned char byte_t;\n"
  printf "  \n"
  printf "  switch (cursor)\n"
  printf "  {\n"
}

//----------------------------------------------------------------
// gen_cxx_tail
// 
gen_cxx_tail()
{
  printf "    default:\n"
  printf "      assert(false);\n"
  printf "      break;\n"
  printf "  }\n}\n"
}

//----------------------------------------------------------------
// main
//
int main()
{
  > the_cursor.hxx
  > the_cursor.cxx
  
  gen_hxx_head >> the_cursor.hxx
  gen_cxx_head >> the_cursor.cxx
  
  NUM="0"
  for i in `/bin/ls -1 *.xbm | grep -v _mask`; do
    cout << " processing $i
    xbm_to_hxx "${i}" "${NUM}"
    xbm_to_cxx "${i}" "${NUM}"
    cout << "
    NUM=$((${NUM} + 1))
  done
  
  gen_hxx_tail >> the_cursor.hxx
  gen_cxx_tail >> the_cursor.cxx
}
