// local includes:
#include "token.hxx"

// system includes:
#include <cassert>
#include <iostream>
#include <fstream>
/*
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>
#include <fstream>
#include <ctype.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <netinet/in.h>
#include <net/if.h>
#include <arpa/inet.h>
*/

// namespace access:
using std::ostream;
using std::fstream;
using std::ios;
using std::cout;
using std::cerr;
using std::endl;


//----------------------------------------------------------------
// xpm_parse_state_t
// 
typedef enum
{
  HAS_NOTHING_E,
  HAS_XPM_COMMENT_E,
  HAS_XPM_DECLARATION_E,
  HAS_XPM_DIMENSIONS_E,
  HAS_XPM_COLOR_E,
  HAS_XPM_SCANLINE_E,
  HAS_EVERYTHING_E
} xpm_parse_state_t;

//----------------------------------------------------------------
// operator <<
// 
ostream &
operator << (ostream & out, xpm_parse_state_t state)
{
  switch (state)
  {
    case HAS_NOTHING_E:			out << "HAS_NOTHING_E";		break;
    case HAS_XPM_COMMENT_E:		out << "HAS_XPM_COMMENT_E";	break;
    case HAS_XPM_DECLARATION_E:	       	out << "HAS_XPM_DECLARATION_E";	break;
    case HAS_XPM_DIMENSIONS_E:		out << "HAS_XPM_DIMENSIONS_E";	break;
    case HAS_XPM_COLOR_E:		out << "HAS_XPM_COLOR_E";	break;
    case HAS_XPM_SCANLINE_E:		out << "HAS_XPM_SCANLINE_E";	break;
    case HAS_EVERYTHING_E:		out << "HAS_EVERYTHING_E";	break;
    default:				out << (unsigned int)(state);	break;
  }
  
  return out;
}

//----------------------------------------------------------------
// some globals:
// 
static char ** FONT[128];
static char ** MASK[128];
static char FONT_NAME[1024];
static unsigned int XPM_WIDTH = 0;
static unsigned int XPM_HEIGHT = 0;
static char * XPM_X_ORIG = NULL;
static char * XPM_Y_ORIG = NULL;
static char * XPM_X_STEP = NULL;
static char * XPM_Y_STEP = NULL;

//----------------------------------------------------------------
// parse_xpm
// 
void
parse_xpm(fstream & fin, const unsigned int & id)
{
  // parse line buffer:
  char buffer[1024];
  
  // parser state:
  xpm_parse_state_t state = HAS_NOTHING_E;
  
  // the name of the xpm:
  char xpm_name[256];
  unsigned int xpm_width = 0;
  unsigned int xpm_height = 0;
  unsigned int xpm_number_of_colors = 2;
  unsigned int xpm_chars_per_color = 1;
  
  unsigned int colors_found = 0;
  unsigned int scanline_index = 0;
  char color[128];
  
  while (true)
  {
    if (fin.eof()) break;
    fin.getline(buffer, 1023, '\n');
    
    switch (state)
    {
      case HAS_NOTHING_E:
      {
	// eat the XPM header:
	if (strcmp(buffer, "/* XPM */") == 0) state = HAS_XPM_COMMENT_E;
      }
      break;
      
      case HAS_XPM_COMMENT_E:
      {
	// read the XPM name:
	Tokenizer tzer_lb('[');
	tzer_lb << buffer;
	char * type_and_name = tzer_lb.get(0);
	
	if (type_and_name != NULL)
	{
	  Tokenizer tzer_st('*');
	  tzer_st << type_and_name;
	  char * name_and_ws = tzer_st.get(1);
	  
	  if (name_and_ws != NULL)
	  {
	    Tokenizer tzer_ws(' ');
	    tzer_ws << name_and_ws;
	    char * name = tzer_ws.get(0);
	    
	    if (name != NULL)
	    {
	      state = HAS_XPM_DECLARATION_E;
	    }
	    else
	    {
	      strcpy(xpm_name, name);
	    }
	  }
	}
	
	if (state != HAS_XPM_DECLARATION_E) state = HAS_NOTHING_E;
      }
      break;
      
      case HAS_XPM_DECLARATION_E:
      {
	// read the number of colors and the dimensions of the XPM:
	Tokenizer tzer_dq('"');
	tzer_dq << buffer;
	char * width_height_ncolors_ncolorchars = NULL;
	for (unsigned int token = 0; token < tzer_dq.count(); token++)
	{
	  if (tzer_dq.len(token) < 7) continue;
	  width_height_ncolors_ncolorchars = tzer_dq.get(token);
	}
	
	if (width_height_ncolors_ncolorchars != NULL)
	{
	  Tokenizer tzer_ws(' ');
	  tzer_ws << width_height_ncolors_ncolorchars;
	  if (tzer_ws.count() == 4)
	  {
	    xpm_width = atol(tzer_ws.get(0));
	    xpm_height = atol(tzer_ws.get(1));
	    xpm_number_of_colors = atol(tzer_ws.get(2));
	    xpm_chars_per_color = atol(tzer_ws.get(3));
	    
	    if ((xpm_width == XPM_WIDTH) &&
		(xpm_height == XPM_HEIGHT) &&
		(xpm_number_of_colors < 3) &&
		(xpm_chars_per_color == 1))
	    {
	      state = HAS_XPM_DIMENSIONS_E;
	    }
	  }
	}
	
	if (state != HAS_XPM_DIMENSIONS_E) state = HAS_NOTHING_E;
      }
      break;
      
      case HAS_XPM_DIMENSIONS_E:
      {
	Tokenizer tzer_dq('"');
	tzer_dq << buffer;
	char * color_char = NULL;
	for (unsigned int token = 0; token < tzer_dq.count(); token++)
	{
	  if ((tzer_dq.len(token) == 8) || (tzer_dq.len(token) == 11))
	  {
	    color_char = tzer_dq.get(token);
	    break;
	  }
	}
	
	if (color_char != NULL)
	{
	  // sanity checking, read the colors:
	  Tokenizer tzer_ws(' ');
	  tzer_ws << color_char;
	  
	  if (tzer_ws.count() == 3)
	  {
	    unsigned int index = (unsigned int)(tzer_ws.get(0)[0]);
	    if (strcmp(tzer_ws.get(2), "None") == 0)
	    {
	      color[index] = '0';
	    }
	    else
	    {
	      color[index] = '1';
	    }
	    
	    colors_found++;
	  }
	  else
	  {
	    state = HAS_NOTHING_E;
	  }
	}
	else
	{
	  state = HAS_NOTHING_E;
	}
	
	if (colors_found == xpm_number_of_colors) state = HAS_XPM_COLOR_E;
      }
      break;
      
      case HAS_XPM_COLOR_E:
      case HAS_XPM_SCANLINE_E:
      {
	Tokenizer tzer_dq('"');
	tzer_dq << buffer;
	char * scanline = NULL;
	for (unsigned int token = 0; token < tzer_dq.count(); token++)
	{
	  if (tzer_dq.len(token) != xpm_width * xpm_chars_per_color)
	  {
	    continue;
	  }
	  
	  scanline = tzer_dq.get(token);
	}
	
	if (scanline != NULL)
	{
	  state = HAS_XPM_SCANLINE_E;
	  
	  if (FONT[id] == NULL)
	  {
	    FONT[id] = new char * [xpm_height];
	    MASK[id] = new char * [xpm_height + 2];
	    
	    for (unsigned int font_row = 0; font_row < xpm_height; font_row++)
	    {
	      unsigned int font_columns = 8 * ((xpm_width + 7) / 8);
	      FONT[id][font_row] = new char [font_columns];
	      for (unsigned int font_col = 0;
		   font_col < font_columns;
		   font_col++)
	      {
		FONT[id][font_row][font_col] = '0';
	      }
	    }
	    
	    for (unsigned int mask_row = 0;
		 mask_row < xpm_height + 2;
		 mask_row++)
	    {
	      unsigned int mask_columns = 8 * ((xpm_width + 9) / 8);
	      MASK[id][mask_row] = new char [mask_columns];
	      for (unsigned int mask_col = 0;
		   mask_col < mask_columns;
		   mask_col++)
	      {
		MASK[id][mask_row][mask_col] = '0';
	      }
	    }
	  }
	  
	  for (unsigned int font_col = 0; font_col < xpm_width; font_col++)
	  {
	    FONT[id][scanline_index][font_col] =
	      color[(unsigned int)(scanline[font_col])];
	  }
	  
	  scanline_index++;
	  if (scanline_index == xpm_height) state = HAS_EVERYTHING_E;
	}
	else
	{
	  state = HAS_NOTHING_E;
	}
      }
      break;
      
      case HAS_EVERYTHING_E: return;
    }
    
    // FIXME: pkoshevoy: for debugging:
    // cout << buffer << " -> " << state << endl;
  }
}

//----------------------------------------------------------------
// max_char
// 
char
max_char(char a, char b)
{
  return (b < a) ? a : b;
}

//----------------------------------------------------------------
// calc_mask
// 
void
calc_mask(const unsigned int & id)
{
  if (FONT[id] == NULL) return;
  
  for (unsigned int row = 0; row < XPM_HEIGHT; row++)
  {
    for (unsigned int col = 0; col < XPM_WIDTH; col++)
    {
      MASK[id][row + 1][col] =
	max_char(FONT[id][row][col], MASK[id][row + 1][col]);
      
      MASK[id][row + 1][col + 2] =
	max_char(FONT[id][row][col], MASK[id][row + 1][col + 2]);
      
      MASK[id][row][col + 1] =
	max_char(FONT[id][row][col], MASK[id][row][col + 1]);
      
      MASK[id][row + 2][col + 1] =
	max_char(FONT[id][row][col], MASK[id][row + 2][col + 1]);
    }
  }
  
  for (unsigned int row = 0; row < XPM_HEIGHT; row++)
  {
    for (unsigned int col = 0; col < XPM_WIDTH; col++)
    {
      if (FONT[id][row][col] == '1') MASK[id][row + 1][col + 1] = '0';
    }
  }
}

//----------------------------------------------------------------
// dump_font
// 
void
dump_font()
{
  for (unsigned int id = 0; id < 128; id++)
  {
    cout << id << ':' << endl;
    if (FONT[id] == NULL) continue;
    
    for (unsigned int row = 0; row < XPM_HEIGHT; row++)
    {
      for (unsigned int col = 0; col < XPM_WIDTH; col++)
      {
	cout << FONT[id][row][col];
      }
      cout << endl;
    }
    cout << endl;
  }
}

//----------------------------------------------------------------
// dump_mask
// 
void
dump_mask()
{
  for (unsigned int id = 0; id < 128; id++)
  {
    cout << id << ':' << endl;
    if (MASK[id] == NULL) continue;
    
    for (unsigned int row = 0; row < XPM_HEIGHT + 2; row++)
    {
      for (unsigned int col = 0; col < XPM_WIDTH + 2; col++)
      {
	cout << MASK[id][row][col];
      }
      cout << endl;
    }
    cout << endl;
  }
}

//----------------------------------------------------------------
// match
// 
bool
match(const char * str_a,
      const char * str_b,
      const unsigned int & chars_to_match)
{
  for (unsigned int i = 0; i < chars_to_match; i++)
  {
    if (str_a[i] != str_b[i]) return false;
  }
  
  return true;
}

//----------------------------------------------------------------
// bin_to_hex
// 
char
bin_to_hex(char * four_bits)
{
  if (match(four_bits, "0000", 4)) return '0';
  if (match(four_bits, "0001", 4)) return '1';
  if (match(four_bits, "0010", 4)) return '2';
  if (match(four_bits, "0011", 4)) return '3';
  if (match(four_bits, "0100", 4)) return '4';
  if (match(four_bits, "0101", 4)) return '5';
  if (match(four_bits, "0110", 4)) return '6';
  if (match(four_bits, "0111", 4)) return '7';
  if (match(four_bits, "1000", 4)) return '8';
  if (match(four_bits, "1001", 4)) return '9';
  if (match(four_bits, "1010", 4)) return 'a';
  if (match(four_bits, "1011", 4)) return 'b';
  if (match(four_bits, "1100", 4)) return 'c';
  if (match(four_bits, "1101", 4)) return 'd';
  if (match(four_bits, "1110", 4)) return 'e';
  if (match(four_bits, "1111", 4)) return 'f';
  
  assert(0);
  return '0';
}

//----------------------------------------------------------------
// str_to_upper
// 
char *
str_to_upper(const char * str)
{
  static char buffer[1024];
  int str_len = strlen(str);
  
  for (int i = 0; i <= str_len; i++)
  {
    buffer[i] = toupper(str[i]);
  }
  return buffer;
}

//----------------------------------------------------------------
// str_to_lower
// 
char *
str_to_lower(const char * str)
{
  static char buffer[1024];
  int str_len = strlen(str);
  
  for (int i = 0; i <= str_len; i++)
  {
    buffer[i] = tolower(str[i]);
  }
  return buffer;
}

//----------------------------------------------------------------
// gen_font_cxx
// 
void
gen_font_cxx(fstream & fout)
{
  unsigned int bytes_per_scanline = (XPM_WIDTH + 7) / 8;
  
  fout << "//----------------------------------------------------------------"
       << endl
       << "// the_" << str_to_lower(FONT_NAME) << "_font_t::font_\n"
       << "// \n"
       << "byte_t\nthe_" << str_to_lower(FONT_NAME)
       << "_font_t::font_[128][" << (XPM_HEIGHT * bytes_per_scanline)
       << "] =\n{" << endl;
  
  for (unsigned int id = 0; id < 128; id++)
  {
    fout << "  { ";
    for (unsigned int row = 0; row < XPM_HEIGHT; row++)
    {
      for (unsigned int byte_idx = 0;
	   byte_idx < bytes_per_scanline;
	   byte_idx++)
      {
	if (FONT[id] == NULL)
	{
	  fout << "0x00";
	}
	else
	{
	  unsigned int row_index = XPM_HEIGHT - row - 1;
	  fout << "0x"
	       << bin_to_hex(&FONT[id][row_index][8 * byte_idx])
	       << bin_to_hex(&FONT[id][row_index][8 * byte_idx + 4]);
	}
      }
      
      if (row + 1 < XPM_HEIGHT) fout << ", ";
    }
    
    if (id < 127)
    {
      fout << " }, " << endl;
    }
    else
    {
      fout << " }" << endl;
    }
  }
  
  fout << "};\n\n";
}

//----------------------------------------------------------------
// gen_mask_cxx
// 
void
gen_mask_cxx(fstream & fout)
{
  unsigned int bytes_per_scanline = (XPM_WIDTH + 9) / 8;
  
  fout << "//----------------------------------------------------------------"
       << endl
       << "// the_" << str_to_lower(FONT_NAME) << "_font_t::mask_\n"
       << "// \n"
       << "byte_t\nthe_" << str_to_lower(FONT_NAME)
       << "_font_t::mask_[128][" << ((XPM_HEIGHT + 2) * bytes_per_scanline)
       << "] =\n{" << endl;
  
  for (unsigned int id = 0; id < 128; id++)
  {
    fout << "  { ";
    for (unsigned int row = 0; row < XPM_HEIGHT + 2; row++)
    {
      for (unsigned int byte_idx = 0;
	   byte_idx < bytes_per_scanline;
	   byte_idx++)
      {
	if (MASK[id] == NULL)
	{
	  fout << "0x00";
	}
	else
	{
	  unsigned int row_index = XPM_HEIGHT - row + 1;
	  fout << "0x"
	       << bin_to_hex(&MASK[id][row_index][8 * byte_idx])
	       << bin_to_hex(&MASK[id][row_index][8 * byte_idx + 4]);
	}
      }
      
      if (row + 1 < XPM_HEIGHT + 2) fout << ", ";
    }
    
    if (id < 127)
    {
      fout << " }, " << endl;
    }
    else
    {
      fout << " }" << endl;
    }
  }
  
  fout << "};\n\n";
}

//----------------------------------------------------------------
// gen_cxx
// 
void
gen_cxx_head(fstream & fout)
{
  fout << "// NOTE: \n"
       << "//       DO NOT EDIT THIS FILE, IT IS AUTOGENERATED.\n"
       << "//       ALL CHANGES WILL BE LOST UPON REGENERATION.\n\n"
       << "// local includes:\n"
       << "#include \"the_" << str_to_lower(FONT_NAME) << "_font.hxx\"\n\n\n";
}

//----------------------------------------------------------------
// gen_cxx_tail
// 
void
gen_cxx_tail(fstream & fout)
{
  fout << "//----------------------------------------------------------------"
       << endl
       << "// THE_" << str_to_upper(FONT_NAME) << "_FONT\n"
       << "// \n"
       << "// a single global instance of the font:\n"
       << "// \n"
       << "the_" << str_to_lower(FONT_NAME) << "_font_t THE_"
       << str_to_upper(FONT_NAME) << "_FONT;" << endl;
}

//----------------------------------------------------------------
// gen_hxx
// 
void
gen_hxx(fstream & fout)
{
  fout << "// NOTE: \n"
       << "//       DO NOT EDIT THIS FILE, IT IS AUTOGENERATED.\n"
       << "//       ALL CHANGES WILL BE LOST UPON REGENERATION.\n\n"
       << "#ifndef THE_" << str_to_upper(FONT_NAME)
       << "_FONT_HXX_\n#define THE_" << str_to_upper(FONT_NAME)
       << "_FONT_HXX_\n\n"
       << "// local includes:\n"
       << "#include \"the_font.hxx\"\n\n\n"
       << "//----------------------------------------------------------------"
       << endl
       << "// the_" << str_to_lower(FONT_NAME) << "_font_t\n"
       << "// \n"
       << "class the_" << str_to_lower(FONT_NAME)
       << "_font_t : public the_font_t\n{\npublic:\n"
       << "  the_" << str_to_lower(FONT_NAME)
       << "_font_t(): the_font_t(\""<< str_to_lower(FONT_NAME)
       << "\") {}\n"
       << "  virtual ~the_" << str_to_lower(FONT_NAME)
       << "_font_t() {}\n\n"
       << "  // virtual:\n"
       << "  unsigned int width() const  { return " << XPM_WIDTH << "; }\n"
       << "  unsigned int height() const { return " << XPM_HEIGHT << "; }\n\n"
       << "  // virtual:\n"
       << "  real_t x_origin() const { return " << XPM_X_ORIG << "; }\n"
       << "  real_t y_origin() const { return " << XPM_Y_ORIG << "; }\n\n"
       << "  // virtual:\n"
       << "  real_t x_step() const { return " << XPM_X_STEP << "; }\n"
       << "  real_t y_step() const { return " << XPM_Y_STEP << "; }\n\n"
       << "  // virtual:\n"
       << "  byte_t * bitmap(const unsigned int & id) const\n"
       << "  { return font_[id]; }\n\n"
       << "  byte_t * bitmap_mask(const unsigned int & id) const\n"
       << "  { return mask_[id]; }\n\n"
       << "private:\n"
       << "  static byte_t font_[128]["
       << (XPM_HEIGHT * ((XPM_WIDTH + 7) / 8))
       << "];\n"
       << "  static byte_t mask_[128]["
       << ((XPM_HEIGHT + 2) * ((XPM_WIDTH + 9) / 8))
       << "];\n"
       << "};\n\n"
       << "//----------------------------------------------------------------"
       << endl
       << "// THE_" << str_to_upper(FONT_NAME) << "_FONT\n"
       << "// \n"
       << "// a single global instance of the font:\n"
       << "// \n"
       << "extern the_" << str_to_lower(FONT_NAME)
       << "_font_t THE_" << str_to_upper(FONT_NAME) << "_FONT;\n"
       << "\n\n#endif // THE_" << str_to_upper(FONT_NAME) << "_FONT_HXX_\n";
}

//----------------------------------------------------------------
// main
// 
int
main(int argc, char ** argv)
{
  if (argc != 8)
  {
    cerr << "usage: " << argv[0]
	 << " name width height x_orig y_orig x_step y_step"
	 << endl
	 << "\nEXAMPLE:\n" << argv[0]
	 << " ascii 5 10 0.0 2.0 7.0 0.0\n\n"
	 << endl;
    exit(1);
  }
  
  strcpy(FONT_NAME, argv[1]);
  XPM_WIDTH = atol(argv[2]);
  XPM_HEIGHT = atol(argv[3]);
  XPM_X_ORIG = argv[4];
  XPM_Y_ORIG = argv[5];
  XPM_X_STEP = argv[6];
  XPM_Y_STEP = argv[7];
  
  char fout_hxx_name[1024];
  strcpy(fout_hxx_name, "the_");
  strcat(fout_hxx_name, str_to_lower(FONT_NAME));
  strcat(fout_hxx_name, "_font.hxx");
  
  fstream fout_hxx;
  fout_hxx.open(fout_hxx_name, ios::out);
  if (fout_hxx.rdbuf()->is_open() == false)
  {
    cerr << "error: could not open " << fout_hxx_name << ", aborting..."
	 << endl;
    exit(1);
  }
  
  char fout_cxx_name[1024];
  strcpy(fout_cxx_name, "the_");
  strcat(fout_cxx_name, str_to_lower(FONT_NAME));
  strcat(fout_cxx_name, "_font.cxx");
  
  fstream fout_cxx;
  fout_cxx.open(fout_cxx_name, ios::out);
  if (fout_cxx.rdbuf()->is_open() == false)
  {
    cerr << "error: could not open " << fout_cxx_name << ", aborting..."
	 << endl;
    exit(1);
  }
  
  for (unsigned int i = 0; i < 128; i++)
  {
    FONT[i] = NULL;
    MASK[i] = NULL;
    
    char file_name[14];
    strcpy(file_name, "ascii_000.xpm");
    
    for (unsigned int temp_id = i, temp_pos = 8; temp_pos > 5; temp_pos--)
    {
      unsigned int n = temp_id % 10;
      temp_id = temp_id / 10;
      file_name[temp_pos] = '0' + char(n);
    }
    
    fstream fin;
    fin.open(file_name, ios::in);
    if (fin.rdbuf()->is_open() == false)
    {
      cerr << "error: could not open " << file_name << ", skipping..." << endl;
      continue;
    }
    
    parse_xpm(fin, i);
    calc_mask(i);
    
    fin.close();
  }
  
  gen_hxx(fout_hxx);
  gen_cxx_head(fout_cxx);
  gen_font_cxx(fout_cxx);
  gen_mask_cxx(fout_cxx);
  gen_cxx_tail(fout_cxx);
  
  return 0;
}
