// author : Paul Koshevoy
// project: none, this file has been accumulating functions I wrote since 1996

// local includes:
#include "token.hxx"

// system includes:
#include <cstddef>
#include <cstring>
#include <cstdlib>


//----------------------------------------------------------------
// Tokenizer::Tokenizer
// 
Tokenizer::Tokenizer(const char & c):
  string_(NULL),
  number_(0),
  delimiter_(c)
{}

//----------------------------------------------------------------
// Tokenizer::~Tokenizer
// 
Tokenizer::~Tokenizer()
{
  delete [] string_;
  string_ = NULL;
}

//----------------------------------------------------------------
// Tokenizer::operator
// 
Tokenizer &
Tokenizer::operator << (const char * given_string)
{
  delete [] string_;
  string_ = NULL;
  
  length_ = strlen(given_string);
  if (length_ == 0)
  {
    index_ = 0;
    number_ = 0;
    return *this;
  }
  
  string_ = new char[length_ + 1];
  strcpy(string_, given_string);
  index_ = 0;
  
  number_ = 0;
  unsigned int state = 0;
  unsigned int sz = 0;
  
  for (unsigned int i = 0; i < length_; i++)
  {
    if (state == 0)
    {
      if (!is_delimiter(string_[i]))
      {
	sz = 1;
	addrs_[number_++] = i;
	state = 1;
      }
    }
    else if (state == 1)
    {
      if (is_delimiter(string_[i]))
      {
	sizes_[number_ - 1] = sz;
	state = 0;
      }
      else
      {
	sz++;
      }
    }
  }
  
  if (number_ > 0)
  {
    sizes_[number_ - 1] = sz;
  }
  
  return *this;
}

//----------------------------------------------------------------
// Tokenizer::get_first
// 
char *
Tokenizer::get_first()
{
  index_ = 0;
  unsigned int pos = 0;
  unsigned int state = 0;
  
  while (true)
  {
    if (string_[index_] == '\0')
    {
      token_[pos] = '\0';
      break;
    }
    
    if (index_ >= length_)
    {
      token_[pos] = '\0';
      break;
    }
    
    if (state == 0)
    {
      if (!is_delimiter(string_[index_]))
      {
	token_[pos++] = string_[index_];
	state = 1;
      }
    }
    else
    {
      if (is_delimiter(string_[index_]))
      {
	token_[pos++] = string_[index_];
      }
      else
      {
	token_[pos] = '\0';
	break;
      }
    }
    
    index_++;
  }
  
  if (pos == 0)
  {
    return NULL;
  }
  
  return token_;
}

//----------------------------------------------------------------
// Tokenizer::get_next
// 
char *
Tokenizer::get_next()
{
  unsigned int pos = 0;
  unsigned int state = 0;
  
  while (true)
  {
    if (string_[index_] == '\0')
    {
      token_[pos] = '\0';
      break;
    }
    
    if (index_ >= length_)
    {
      token_[pos] = '\0';
      break;
    }
    
    if (state == 0)
    {
      if (!is_delimiter(string_[index_]))
      {
	token_[pos++] = string_[index_];
	state = 1;
      }
    }
    else
    {
      if (!is_delimiter(string_[index_]))
      {
	token_[pos++] = string_[index_];
      }
      else
      {
	token_[pos] = '\0';
	break;
      }
    }
    
    index_++;
  }
  
  if (pos == 0)
  {
    return NULL;
  }
  
  return token_;
}

//----------------------------------------------------------------
// Tokenizer::get
// 
char *
Tokenizer::get(const unsigned int & token_index)
{
  if (token_index >= number_) return NULL;
  
  index_ = addrs_[token_index];
  unsigned int pos = 0;
  
  while (true)
  {
    if (string_[index_] == '\0')
    {
      token_[pos] = '\0';
      break;
    }
    
    if (index_ >= length_)
    {
      token_[pos] = '\0';
      break;
    }
    
    if (!is_delimiter(string_[index_]))
    {
      token_[pos++] = string_[index_];
    }
    else
    {
      token_[pos] = '\0';
      break;
    }
    
    index_++;
  }
  
  if (pos == 0)
  {
    return NULL;
  }
  
  return token_;
}

//----------------------------------------------------------------
// Tokenizer::is_delimiter
// 
bool
Tokenizer::is_delimiter(const char & c) const
{
  return (c == '\n' ||
	  c == delimiter_ ||
	  c == '\t' ||
	  c == char(13));
}
