#include <iostream>
#include <QFile>
#include "ImageGroup.h"

using namespace std;

QString ImageGroup::getGroupName ()
{
  return groupName;
}

void ImageGroup::setGroupName (QString imagesGroupName)
{
  groupName = imagesGroupName;
  cout << qPrintable(imagesGroupName) << endl;
}

void ImageGroup::addImageFromFile (QString imageFileName)
{
  QPixmap imagePixmap(imageFileName,0,Qt::AutoColor);
  pictureItem = new QGraphicsPixmapItem();
  pictureItem->setPixmap(imagePixmap);
  action = new QAction(imageFileName, this);
  action->setCheckable(TRUE);
  action->setChecked(TRUE);
  connect(action, SIGNAL(triggered()), this, SLOT(setImageVisibilities()));
  this->addImageToGroup(pictureItem, action);
}

void ImageGroup::setImageVisibilities()
{
  int nActions = this->getNumberOfImages();
  for (int iAction = 0; iAction < nActions; iAction++)
  {
 	getImageAt(iAction)->setVisible(getActionAt(iAction)->isChecked());
	if (getActionAt(iAction)->isChecked())
	  currentImage = iAction;
  }
  printf ("Current image index = %d\n", currentImage);
}

void ImageGroup::unCheckAllActions()
{
  int nActions = this->getNumberOfActions();
  if (nActions > 0)
  {
    for (int iAction = 0; iAction < nActions; iAction++)
	{
	  getActionAt(iAction)->setChecked(FALSE);
	}
  }
}

VectorOfImages ImageGroup::getImageGroup ()
{
  return imageGroup;
}

VectorOfActions ImageGroup::getActionGroup ()
{
  return actionGroup;
}

QGraphicsPixmapItem* ImageGroup::getImageAt (int index)
{
  return imageGroup.at(index);
}

QAction* ImageGroup::getActionAt (int index)
{
  return actionGroup.at(index);
}

int ImageGroup::getCurrentImageIndex ()
{
  return currentImage;
}

QGraphicsPixmapItem* ImageGroup::getCurrentImage ()
{
  //printf ("Retrieving image at %d\n", currentImage);
  return getImageAt(currentImage);
}

QAction* ImageGroup::getCurrentAction()
{
  return getActionAt(currentImage);
}

void ImageGroup::setCurrentImage (int activeImage)
{
  currentImage = activeImage;
  this->setImageVisibilities();
}

int ImageGroup::getNumberOfImages ()
{
  return imageGroup.size();
}

int ImageGroup::getNumberOfActions()
{
  return actionGroup.size();
}

void ImageGroup::addImageToGroup (QGraphicsPixmapItem* image, QAction* action)
{
  imageGroup.push_back(image);
  this->unCheckAllActions();
  actionGroup.push_back(action);
  this->setCurrentImage(getNumberOfImages() - 1);
  //printf ("Adding new image\n");
}

void ImageGroup::removeImageFromGroup (QGraphicsPixmapItem* image)
{
  int imagePosition = imageGroup.indexOf(image);
  this->removeImageFromGroup (imagePosition);
}

void ImageGroup::removeImageFromGroup (int imageIndex)
{
  imageGroup.remove(imageIndex);
  actionGroup.remove(imageIndex);
  this->setCurrentImage(getNumberOfImages() - 1);
}
