#include <iostream>
#include "SynapseGroup.h"
#include <QFile>

using namespace std;

QString SynapseGroup::getGroupName ()
{
	return groupName;
}

void SynapseGroup::setGroupName (QString synapsesGroupName)
{
	groupName = synapsesGroupName;
	cout << qPrintable(synapsesGroupName) << endl;
	//cout << groupName << endl;
}

void SynapseGroup::addSynapsesFromFile (QString synapsesFileName)
{
  setGroupName(synapsesFileName);
  QFile xmlFile (synapsesFileName);
  QDomDocument* domSynapses = new QDomDocument ();

  domSynapses->setContent(&xmlFile);
  QDomElement root = domSynapses->documentElement();

  QDomElement synapseElement = root.firstChildElement("roi");

  while (!synapseElement.isNull())
  {
    SynapseMarker* synapseMarker = new SynapseMarker;

    //Setting Synapse variables
    synapseMarker->setID (synapseElement.firstChildElement("id").text());
    synapseMarker->setType (synapseElement.firstChildElement("type").text());
    synapseMarker->setName (synapseElement.firstChildElement("name").text());
    synapseMarker->setMarkerLabel (synapseElement.firstChildElement("keyWords").firstChildElement("keyWord").text());

    QPointF position;
    position.setX (synapseElement.firstChildElement("position").firstChildElement("xcoord").text().toDouble());
    position.setY (4188 - synapseElement.firstChildElement("position").firstChildElement("ycoord").text().toDouble());
    synapseMarker->setPosition (position);
    synapseMarker->scale (0.25, 0.25);

    int redC = int (synapseElement.firstChildElement("color").firstChildElement("redComponent").text().toDouble() * 255);
    int greenC = int (synapseElement.firstChildElement("color").firstChildElement("greenComponent").text().toDouble() * 255);
    int blueC = int (synapseElement.firstChildElement("color").firstChildElement("blueComponent").text().toDouble() * 255);

    QColor color(redC, greenC, blueC, 255);
    synapseMarker->setColor (color);
	//printf ("Marker Color = (%d, %d, %d)\n",redC, greenC, blueC);
	//flush(stdout);
    //std::cout << "Marker Color = (" << redC << ", " << greenC << ", " << blueC << ")" << endl;
	//std::cout.flush();

    synapseMarker->setDirection (synapseElement.firstChildElement("angle").text().toDouble());

    this->addSynapseToGroup(synapseMarker);
    synapseElement = synapseElement.nextSiblingElement("roi");
  }

  viewGroupPictureAct = new QAction(synapsesFileName, this);
  connect(viewGroupPictureAct, SIGNAL(triggered()), this, SLOT(toggleGroupVisibility()));
  groupVisibility = TRUE;
}

QAction* SynapseGroup::getGroupMenuItem()
{
	return viewGroupPictureAct;
}

void SynapseGroup::toggleGroupVisibility ()
{
  if (groupVisibility)
  {
    groupVisibility = FALSE;
    printf ("Setting Group visibility to False\n"); //flush(stdout);
  }
  else
  {
    groupVisibility = TRUE;
    printf ("Setting Group visibility to True\n"); //flush(stdout);
  }

  int nSynapseMarker = this->getNumberOfSynapses();
  for (int iSynapseMarker = 0; iSynapseMarker < nSynapseMarker; iSynapseMarker++)
  {
    if (groupVisibility)
	  getSynapseAt(iSynapseMarker)->show();
	else
	  getSynapseAt(iSynapseMarker)->hide();
  }
}

VectorOfSynapses SynapseGroup::getSynapseGroup ()
{
	return synapseGroup;
}

SynapseMarker* SynapseGroup::getSynapseAt (int index)
{
	return synapseGroup.at(index);
}

int SynapseGroup::getCurrentSynapseIndex ()
{
  return currentSynapse;
}

SynapseMarker* SynapseGroup::getCurrentSynapse ()
{
	return getSynapseAt(currentSynapse);
}

void SynapseGroup::setCurrentSynapse (int activeSynapse)
{
  currentSynapse = activeSynapse;
}

int SynapseGroup::getNumberOfSynapses ()
{
	return synapseGroup.size();
}

void SynapseGroup::addSynapseToGroup (SynapseMarker* synapse)
{
  synapseGroup.push_back(synapse);
  currentSynapse = synapseGroup.size() - 1;
}

void SynapseGroup::removeSynapseFromGroup (SynapseMarker* synapse)
{
  int synapsePosition = synapseGroup.indexOf(synapse);
  this->removeSynapseFromGroup (synapsePosition);
  currentSynapse = synapseGroup.size() - 1;
}

void SynapseGroup::removeSynapseFromGroup (int synapseIndex)
{
  synapseGroup.remove(synapseIndex);
  currentSynapse = synapseGroup.size() - 1;
}
