#include "SynapseGroupPropertyEditor.h"

using namespace std;

SynapseGroupPropertyEditor::SynapseGroupPropertyEditor()
{
  StandardMethodInit();
  printf ("Intantiating SynapseGroupPropertyEditor\n");
  setUpDialogWidgets ();
  setUpDialog ();
}

SynapseGroupPropertyEditor::~SynapseGroupPropertyEditor()
{
  delete synapseGroupManager;
  delete groupListComboBox;
  delete groupListLabel;
  delete colorLabel;
  delete colorText;
  delete sizeLabel;
  delete sizeText;
  delete defaultOrientationLabel;
  delete defaultOrientationText;
  delete windowLayout;
}

void SynapseGroupPropertyEditor::setDialogDefaults(SynapseGroupManager* sgm)
{
  synapseGroupManager = sgm;
  groupListComboBox->setSizeAdjustPolicy (QComboBox::AdjustToContents);
  groupListComboBox->addItems (sgm->getSynapseGroupFileNames());
  QString colorRGB, colorR, colorG, colorB, colorAlpha;
  colorR = colorR.setNum(sgm->getCurrentSynapseGroup()->getCurrentSynapse()->getColor().red());
  colorG = colorG.setNum(sgm->getCurrentSynapseGroup()->getCurrentSynapse()->getColor().green());
  colorB = colorB.setNum(sgm->getCurrentSynapseGroup()->getCurrentSynapse()->getColor().blue());
  colorRGB = "(" + colorR + ", " + colorG + ", " + colorB + ")";
  colorText->setText (colorRGB);
  colorAlpha = colorAlpha.setNum(sgm->getCurrentSynapseGroup()->getCurrentSynapse()->getColor().alpha());
  transperancyText->setText(colorAlpha);
  QString sizeString;
  sizeString = sizeString.setNum(sgm->getCurrentSynapseGroup()->getCurrentSynapse()->getSize());
  sizeText->setText (sizeString);
  defaultOrientationText->setText ("0");
}

void SynapseGroupPropertyEditor::setUpDialogWidgets()
{
  synapseGroupManager = new SynapseGroupManager();
  groupListComboBox = new QComboBox();
  groupListLabel = new QLabel("List of Groups");
  colorLabel = new QLabel("Synapse Color");
  colorText = new QLineEdit ("Synaspe color Value");
  transperancyLabel = new QLabel("Transperancy");
  transperancyText = new QLineEdit ("Synaspe alpha Value");
  colorButton = new QPushButton (tr("Select Color"));
  connect(colorButton, SIGNAL(clicked()), this, SLOT(setColor()));
  sizeLabel = new QLabel("Synapse Size");
  sizeText = new QLineEdit ("Synapse Size Value");
  defaultOrientationLabel = new QLabel("Default Orientation");
  defaultOrientationText = new QLineEdit ("Default Orientation Value");

  OKButton = new QPushButton (tr("OK"));
  connect(OKButton, SIGNAL(clicked()), this, SLOT(OKClickEvent()));

  applyButton = new QPushButton (tr("Apply"));
  connect(applyButton, SIGNAL(clicked()), this, SLOT(applyClickEvent()));

  cancelButton = new QPushButton (tr("Cancel"));
  connect(cancelButton, SIGNAL(clicked()), this, SLOT(close()));
}

void SynapseGroupPropertyEditor::OKClickEvent ()
{
  setPropertiesToGroup();
  close ();
}

void SynapseGroupPropertyEditor::applyClickEvent ()
{
  setPropertiesToGroup();
}

void SynapseGroupPropertyEditor::setPropertiesToGroup()
{
  int choosenGroup = groupListComboBox->currentIndex();
  int nSynapses = synapseGroupManager->getSynapseGroupAt(choosenGroup)->getNumberOfSynapses();
  color.setAlpha (transperancyText->text().toInt());
  for (int iSynapse = 0; iSynapse < nSynapses; iSynapse++)
  {
    synapseGroupManager->getSynapseGroupAt(choosenGroup)->getSynapseAt(iSynapse)->setColor(color);
    synapseGroupManager->getSynapseGroupAt(choosenGroup)->getSynapseAt(iSynapse)->setSize(sizeText->text().toFloat());
  }
}

void SynapseGroupPropertyEditor::setUpDialog ()
{
  windowLayout = new QGridLayout();
  windowLayout->addWidget (groupListLabel, 1, 1, Qt::AlignRight);
  windowLayout->addWidget (groupListComboBox, 1, 2, Qt::AlignLeft);
  windowLayout->addWidget (colorLabel, 2, 1, Qt::AlignRight);
  QGridLayout* colorWigdetsLayout = new QGridLayout();
  colorWigdetsLayout->addWidget(colorText, 1, 1, Qt::AlignLeft);
  colorWigdetsLayout->addWidget(colorButton, 1, 2, Qt::AlignLeft);
  windowLayout->addLayout (colorWigdetsLayout, 2, 2, Qt::AlignLeft);
  windowLayout->addWidget (transperancyLabel, 3, 1, Qt::AlignRight);
  windowLayout->addWidget (transperancyText, 3, 2, Qt::AlignLeft);
  windowLayout->addWidget (sizeLabel, 4, 1, Qt::AlignRight);
  windowLayout->addWidget (sizeText, 4, 2, Qt::AlignLeft);
  windowLayout->addWidget (defaultOrientationLabel, 5, 1, Qt::AlignRight);
  windowLayout->addWidget (defaultOrientationText, 5, 2, Qt::AlignLeft);
  QGridLayout* commandButtonsLayout = new QGridLayout();
  commandButtonsLayout->addWidget(OKButton, 1, 1, Qt::AlignLeft);
  commandButtonsLayout->addWidget(applyButton, 1, 2, Qt::AlignLeft);
  commandButtonsLayout->addWidget(cancelButton, 1, 3, Qt::AlignLeft);
  windowLayout->addLayout (commandButtonsLayout, 6, 2, Qt::AlignLeft);
  this->setLayout (windowLayout);
  this->setWindowTitle(tr("Synapses Viewer : Set Synapse Group Properties"));
  this->resize(400, 250); 
  this->show();
}

void SynapseGroupPropertyEditor::setColor ()
{
  color = QColorDialog::getColor(Qt::green, this);
  if (color.isValid()) 
  {
    QString colorRGB, colorR, colorG, colorB;
    colorR = colorR.setNum(color.red());
    colorG = colorG.setNum(color.green());
    colorB = colorB.setNum(color.blue());
    colorRGB = "(" + colorR + ", " + colorG + ", " + colorB + ")";
    colorText->setText (colorRGB);
	colorText->setPalette(QPalette(color));
	colorText->setAutoFillBackground(true);
  }
}