#include "SynapseMarker.h"

#include <QGraphicsScene>
#include <QPainter>
#include <QStyleOption>

#include <math.h>

static const QPointF arrow[7] = {QPointF(0.0, -25), QPointF(30.0, 0), QPointF(0.0, 25), 
                                 QPointF(0.0, 10), QPointF(-40.0, 10), QPointF(-40.0, -10.0), QPointF(0.0, -10)};

SynapseMarker::SynapseMarker()
{
  Size = 1.0;
}

QRectF SynapseMarker::boundingRect() const
{
  qreal adjust = 0.5;
  return QRectF(-18 - adjust, -22 - adjust, 36 + adjust, 60 + adjust);
}

void SynapseMarker::paint(QPainter *painter, const QStyleOptionGraphicsItem *, QWidget *)
{
  painter->setBrush(Color);
  painter->scale(Size, Size);
  painter->drawPolygon(arrow, 7);
}

void SynapseMarker::setID (QString id)
{
  Id = id;
}

void SynapseMarker::setType (QString type)
{
  Type = type;
}

void SynapseMarker::setName (QString name)
{
  Name = name;
}

void SynapseMarker::setMarkerLabel (QString markerLabel)
{
  MarkerLabel = markerLabel;
}

void SynapseMarker::setSize (qreal nSize)
{
  Size = nSize;
}

void SynapseMarker::setColor (QColor color)
{
  Color = color;
}

void SynapseMarker::setDirection (qreal direction)
{
  Direction = direction;
}

void SynapseMarker::setPosition (QPointF position)
{
  Position = position;
}

QString SynapseMarker::getID ()
{
  return Id;
}

QString SynapseMarker::getType ()
{
  return Type;
}

QString SynapseMarker::getName ()
{
  return Name;
}

QString SynapseMarker::getMarkerLabel ()
{
  return MarkerLabel;
}

qreal SynapseMarker::getSize ()
{
  return Size;
}

QColor SynapseMarker::getColor ()
{
  return Color;
}

qreal SynapseMarker::getDirection ()
{
  return Direction;
}

QPointF SynapseMarker::getPosition ()
{
  return Position;
}
