#ifndef SYNAPSESVIEWER_H
#define SYNAPSESVIEWER_H

#include <QDialog>
#include <QGroupBox>
#include <QPushButton>
#include <QMainWindow>
#include <QSlider>
#include <QSpinBox>
#include <QtXml/qdom.h>
#include <QVector>
#include "ImageGroup.h"
#include "SynapseMarker.h"
#include "SynapseGroup.h"
#include "SynapseGroupManager.h"
#include "SynapseGroupPropertyEditor.h"
#include "ViewerController.h"

#define ADD_SYNAPSE_MODE	0;
#define MOVE_SYNAPSE_MODE	1;
#define REMOVE_SYNAPSE_MODE	2;

class QGraphicsScene;
class QGraphicsView;
class QSlider;
class QDomDocument;
class QPushButton;
class QGridLayout;
class QVBoxLayout;

class SynapsesViewer : public QMainWindow
{
    Q_OBJECT

public:
  SynapsesViewer();
  ~SynapsesViewer();
  SynapsesViewer(int argc, char **argv);

private slots:
  void editSynapseGroup();
  void moveSceneRight();
  void moveSceneLeft();
  void moveSceneUp();
  void moveSceneDown();
  void openPicture();
  void openSynapses();
  void openSVF();
  void openSVFFile(QString svfFileName);
  void setActivePicture();
  void setAddSynapseMode();
  void setMoveSynapseMode();
  void setRemoveSynapseMode();
  void zoomInPicture();
  void zoomOutPicture();
  void setScale (int newScale);

private:
  int synapseMode;
  int imageScale;
  SynapseGroupManager *synapseGroupManager;
  SynapseGroupPropertyEditor *synapseGroupPropertyEditor;
  ImageGroup *imageGroup;
  ViewerController *viewerController;
  QString synapsesFilename;
  QString pictureFilename;
  QAction *exitAct;
  QAction *openPictureAct;
  QAction *openSVFAct;
  QAction *openSynapsesAct;
  QAction *zoomInPictureAct;
  QAction *zoomOutPictureAct;
  QAction *synapseGroupVisibilityAct;
  QAction *imageGroupVisibilityAct;
  QAction *setSynapseGroupPropertiesAct;
  QAction *addSynapseModeAct;
  QAction *removeSynapseModeAct;
  QAction *moveSynapseModeAct;
  QActionGroup *imageVisibilityGroup;
  QDomDocument *domSynapses;
  QDomDocument *svfElements;
  QGraphicsPixmapItem *pictureItem;
  QGraphicsScene *scene;
  QGraphicsView *view;
  QMenu *fileMenu;
  QMenu *viewMenu;
  QMenu *toolsMenu;
  QMenu *synapseGroupVisibilityMenu;
  QMenu *imageGroupVisibilityMenu;
  //QSlider *scaleSlider;
  //QGridLayout *translatorsLayout;
  //QGridLayout *scalingLayout;
  QGridLayout *setImageLayout;

  void createActions();
  void initializeApplication();
  void initializeViewController();
  void initializeWidgets();
  void initializeSynapseGroupManager();
  void initializeImageGroup();
  void createMenus();
  //void createScalingWidgets();
  //void createTranslatingWidgets();
  void setPicture (QString fileName);
  void setSynapses (QString fileName);
  void setSynapses (SynapseGroup* newSynapseGroup);
  void updateViewer(QString pictureFileName, QString synapsesFileName);
};

#endif //SYNAPSESVIEWER_H

