// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : mosaic_cache_pyramid.cxx
// Author       : Pavel A. Koshevoy
// Created      : Fri Oct 20 12:18:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : mosaic cache pyramid -- supports on-demand
//                coarse-to-fine mosaic image regeneration.

// local includes:
#include "mosaic_cache_pyramid.hxx"
#include "mosaic_cache.hxx"

// system includes:
#include <math.h>


//----------------------------------------------------------------
// mosaic_cache_pyramid_t::mosaic_cache_pyramid_t
// 
mosaic_cache_pyramid_t::mosaic_cache_pyramid_t():
  cache_(0, 16, NULL)
{}

//----------------------------------------------------------------
// mosaic_cache_pyramid_t::~mosaic_cache_pyramid_t
// 
mosaic_cache_pyramid_t::~mosaic_cache_pyramid_t()
{
  clear();
}

//----------------------------------------------------------------
// mosaic_cache_pyramid_t::clear
// 
void
mosaic_cache_pyramid_t::clear()
{
  for (unsigned int i = 0; i < cache_.size(); i++)
  {
    delete cache_[i];
    cache_[i] = NULL;
  }
  
  cache_.resize(0);
}

//----------------------------------------------------------------
// mosaic_cache_pyramid_t::setup
// 
void
mosaic_cache_pyramid_t::setup(const double w_max,
			      const double h_max,
			      const double spacing_x,
			      const double spacing_y)
{
#if 0
  // FIXME: force the single-res pyramid:
  const unsigned int num_levels = 1;
  
#else
  // calculate the number of levels of the multi-res pyramid:
  static const double LOG_2 = log(2.0);
  
  const double x_discrete_levels = floor(0.5 + w_max / spacing_x);
  const double y_discrete_levels = floor(0.5 + h_max / spacing_y);
  const double discrete_levels = std::max(x_discrete_levels,
					  y_discrete_levels);
  const double min_discrete_levels = 64.0;
  
  const double divisions =
    std::max(log(double(discrete_levels) /
		 double(min_discrete_levels)) / LOG_2,
	     0.0);
  
  const unsigned int num_levels = 1 + (unsigned int)(floor(divisions));
#endif
  
  setup(num_levels, spacing_x, spacing_y);
}

//----------------------------------------------------------------
// mosaic_cache_pyramid_t::setup
// 
void
mosaic_cache_pyramid_t::setup(const unsigned int num_levels,
			      const double spacing_x,
			      const double spacing_y)
{
  clear();
  cache_.resize(num_levels);
  
  double two_to_i = 1.0;
  for (unsigned int i = 0; i < num_levels; i++)
  {
    cache_[i] = new mosaic_cache_t(i,
				   spacing_x * two_to_i,
				   spacing_y * two_to_i);
    two_to_i *= 2.0;
  }
}
