// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : mosaic_cache_pyramid.hxx
// Author       : Pavel A. Koshevoy
// Created      : Fri Oct 20 11:44:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : mosaic cache pyramid -- supports on-demand
//                coarse-to-fine mosaic image regeneration.

#ifndef MOSAIC_CACHE_PYRAMID_HXX_
#define MOSAIC_CACHE_PYRAMID_HXX_

// thelib includes:
#include <utils/the_dynamic_array.hxx>

// forward declarations:
class mosaic_cache_t;


//----------------------------------------------------------------
// mosaic_cache_pyramid_t
// 
class mosaic_cache_pyramid_t
{
public:
  mosaic_cache_pyramid_t();
  ~mosaic_cache_pyramid_t();
  
  // remove all cache levels:
  void clear();
  
  // Initialize the pyramid to store multi-resolution mosaic caches
  // such that the first pyramid level has the highest resolution
  // and the resolution is reduced in half at each consecutive level
  // 
  // NOTE: this will destroy the existing cache levels:
  void setup(const double w_max,
	     const double h_max,
	     const double spacing_x,
	     const double spacing_y);
  
  void setup(const unsigned int num_levels,
	     const double spacing_x,
	     const double spacing_y);
  
  // cache levels:
  the_dynamic_array_t<mosaic_cache_t *> cache_;
};


#endif // MOSAIC_CACHE_PYRAMID_HXX_
