// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : mosaic_layout_controller.hxx
// Author       : Pavel A. Koshevoy
// Created      : Wed Mar 21 14:48:49 MDT 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Wrapper class for automatic mosaic layout.

#ifndef MOSAIC_LAYOUT_CONTROLLER_HXX_
#define MOSAIC_LAYOUT_CONTROLLER_HXX_

// system includes:
#include <vector>
#include <list>

// the includes:
#include <itk/fft_common.hxx>
#include <itk/mosaic_layout_common.hxx>
#include <thread/the_transaction.hxx>

// local includes:
#include "the_mosaic_proc.hxx"


//----------------------------------------------------------------
// mosaic_layout_controller_t
// 
class mosaic_layout_controller_t
{
public:
  mosaic_layout_controller_t(std::list<the_tile_t *> & tile_primitives,
			     const unsigned int & num_tiles);
  
  ~mosaic_layout_controller_t();
  
  void layout(the_log_t & log);
  
  // number of tiles to layout:
  unsigned int num_tiles_;
  
  // maximum number of intermediate tiles in cascaded mappings:
  unsigned int max_cascade_len_;
  
  // all the tiles in the mosaic, and their masks:
  array2d(native_image_t::Pointer) tile_pyramid_;
  array2d(mask_t::ConstPointer) mask_pyramid_;
  
  // the computed transforms mapping into the target tile:
  std::vector<affine_transform_t::Pointer> affine_;
  
  // a vector identifying which tiles have been successfully matched:
  std::vector<bool> matched_;
  
  // cost[num_intermediate_tiles][i][j] contains the maximum error metric
  // due to the image mapping between each image in the cascading chain:
  array3d(double) cost_;
  
  // path[num_intermediate_tiles][i][j] contains the transforms
  // associated with the specified number of intermediate images
  // that maps image i to image j:
  array3d(translate_transform_t::Pointer) path_;
  
  // this map is used to match the mismatched images:
  // index  = image id
  // first  = neighbor image id
  // second = list of peaks in the corresponding displacement PDF
  array2d(std::list<local_max_t>) peaks_;
  
  // pyramid level used for matching:
  unsigned int optimal_level_;
  
  // min/max tile overlap parameters:
  bool use_std_mask_;
  double overlap_min_;
  double overlap_max_;
  unsigned int max_peaks_;
};


//----------------------------------------------------------------
// tile_match_transaction_t
// 
class tile_match_transaction_t : public the_transaction_t
{
public:
  tile_match_transaction_t(mosaic_layout_controller_t * controller);
  
  // virtual:
  void execute(the_thread_interface_t * thread);
  
  mosaic_layout_controller_t * controller_;
};


#endif // MOSAIC_LAYOUT_CONTROLLER_HXX_
