// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : mosaic_refinement_controller.hxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Mar 26 10:19:43 MDT 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Wrapper class for automatic mosaic refinement.

#ifndef MOSAIC_REFINEMENT_CONTROLLER_HXX_
#define MOSAIC_REFINEMENT_CONTROLLER_HXX_

// system includes:
#include <vector>
#include <list>

// the includes:
#include <itk/fft_common.hxx>
#include <itk/itkGridTransform.h>
#include <thread/the_transaction.hxx>

// local includes:
#include "the_mosaic_proc.hxx"


//----------------------------------------------------------------
// USE_GRID_REFINEMENT
// 
#define USE_GRID_REFINEMENT


//----------------------------------------------------------------
// mosaic_refinement_controller_t
// 
class mosaic_refinement_controller_t
{
public:
  mosaic_refinement_controller_t(std::list<the_tile_t *> & tile_primitives,
				 const unsigned int & num_tiles);
  
  ~mosaic_refinement_controller_t();
  
  void refine(the_log_t & log);
  
  // all the tiles in the mosaic, and their masks:
  array2d(native_image_t::Pointer) tile_pyramid_;
  array2d(mask_t::Pointer) mask_pyramid_;
  std::vector<native_image_t::Pointer> tile_;
  std::vector<mask_t::ConstPointer> mask_;
  std::vector<the_grid_transform_t> grid_;
  
  // the computed transforms mapping into the target tile:
#ifdef USE_GRID_REFINEMENT
  std::vector<itk::GridTransform::Pointer> transform_;
#else
  std::vector<base_transform_t::Pointer> transform_;
#endif
};


//----------------------------------------------------------------
// mosaic_refinement_transaction_t
// 
class mosaic_refinement_transaction_t : public the_transaction_t
{
public:
  mosaic_refinement_transaction_t(mosaic_refinement_controller_t * controller);
  
  // virtual:
  void execute(the_thread_interface_t * thread);
  
  mosaic_refinement_controller_t * controller_;
};


#endif // MOSAIC_REFINEMENT_CONTROLLER_HXX_
