static const char * shader_la_src = 
"//----------------------------------------------------------------\n"
"// decode_la\n"
"// \n"
"float2\n"
"decode_la(float pixel)\n"
"{\n"
"  // decode the pixel here:\n"
"  float byte = floor(0.5 + pixel * 255.0f);\n"
"  \n"
"  float2 la;\n"
"  if (byte < 120.0)\n"
"  {\n"
"    // decode the alpha/luminance indices:\n"
"    float a_bin = min(14.0, floor((sqrt(1.0 + 8.0 * byte) - 1.0) / 2.0));\n"
"    float offset = a_bin * (a_bin + 1.0) / 2.0;\n"
"    float l_bin = byte - offset;\n"
"    \n"
"    la.x = (l_bin + 0.5) / (a_bin + 1.0);\n"
"    la.y = (a_bin) / 15.0;\n"
"  }\n"
"  else\n"
"  {\n"
"    la.x = floor(byte - 120.0) / 136.0;\n"
"    la.y = 1.0;\n"
"  }\n"
"  \n"
"  return la;\n"
"}\n"
"\n"
"//----------------------------------------------------------------\n"
"// get_la\n"
"// \n"
"float2\n"
"get_la(uniform sampler2D tile_texture,\n"
"       float s,\n"
"       float t)\n"
"{\n"
"  // s = min(1.0, max(0.0, s));\n"
"  // t = min(1.0, max(0.0, t));\n"
"  return decode_la(tex2D(tile_texture, float2(s, t)).r);\n"
"}\n"
"\n"
"//----------------------------------------------------------------\n"
"// main\n"
"// \n"
"void\n"
"main(float2 st : TEX0,\n"
"     uniform sampler2D tile_texture,\n"
"     uniform float2 tile_size,\n"
"     uniform float3 tint,\n"
"     uniform bool mask,\n"
"     uniform bool antialias,\n"
"     out float4 frag_color : COLOR)\n"
"{\n"
"#ifndef DISABLE_AA\n"
"  if (antialias)\n"
"  {\n"
"    float nx = tile_size.x;\n"
"    float ny = tile_size.y;\n"
"    \n"
"    float ix[2];\n"
"    ix[0] = st.s * nx;\n"
"    ix[1] = floor(ix[0]) + 1.0;\n"
"    \n"
"    float wx[2];\n"
"    wx[1] = ix[0] - (ix[1] - 1.0);\n"
"    wx[0] = 1.0 - wx[1];\n"
"    \n"
"    ix[0] = (ix[1] - 1.0) / nx;\n"
"    ix[1] /= nx;\n"
"    \n"
"    float iy[2];\n"
"    iy[0] = st.t * ny;\n"
"    iy[1] = floor(iy[0]) + 1.0;\n"
"    \n"
"    float wy[2];\n"
"    wy[1] = iy[0] - (iy[1] - 1.0);\n"
"    wy[0] = 1.0 - wy[1];\n"
"    \n"
"    iy[0] = (iy[1] - 1.0) / ny;\n"
"    iy[1] /= ny;\n"
"    \n"
"    float2 la = (wx[0] * wy[0] * get_la(tile_texture, ix[0], iy[0]) +\n"
"		 wx[1] * wy[0] * get_la(tile_texture, ix[1], iy[0]) +\n"
"		 wx[0] * wy[1] * get_la(tile_texture, ix[0], iy[1]) +\n"
"		 wx[1] * wy[1] * get_la(tile_texture, ix[1], iy[1]));\n"
"    \n"
"    \n"
"    if (!mask)\n"
"    {\n"
"      frag_color = float4(tint.r * la.x,\n"
"			  tint.g * la.x,\n"
"			  tint.b * la.x,\n"
"			  la.y);\n"
"    }\n"
"    else\n"
"    {\n"
"      frag_color = float4(0, 0, 0, la.y);\n"
"    }\n"
"  }\n"
"  else\n"
"#endif // DISABLE_AA\n"
"  {\n"
"    float2 la = get_la(tile_texture, st.x, st.y);\n"
"    if (!mask)\n"
"    {\n"
"      frag_color = float4(tint.r * la.x,\n"
"			  tint.g * la.x,\n"
"			  tint.b * la.x,\n"
"			  la.y);\n"
"    }\n"
"    else\n"
"    {\n"
"      frag_color = float4(0, 0, 0, la.y);\n"
"    }\n"
"  }\n"
"}\n";
