static const char * shader_rgba16_src = 
"//----------------------------------------------------------------\n"
"// decode_rgba16\n"
"// \n"
"float4\n"
"decode_rgba16(float b0, float b1)\n"
"{\n"
"  float4 rgba;\n"
"  \n"
"#if 1\n"
"  float r = b0 / 8;		// r  = b0 >> 3\n"
"  float g0 = fmod(b0, 8);	// g0 = b0 & 7\n"
"  float g1 = b1 / 64;		// g1 = b1 >> 6\n"
"  float g = g0 * 4 + g1;	// g  = (g0 << 2) | g1\n"
"  float b = fmod((b1 / 2), 32);	// b  = (b1 >> 1) & 31\n"
"  float a = fmod(b1, 2);	// a  = b1 & 1\n"
"#elif 0\n"
"  // decode R5-G3-B5-A1-g2\n"
"  float r = floor(0.5 + b0 / 8);\n"
"  float b = floor(0.5 + b1 / 8);\n"
"  float g0 = floor(0.5 + fmod(b0, 8));\n"
"  float a = floor(0.5 + fmod(b1, 8) / 4);\n"
"  float g1 = floor(0.5 + fmod(b1, 4));\n"
"  float g = g0 * 4 + g1;\n"
"#else\n"
"  // decode A1-R5-g2-G3-B5\n"
"  float a = b0 / 128;\n"
"  float r = fmod(b0, 128) / 4;\n"
"  float g1 = fmod(b0, 4);\n"
"  float g0 = b1 / 32;\n"
"  float b = fmod(b1, 32);\n"
"  float g = g0 * 4 + g1;\n"
"#endif\n"
"  \n"
"  rgba.r = float(r) / 32.0f;\n"
"  rgba.g = float(g) / 32.0f;\n"
"  rgba.b = float(b) / 32.0f;\n"
"  rgba.a = float(a);\n"
"  return rgba;\n"
"}\n"
"\n"
"//----------------------------------------------------------------\n"
"// get_la\n"
"// \n"
"float\n"
"get_byte(uniform sampler2D tile_texture,\n"
"	 float s,\n"
"	 float t)\n"
"{\n"
"  return floor(0.5f + tex2D(tile_texture, float2(s, t)).r * 255.0f);\n"
"}\n"
"\n"
"//----------------------------------------------------------------\n"
"// main\n"
"// \n"
"void\n"
"main(float2 st : TEX0,\n"
"     uniform sampler2D tile_texture,\n"
"     uniform float2 tile_size,\n"
"     uniform bool antialias,\n"
"     out float4 frag_color : COLOR)\n"
"{\n"
"#ifndef DISABLE_AA\n"
"  if (antialias)\n"
"  {\n"
"    float nx = tile_size.x / 2.0;\n"
"    float ny = tile_size.y;\n"
"    \n"
"    // lookup the first byte:\n"
"    float ix[2];\n"
"    ix[0] = st.s * nx;\n"
"    ix[1] = floor(ix[0]) + 1.0;\n"
"    \n"
"    float wx[2];\n"
"    wx[1] = ix[0] - (ix[1] - 1.0);\n"
"    wx[0] = 1.0 - wx[1];\n"
"    \n"
"    ix[0] = (ix[1] - 1.0) / nx;\n"
"    ix[1] /= nx;\n"
"    \n"
"    float iy[2];\n"
"    iy[0] = st.t * ny;\n"
"    iy[1] = floor(iy[0]) + 1.0;\n"
"    \n"
"    float wy[2];\n"
"    wy[1] = iy[0] - (iy[1] - 1.0);\n"
"    wy[0] = 1.0 - wy[1];\n"
"    \n"
"    iy[0] = (iy[1] - 1.0) / ny;\n"
"    iy[1] /= ny;\n"
"    \n"
"    float b00 = get_byte(tile_texture, ix[0], iy[0]);\n"
"    float b01 = get_byte(tile_texture, ix[1], iy[0]);\n"
"    float b02 = get_byte(tile_texture, ix[0], iy[1]);\n"
"    float b03 = get_byte(tile_texture, ix[1], iy[1]);\n"
"    \n"
"    // lookup the second byte:\n"
"    float ds = 0.5 / nx;\n"
"    ix[0] += ds;\n"
"    ix[1] += ds;\n"
"    \n"
"    float b10 = get_byte(tile_texture, ix[0], iy[0]);\n"
"    float b11 = get_byte(tile_texture, ix[1], iy[0]);\n"
"    float b12 = get_byte(tile_texture, ix[0], iy[1]);\n"
"    float b13 = get_byte(tile_texture, ix[1], iy[1]);\n"
"    \n"
"    frag_color = (wx[0] * wy[0] * decode_rgba16(b00, b10) +\n"
"		  wx[1] * wy[0] * decode_rgba16(b01, b11) +\n"
"		  wx[0] * wy[1] * decode_rgba16(b02, b12) +\n"
"		  wx[1] * wy[1] * decode_rgba16(b03, b13));\n"
"  }\n"
"  else\n"
"#endif // DISABLE_AA\n"
"  {\n"
"    float ix[2];\n"
"    \n"
"    float nx = tile_size.x / 2.0;\n"
"    ix[0] = floor(st.s * nx);\n"
"    ix[1] = ix[0] + 0.5;\n"
"    ix[0] /= nx;\n"
"    ix[1] /= nx;\n"
"    \n"
"    float b0 = get_byte(tile_texture, ix[0], st.t);\n"
"    float b1 = get_byte(tile_texture, ix[1], st.t);\n"
"    frag_color = decode_rgba16(b0, b1);\n"
"  }\n"
"}\n";
