// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// Copyright    : (C) 2007, 2008 University of Utah

// system includes:
#include <iostream>
#include <fstream>
#include <string>

// namespace access:
using std::cin;
using std::cout;
using std::cerr;
using std::endl;
using std::fstream;
using std::ios;
using std::string;


//----------------------------------------------------------------
// usage
// 
static void
usage(char ** argv, const char * message = NULL)
{
  cerr << "USAGE: " << argv[0] << " "
       << "-name name "
       << "-load file_load "
       << "-save file_save "
       << endl
       << "EXAMPLE: " << argv[0] << " "
       << "-name la_shader_src "
       << "-load la_shader.fs "
       << "-save la_shader.hxx "
       << endl;
  
  if (message != NULL)
  {
    cerr << "ERROR: " << message << endl;
  }
  
  exit(1);
}

//----------------------------------------------------------------
// main
// 
int
main(int argc, char ** argv)
{
  // parse the command line arguments:
  string name;
  string load;
  string save;
  
  for (int i = 1; i < argc; i++)
  {
    if (strcmp(argv[i], "-name") == 0)
    {
      if ((argc - i) <= 1) usage(argv, "could not parse -name option");
      
      i++;
      name = string(argv[i]);
    }
    else if (strcmp(argv[i], "-load") == 0)
    {
      if ((argc - i) <= 1) usage(argv, "could not parse -load option");
      
      i++;
      load = string(argv[i]);
    }
    else if (strcmp(argv[i], "-save") == 0)
    {
      if ((argc - i) <= 1) usage(argv, "could not parse -save option");
      
      i++;
      save = string(argv[i]);
    }
    else
    {
      usage(argv,
	    (string("unknown option \'") +
	     string(argv[i]) +
	     string("\'")).c_str());
    }
  }
  
  if (name.empty())
  {
    usage(argv, "you must specify the -name option");
  }
  
  fstream file_load;
  file_load.open(load.c_str(), ios::in);
  if (!file_load.is_open())
  {
    usage(argv, (string("could not load file: ") + load).c_str());
  }
  
  fstream file_save;
  file_save.open(save.c_str(), ios::out);
  if (!file_save.is_open())
  {
    usage(argv, (string("could not save file: ") + save).c_str());
  }
  
  file_save << "static const char * " << name << " = " << endl;
  
  bool first = true;
  while (true)
  {
    string line;
    getline(file_load, line);
    if (file_load.eof()) break;
    if (!first) file_save << endl;
    
    file_save << '"' << line << "\\n\"";
    first = false;
  }
  file_save << ";" << endl;
  
  return 0;
}
