// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_main_window.hxx
// Author       : Pavel A. Koshevoy
// Created      : 2007/03/21 12:01
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : main application window widget

#ifndef THE_MAIN_WINDOW_HXX_
#define THE_MAIN_WINDOW_HXX_

// system includes:
#include <list>

// Qt includes:
#include <QtGui>
#include <QGridLayout>
#include <QMainWindow>
#include <QTimer>
#include <QStringList>
#include <QAbstractListModel>

// local includes:
#include "ui_MainWindow.h"
#include "ui_AboutDialog.h"
#include "PreferencesDialog.h"

// the includes:
#include <doc/the_document.hxx>
#include <ui/the_document_ui.hxx>
#include <Qt/the_qt_view.hxx>

// forward declarations:
class the_mosaic_proc_t;
class the_mosaic_proc_ui_t;
class the_transaction_t;
class the_tile_t;
class the_tile_transaction_t;


//----------------------------------------------------------------
// the_tiles_model_t
// 
class the_tiles_model_t : public QAbstractListModel
{
  Q_OBJECT
  
public:
  the_tiles_model_t(the_document_so_t * doc_so, QObject * parent = 0);
  
  // virtual:
  int rowCount(const QModelIndex & parent = QModelIndex()) const;
  
  // virtual:
  QVariant data(const QModelIndex & index, int role = Qt::DisplayRole) const;
  
  // helpers:
  the_mosaic_proc_t * proc() const;
  the_tile_t * tile(const QModelIndex & index) const;
  
  // return a model index corresponding to a given tile:
  QModelIndex index(const QString & tile_absolute_file_path) const;
  
  void trigger_update(const QModelIndex & index);
  void trigger_reset();
  
private:
  the_document_so_t * doc_so_;
};

//----------------------------------------------------------------
// the_main_window_t
// 
class the_main_window_t : public QMainWindow,
			  public Ui::MainWindow,
			  public the_document_ui_t
{
  Q_OBJECT
  
public:
  the_main_window_t();
  ~the_main_window_t();
  
public slots:
  void tile_ready(the_transaction_t *);
  void selection_changed(const QModelIndex &, const QModelIndex &);
  
  // the file menu:
  void file_open();
  void file_open(const QString & filename);
  bool file_save();
  void file_exit();
  
  // the edit menu:
  void edit_undo();
  void edit_redo();
  void edit_unwarp();
  void edit_prefs();
  
  // the view menu:
  void view_restore();
  
  // the tiles menu:
  void tiles_add();
  void tiles_add(const QStringList & filenames,
		 unsigned int shrink_factor,
		 double spacing_x,
		 double spacing_y);
  void tiles_remove();
  void tiles_clear();
  
  // the mosaic menu:
  void mosaic_layout();
  void mosaic_refine();
  
  // the help menu:
  void help_about();
  
public:
  // add a tile primitive to the procedure and return a transaction
  // to load the tile:
  the_tile_transaction_t * tile_load_transaction(const QString & filename,
						 unsigned int shrink_factor,
						 double spacing_x,
						 double spacing_y);
  
  // virtual: ui update mechanism:
  void sync_ui();
  
  // virtual:
  void kill_ui();
  
  // collect the visible views:
  void get_visible_views(std::vector<the_view_t *> & views) const;
  
  // if in single view mode, return the view:
  the_view_t * single_view() const;
  
  // accessor to the mosaic procedure ui:
  inline the_mosaic_proc_ui_t * mosaic_proc_ui() const
  { return mosaic_proc_ui_; }
  
  // preference accessor:
  inline PreferencesDialog & preferences()
  { return preferences_ui_; }
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  void dragEnterEvent(QDragEnterEvent * e);
  void dropEvent(QDropEvent * e);
  
  // the list view model:
  the_tiles_model_t * tiles_model_;
  
  // the procedure ui:
  the_mosaic_proc_ui_t * mosaic_proc_ui_;
  
  // the preferences ui:
  PreferencesDialog preferences_ui_;
};


//----------------------------------------------------------------
// AboutDialog
// 
class AboutDialog : public QDialog,
		    public Ui::AboutDialog
{
  Q_OBJECT
  
public:
  AboutDialog(QWidget * parent = 0, Qt::WFlags f = 0):
    QDialog(parent, f),
    Ui::AboutDialog()
  {
    Ui::AboutDialog::setupUi(this);
  }
};


#endif // THE_MAIN_WINDOW_HXX_
