// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_mosaic_proc_ui.hxx
// Author       : Pavel A. Koshevoy
// Created      : Wed Feb 14 12:11:00 MST 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : UI for the mosaic layout procedure.

#ifndef THE_MOSAIC_PROC_UI_HXX_
#define THE_MOSAIC_PROC_UI_HXX_

// system includes:
#include <list>

// the includes:
#include <ui/the_procedure_ui.hxx>
#include <eh/the_document_eh.hxx>
#include <doc/the_document.hxx>

// Qt includes:
#include <QObject>
#include <QTimer>

// forward declarations:
class the_registry_t;
class the_main_window_t;
class the_mosaic_proc_t;
class the_transaction_t;


//----------------------------------------------------------------
// the_mosaic_proc_eh_t
// 
class the_mosaic_proc_eh_t : public the_document_eh_t<the_document_t>
{
public:
  the_mosaic_proc_eh_t(the_document_ui_t * doc_ui);
  
  // virtual:
  const char * name() const
  { return "the_mosaic_proc_eh_t"; }
  
  // virtual: check if a given modifier is currently active:
  bool is_modifier_active_locally(the_eh_modifier_t mod) const;
  
  // helpers, shortcuts:
  the_registry_t * registry() const;
  
  // helper functions:
  the_mosaic_proc_t * proc() const;
  
  // virtual:
  bool view_handler(the_view_t * view);
  
protected:
  // virtual: Override the processor function.
  bool processor(the_input_device_eh_t::the_event_type_t event_type);
  
  // processor helper functions:
  bool process_mouse();
  
  typedef enum {
    HAS_NOTHING_E,
    HAS_TILE_E,
    HAS_POINTS_E,
    MOVE_TILE_E,
    MOVE_POINTS_E,
    WARP_MESH_LOCALLY_E
  } mosaic_state_t;
  
  mosaic_state_t mosaic_state_;
};

//----------------------------------------------------------------
// the_mosaic_proc_ui_t
// 
class the_mosaic_proc_ui_t : public QObject,
			     public the_procedure_ui_t
{
  Q_OBJECT
  
public:
  the_mosaic_proc_ui_t(the_main_window_t * doc_ui);
  ~the_mosaic_proc_ui_t();
  
  // virtual: start/stop the ui:
  void install();
  void uninstall();
  
  // virtual: ui update mechanism:
  void sync_ui();
  
  // helper functions:
  the_mosaic_proc_t * proc();
  
public slots:
  void rgbaDone(the_transaction_t * t);
  
private slots:
  // this slot will be connected to the repaint timer:
  void repaintMosaic();
  
protected:
  // virtual:
  void customEvent(QEvent * e);
  
private:
  // the event handler that will update the procedure:
  the_mosaic_proc_eh_t eh_;
  
  // this timer will be used to buffer OpenGL updates -- instead of
  // executing each repaint immediately the repaint will be carried
  // out after a timeout event:
  QTimer repaint_timer_;
};


#endif // THE_MOSAIC_PROC_UI_HXX_
