// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_tile_transaction.hxx
// Author       : Pavel A. Koshevoy
// Created      : Fri Feb 23 12:43:00 MST 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Load a mosaic tile, generate a multiresolutions pyramid
//                and OpenGL textures.

#ifndef THE_TILE_TRANSACTION_HXX_
#define THE_TILE_TRANSACTION_HXX_

// Boost includes:
#include <boost/shared_ptr.hpp>

// the includes:
#include <thread/the_transaction.hxx>
#include <thread/the_thread_interface.hxx>

// local includes:
#include "the_mosaic_proc.hxx"


//----------------------------------------------------------------
// the_tile_transaction_t
// 
class the_tile_transaction_t : public the_transaction_t
{
public:
  the_tile_transaction_t(unsigned int tile_id,
			 boost::shared_ptr<the_image_pyramid_t> & pyramid,
			 unsigned int shrink_factor,
			 unsigned int max_texture,
			 bool use_fragment_shader,
			 bool use_std_mask,
			 bool use_luminance_textures);
  
  // virtual:
  void execute(the_thread_interface_t * thread);
  
  // registry id of the tile primitive:
  unsigned int tile_id_;
  
  // the pyramid that has to be loaded:
  boost::shared_ptr<the_image_pyramid_t> pyramid_;
  
  // mask and texture parameters:
  const unsigned int shrink_factor_;
  const unsigned int max_texture_;
  const bool use_fragment_shader_;
  const bool use_std_mask_;
  const bool use_luminance_textures_;
};


#endif // THE_TILE_TRANSACTION_HXX_
