// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_warped_image_dl_elem.hxx
// Author       : Pavel A. Koshevoy
// Created      : Wed Mar 3 12:10:00 MST 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : An OpenGL display list element for an image
//                warped with the grid transform.

#ifndef THE_WARPED_IMAGE_DL_ELEM_HXX_
#define THE_WARPED_IMAGE_DL_ELEM_HXX_

// the includes:
#include <opengl/the_disp_list.hxx>
#include <opengl/image_tile_dl_elem.hxx>
#include <image/image_tile_generator.hxx>
#include <itk/the_grid_transform.hxx>

// Cg includes:
#include <Cg/cgGL.h>


//----------------------------------------------------------------
// the_warped_image_dl_elem_t
// 
class the_warped_image_dl_elem_t : public the_dl_elem_t
{
public:
  the_warped_image_dl_elem_t(boost::shared_ptr<image_tile_dl_elem_t> & image,
			     const the_grid_transform_t & transform);
  
  the_warped_image_dl_elem_t(const image_tile_generator_t & image,
			     const the_grid_transform_t & transform,
			     GLenum min_filter,
			     GLenum mag_filter
			     #ifdef USE_CG
			     ,const CGprogram * fragment_program
			     #endif
			     );
  
  ~the_warped_image_dl_elem_t();
  
  // calculate the warped image quad strips and texture coordinates
  // of each quad relative to each image tile:
  void update(const the_grid_transform_t & transform);
  
  // virtual:
  void draw() const;
  
  void draw(const the_color_t & color, const bool & use_textures) const;
  
  static void draw_tile(const void * data, const unsigned int & tile_index);
  
  // virtual:
  void update_bbox(the_bbox_t & bbox) const;
  
#ifdef USE_CG
  // helper:
  inline bool has_fragment_program() const
  { return image_->has_fragment_program(); }
#endif
  
  //----------------------------------------------------------------
  // quad_strip_edge_t
  // 
  class quad_strip_edge_t
  {
  public:
    // texture coordinates
    float u_;
    float v0_;
    float v1_;
    
    float x0_;
    float y0_;
    
    float x1_;
    float y1_;
  };
  
  // the unwarped image:
  boost::shared_ptr<image_tile_dl_elem_t> image_;
  
  // cached quad strip data:
  fast_array_t<fast_array_t<fast_array_t<quad_strip_edge_t> > >
  image_tile_quad_strips_;
  
  // image bounding box:
  the_bbox_t bbox_;
  
private:
  the_warped_image_dl_elem_t();

  mutable the_color_t color_;
  mutable bool use_textures_;
};


#endif // THE_WARPED_IMAGE_DL_ELEM_HXX_
