// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : IRImageLoader.hxx
// Author       : Joel Spaltenstein
// Created      : 2008/02/20
// Copyright    : (C) 2008 University of Utah
// License      : GPLv2
// Description  :

#ifndef __IR_IMAGE_LOADER_HXX__
#define __IR_IMAGE_LOADER_HXX__

#include <vector>
#include <map>

#include <itk/common.hxx>
#include <itkTransformBase.h>

#include <itk/itkImageMosaicVarianceMetric.h>
#include <utils/the_text.hxx>

class IRTransform;

class IRImageLoader
{
struct lt_the_text {
	bool operator()(const the_text_t s1, const the_text_t s2) const {return (s1 < s2);}
};

typedef std::map<the_text_t, IRTransform*, struct lt_the_text> transformationMap;
typedef std::map<the_text_t, image_t::Pointer, struct lt_the_text> imageMap;
typedef std::vector<IRTransform*> IRTransformVector;
typedef std::vector<the_text_t> TheTextVector;

public:
	static IRImageLoader* sharedImageLoader();

	virtual ~IRImageLoader();
	
	void setTransformList(const IRTransformVector& transforms, const TheTextVector& imageIDs);
	
	void setShrinkFactor(unsigned int shrinkFactor);
	unsigned int shrinkFactor();
	void setPixelSpacing(double pixelSpacing);
	double pixelSpacing();
	
	image_t::Pointer getImage(const the_text_t& imageID);
	image_t::Pointer getFullResImage(const the_text_t& imageID); // this does not cache the image
	vec2d_t getImageSize(const the_text_t& imageID, base_transform_t::Pointer transform);
	
private:
	IRImageLoader();
	
	void imageCheckLoad(const the_text_t& imageID); 
	
	unsigned int _shrinkFactor;
	double _pixelSpacing;
	imageMap _images;
	transformationMap _transformations;
};



#endif
