// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : IRRefineTranslateCanvas.hxx
// Author       : Joel Spaltenstein
// Created      : 2008/02/20
// Copyright    : (C) 2008 University of Utah
// License      : GPLv2
// Description  :

#ifndef __IR_REFINE_TRANSLATE_CANVAS_HXX__
#define __IR_REFINE_TRANSLATE_CANVAS_HXX__

// system includes:
#include <vector>
#include <list>

// ITK includes:
#include <itkTransformBase.h>

// the includes:
#include <utils/the_text.hxx>
#include <itk/common.hxx>

// forward declarations:
class IRTransform;
class IRConnection;

//----------------------------------------------------------------
// IRConnectionVector
// 
typedef std::vector<IRConnection*> IRConnectionVector;


//----------------------------------------------------------------
// IRRefineTranslateCanvas
// 
class IRRefineTranslateCanvas
{
	typedef std::vector<IRTransform*> IRTransformationVector;
	typedef std::vector<base_transform_t::Pointer> TrasformBasePointerVector;
	typedef std::vector<the_text_t> TheTextVector;
	typedef std::list<the_text_t> TheTextList;
	
public:
	
	IRRefineTranslateCanvas();
	virtual ~IRRefineTranslateCanvas();
	
	// takes ownership of the trans
	void setBaseTransforms(const TrasformBasePointerVector& transforms,
						   const TheTextVector& imageIDs);
	
	void fillTransformAndImageIDVectors(TrasformBasePointerVector& transforms,
										TheTextVector& imageIDs);
	
	// silly other version that uses a list because fo some reason
	// paul likes using lists for the list of image IDs........
	void fillTransformAndImageIDVectors(TrasformBasePointerVector& transforms,
										TheTextList& imageIDs);

	// based on the current positions of each of the transforms
	// (which, when this is called will be the positions in the
	// .mosaic file that was loaded) this routine creates IRConnections
	// for each overlap and puts them in _preProcessedConnectionVector,
	// the connections are not added to the individual transforms
	// until findIdealTransformationOffsets is called
	void buildConnections();

	// initializes the groupIDs so that all the connected transforms
	// have the same groupID
	void fillGroupIDs();

	// removes transforms that are in a group that make up less than
	// cutoffPercentage of the mosaic
	void pruneSmallGroups(float cutoffPercentage);
	
	void findIdealTransformationOffsets(unsigned int numThreads);
	
	void releaseTensionOnSystem();
	
	float systemEnergy();
	float maximumTension();
	float maximumPullOnTransformation();

	// what is the signature of a thread entry again? keep on trying t
	void findOffsetsThreadEntry();
	
private:
	void addProcessedConnection(IRConnection* connection);
	
	// returns NULL if there are no more entries to process
	IRConnection* getConectionToProcess();
	
	void setTransformAndNeighborsToGroupID(IRTransform* transform,
										   long groupID);
	
	void removeTransformsWithGroupID(long groupID);
	
	// will be populated by buildConnections()
	IRConnectionVector _preProcessedConnectionVector;

	IRTransformationVector _transformationVector;
	IRConnectionVector _connectionVector;
};


#endif // __IR_REFINE_TRANSLATE_CANVAS_HXX__
