// -*- Mode: c++; tab-width: 4; c-basic-offset: 4; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : IRTransform.hxx
// Author       : Joel Spaltenstein
// Created      : 2008/02/19
// Copyright    : (C) 2008 University of Utah
// License      : GPLv2
// Description  : 

#ifndef __IR_TRANSFORMATION_HXX__
#define __IR_TRANSFORMATION_HXX__

// system includes:
#include <vector>

// the includes:
#include <itk/common.hxx>

// ITK includes:
#include <itkTransformBase.h>

// forward declarations:
class IRConnection;


//----------------------------------------------------------------
// IRTransform
// 
class IRTransform
{
	typedef std::vector<IRConnection*> IRConnectionVector;

public:
	IRTransform(base_transform_t::Pointer transformBase,
				const the_text_t& imageID);
	virtual ~IRTransform();
	
	const pnt2d_t & position() const;
	const vec2d_t & size() const;
	
	long groupID() const;
	void setGroupID(long newID);
	
	const IRConnectionVector & connections() const;
	
	bool overlapsTransform(const IRTransform* otherTransform) const;
	
	// sets start and size to the area that is covered by otherTransfrom,
	// the value returned by this function are with respect the the corner of
	// the transformation.
	// if they don't overlap at all this function returns an error (-1)
	int trasformOverlapArea(const IRTransform* otherTransform,
							pnt2d_t *start,
							vec2d_t *size) const;
	
	vec2d_t totalTension();
	void releaseTension();
	
	
	void addConnection(IRConnection* connection);
	void removeConnection(IRConnection* connection);
	base_transform_t::Pointer transformBase();
	const the_text_t & imageID() const;

	// DEBUG stuff
	void printImageID();

private:
	IRConnectionVector _connections;
	base_transform_t::Pointer _transformBase;
	the_text_t _imageID;
	
	pnt2d_t _position;
	vec2d_t _size;

	// all tiles that are connected share the same groupID once this
	long _groupID;
};


#endif // __IR_TRANSFORMATION_HXX__
