// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : visualize.hxx
// Author       : Pavel A. Koshevoy
// Created      : 2006/04/05 11:09
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Helper functions for visualizing unmatched
//                and matched SIFT keys and descriptors.

#ifndef VISUALIZE_HXX_
#define VISUALIZE_HXX_

// local includes:
#include "extrema.hxx"
#include "pyramid.hxx"
#include "match.hxx"

// the includes:
#include <itk/common.hxx>
#include <itk/draw.hxx>

// ITK includes:
#include <itkVector.h>

// system includes:
#include <list>
#include <vector>


//----------------------------------------------------------------
// draw_feature_key
// 
extern void
draw_feature_key(native_image_t::Pointer * image,
		 const double & x0,
		 const double & y0,
		 const double & direction,
		 const double & radius,
		 const xyz_t & color);

//----------------------------------------------------------------
// draw_feature_vector
// 
// x0, y0 are expressed in physical coordinates:
// 
extern void
draw_feature_vector(native_image_t::Pointer * image,
		    double x0,
		    double y0,
		    const double & direction,
		    const xyz_t & color,
		    const std::vector<double> & feature,
		    double scale = 1.0);

//----------------------------------------------------------------
// draw_keys
// 
extern void
draw_keys(native_image_t::Pointer * image,
	  const std::list<descriptor_t> & keys,
	  const xyz_t & rgb,
	  const double & r0,
	  const double & r1,
	  const unsigned int octave = 0);

//----------------------------------------------------------------
// visualize_matches
// 
// visualize the matching keys:
// 
extern void
visualize_matches(const pyramid_t & a,
		  const pyramid_t & b,
		  const unsigned int & io, // octave index
		  const unsigned int & is, // scale index
		  const std::list<match_t> & ab,
		  const the_text_t & fn_prefix,
		  unsigned int num_keys = ~0);

//----------------------------------------------------------------
// visualize_best_fit
// 
extern void
visualize_best_fit(const the_text_t & fn_prefix,
		   const image_t * a_img,
		   const image_t * b_img,
		   const base_transform_t * t_ab,
		   const std::vector<const match_t *> & ab,
		   const std::list<unsigned int> & inliers,
		   const mask_t * a_mask = NULL,
		   const mask_t * b_mask = NULL);

//----------------------------------------------------------------
// visualize_matches_v2
// 
// visualize the matching keys:
// 
extern void
visualize_matches_v2(const pyramid_t & a,
		     const pyramid_t & b,
		     const std::list<const match_t *> & ab,
		     const the_text_t & fn_prefix,
		     unsigned int num_keys = ~0);

//----------------------------------------------------------------
// visualize_nn
// 
extern void
visualize_nn(const the_text_t & fn_prefix,
	     const pyramid_t & a,
	     const pyramid_t & b,
	     const ext_wrapper_t & b_key_wrapper,
	     const std::vector<ext_wrapper_t> & a_key_wrappers,
	     const double & window_radius);


//----------------------------------------------------------------
// dump
// 
extern void
dump(std::ostream & so,
     const double * data,
     const unsigned int & size,
     const unsigned int & height = 10,
     const bool & normalize = false);


//----------------------------------------------------------------
// dump
// 
extern void
dump(std::ostream & so,
     const double * data_a,
     const double * data_b,
     const unsigned int & size,
     const unsigned int & height = 10,
     const bool & normalize = false);


#endif // VISUALIZE_HXX_
