// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : ImageProcessingDialog.cpp
// Author       : Pavel A. Koshevoy
// Created      : Tue Jul 25 16:30:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : A modal dialog for the image processing thread.

// local includes:
#include "ImageProcessingDialog.h"

// thelib includes:
#include <ui/the_trail.hxx>
#include <thread/the_transaction.hxx>
#include <thread/the_mutex_interface.hxx>

// Qt includes:
#include <QPushButton>
#include <QTextEdit>
#include <QCloseEvent>
#include <QTimer>
#include <QTime>


//----------------------------------------------------------------
// ImageProcessingDialog::ImageProcessingDialog
// 
ImageProcessingDialog::ImageProcessingDialog(QWidget * parent, Qt::WFlags f):
  QDialog(parent, f),
  Ui::ImageProcessingDialog()
{
  Ui::ImageProcessingDialog::setupUi(this);
  
  connect(cancelButton, SIGNAL(clicked()),
	  this, SLOT(cancel()));
  
  connect(&thread_, SIGNAL(status_update(const QString &)),
	  this, SLOT(updateStatus(const QString &)));
  
  connect(&thread_, SIGNAL(transaction_finished(the_transaction_t *)),
	  this, SLOT(transactionFinished(the_transaction_t *)));
  /*
  connect(&thread_, SIGNAL(thread_stopped(the_thread_interface_t *, bool)),
	  this, SLOT(threadStopped(the_thread_interface_t *, bool)));
  */
}

//----------------------------------------------------------------
// ImageProcessingDialog::~ImageProcessingDialog
// 
ImageProcessingDialog::~ImageProcessingDialog()
{
  if (thread_.isRunning())
  {
    thread_.wait();
    assert(0);
  }
}

//----------------------------------------------------------------
// ImageProcessingDialog::exec
// 
int
ImageProcessingDialog::exec()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  QTimer::singleShot(0, this, SLOT(startThread()));
  return QDialog::exec();
}

//----------------------------------------------------------------
// ImageProcessingDialog::next
// 
void
ImageProcessingDialog::startThread()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (!thread_.isRunning())
  {
    thread_.start();
  }
}

//----------------------------------------------------------------
// ImageProcessingDialog::cancel
// 
void
ImageProcessingDialog::cancel()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (thread_.isRunning())
  {
    /*
    cerr << "\n\nTRANSACTION ABORT REQUESTED AT "
	 << qPrintable(QTime::currentTime().toString()) << endl << endl;
    */
    
    // NOTE: this will block the UI:
    thread_.stop();
    thread_.wait();
  }
  
  done(QDialog::Rejected);
}

//----------------------------------------------------------------
// ImageProcessingDialog::transactionFinished
// 
void
ImageProcessingDialog::transactionFinished(the_transaction_t * t)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  /*
  cerr << "\n\nTRANSACTION STOPPED AT "
       << qPrintable(QTime::currentTime().toString()) << endl << endl;
  */
  if (t->state() == the_transaction_t::DONE_E)
  {
    if (!thread_.has_work())
    {
      thread_.wait();
      done(QDialog::Accepted);
    }
  }
}

//----------------------------------------------------------------
// ImageProcessingDialog::updateStatus
// 
void
ImageProcessingDialog::updateStatus(const QString & status)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  statusTextView->append(status);
}

//----------------------------------------------------------------
// ImageProcessingDialog::threadStopped
// 
void
ImageProcessingDialog::threadStopped(the_thread_interface_t * /* thread */,
				     bool all_transactions_completed)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (all_transactions_completed)
  {
    thread_.wait();
    done(QDialog::Accepted);
  }
}

//----------------------------------------------------------------
// ImageProcessingDialog::closeEvent
// 
void
ImageProcessingDialog::closeEvent(QCloseEvent * e)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  e->ignore();
  ImageProcessingDialog::cancel();
}
