// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : PreferencesDialog.h
// Author       : Pavel A. Koshevoy
// Created      : Sat Sep 09 20:50:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : UI for the user preferences.

#ifndef PREFERENCES_DIALOG_H_
#define PREFERENCES_DIALOG_H_

// local includes:
#include "ui_PreferencesDialog.h"

// Qt includes:
#include <QDialog>

// forward declarations:
extern bool prefsAllowLinearFiltering();
extern bool prefsUseLuminanceTextures();
extern bool prefsUseFragmentShader();
extern void prefsSetFragmentShaderAvailable(bool has_shader);
extern void prefsSetUseFragmentShader(bool use_shader);


//----------------------------------------------------------------
// PreferencesDialog
// 
class PreferencesDialog : public QDialog,
			  public Ui::PreferencesDialog
{
  Q_OBJECT
  
public:
  PreferencesDialog(QWidget * parent = 0, Qt::WFlags f = 0);
  
  // helper:
  void loadSettings();
  
  // virtual:
  int exec();
  
  // accessors:
  inline const bool & allowLinearFiltering() const
  { return allow_linear_filtering_; }
  
  inline const bool & useLuminanceTextures() const
  { return use_luminance_textures_; }
  
  bool useFragmentShader() const;
  void setFragmentShaderAvailable(bool has_shader);
  void setUseFragmentShader(bool use_shader);
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  
  bool allow_linear_filtering_;
  bool use_luminance_textures_;
  bool use_fragment_shader_;
};


#endif // PREFERENCES_DIALOG_H_
