// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : StdTileTransaction.h
// Author       : Pavel A. Koshevoy
// Created      : Tue Sep 19 16:15:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : transaction for loading a image tile for a mosaic
//		  (masks out a defect at the lower left corner)

#ifndef STD_TILE_TRANSACTION_H_
#define STD_TILE_TRANSACTION_H_

// the includes:
#include <thread/the_transaction.hxx>
#include <thread/the_thread_interface.hxx>
#include <image/image_tile_generator.hxx>
#include <image/la_pixel_converter.hxx>
#include <itk/common.hxx>

// Cg includes:
#include <Cg/cgGL.h>

// forward declarations:
extern CGprogram * CG_LA_FRAGMENT_PROGRAM_PTR;


//----------------------------------------------------------------
// StdTileTransaction
// 
template <typename TImage>
class StdTileTransaction : public the_transaction_t
{
public:
  StdTileTransaction(const the_text_t & filename,
		     const bool & use_std_mask,
		     const unsigned int & shrink_factor,
		     const double & pixel_spacing,
		     const bool blab = true):
    filename_(filename),
    use_std_mask_(use_std_mask),
    shrink_factor_(shrink_factor),
    pixel_spacing_(pixel_spacing),
    blab_(blab)
  {}
  
  // virtual:
  void execute(the_thread_interface_t * thread)
  {
    if (blab_)
    {
      std::ostringstream os;
      os << "loading " << filename_;
      blab(thread, tr(os.str().c_str()));
    }
    
    typename TImage::Pointer tile = ::std_tile<TImage>(filename_,
						       shrink_factor_,
						       pixel_spacing_,
						       false);
    mask_t::Pointer tile_mask = ::std_mask<TImage>(tile, use_std_mask_);
    
    if (blab_)
    {
      blab(thread, tr("converting to OpenGL texture"));
    }

    convert_tile<TImage, mask_t>(tile_,
				 tile,
				 tile_mask,
				 CG_LA_FRAGMENT_PROGRAM_PTR != NULL);
  }
  
  // trasaction parameters:
  the_text_t filename_;
  bool use_std_mask_;
  unsigned int shrink_factor_;
  double pixel_spacing_;
  bool blab_;
  
  // transaction output:
  image_tile_generator_t tile_;
};


#endif // STD_TILE_TRANSACTION_H_
