// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_main_window.hxx
// Author       : Pavel A. Koshevoy
// Created      : 2007/01/03 17:20
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : main application window widget

#ifndef THE_MAIN_WINDOW_HXX_
#define THE_MAIN_WINDOW_HXX_

// system includes:
#include <list>

// Qt includes:
#include <QtGui>
#include <QGridLayout>
#include <QMainWindow>
#include <QTimer>
#include <QStringList>

// local includes:
#include "ui_MainWindow.h"
#include "ui_AboutDialog.h"
#include "PreferencesDialog.h"

// thelib includes:
#include <ui/the_document_ui.hxx>
#include <doc/the_document.hxx>

// forward declarations:
class the_track_proc_ui_t;


//----------------------------------------------------------------
// the_main_window_t
// 
class the_main_window_t : public QMainWindow,
			  public Ui::MainWindow,
			  public the_document_ui_t
{
  Q_OBJECT
  
public:
  the_main_window_t();
  ~the_main_window_t();
  
public slots:
  // the file menu:
  void file_new();
  void file_exit();
  
  // the edit menu:
  void edit_undo();
  void edit_redo();
  void edit_preferences();
  
  // the view menu:
  void toggle_track_ui(bool on);
  void orientation_restore();
  
  // the help menu:
  void helpAbout();
  
  // slots:
  void tool_track();
  
public:
  // virtual: ui update mechanism:
  void sync_ui();
  
  // virtual:
  void kill_ui();
  
  // shortcut:
  inline the_document_t * document() const
  { return doc_so().document(); }
  
  // collect the visible views:
  void get_visible_views(std::vector<the_view_t *> & views) const;
  
  // if in single view mode, return the view:
  the_view_t * single_view() const;
  
  // accessor to the track procedure ui:
  inline the_track_proc_ui_t * track_proc_ui() const
  { return track_proc_ui_; }
  
  // preference accessor:
  inline PreferencesDialog & preferences()
  { return preferences_ui_; }
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  
  // the procedure ui:
  the_track_proc_ui_t * track_proc_ui_;
  
  // the preferences ui:
  PreferencesDialog preferences_ui_;
};


//----------------------------------------------------------------
// AboutDialog
// 
class AboutDialog : public QDialog,
		    public Ui::AboutDialog
{
  Q_OBJECT
  
public:
  AboutDialog(QWidget * parent = 0, Qt::WFlags f = 0):
    QDialog(parent, f),
    Ui::AboutDialog()
  {
    Ui::AboutDialog::setupUi(this);
  }
};


#endif // THE_MAIN_WINDOW_HXX_
