// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_track_proc_ui.cxx
// Author       : Pavel A. Koshevoy
// Created      : Tue Jan 2 16:06:00 MDT 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : UI for the feature tracking procedure.

// local includes:
#include "the_track_proc_ui.hxx"
#include "the_track_proc.hxx"
#include "the_main_window.hxx"
#include "ImageProcessingDialog.h"
#include "ImageDataTransaction.h"

// thelib includes:
#include <geom/the_point.hxx>
#include <doc/the_graph.hxx>
#include <sel/the_pick_list.hxx>
#include <math/the_bbox.hxx>
#include <math/the_aa_bbox.hxx>
#include <image/image_tile_generator.hxx>
#include <itk/common.hxx>
#include <itk/stos.hxx>
#include <itk/stos_common.hxx>
#include <itk/grid_common.hxx>
#include <itk/itkCascadedTransform.h>
#include <Qt/the_qt_trail.hxx>

// system includes:
#include <fstream>
#include <memory>

// Qt includes:
#include <QAction>
#include <QLineEdit>
#include <QButtonGroup>
#include <QRadioButton>
#include <QPushButton>
#include <QStackedWidget>
#include <QComboBox>
#include <QCheckBox>


//----------------------------------------------------------------
// the_track_proc_eh_t::the_track_proc_eh_t
// 
the_track_proc_eh_t::the_track_proc_eh_t(the_document_ui_t * doc_ui):
  the_document_eh_t<the_document_t>(doc_ui)
{}

//----------------------------------------------------------------
// the_track_proc_eh_t::is_modifier_active_locally
// 
bool
the_track_proc_eh_t::is_modifier_active_locally(the_eh_modifier_t mod) const
{
#if 0 
  cout << "--------------------------------------------------------" << endl;
  ::dump(THE_MOUSE);
  ::dump(THE_KEYBD);
#endif
  
  switch (mod)
  {
    case THE_EH_MOD_VIEW_ZOOM_E:
      return ((state() != THE_EH_NOT_READY_E) &&
	      THE_MOUSE.verify_pressed_btns(THE_MOUSE.m_btn()));
      
    case THE_EH_MOD_VIEW_PAN_E:
      return ((state() != THE_EH_NOT_READY_E) &&
	      THE_MOUSE.verify_pressed_btns(THE_MOUSE.l_btn()));
      
    default:
      break;
  }
  
  return the_input_device_eh_t::is_modifier_active_locally(mod);
}

//----------------------------------------------------------------
// the_track_proc_eh_t::delete_something
// 
void
the_track_proc_eh_t::delete_something()
{
  cerr << "delete_something" << endl;
  
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  // regenerate and update the UI:
  doc_ui().sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_eh_t::clear_all
// 
void
the_track_proc_eh_t::clear_all()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  // FIXME: write me:
  return;
}

//----------------------------------------------------------------
// the_track_proc_eh_t::processor
// 
bool
the_track_proc_eh_t::
processor(the_input_device_eh_t::the_event_type_t event_type)
{
  bool processed = false;
  
  if (event_type == THE_MOUSE_EVENT_E)
  {
    processed = process_mouse();
  }
  else if (event_type == THE_KEYBD_EVENT_E)
  {
    const the_keybd_key_t & key_up = THE_KEYBD.key(Qt::Key_Up);
    const the_keybd_key_t & key_dn = THE_KEYBD.key(Qt::Key_Down);
    
    if (key_up.down() || key_dn.down())
    {
      the_track_proc_t * p = proc();
      const int num_slices = p->exact_local_.size();
      
      the_track_proc_ui_t * proc_ui =
	doc_ui().active_proc_ui<the_track_proc_ui_t>();
      
      QItemSelectionModel * sel_model =
	proc_ui->imageListView->selectionModel();
      
      QModelIndex index = sel_model->currentIndex();
      
      if (key_dn.down())
      {
	index = proc_ui->model_->index(std::min(index.row() + 1,
						num_slices - 1),
				       index.column());
      }
      else
      {
	index = proc_ui->model_->index(std::max(index.row() - 1, 0),
				       index.column());
      }
      
      sel_model->setCurrentIndex(index, QItemSelectionModel::SelectCurrent);
      processed = true;
    }
    else
    {
      processed = false;
    }
  }
  
  if (processed)
  {
    // call document sync_ui, which will in turn call proc()->regenerate():
    doc_ui().sync_ui();
  }
  
  return processed;
}

//----------------------------------------------------------------
// the_track_proc_eh_t::registry
// 
the_registry_t *
the_track_proc_eh_t::registry() const
{
  return proc()->registry();
}

//----------------------------------------------------------------
// the_track_proc_eh_t::proc
// 
the_track_proc_t *
the_track_proc_eh_t::proc() const
{
  the_document_t * d = document();
  if (d == NULL) return NULL;
  return d->active_procedure<the_track_proc_t>();
}

//----------------------------------------------------------------
// the_track_proc_eh_t::process_mouse
// 
bool
the_track_proc_eh_t::process_mouse()
{
  if (THE_KEYBD.verify_pressed_keys(THE_KEYBD.ctl()) &&
      THE_MOUSE.verify_pressed_btns(THE_MOUSE.l_btn()))
  {
    // create a new mask point:
#if 0
    p3x1_t wcs_a = a_wcs();
#else
    p3x1_t wcs_a = b_wcs();
#endif
    
    p3x1_t wcs_b = b_wcs();
    pnt2d_t mosaic_a = pnt2d(wcs_a[0], wcs_a[1]);
    pnt2d_t mosaic_b = pnt2d(wcs_b[0], wcs_b[1]);
    
    // shortcuts:
    the_track_proc_t * p = proc();
    const image_data_t & image = p->image();
    
    const base_transform_t * mosaic_to_tile =
      p->approx_global_[p->active_slice_];
    
    base_transform_t::Pointer tile_to_mosaic = mosaic_to_tile->GetInverse();
    
    the::unique_list<mask_pt_t> & mask =
      p->mask_[p->active_label_][p->active_slice_];
    
    // transform the mosaic point into the tile space:
    pnt2d_t tile_a = mosaic_to_tile->TransformPoint(mosaic_a);
    pnt2d_t tile_b = mosaic_to_tile->TransformPoint(mosaic_b);
    
    vec2d_t vec_ab = tile_b - tile_a;
    double dist_ab = vec_ab.GetNorm();
    
    unsigned int num_samples =
      1 + int(1.5 * dist_ab / std::min(image.tile_generator_.spacing_x_,
				       image.tile_generator_.spacing_y_));

    double ox = image.tile_generator_.origin_x_;
    double oy = image.tile_generator_.origin_y_;
    double sx = image.tile_generator_.spacing_x_;
    double sy = image.tile_generator_.spacing_y_;
    
    for (unsigned int i = 0; i < num_samples; i++)
    {
      double t = num_samples == 1 ? 0 : double(i) / double(num_samples - 1);
      pnt2d_t tile_pt = tile_a + vec_ab * t;
      
      // confine the point to be as accurate as the granularity of the
      // image data:
      pnt2d_t index;
      index[0] = floor((tile_pt[0] - ox) / sx);
      index[1] = floor((tile_pt[1] - oy) / sy);
      
      mask_pt_t mask_pt;
      mask_pt.tile_pt_[0] = (ox + sx * index[0]);
      mask_pt.tile_pt_[1] = (oy + sy * index[1]);
      
      mask_pt.mosaic_quad_[0] =
	tile_to_mosaic->TransformPoint(mask_pt.tile_pt_);
      
      mask_pt.mosaic_quad_[1] =
	tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ + vec2d(sx, 0));
      
      mask_pt.mosaic_quad_[2] =
	tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ + vec2d(sx, sy));
      
      mask_pt.mosaic_quad_[3] =
	tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ + vec2d(0, sy));
      
      mask.push_back(mask_pt);
    }
    
    reset_anchor();
    return true;
  }
  
  // reject all other events:
  return false;
}


//----------------------------------------------------------------
// the_track_proc_ui_t::the_track_proc_ui_t
// 
the_track_proc_ui_t::the_track_proc_ui_t(the_main_window_t * doc_ui):
  QWidget(doc_ui,
	  Qt::Window |
	  Qt::WindowTitleHint |
	  Qt::WindowSystemMenuHint |
	  Qt::WindowMinMaxButtonsHint ),
  the_procedure_ui_t(*doc_ui),
  eh_(doc_ui),
  model_(NULL)
{
  setupUi(this);
  
  model_ = new ReadOnlyStringListModel(this);
  imageListView->setModel(model_);
  
  // attach the event handler to the document views:
  std::vector<the_view_t *> & views = doc_ui_.views();
  unsigned int num_views = views.size();
  for (unsigned int i = 0; i < num_views; i++)
  {
    eh_.attach_view(views[i]);
  }
}

//----------------------------------------------------------------
// the_track_proc_ui_t::~the_track_proc_ui_t
// 
the_track_proc_ui_t::~the_track_proc_ui_t()
{
  eh_.uninstall();
  
  std::vector<the_view_t *> & views = doc_ui_.views();
  unsigned int num_views = views.size();
  for (unsigned int i = 0; i < num_views; i++)
  {
    eh_.detach_view(views[i]);
  }
  
  // remove the data model:
  delete model_;
  model_ = NULL;
}

//----------------------------------------------------------------
// the_track_proc_ui_t::install
// 
void
the_track_proc_ui_t::install()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (installed()) return;
  
  show();
  
  // make a new procedure:
  document()->add_proc(new the_track_proc_t());
  the_procedure_ui_t::install();
  eh_.install();
  
  // hookup the slots:
  the_main_window_t * main_window =
    dynamic_cast<the_main_window_t *>(&doc_ui_);
  
  connect(main_window->actionLoad, SIGNAL(triggered()),
	  this, SLOT(file_load()));
  connect(main_window->actionDelete, SIGNAL(triggered()),
	  this, SLOT(edit_delete()));
  connect(main_window->actionClearAll, SIGNAL(triggered()),
	  this, SLOT(clear_all()));
  
  QItemSelectionModel * sel_model = imageListView->selectionModel();
  connect(sel_model, SIGNAL(currentChanged(const QModelIndex &,
					   const QModelIndex &)),
	  this, SLOT(selection_changed()));
  
  connect(labelComboBox, SIGNAL(currentIndexChanged(int)),
	  this, SLOT(active_label_changed()));
  
  connect(applyMaskButton, SIGNAL(clicked(bool)),
	  this, SLOT(propagate_active_label()));
  
  connect(clearMaskButton, SIGNAL(clicked(bool)),
	  this, SLOT(clear_active_label()));
  
  doc_ui().sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::uninstall
// 
void
the_track_proc_ui_t::uninstall()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (!installed()) return;
  
  sync_ui();
  hide();
  
  // unhook the slots:
  the_main_window_t * main_window =
    dynamic_cast<the_main_window_t *>(&doc_ui_);
  
  disconnect(main_window->actionLoad, SIGNAL(triggered()),
	     this, SLOT(file_load()));
  disconnect(main_window->actionDelete, SIGNAL(triggered()),
	     this, SLOT(edit_delete()));
  disconnect(main_window->actionClearAll, SIGNAL(triggered()),
	     this, SLOT(clear_all()));
  
  QItemSelectionModel * sel_model = imageListView->selectionModel();
  disconnect(sel_model, SIGNAL(currentChanged(const QModelIndex &,
					      const QModelIndex &)),
	     this, SLOT(selection_changed()));
  
  disconnect(labelComboBox, SIGNAL(currentIndexChanged(int)),
	     this, SLOT(active_label_changed()));
  
  disconnect(applyMaskButton, SIGNAL(clicked(bool)),
	     this, SLOT(propagate_active_label()));
  
  disconnect(clearMaskButton, SIGNAL(clicked(bool)),
	     this, SLOT(clear_active_label()));
  
  the_procedure_ui_t::uninstall();
  eh_.uninstall();
  
  doc_ui().sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::sync_ui
// 
void
the_track_proc_ui_t::sync_ui()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  the_track_proc_t * p = proc();
  if (p == NULL) return;
  p->regenerate();
  
  // FIXME: write me: enable/disable context sensitive actions here:
  // imageListView->update();
  const the_dynamic_array_t<the_color_t> & labels = p->color_;
  if ((unsigned int)(labelComboBox->count()) != labels.size())
  {
    labelComboBox->clear();
    for (unsigned int i = 0; i < labels.size(); i++)
    {
      const the_color_t & c = labels[i];
      QPixmap pixmap(14, 14);
      pixmap.fill(QColor::fromRgbF(c[0], c[1], c[2]));
      QIcon icon(pixmap);
      QString text = tr("label ") + QString::number(i);
      labelComboBox->addItem(icon, text);
    }
  }
  
  labelComboBox->setCurrentIndex(p->active_label_);
}

//----------------------------------------------------------------
// the_track_proc_ui_t::file_load
// 
void
the_track_proc_ui_t::file_load(const QStringList & filenames)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (filenames.empty()) return;
  
  // ask for the shrink factor:
  shrink_factor_dialog_.exec();
  file_load(filenames, shrink_factor_dialog_.shrink_factor());
}

//----------------------------------------------------------------
// the_track_proc_ui_t::file_load
// 
void
the_track_proc_ui_t::file_load(const QStringList & filenames,
			       unsigned int shrink_factor)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (filenames.empty()) return;
  
  // load the pair transforms:
  std::list<stos_t<native_image_t> > stos_list;
  unsigned int num_stos = 0;
  
  for (QStringList::const_iterator iter = filenames.begin();
       iter != filenames.end(); ++iter)
  {
    const QString & fn = *iter;
    the_text_t fn_load(qPrintable(fn), fn.size());
    
    stos_t<native_image_t> tmp;
    if (!tmp.load(fn_load, false))
    {
      QMessageBox::critical(this,
			    QString("Error loading transform"),
			    QString("Could not load ") + *iter +
			    QString(", skipping..."),
			    QMessageBox::Ok,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      continue;
    }
    
    stos_list.push_back(tmp);
    num_stos++;
  }
  
  // sort the transforms:
  if (!sort<native_image_t>(stos_list))
  {
    QMessageBox::critical(this,
			  QString("Error sorting transforms"),
			  QString("Could not sort pair-wise transforms") +
			  QString(", aborting..."),
			  QMessageBox::Ok,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return;
  }
  
  // update the image list view model:
  the_dynamic_array_t<stos_t<native_image_t> > stos_arr;
  {
    unsigned int index = 0;
    for (std::list<stos_t<native_image_t> >::const_iterator
	   iter = stos_list.begin(); iter != stos_list.end(); ++iter, index++)
    {
      stos_arr[index] = *iter;
    }
  }
  
  QStringList image_list;
  if (num_stos > 0)
  {
    image_list.push_back(QString(stos_arr[0].fn_[0]));
  }
  
  for (unsigned int i = 0; i < num_stos; i++)
  {
    image_list.push_back(QString(stos_arr[i].fn_[1]));
  }
  
  // lookup the prefered internal format for the grayscale images:
  GLenum internal_format =
    prefsUseLuminanceTextures() ?
    GL_LUMINANCE6_ALPHA2 :
    GL_RGBA8;
  
  // setup rough slice alignment transforms:
  const unsigned int num_images = num_stos + 1;
  typedef itk::CascadedTransform<double, 2> cascaded_transform_t;
  typedef itk::LegendrePolynomialTransform<double, 1> approx_transform_t;
  the_dynamic_array_t<base_transform_t::ConstPointer>
    exact_local(num_images);
  the_dynamic_array_t<approx_transform_t::Pointer>
    approx_local(num_images);
  the_dynamic_array_t<base_transform_t::ConstPointer>
    approx_global(num_images);
  
  for (unsigned int index = 0; index < num_images; index++)
  {
    int i0 = index - 1;
    int i1 = (i0 == -1) ? 0 : 1;
    i0 = std::max(0, i0);
    
    const stos_t<native_image_t> & stos = stos_arr[i0];
    exact_local[index] =
      (i1 == 1) ?
      stos.t01_.GetPointer() :
      identity_transform_t::New().GetPointer();
    
    pnt2d_t tile_min = pnt2d(0, 0);
    pnt2d_t tile_max = pnt2d(stos.sp_[i1][0] * stos.sz_[i1][0],
			     stos.sp_[i1][1] * stos.sz_[i1][1]);
    
    approx_local[index] =
      approx_transform<approx_transform_t>(tile_min,
					   tile_max,
					   NULL,
					   exact_local[index],
					   16,
					   1,
					   true);
    
    cascaded_transform_t::Pointer cascade = cascaded_transform_t::New();
    for (unsigned int j = 0; j <= index; j++)
    {
      cascade->append(approx_local[j].GetPointer());
    }
    
    approx_global[index] =
      approx_transform<approx_transform_t>(tile_min,
					   tile_max,
					   NULL,
					   cascade,
					   16,
					   1,
					   true).GetPointer();
  }
  
  // start loading images:
  typedef ImageDataTransaction<native_image_t> IDT_t;
  ImageProcessingDialog img_proc_dialog(this);
  std::list<IDT_t> transactions;
  
  for (unsigned int index = 0; index < num_images; index++)
  {
    int i0 = index - 1;
    int i1 = (i0 == -1) ? 0 : 1;
    i0 = std::max(0, i0);
    
    const stos_t<native_image_t> & stos = stos_arr[i0];
    IDT_t t(stos.fn_[i1],
	    shrink_factor,
	    stos.flipped_[i1],
	    OpenGL(0).max_texture_,
	    internal_format);
    transactions.push_back(t);
    img_proc_dialog.thread_.push_back(&(*(transactions.rbegin())));
  }
  
  // execute the transactions:
  int r = img_proc_dialog.exec();
  if (r == QDialog::Accepted)
  {
    // extract the image data:
    the_dynamic_array_t<image_data_t> image_data(num_images);
    unsigned int index = 0;
    for (std::list<IDT_t>::const_iterator iter =
	   transactions.begin(); iter != transactions.end(); ++iter, index++)
    {
      image_data[index] = (*iter).data_;

      const base_transform_t * forward = approx_global[index].GetPointer();
      base_transform_t::Pointer inverse = forward->GetInverse();
      
      // rotate the tiles according to the approximate alignment transform:
      std::vector<image_tile_t> & tiles =
	image_data[index].tile_generator_.tiles_;
      
      unsigned int num_tiles = tiles.size();
      for (unsigned int i = 0; i < num_tiles; i++)
      {
	image_tile_t & tile = tiles[i];
	pnt2d_t tile_corner[] = {
	  pnt2d(tile.corner_[0][0], tile.corner_[0][1]),
	  pnt2d(tile.corner_[1][0], tile.corner_[1][1]),
	  pnt2d(tile.corner_[2][0], tile.corner_[2][1]),
	  pnt2d(tile.corner_[3][0], tile.corner_[3][1])
	};
	
	int i0 = index - 1;
	int i1 = (i0 == -1) ? 0 : 1;
	i0 = std::max(0, i0);
	
	const stos_t<native_image_t> & stos = stos_arr[i0];
	pnt2d_t tile_min = pnt2d(0, 0);
	pnt2d_t tile_max = pnt2d(stos.sp_[i1][0] * stos.sz_[i1][0],
				 stos.sp_[i1][1] * stos.sz_[i1][1]);
	
	for (unsigned int j = 0; j < 4; j++)
	{
	  pnt2d_t mosaic_corner;
	  find_inverse(tile_min,
		       tile_max,
		       forward,
		       inverse.GetPointer(),
		       tile_corner[j],
		       mosaic_corner);
	  
	  tile.corner_[j][0] = mosaic_corner[0];
	  tile.corner_[j][1] = mosaic_corner[1];
	  tile.corner_[j][2] = 0;
	}
      }
    }
    
    // update the procedure:
    proc()->reset(stos_arr, image_data, exact_local, approx_global);
    
    QItemSelectionModel * sel_model = imageListView->selectionModel();
    model_->setStringList(image_list);
    sel_model->clear();
    sel_model->setCurrentIndex(model_->index(0, 0, QModelIndex()),
			       QItemSelectionModel::SelectCurrent);
  }
  
  // update the UI:
  doc_ui_.view(0)->restore_orientation();
  doc_ui_.sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::file_load
// 
void
the_track_proc_ui_t::file_load()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  QStringList filenames
    (QFileDialog::getOpenFileNames(this,
				  "Load registration transforms",
				  QString(),
				  "slice-to-slice transforms (*.stos)"));
  file_load(filenames);
}

//----------------------------------------------------------------
// the_track_proc_ui_t::edit_delete
// 
void
the_track_proc_ui_t::edit_delete()
{
  eh_.delete_something();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::clear_all
// 
void
the_track_proc_ui_t::clear_all()
{
  eh_.clear_all();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::select_slice
// 
void
the_track_proc_ui_t::select_slice(QModelIndex index)
{
  if (!model_->hasIndex(index.row(), index.column()))
  {
    return;
  }
  
  QItemSelectionModel * sel_model = imageListView->selectionModel();
  sel_model->setCurrentIndex(model_->index(index.row(), index.column()),
			     QItemSelectionModel::SelectCurrent);
}

//----------------------------------------------------------------
// the_track_proc_ui_t::selection_changed
// 
void
the_track_proc_ui_t::selection_changed()
{
  QItemSelectionModel * sel_model = imageListView->selectionModel();
  QModelIndex index = sel_model->currentIndex();
  QString data = model_->data(index, Qt::DisplayRole).toString();
  /*
  cerr << "selection_changed(" << index.row() << ", " << index.column()
       << ") = " << qPrintable(data) << endl;
  */
  
  // shortcuts:
  the_main_window_t * main_window =
    dynamic_cast<the_main_window_t *>(the_document_ui_t::doc_ui());
  the_track_proc_t * p = proc();
  
  p->set_active_slice(index.row());
  main_window->sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::active_label_changed
// 
void
the_track_proc_ui_t::active_label_changed()
{
  the_track_proc_t * p = proc();
  p->set_active_label(labelComboBox->currentIndex());
}


//----------------------------------------------------------------
// LabelsTransaction
// 
class LabelsTransaction : public the_transaction_t
{
public:
  // constructor:
  LabelsTransaction(the_track_proc_t * proc,
		    bool refine):
    proc_(proc),
    refine_(refine)
  {}
  
  // virtual:
  void execute(the_thread_interface_t * thread)
  {
    // a handle object allowing the user to interrupt transaction execution:
    itk_terminator_t terminator("LabelsTransaction::execute");
    
    the_transaction_log_t trans_log(thread);
    
    // pre-allocate image/mask data:
    static const unsigned int neighborhood = 64;
    static const unsigned int neighborhood_half = neighborhood / 2;
    static const unsigned int neighborhood_double = neighborhood * 2;
    
    native_image_t::Pointer tile =
      make_image<native_image_t>(neighborhood_double,
				 neighborhood_double,
				 1.0,
				 0);
    mask_t::Pointer mask =
      make_image<mask_t>(neighborhood_double,
			 neighborhood_double,
			 1.0,
			 0);
    
    native_image_t::Pointer img[] = {
      make_image<native_image_t>(neighborhood, neighborhood, 1.0, 0),
      make_image<native_image_t>(neighborhood, neighborhood, 1.0, 0)
    };
    
    mask_t::Pointer msk[] = {
      make_image<mask_t>(neighborhood, neighborhood, 1.0, 0),
      make_image<mask_t>(neighborhood, neighborhood, 1.0, 0)
    };
    
    // temporary:
    native_image_t::IndexType itk_index;
    pnt2d_t itk_pt;
    const unsigned char * four_pixels[4];
    double four_weights[4];
    
    // shortcut:
    const int num_slices = proc_->exact_local_.size();
    
    // propagate up:
    for (int i = proc_->active_slice_; i > 0; i--)
    {
      blab(thread,
	   tr("propagating mask to level ") +
	   the_text_t::number(i - 1));
      
      const std::list<mask_pt_t> & mask_a =
	proc_->mask_[proc_->active_label_][i];
      
      std::list<mask_pt_t> & mask_b =
	proc_->mask_[proc_->active_label_][i - 1];
      mask_b.clear();
      
      const base_transform_t * exact_local = proc_->exact_local_[i];
      base_transform_t::Pointer exact_inverse = exact_local->GetInverse();
      const base_transform_t * approx_global = proc_->approx_global_[i - 1];
      base_transform_t::Pointer tile_to_mosaic = approx_global->GetInverse();
      
      const image_data_t & image = proc_->image_->elem(i);
      double ox = image.tile_generator_.origin_x_;
      double oy = image.tile_generator_.origin_y_;
      double sx = image.tile_generator_.spacing_x_;
      double sy = image.tile_generator_.spacing_y_;
      unsigned int nx = image.tile_generator_.w_;
      unsigned int ny = image.tile_generator_.h_;
      
      pnt2d_t tile_min = pnt2d(ox, oy);
      pnt2d_t tile_max = pnt2d(ox + sx * double(nx), oy + sy * double(ny));
      
      for (std::list<mask_pt_t>::const_iterator iter = mask_a.begin();
	   iter != mask_a.end(); ++iter)
      {
	// make sure there hasn't been an interrupt:
	terminator.terminate_on_request();
	
	const pnt2d_t & pt_a = (*iter).tile_pt_;
	
	mask_pt_t mask_pt;
	if (!find_inverse(tile_min,
			  tile_max,
			  exact_local,
			  exact_inverse.GetPointer(),
			  pt_a,
			  mask_pt.tile_pt_))
	{
	  continue;
	}
	
	// calculate the quad corner points in the mosaic space:
	mask_pt.mosaic_quad_[0] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_);
	
	mask_pt.mosaic_quad_[1] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ + vec2d(sx, 0));
	
	mask_pt.mosaic_quad_[2] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ + vec2d(sx, sy));
	
	mask_pt.mosaic_quad_[3] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ + vec2d(0, sy));
	
	mask_b.push_back(mask_pt);
      }
    }
    
    // propagate down:
    for (int i = proc_->active_slice_; i < num_slices - 1; i++)
    {
      blab(thread,
	   tr("propagating mask to level ") +
	   the_text_t::number(i + 1));
      
      const std::list<mask_pt_t> & mask_a =
	proc_->mask_[proc_->active_label_][i];
      
      std::list<mask_pt_t> & mask_b =
	proc_->mask_[proc_->active_label_][i + 1];
      mask_b.clear();
      
      const base_transform_t * exact_local = proc_->exact_local_[i + 1];
      base_transform_t::Pointer exact_inverse = exact_local->GetInverse();
      const base_transform_t * approx_global = proc_->approx_global_[i + 1];
      base_transform_t::Pointer tile_to_mosaic = approx_global->GetInverse();
      
      // shortcuts:
      const image_tile_generator_t & image_0 =
	proc_->image_->elem(i).tile_generator_;
      
      const image_tile_generator_t & image_1 =
	proc_->image_->elem(i + 1).tile_generator_;
      
      const unsigned int bpp[] = {
	image_0.bytes_per_pixel_,
	image_1.bytes_per_pixel_
      };
      
      // setup spacing:
      native_image_t::SpacingType sp[2];
      sp[0][0] = image_0.spacing_x_;
      sp[0][1] = image_0.spacing_y_;
      sp[1][0] = image_1.spacing_x_;
      sp[1][1] = image_1.spacing_y_;
      tile->SetSpacing(sp[0]);
      mask->SetSpacing(sp[0]);
      img[0]->SetSpacing(sp[0]);
      msk[0]->SetSpacing(sp[0]);
      img[1]->SetSpacing(sp[1]);
      msk[1]->SetSpacing(sp[1]);
      
      pnt2d_t tile1_min = pnt2d(image_1.origin_x_, image_1.origin_y_);
      pnt2d_t tile1_max = tile1_min + vec2d(sp[1][0] * double(image_1.w_),
					    sp[1][1] * double(image_1.h_));
      
      for (std::list<mask_pt_t>::const_iterator iter = mask_a.begin();
	   iter != mask_a.end(); ++iter)
      {
	// make sure there hasn't been an interrupt:
	terminator.terminate_on_request();
	
	// shortcut:
	const pnt2d_t & pt_a = (*iter).tile_pt_;
	
	mask_pt_t mask_pt;
	mask_pt.tile_pt_ = exact_local->TransformPoint(pt_a);
	if (mask_pt.tile_pt_[0] == std::numeric_limits<double>::max())
	{
	  // discontinuous transforms will map to extreme coordinates
	  // when a mapping doesn't exist:
	  continue;
	}
	
	// refine the point mapping via local feature tracking:
	if (refine_)
	{
	  // extract 3 regions of image data and mask data from
	  // the slices -- 2 small neighborhood regions from
	  // the fixed and moving image, and 1 larger image/mask
	  // from the fixed image. The larger image region is
	  // optional (and the small region may be subsituted instead),
	  // but it can be used to improve the robustness of the refinement:
	  
	  // find the pixel coordinates of the tile point in the fixed image:
	  int ix;
	  int iy;
	  if (!image_0.get_pixel_coords(pt_a[0], pt_a[1], ix, iy))
	  {
	    continue;
	  }
	  
	  // extract the large neighborhood centered at the tile point:
	  for (unsigned int y = 0; y < neighborhood_double; y++)
	  {
	    itk_index[1] = y;
	    
	    int iy1 = iy + y - neighborhood;
	    const unsigned char * scanline = image_0.scanline(iy1);
	    
	    for (unsigned int x = 0; x < neighborhood_double; x++)
	    {
	      itk_index[0] = x;
	      
	      unsigned char l = 0;
	      unsigned char a = 0;
	      
	      int ix1 = ix + x - neighborhood;
	      if (image_0.valid_pixel_coords(ix1, iy1))
	      {
		const unsigned char * pixel = scanline + ix1 * bpp[0];
		if (bpp[0] == 1)
		{
		  decode_la(pixel[0], l, a);
		}
		else
		{
		  l = pixel[0];
		  a = pixel[1];
		}
	      }
	      
	      tile->SetPixel(itk_index, l);
	      mask->SetPixel(itk_index, a);
	    }
	  }
	  
	  // copy a portion of the large neighborhood of the fixed image
	  // into the small neighborhood image:
	  {
	    native_image_t::SizeType rn_sz;
	    rn_sz[0] = neighborhood;
	    rn_sz[1] = neighborhood;
	    
	    native_image_t::IndexType rn_ix;
	    rn_ix[0] = neighborhood_half;
	    rn_ix[1] = neighborhood_half;
	    
	    native_image_t::RegionType rn;
	    rn.SetIndex(rn_ix);
	    rn.SetSize(rn_sz);
	    
	    typedef itk::ImageRegionConstIteratorWithIndex<native_image_t>
	      itex_t;
	    itex_t itex(tile, rn);
	    for (itex.GoToBegin(); !itex.IsAtEnd(); ++itex)
	    {
	      unsigned char l = itex.Get();
	      
	      itk_index = itex.GetIndex();
	      unsigned char a = mask->GetPixel(itk_index);
	      
	      itk_index[0] -= neighborhood_half;
	      itk_index[1] -= neighborhood_half;
	      img[0]->SetPixel(itk_index, l);
	      msk[0]->SetPixel(itk_index, a);
	    }
	  }
	  
	  tile = histogram_equalization<native_image_t>
	    (tile, 256, 0, 255, mask);
	  img[0] = histogram_equalization<native_image_t>
	    (img[0], 256, 0, 255, msk[0]);
	  
	bool ok = true;
	for (int j = 0; j < 10; j++)
	{
	  pnt2d_t fi_pt;
	  if (!find_inverse(tile1_min,
			    tile1_max,
			    exact_local,
			    exact_inverse.GetPointer(),
			    mask_pt.tile_pt_,
			    fi_pt))
	  {
	    ok = false;
	    break;
	  }
	  
	  // extract the small neighborhood of the moving image
	  // centered at the tile point:
	  for (unsigned int y = 0; y < neighborhood; y++)
	  {
	    itk_index[1] = y;
	    int iy1 = y - neighborhood_half;
	    
	    for (unsigned int x = 0; x < neighborhood; x++)
	    {
	      itk_index[0] = x;
	      int ix1 = x - neighborhood_half;
	      
	      unsigned char l = 0;
	      unsigned char a = 0;
	      
	      // get physical coordinates:
	      itk_pt[0] = fi_pt[0] + sp[0][0] * double(ix1);
	      itk_pt[1] = fi_pt[1] + sp[0][1] * double(iy1);
	      
	      // see where they map to:
	      itk_pt = exact_local->TransformPoint(itk_pt);
	      
	      if (image_1.evaluate(itk_pt[0],
				   itk_pt[1],
				   four_pixels,
				   four_weights))
	      {
		double luminance = 0;
		double alpha = 0;
		
		if (bpp[1] == 1)
		{
		  interpolate_compressed_luminance_alpha(luminance,
							 alpha,
							 four_pixels,
							 four_weights);
		}
		else
		{
		  interpolate_luminance_alpha(luminance,
					      alpha,
					      four_pixels,
					      four_weights);
		}
		
		l = (unsigned char)(luminance);
		a = (unsigned char)(alpha);
	      }
	      
	      img[1]->SetPixel(itk_index, l);
	      msk[1]->SetPixel(itk_index, a);
	    }
	  }
	  img[1] = histogram_equalization<native_image_t>
	    (img[1], 256, 0, 255, msk[1]);
	  
	  // FIXME:
	  /*
	  save<native_image_t>(tile,
			       the_text_t("/tmp/tile-") +
			       the_text_t::number(i, 2, '0') + ".png");
	  save<native_image_t>(img[0],
			       the_text_t("/tmp/img0-") +
			       the_text_t::number(i, 2, '0') + ".png");
	  save<native_image_t>(img[1],
			       the_text_t("/tmp/img1-") +
			       the_text_t::number(i, 2, '0') + ".png");
	  */
	  
	  pnt2d_t origin;
	  origin[0] = double(neighborhood_half) * sp[0][0];
	  origin[1] = double(neighborhood_half) * sp[0][1];
	  
	  vec2d_t shift;
	  bool refined =
#if 1
	    refine_one_point_fft<native_image_t>(trans_log,
						 shift,
						 origin,
						 tile.GetPointer(),
						 mask.GetPointer(),
						 img[0].GetPointer(),
						 msk[0].GetPointer(),
						 img[1].GetPointer(),
						 msk[1].GetPointer());
#else
	    refine_one_point_nofft<native_image_t>(trans_log,
						   shift,
						   origin,
						   tile.GetPointer(),
						   mask.GetPointer(),
						   img[1].GetPointer(),
						   msk[1].GetPointer());
#endif
	  
	  if (refined)
	  {
	    // correct the point coordinates:
	    mask_pt.tile_pt_ -= shift;
	    blab(thread,
		 qPrintable(QString("%1. adjusting by %2 %3")
			    .arg(j)
			    .arg(shift[0])
			    .arg(shift[1])));
	    
	    if (shift.GetSquaredNorm() < 1e-4) break;
	  }
	  else
	  {
	    break;
	  }
	}
	if (!ok) continue;
	}
	
	
	mask_pt.mosaic_quad_[0] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_);
	
	mask_pt.mosaic_quad_[1] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ +
					 vec2d(sp[1][0], 0));
	
	mask_pt.mosaic_quad_[2] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ +
					 vec2d(sp[1][0], sp[1][1]));
	
	mask_pt.mosaic_quad_[3] =
	  tile_to_mosaic->TransformPoint(mask_pt.tile_pt_ +
					 vec2d(0, sp[1][1]));
	
	mask_b.push_back(mask_pt);
      }
    }
  }
  
  // trasaction parameters:
  the_track_proc_t * proc_;
  bool refine_;
};

//----------------------------------------------------------------
// the_track_proc_ui_t::propagate_active_label
// 
void
the_track_proc_ui_t::propagate_active_label()
{
  the_track_proc_t * p = proc();
  
  LabelsTransaction trans(p, refineCheckBox->isChecked());
  ImageProcessingDialog img_proc_dialog(this);
  img_proc_dialog.thread_.push_back(&trans);
  img_proc_dialog.exec();
  
  p->request_regeneration();
  doc_ui_.sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::clear_active_label
// 
void
the_track_proc_ui_t::clear_active_label()
{
  the_track_proc_t * p = proc();
  const unsigned int num_slices = p->exact_local_.size();
  
  for (unsigned int i = 0; i < num_slices; i++)
  {
    std::list<mask_pt_t> & mask = p->mask_[p->active_label_][i];
    mask.clear();
  }
  
  p->request_regeneration();
  doc_ui_.sync_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::closeEvent
// 
void
the_track_proc_ui_t::closeEvent(QCloseEvent * e)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  e->ignore();
  the_main_window_t * main_window =
    dynamic_cast<the_main_window_t *>(&doc_ui());
  main_window->kill_ui();
}

//----------------------------------------------------------------
// the_track_proc_ui_t::proc
// 
the_track_proc_t *
the_track_proc_ui_t::proc()
{
  the_document_t * d = document();
  if (d == NULL) return NULL;
  return d->active_procedure<the_track_proc_t>();
}
