// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_track_proc_ui.hxx
// Author       : Pavel A. Koshevoy
// Created      : Tue Jan 2 16:07:00 MDT 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : UI for the feature tracking procedure.

#ifndef THE_TRACK_PROC_UI_HXX_
#define THE_TRACK_PROC_UI_HXX_

// system includes:
#include <list>

// local includes:
#include "ui_LabelDialog.h"
#include "ui_ShrinkFactorDialog.h"
#include "the_track_proc.hxx"

// the includes:
#include <ui/the_procedure_ui.hxx>
#include <eh/the_document_eh.hxx>
#include <doc/the_document.hxx>
#include <doc/the_registry.hxx>
#include <geom/the_point.hxx>
#include <thread/the_thread_interface.hxx>

// Qt includes:
#include <QObject>
#include <QWidget>
#include <QDialog>
#include <QSpinBox>
#include <QTimer>
#include <QStringListModel>

// forward declarations:
class the_pick_list_t;
class the_main_window_t;


//----------------------------------------------------------------
// the_track_proc_eh_t
// 
class the_track_proc_eh_t : public the_document_eh_t<the_document_t>
{
public:
  the_track_proc_eh_t(the_document_ui_t * doc_ui);
  
  // virtual:
  const char * name() const
  { return "the_track_proc_eh_t"; }
  
  // virtual: check if a given modifier is currently active:
  bool is_modifier_active_locally(the_eh_modifier_t mod) const;
  
  // the UI will call this:
  void delete_something();
  void clear_all();
  
  // helpers, shortcuts:
  the_registry_t * registry() const;
  
  // helper functions:
  the_track_proc_t * proc() const;
  
protected:
  // virtual: Override the processor function.
  bool processor(the_input_device_eh_t::the_event_type_t event_type);
  
  // processor helper functions:
  bool process_mouse();
};

//----------------------------------------------------------------
// ReadOnlyStringListModel
// 
class ReadOnlyStringListModel : public QStringListModel
{
public:
  ReadOnlyStringListModel(QObject * parent = 0):
    QStringListModel(parent)
  {}
  
  // virtual:
  Qt::ItemFlags flags(const QModelIndex & index) const
  {
    if (index.isValid())
    {
      return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
    }
    
    return QStringListModel::flags(index);
  }
};

//----------------------------------------------------------------
// ShrinkFactorDialog
// 
class ShrinkFactorDialog : public QDialog,
			   public Ui::ShrinkFactorDialog
{
  Q_OBJECT
  
public:
  ShrinkFactorDialog(QWidget * parent = 0, Qt::WFlags f = 0):
    QDialog(parent, f),
    Ui::ShrinkFactorDialog()
  {
    Ui::ShrinkFactorDialog::setupUi(this);
  }
  
  // get the shrink factor:
  inline int shrink_factor() const
  { return shrinkFactorSpinBox->value(); }
};

//----------------------------------------------------------------
// the_track_proc_ui_t
// 
class the_track_proc_ui_t : public QWidget,
			    public Ui::LabelDialog,
			    public the_procedure_ui_t
{
  Q_OBJECT
public:
  the_track_proc_ui_t(the_main_window_t * doc_ui);
  ~the_track_proc_ui_t();
  
  // virtual: start/stop the ui:
  void install();
  void uninstall();
  
  // virtual: ui update mechanism:
  void sync_ui();
  
  // helpers:
  void select_slice(QModelIndex index);
  void file_load(const QStringList & filenames);
  void file_load(const QStringList & filenames,
		 unsigned int shrink_factor);
  
public slots:
  void file_load();
  void edit_delete();
  void clear_all();
  void selection_changed();
  void active_label_changed();
  void propagate_active_label();
  void clear_active_label();
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  
private:
  // helper functions:
  the_track_proc_t * proc();
  
  // the event handler that will update the procedure:
  the_track_proc_eh_t eh_;
  
  // the dialog used to select image shrink factor:
  ShrinkFactorDialog shrink_factor_dialog_;
  
public:
  // data model:
  ReadOnlyStringListModel * model_;
};


#endif // THE_TRACK_PROC_UI_HXX_
