// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : PreferencesDialog.cpp
// Author       : Pavel A. Koshevoy
// Created      : Sat Sep 09 20:50:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : UI for the user preferences.

// local includes:
#include "PreferencesDialog.h"
#include "the_main_window.hxx"
#include "the_stos_proc_ui.hxx"

// the includes:
#include <ui/the_trail.hxx>
#include <opengl/OpenGLCapabilities.h>

// Qt includes:
#include <QSettings>
#include <QCheckBox>
#include <QCloseEvent>


//----------------------------------------------------------------
// prefsAllowLinearFiltering
// 
bool
prefsAllowLinearFiltering()
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  return main_window->preferences().allowLinearFiltering();
}

//----------------------------------------------------------------
// prefsUseLuminanceTextures
// 
bool
prefsUseLuminanceTextures()
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  return main_window->preferences().useLuminanceTextures();
}

//----------------------------------------------------------------
// prefsUseFragmentShader
// 
bool
prefsUseFragmentShader()
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  return main_window->preferences().useFragmentShader();
}

//----------------------------------------------------------------
// prefsSetFragmentShaderAvailable
// 
void
prefsSetFragmentShaderAvailable(bool has_shader)
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  
  main_window->preferences().setFragmentShaderAvailable(has_shader);
}

//----------------------------------------------------------------
// prefsSetUseFragmentShader
// 
void
prefsSetUseFragmentShader(bool use_shader)
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  
  main_window->preferences().setUseFragmentShader(use_shader);
}

//----------------------------------------------------------------
// prefsAutoRegeneration
// 
bool
prefsAutoRegeneration()
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  
  return (main_window->preferences().autoRegeneration() &&
	  main_window->stos_proc_ui()->isVisible());
}

//----------------------------------------------------------------
// prefsDisplayLandmarks
// 
bool
prefsDisplayLandmarks()
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  return main_window->preferences().displayLandmarks();
}

//----------------------------------------------------------------
// prefsUseOpenGLForMosaicking
// 
bool
prefsUseOpenGLForMosaicking()
{
  the_main_window_t * main_window =
    (the_main_window_t *)(the_document_ui_t::doc_ui());
  
  return main_window->preferences().useOpenGLForMosaicking();
}


//----------------------------------------------------------------
// to_bool
// 
static inline bool to_bool(const Qt::CheckState & chst)
{ return chst == Qt::Checked; }

//----------------------------------------------------------------
// to_chst
// 
static inline Qt::CheckState to_chst(const QVariant & qvar)
{ return qvar.toBool() ? Qt::Checked : Qt::Unchecked; }

//----------------------------------------------------------------
// to_chst
// 
static inline Qt::CheckState to_chst(const bool & checked)
{ return checked ? Qt::Checked : Qt::Unchecked; }

//----------------------------------------------------------------
// PreferencesDialog::PreferencesDialog
// 
PreferencesDialog::PreferencesDialog(QWidget * parent, Qt::WFlags f):
  QDialog(parent, f),
  Ui::PreferencesDialog()
{
  Ui::PreferencesDialog::setupUi(this);
}

//----------------------------------------------------------------
// PreferencesDialog::loadSettings
// 
void
PreferencesDialog::loadSettings()
{
  // load the preferences:
  QCoreApplication::setOrganizationName("SCII");
  QCoreApplication::setOrganizationDomain("sci.utah.edu");
  QCoreApplication::setApplicationName("ir-tweak");
  QSettings settings;
  
  linearFilteringCheckBox->setCheckState
    (to_chst(settings.value("allow_linear_filtering_",
			    to_bool(linearFilteringCheckBox->checkState()))));
  allow_linear_filtering_ = to_bool(linearFilteringCheckBox->checkState());
  
  luminanceTexturesCheckBox->setCheckState
    (to_chst(settings.
	     value("use_luminance_textures_",
		   to_bool(luminanceTexturesCheckBox->checkState()))));
  use_luminance_textures_ = to_bool(luminanceTexturesCheckBox->checkState());
  
  fragmentShaderCheckBox->setCheckState
    (to_chst(settings.
	     value("use_fragment_shader_",
		   to_bool(fragmentShaderCheckBox->checkState()))));
  use_fragment_shader_ = to_bool(fragmentShaderCheckBox->checkState());
  
  autoRegenMosaicCheckBox->setCheckState
    (to_chst(settings.value("auto_regeneration_",
			    to_bool(autoRegenMosaicCheckBox->checkState()))));
  auto_regeneration_ = to_bool(autoRegenMosaicCheckBox->checkState());
  
  displayLandmarksCheckBox->setCheckState
    (to_chst(settings.value("display_landmarks_",
			    to_bool(displayLandmarksCheckBox->checkState()))));
  display_landmarks_ = to_bool(displayLandmarksCheckBox->checkState());
  
  useOpenGLMosaicCheckBox->setCheckState
    (to_chst(settings.value("use_opengl_mosaicking_",
			    to_bool(useOpenGLMosaicCheckBox->checkState()))));
  use_opengl_mosaicking_ = to_bool(useOpenGLMosaicCheckBox->checkState());
}

//----------------------------------------------------------------
// PreferencesDialog::exec
// 
int
PreferencesDialog::exec()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  int r = QDialog::exec();
  if (r == QDialog::Accepted)
  {
    allow_linear_filtering_ = to_bool(linearFilteringCheckBox->checkState());
    use_luminance_textures_ = to_bool(luminanceTexturesCheckBox->checkState());
    use_fragment_shader_ = to_bool(fragmentShaderCheckBox->checkState());
    auto_regeneration_ = to_bool(autoRegenMosaicCheckBox->checkState());
    display_landmarks_ = to_bool(displayLandmarksCheckBox->checkState());
    use_opengl_mosaicking_ = to_bool(useOpenGLMosaicCheckBox->checkState());
    
    QSettings settings;
    settings.setValue("allow_linear_filtering_", allow_linear_filtering_);
    settings.setValue("use_luminance_textures_", use_luminance_textures_);
    settings.setValue("use_fragment_shader_", use_fragment_shader_);
    settings.setValue("auto_regeneration_", auto_regeneration_);
    settings.setValue("display_landmarks_", display_landmarks_);
    settings.setValue("use_opengl_mosaicking_", use_opengl_mosaicking_);
  }
  
  return r;
}

//----------------------------------------------------------------
// PreferencesDialog::useFragmentShader
// 
bool
PreferencesDialog::useFragmentShader() const
{
  return fragmentShaderCheckBox->isEnabled() && use_fragment_shader_;
}

//----------------------------------------------------------------
// PreferencesDialog::setFragmentShaderAvailable
// 
void
PreferencesDialog::setFragmentShaderAvailable(bool has_shader)
{
  fragmentShaderCheckBox->setEnabled(has_shader);
}

//----------------------------------------------------------------
// PreferencesDialog::setUseFragmentShader
// 
void
PreferencesDialog::setUseFragmentShader(bool use_shader)
{
  use_fragment_shader_ = use_shader;
  fragmentShaderCheckBox->setCheckState(to_chst(use_shader));
}

//----------------------------------------------------------------
// PreferencesDialog::closeEvent
// 
void
PreferencesDialog::closeEvent(QCloseEvent * e)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  e->ignore();
  done(QDialog::Rejected);
}
