// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_stos_proc.hxx
// Author       : Pavel A. Koshevoy
// Created      : 2006/07/11 13:20
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : main application window widget

// system includes:
#include <vector>

// Qt includes:
#include <QWidget>
#include <QTabWidget>
#include <QApplication>
#include <QMessageBox>
#include <QInputDialog>
#include <QFileDialog>
#include <QFileInfo>
#include <QAction>
#include <QListView>
#include <QPixmap>
#include <QGridLayout>
#include <QList>
#include <QDateTime>

// local includes:
#include "the_main_window.hxx"
#include "the_stos_proc_ui.hxx"
#include "the_stos_proc.hxx"

// the includes:
#include <math/the_color.hxx>
#include <opengl/the_appearance.hxx>
#include <eh/the_view_mgr_eh.hxx>
#include <io/the_file_io.hxx>
#include <utils/the_indentation.hxx>
#include <Qt/the_qt_trail.hxx>
#include <Qt/the_qt_view.hxx>

// forward declarations:
extern CGprogram CG_LA_FRAGMENT_PROGRAM;
extern CGprogram CG_MOSAIC_FRAGMENT_PROGRAM;
extern CGprogram * CG_LA_FRAGMENT_PROGRAM_PTR;
extern CGprogram * CG_MOSAIC_FRAGMENT_PROGRAM_PTR;
extern bool CG_LA_FRAGMENT_PROGRAM_OK;
extern bool CG_MOSAIC_FRAGMENT_PROGRAM_OK;


//----------------------------------------------------------------
// orientation
// 
static the_view_mgr_orientation_t orientation[] =
{
  THE_BOTTOM_VIEW_E,
  THE_BOTTOM_VIEW_E,
  THE_BOTTOM_VIEW_E
};


//----------------------------------------------------------------
// the_main_window_t::the_main_window_t
// 
the_main_window_t::the_main_window_t():
  QMainWindow(NULL, 0),
  the_document_ui_t("ir-tweak/20081009"),
  stos_proc_ui_(NULL),
  preferences_ui_(this)
{
  the_qt_view_t * shared = new the_qt_view_t(this, "", NULL, orientation[2]);
  the_view_t * qt_view[3];
  qt_view[0] = new the_qt_view_t(this, "fixed image", shared, orientation[0]);
  qt_view[1] = new the_qt_view_t(this, "moving image", shared, orientation[1]);
  qt_view[2] = shared;
  setup(NULL, qt_view, 3);
  
  view<the_qt_view_t>(0)->setObjectName("fixed");
  view<the_qt_view_t>(1)->setObjectName("moving");
  view<the_qt_view_t>(2)->setObjectName("mosaic");
  
  // disable 3D rotation:
#ifndef DEBUG_TEXTURES
  view_mgr_eh_[0]->allow_spin(false);
  view_mgr_eh_[1]->allow_spin(false);
  view_mgr_eh_[2]->allow_spin(false);
#endif
  
  setupUi(this);
  
  // signals and slots connections:
  actionNew->setShortcutContext(Qt::ApplicationShortcut);
  actionLoad->setShortcutContext(Qt::ApplicationShortcut);
  actionSave->setShortcutContext(Qt::ApplicationShortcut);
  actionAutoSave->setShortcutContext(Qt::ApplicationShortcut);
  actionQuit->setShortcutContext(Qt::ApplicationShortcut);
  actionRedo->setShortcutContext(Qt::ApplicationShortcut);
  actionUndo->setShortcutContext(Qt::ApplicationShortcut);
  actionDelete->setShortcutContext(Qt::ApplicationShortcut);
  actionMosaic->setShortcutContext(Qt::ApplicationShortcut);
  actionLinkViews->setShortcutContext(Qt::ApplicationShortcut);
  actionRestore->setShortcutContext(Qt::ApplicationShortcut);
  actionChangeFixed->setShortcutContext(Qt::ApplicationShortcut);
  actionChangeMoving->setShortcutContext(Qt::ApplicationShortcut);
  actionSwapImages->setShortcutContext(Qt::ApplicationShortcut);
  actionAbout->setShortcutContext(Qt::ApplicationShortcut);
  
  connect(actionNew, SIGNAL(triggered()),
	  this, SLOT(file_new()));
  connect(actionLoad, SIGNAL(triggered()),
	  this, SLOT(file_load()));
  connect(actionSave, SIGNAL(triggered()),
	  this, SLOT(file_save_as()));
  connect(actionAutoSave, SIGNAL(toggled(bool)),
	  this, SLOT(file_auto_save(bool)));
  connect(actionQuit, SIGNAL(triggered()),
	  this, SLOT(file_exit()));
  
  connect(actionRedo, SIGNAL(triggered()),
	  this, SLOT(edit_redo()));
  connect(actionUndo, SIGNAL(triggered()),
	  this, SLOT(edit_undo()));
  connect(actionPreferences, SIGNAL(triggered()),
	  this, SLOT(edit_preferences()));
  
  connect(actionMosaic, SIGNAL(toggled(bool)),
	  this, SLOT(toggle_stos_ui(bool)));
  connect(actionMosaicIcon, SIGNAL(toggled(bool)),
	  this, SLOT(toggle_stos_ui(bool)));
  connect(actionLinkViews, SIGNAL(toggled(bool)),
	  this, SLOT(toggle_link_views(bool)));
  connect(actionRestore, SIGNAL(triggered()),
	  this, SLOT(orientation_restore()));
  
  connect(actionAbout, SIGNAL(triggered()),
	  this, SLOT(helpAbout()));
  
  connect(&autosave_, SIGNAL(timeout()),
	  this, SLOT(auto_save()));
  
  // UI objects of various procedures:
  stos_proc_ui_ = new the_stos_proc_ui_t(this);
  
  // fixed:
  layout_[0] = new QGridLayout(frameFixed);
  layout_[0]->setSpacing(0);
  layout_[0]->setMargin(0);
  view<the_qt_view_t>(0)->setParent(frameFixed);
  layout_[0]->addWidget(view<the_qt_view_t>(0), 0, 0);
  view<the_qt_view_t>(0)->show();
  
  // moving:
  layout_[1] = new QGridLayout(frameMoving);
  layout_[1]->setSpacing(0);
  layout_[1]->setMargin(0);
  view<the_qt_view_t>(1)->setParent(frameMoving);
  layout_[1]->addWidget(view<the_qt_view_t>(1), 0, 0);
  view<the_qt_view_t>(1)->show();
  
  // mosaic:
  layout_[2] = new QGridLayout(stos_proc_ui_->frameMosaic);
  layout_[2]->setSpacing(0);
  layout_[2]->setMargin(0);
  view<the_qt_view_t>(2)->setParent(stos_proc_ui_->frameMosaic);
  layout_[2]->addWidget(view<the_qt_view_t>(2), 0, 0);
  view<the_qt_view_t>(2)->show();
  
  // initialize the UI:
  sync_ui();
}

//----------------------------------------------------------------
// the_main_window_t::~the_main_window_t
// 
the_main_window_t::~the_main_window_t()
{
  assert(stos_proc_ui_ == NULL);
}

//----------------------------------------------------------------
// ok_to_discard_current_document
// 
static bool
ok_to_discard_current_document(the_main_window_t * main_window,
			       the_document_so_t & doc_so)
{
  if (doc_so.changes_saved()) return true;
  
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  QMessageBox mb(main_window);
  mb.setIcon(QMessageBox::Question);	
  mb.setWindowTitle(QObject::tr("Save changes?"));
  
  std::ostringstream os;
  os << "The document \"" << doc_so.document()->name()
     <<"\" has unsaved changes." << endl;
  mb.setText(QString::fromUtf8(os.str().c_str()));
  
  os.str("");
  os << "Click [Save] to save the changes and proceed." << endl
     << "Click [Discard] to discard the changes and proceed." << endl
     << "Click [Cancel] to keep the changes and continue working." << endl;
  mb.setDetailedText(QString::fromUtf8(os.str().c_str()));
  
  QAbstractButton * btn_save =
    mb.addButton(QObject::tr("Save"), QMessageBox::ActionRole);
  
  QAbstractButton * btn_discard =
    mb.addButton(QObject::tr("Discard"), QMessageBox::ActionRole);
  
  // QAbstractButton * btn_cancel =
  mb.addButton(QObject::tr("Cancel"), QMessageBox::ActionRole);
  
  mb.exec();
  if (mb.clickedButton() == btn_save)
  {
    main_window->file_save();
    if (doc_so.changes_saved()) return true;
    
    QMessageBox::critical(main_window,
			  QObject::tr("Could not save changes"),
			  QObject::tr("Could not save the document, "
				      "aborting..."),
			  QMessageBox::Ok,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
    return false;
  }
  
  return (mb.clickedButton() == btn_discard);
}

//----------------------------------------------------------------
// the_main_window_t::file_new
// 
void
the_main_window_t::file_new()
{
  // check whether the existing document needs to be saved:
  if (ok_to_discard_current_document(this, doc_so()) == false) return;
  
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  // this is potentially better:
  MOSAIC_THREAD_POOL()->stop();
  MOSAIC_THREAD_POOL()->wait();
  QString time_stamp = QDateTime::currentDateTime().toString("yyyyMMdd-hhmm");
  doc_so().new_document(new the_document_t((time_stamp + ".tweak").
					   toUtf8().constData()));
  if (!stos_proc_ui_->installed())
  {
    tool_stos();
  }
  else
  {
    document()->add_proc(new the_stos_proc_t());
  }
  
  sync_ui();
}

//----------------------------------------------------------------
// the_main_window_t::file_load
// 
void
the_main_window_t::file_load()
{
  // check whether the existing document needs to be saved:
  if (ok_to_discard_current_document(this, doc_so()) == false) return;
  
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  QString filename
    (QFileDialog::getOpenFileName(this,
				  tr("Open document"),
				  QString(),
				  tr("ir-tweak documents (*.tweak)")));
  file_load(filename);
}

// instantiate the loaders:
template bool the_loader
<the_primitive_t, the_stos_proc_t>(std::istream &, the_primitive_t *&);

template bool the_loader
<the_primitive_t, the_hard_point_t>(std::istream &, the_primitive_t *&);

template bool the_loader
<the_primitive_t, point_pair_t>(std::istream &, the_primitive_t *&);

template bool the_loader
<the_primitive_t, the_image_t>(std::istream &, the_primitive_t *&);

template bool the_loader
<the_primitive_t, the_xform_t>(std::istream &, the_primitive_t *&);

template bool the_loader
<the_primitive_t, the_mosaic_t>(std::istream &, the_primitive_t *&);

//----------------------------------------------------------------
// the_main_window_t::file_load
// 
void
the_main_window_t::file_load(const QString & filename)
{
  // check whether the existing document needs to be saved:
  if (ok_to_discard_current_document(this, doc_so()) == false) return;
  
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (filename.isEmpty()) return;
  
  // change current working directory:
  QDir::setCurrent(QFileInfo(filename).dir().absolutePath());
  
  // FIXME: experimental:
  MOSAIC_THREAD_POOL()->stop();
  MOSAIC_THREAD_POOL()->wait();
  
  // setup the file io handlers:
  {
    the_file_io_t<the_primitive_t> & io = the_primitive_file_io();
    
    typedef the_loader_t<the_primitive_t> loader_t;
    io.add(loader_t("the_stos_proc_t",
		    the_loader<the_primitive_t, the_stos_proc_t>));
    io.add(loader_t("the_hard_point_t",
		    the_loader<the_primitive_t, the_hard_point_t>));
    io.add(loader_t("point_pair_t",
		    the_loader<the_primitive_t, point_pair_t>));
    io.add(loader_t("the_image_t",
		    the_loader<the_primitive_t, the_image_t>));
    io.add(loader_t("the_xform_t",
		    the_loader<the_primitive_t, the_xform_t>));
    io.add(loader_t("the_mosaic_t",
		    the_loader<the_primitive_t, the_mosaic_t>));
  }
  
  // setup file io handlers for references:
  {
    the_file_io_t<the_reference_t> & io = the_reference_file_io();
    
    typedef the_loader_t<the_reference_t> loader_t;
    io.add(loader_t("the_point_ref_t",
		    the_loader<the_reference_t, the_point_ref_t>));
  }
  
  // try to load the document:
  if (!doc_so().load_document(filename.toUtf8().constData()))
  {
    // could not open the document:
    QMessageBox::critical(this,
			  tr("Could not load the document"),
			  tr("Could not open ") + filename,
			  QMessageBox::Ok,
			  QMessageBox::NoButton,
			  QMessageBox::NoButton);
  }
  
  if (!stos_proc_ui_->installed())
  {
    tool_stos();
  }
  
  view(0)->restore_orientation();
  view(1)->restore_orientation();
  view(2)->restore_orientation();
  sync_ui();
}

//----------------------------------------------------------------
// the_main_window_t::file_save_as
// 
bool
the_main_window_t::file_save_as()
{
  return file_save(true);
}

//----------------------------------------------------------------
// the_main_window_t::file_save
// 
bool
the_main_window_t::file_save(bool save_as)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (!doc_so().has_filename() || save_as)
  {
    QString suggest_fn = QString::fromUtf8(document()->name().text());
    QString filename
      (QFileDialog::getSaveFileName(this,
				    tr("Save document"),
				    suggest_fn,
				    tr("ir-tweak documents (*.tweak)")));
    if (filename.isEmpty()) return false;
    doc_so().set_filename(filename.toUtf8().constData());
  }
  
  if (!auto_save())
  {
    return false;
  }
  
  sync_ui();
  return true;
}

//----------------------------------------------------------------
// the_main_window_t::file_auto_save
// 
void
the_main_window_t::file_auto_save(bool on)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  // stop the timer:
  autosave_.stop();
  if (!on) return;
  
  // make sure we will be able to autosave successfully,
  // if not -- turn of autosave:
  if (!file_save())
  {
    QMessageBox::critical
      (this,
       tr("Auto save canceled"),
       tr("Auto save requires a valid file name, aborting..."),
       QMessageBox::Ok,
       QMessageBox::NoButton,
       QMessageBox::NoButton);
    actionAutoSave->setChecked(false);
    return;
  }
  
  // auto-save the file every 30 seconds:
  static const unsigned int sec = 30;
  autosave_.start(sec * 1000);
}

//----------------------------------------------------------------
// the_main_window_t::file_exit
// 
void
the_main_window_t::file_exit()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  // check whether the existing document needs to be saved:
  if (ok_to_discard_current_document(this, doc_so()) == false) return;
  
  // stop trail playback:
  THE_TRAIL.stop();
  
  // shut down the procedure UI:
  stos_proc_ui_->uninstall();
  
  // close the document:
  doc_so().close_document();
  
  // delete the mosaic view:
  view<the_qt_view_t>(2)->setParent(NULL);
  delete stos_proc_ui_;
  stos_proc_ui_ = NULL;
  
  // shut dowm the document UI:
  shutdown();
  
  // done:
  qApp->quit();
}

//----------------------------------------------------------------
// the_main_window_t::edit_undo
// 
void
the_main_window_t::edit_undo()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  doc_so().undo();
  document()->active_procedure<the_stos_proc_t>()->request_regeneration();
  sync_ui();
}

//----------------------------------------------------------------
// the_main_window_t::edit_redo
// 
void
the_main_window_t::edit_redo()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  doc_so().redo();
  document()->active_procedure<the_stos_proc_t>()->request_regeneration();
  sync_ui();
}

//----------------------------------------------------------------
// the_main_window_t::edit_preferences
// 
void
the_main_window_t::edit_preferences()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;

  bool prev_gpu_pref = prefsUseFragmentShader();
  
  int r = preferences_ui_.exec();
  if (r == QDialog::Accepted)
  {
    bool curr_gpu_pref = prefsUseFragmentShader();
    if (curr_gpu_pref != prev_gpu_pref)
    {
      if (!curr_gpu_pref)
      {
	CG_LA_FRAGMENT_PROGRAM_PTR = NULL;
	CG_MOSAIC_FRAGMENT_PROGRAM_PTR = NULL;
      }
      else
      {
	CG_LA_FRAGMENT_PROGRAM_PTR =
	  CG_LA_FRAGMENT_PROGRAM_OK ?
	  &CG_LA_FRAGMENT_PROGRAM :
	  NULL;
	
	CG_MOSAIC_FRAGMENT_PROGRAM_PTR =
	  CG_MOSAIC_FRAGMENT_PROGRAM_OK ?
	  &CG_MOSAIC_FRAGMENT_PROGRAM :
	  NULL;
      }
      
      // reload the fixed/moving images in correct format:
      the_stos_proc_t * proc = document()->active_procedure<the_stos_proc_t>();
      proc->reload();
      proc->request_regeneration();
      proc->regenerate();
    }
    
    document()->active_procedure()->request_regeneration();
    sync_ui();
  }
}

//----------------------------------------------------------------
// the_main_window_t::toggle_stos_ui
// 
void
the_main_window_t::toggle_stos_ui(bool on)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (on && stos_proc_ui_->isHidden())
  {
    stos_proc_ui_->show();
    
    if (!actionMosaic->isChecked())
    {
      actionMosaic->setChecked(true);
    }
    
    if (!actionMosaicIcon->isChecked())
    {
      actionMosaicIcon->setChecked(true);
    }
  }
  else if (!on && stos_proc_ui_->isVisible())
  {
    stos_proc_ui_->hide();
    
    if (actionMosaic->isChecked())
    {
      actionMosaic->setChecked(false);
    }
    
    if (actionMosaicIcon->isChecked())
    {
      actionMosaicIcon->setChecked(false);
    }
  }
}

//----------------------------------------------------------------
// the_main_window_t::toggle_link_views
// 
void
the_main_window_t::toggle_link_views(bool on)
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  if (on)
  {
    view_mgr_eh_[0]->add_linked_view(view_[1]);
    view_mgr_eh_[2]->add_linked_view(view_[0]);
    view_mgr_eh_[2]->add_linked_view(view_[1]);
  }
  else
  {
    view_mgr_eh_[0]->del_linked_view(view_[1]);
    view_mgr_eh_[2]->del_linked_view(view_[0]);
    view_mgr_eh_[2]->del_linked_view(view_[1]);
  }
}

//----------------------------------------------------------------
// the_main_window_t::orientation_restore
// 
void
the_main_window_t::orientation_restore()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  std::vector<the_view_t *> views;
  get_visible_views(views);
  unsigned int num_views = views.size();
  for (unsigned int i = 0; i < num_views; i++)
  {
    views[i]->restore_orientation();
  }
}

//----------------------------------------------------------------
// the_main_window_t::helpAbout
// 
void
the_main_window_t::helpAbout()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  AboutDialog about(this);
  about.exec();
}

//----------------------------------------------------------------
// the_main_window_t::tool_stos
// 
void
the_main_window_t::tool_stos()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  toggle_stos_ui(true);
  stos_proc_ui_->install();
}

//----------------------------------------------------------------
// the_main_window_t::auto_save
// 
bool
the_main_window_t::auto_save()
{
  if (!doc_so().changes_saved())
  {
    if (!doc_so().save_document())
    {
      QMessageBox::critical(this,
			    tr("Could not save the document"),
			    tr("Could not save ") +
			    QString::fromUtf8(doc_so().filename()),
			    QMessageBox::Ok,
			    QMessageBox::NoButton,
			    QMessageBox::NoButton);
      return false;
    }
  }
  
  return true;
}

//----------------------------------------------------------------
// the_main_window_t::sync_ui
// 
void
the_main_window_t::sync_ui()
{
  // setup 3D viewing (necessary in order for the compiled display
  // lists to tesselate the splines correctly):
  // shared()->gl_make_current();
  
  // shortcut:
  const the_document_t * doc = document();
  bool has_doc = (doc != NULL);
  
  // shortcut:
  the_procedure_ui_t * proc_ui = active_proc_ui();
  if (proc_ui != NULL) proc_ui->sync_ui();
  
  // update the views:
  for (unsigned int i = 0; i < view_.size(); i++)
  {
    the_qt_view_t * v = view<the_qt_view_t>(i);
    if (v->isHidden()) continue;
    v->refresh();
  }
  
  // update the actions:
  actionSave->setEnabled(has_doc);

  bool undo_ok = has_doc && !doc_so().undo_stack_empty();
  actionUndo->setEnabled(undo_ok);
  actionUndoIcon->setEnabled(undo_ok);

  bool redo_ok = has_doc && !doc_so().redo_stack_empty();
  actionRedo->setEnabled(redo_ok);
  actionRedoIcon->setEnabled(redo_ok);
  
  // FIXME:
#if 0
  if (!view(2)->is_hidden())
  {
    static unsigned int frame = 0;
    the_qt_view_t * v = view<the_qt_view_t>(2);
    QImage img = v->grabFrameBuffer();
    static const the_text_t fn_prefix("/scratch/crcns/frames/ir-tweak/");
    the_text_t fn = fn_prefix + the_text_t::number(frame, 6, '0') + ".png";
    img.save(QString::fromUtf8(fn.text()));
    frame++;
  }
#endif
}

//----------------------------------------------------------------
// the_main_window_t::kill_ui
// 
void
the_main_window_t::kill_ui()
{
  // setup the milestone markers:
  the_trail_t::milestone_t milestone;
  
  file_exit();
}

//----------------------------------------------------------------
// the_main_window_t::get_visible_views
// 
void
the_main_window_t::get_visible_views(std::vector<the_view_t *> & views) const
{
  if (stos_proc_ui_->isHidden())
  {
    views.resize(2);
  }
  else
  {
    views.resize(3);
    views[2] = view_[2];
  }
  
  views[0] = view_[0];
  views[1] = view_[1];
}

//----------------------------------------------------------------
// the_main_window_t::closeEvent
// 
void
the_main_window_t::closeEvent(QCloseEvent * e)
{
  e->ignore();
  kill_ui();
}
