// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_stos_proc_ui.hxx
// Author       : Pavel A. Koshevoy
// Created      : 2006/07/11 13:20
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : UI for the landmark registration tool
//                used for slice to slice registration.

#ifndef THE_STOS_PROC_UI_HXX_
#define THE_STOS_PROC_UI_HXX_

// system includes:
#include <list>

// local includes:
#include "ui_MosaicWindow.h"
#include "the_stos_proc.hxx"
#include "ImageLoaderDialog.h"

// thelib includes:
#include <ui/the_procedure_ui.hxx>
#include <eh/the_document_eh.hxx>
#include <doc/the_document.hxx>
#include <doc/the_registry.hxx>
#include <geom/the_point.hxx>
#include <Qt/the_qt_thread_pool.hxx>

// Qt includes:
#include <QObject>
#include <QDialog>
#include <QTimer>

// forward declarations:
class the_point_t;
class the_graph_t;
class the_pick_list_t;
class the_main_window_t;
class the_transaction_t;


//----------------------------------------------------------------
// MOSAIC_THREAD_POOL
// 
extern the_qt_thread_pool_t * MOSAIC_THREAD_POOL();


//----------------------------------------------------------------
// the_stos_proc_eh_t
// 
class the_stos_proc_eh_t : public the_document_eh_t<the_document_t>
{
public:
  the_stos_proc_eh_t(the_document_ui_t * doc_ui);
  
  // virtual:
  const char * name() const
  { return "the_stos_proc_eh_t"; }
  
  // virtual: check if a given modifier is currently active:
  bool is_modifier_active_locally(the_eh_modifier_t mod) const;
  
  // the UI will call this:
  void delete_something();
  void clear_all();
  
  // helpers, shortcuts:
  the_registry_t * registry() const;
  
  // helper functions:
  the_stos_proc_t * proc() const;
  
  // helper:
  the_point_t * active_pt() const
  {
    return
      (proc()->selection_ == std::numeric_limits<unsigned int>::max()) ?
      NULL :
      registry()->elem<the_point_t>(proc()->selection_);
  }
  
  // virtual:
  bool view_handler(the_view_t * view);
  
protected:
  // virtual: Override the processor function.
  bool processor(the_input_device_eh_t::the_event_type_t event_type);
  
  // processor helper functions:
  bool process_mouse();
};

//----------------------------------------------------------------
// the_stos_proc_ui_t
// 
class the_stos_proc_ui_t : public QDialog,
			   public Ui::MosaicWindow,
			   public the_procedure_ui_t
{
  Q_OBJECT
public:
  the_stos_proc_ui_t(the_main_window_t * doc_ui);
  ~the_stos_proc_ui_t();
  
  // virtual: start/stop the ui:
  void install();
  void uninstall();
  
  // virtual: ui update mechanism:
  void sync_ui();
  
public slots:
  // the file menu:
  void exportStoS();
  void exportMosaic();
  void exportWarped();
  void exportRegistered();
  
  // the image menu:
  void changeFixed();
  void changeMoving();
  void flipFixed();
  void flipMoving();
  void swapImages();
  
  void slot_edit_delete()
  { eh_.delete_something(); }
  
  void slot_clear_all()
  { eh_.clear_all(); }
  
  void rgbaDone(the_transaction_t * t);
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  void customEvent(QEvent * e);
  
private slots:
  // this slot will be connected to the repaint timer:
  void repaintMosaic();
  
private:
  // helper functions:
  the_stos_proc_t * proc();
  
  // the event handler that will update the procedure:
  the_stos_proc_eh_t eh_;
  
  // the image loader dialogs:
  ImageLoaderDialog image_loader_;
  
  // this timer will be used to buffer OpenGL updates -- instead of
  // executing each repaint immediately the repaint will be carried
  // out after a timeout event:
  QTimer repaint_timer_;
};


#endif // THE_STOS_PROC_UI_HXX_
