// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_warped_image_dl_elem.cxx
// Author       : Pavel A. Koshevoy
// Created      : Wed Mar 3 12:10:00 MST 2007
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : An OpenGL display list element for an image
//                warped with the grid transform.

// GLEW includes:
#define GLEW_STATIC 1
#include <GL/glew.h>

// local includes:
#include "the_warped_image_dl_elem.hxx"

// the includes:
#include <image/texture.hxx>

#ifdef USE_CG
extern CGcontext cg_context;
extern CGprofile cg_profile;
#endif


#if 0
//----------------------------------------------------------------
// operator <<
// 
static std::ostream &
operator << (std::ostream & so,
	     const the_warped_image_dl_elem_t::quad_strip_edge_t & e)
{
  so << e.x0_ << ", " << e.y0_ << " -- " << e.x1_ << ", " << e.y1_;
  return so;
}
#endif


//----------------------------------------------------------------
// the_warped_image_dl_elem_t::the_warped_image_dl_elem_t
// 
the_warped_image_dl_elem_t::
the_warped_image_dl_elem_t(boost::shared_ptr<image_tile_dl_elem_t> & image,
			   const the_grid_transform_t & transform):
  image_(image)
{
  update(transform);
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::the_warped_image_dl_elem_t
// 
the_warped_image_dl_elem_t::
the_warped_image_dl_elem_t(const image_tile_generator_t & image,
			   const the_grid_transform_t & transform,
			   GLenum min_filter,
			   GLenum mag_filter
			   #ifdef USE_CG
			   ,const CGprogram * fragment_program
			   #endif
			   ):
  image_(boost::shared_ptr<image_tile_dl_elem_t>
	 (new image_tile_dl_elem_t(image,
				   min_filter,
				   mag_filter
				   #ifdef USE_CG
				   , fragment_program
				   #endif
				   )))
{
  update(transform);
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::~the_warped_image_dl_elem_t
// 
the_warped_image_dl_elem_t::~the_warped_image_dl_elem_t()
{
  // FIXME_OPENGL("the_warped_image_dl_elem_t::~the_warped_image_dl_elem_t");
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::update
// 
void
the_warped_image_dl_elem_t::update(const the_grid_transform_t & transform)
{
  bbox_ = the_bbox_t();
  bbox_ << p3x1_t(transform.grid_min_[0],
		  transform.grid_min_[1], 0)
	<< p3x1_t(transform.grid_min_[0] + transform.grid_ext_[0],
		  transform.grid_min_[1] + transform.grid_ext_[1], 0);
  
  // shortcuts:
  const image_tile_generator_t & image = image_->data_;
  const image_tile_t * tiles = &(image.tiles_[0]);
  const unsigned int num_tiles = image.tiles_.size();
  image_tile_quad_strips_.resize(num_tiles);
  
  for (unsigned int i = 0; i < num_tiles; i++)
  {
    const image_tile_t & tile = tiles[i];
    const texture_base_t & tex = *(tile.texture_.get());
    fast_array_t<fast_array_t<quad_strip_edge_t> > & tile_quad_strips =
      image_tile_quad_strips_[i];
    
    double ou = double(tex.skip_pixels_) / double(image.w_);
    double ov = double(tex.skip_rows_) / double(image.h_);
    double du = double(tex.width_ - 2) / double(image.w_);
    double dv = double(tex.height_ - 2) / double(image.h_);
    /*
      cerr << tex.skip_pixels_ << ":" << tex.skip_rows_ << "  "
      << tex.width_ << " x " << tex.height_ << ",  "
      << image.w_ << " x " << image.h_
      << endl;
    */
    // draw a quad strip:
    double ds = tile.s1_ - tile.s0_;
    double dt = tile.t1_ - tile.t0_;
    /*
      cerr << tile.s0_ << ", " << tile.s1_ << "  x  "
      << tile.t0_ << ", " << tile.t1_ << endl;
    */
    unsigned int n_rows =
      (unsigned int)(std::max(1.0, dv * double(transform.rows_)));
    unsigned int n_cols =
      (unsigned int)(std::max(1.0, du * double(transform.cols_)));
    
    tile_quad_strips.resize(n_rows);
      
    // temporaries:
    pnt2d_t itk_xy;
    pnt2d_t itk_uv;
    
    for (unsigned int row = 0; row < n_rows; row++)
    {
      // shortcut:
      fast_array_t<quad_strip_edge_t> & quad_strip = tile_quad_strips[row];
      quad_strip.resize(n_cols + 1);
      
      double v0 = double(row) / double(n_rows);
      double v1 = double(row + 1) / double(n_rows);
      
      // if (row != 4) continue;
      
      for (unsigned int col = 0; col <= n_cols; col++)
      {
	quad_strip_edge_t & edge = quad_strip[col];
	double u = double(col) / double(n_cols);
	
	edge.u_ = u * ds + tile.s0_;
	edge.v0_ = v0 * dt + tile.t0_;
	edge.v1_ = v1 * dt + tile.t0_;
	
	itk_uv[0] = std::min(1.0, ou + u * du);
	itk_uv[1] = std::min(1.0, ov + v0 * dv);
	transform.transform_inv(itk_uv, itk_xy);
	edge.x0_ = itk_xy[0];
	edge.y0_ = itk_xy[1];
	
	itk_uv[1] = std::min(1.0, ov + v1 * dv);
	transform.transform_inv(itk_uv, itk_xy);
	edge.x1_ = itk_xy[0];
	edge.y1_ = itk_xy[1];
	
	// cerr << edge << endl;
      }
      // cerr << endl;
    }
  }
  // cerr << " - - - - - - - - - - - - - - - - - - - - - - - - - - -" << endl;
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::draw
// 
void
the_warped_image_dl_elem_t::draw() const
{
  draw(the_color_t::WHITE, true);
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::draw
// 
void
the_warped_image_dl_elem_t::draw(const the_color_t & color,
				 const bool & use_textures) const
{
  color_ = color;
  use_textures_ = use_textures;
  image_->draw(draw_tile, this, color, use_textures);
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::draw_tile
// 
void
the_warped_image_dl_elem_t::draw_tile(const void * data,
				      const unsigned int & tile_index)
{
  const the_warped_image_dl_elem_t * warped =
    (const the_warped_image_dl_elem_t *)(data);
  
  const image_tile_dl_elem_t * image = warped->image_.get();
  
#ifdef USE_CG
  if (image->has_fragment_program())
  {
    const CGprogram & fragment_program = *(image->fragment_program_);
    CGparameter tint_param = cgGetNamedParameter(fragment_program, "tint");
    CGparameter mask_param = cgGetNamedParameter(fragment_program, "mask");
    
    cgGLSetParameter3f(tint_param,
		       warped->color_[0],
		       warped->color_[1],
		       warped->color_[2]);
    cgGLSetParameter1f(mask_param, float(!warped->use_textures_));
  }
#endif
  
  // shortcuts:
  const fast_array_t<fast_array_t<quad_strip_edge_t> > & quad_strips =
    warped->image_tile_quad_strips_[tile_index];
  
  const unsigned int n_strips = quad_strips.size_;
  const unsigned int n_edges = quad_strips[0].size_;
  
  for (unsigned int strip = 0; strip < n_strips; strip++)
  {
    // shortcut:
    const quad_strip_edge_t * edges = &(quad_strips[strip][0]);
    
    // if (strip != 4) continue;
    // cerr << " . . . . . . . . . . . . . . . . . . . . . . . . . . " << endl;
    glBegin(GL_QUAD_STRIP);
    {
      for (unsigned int edge = 0; edge < n_edges; edge++)
      {
	const quad_strip_edge_t & e = edges[edge];
	// cerr << e << endl;
	glTexCoord2f(e.u_, e.v0_);
	glVertex2f(e.x0_, e.y0_);
	
	glTexCoord2f(e.u_, e.v1_);
	glVertex2f(e.x1_, e.y1_);
      }
    }
    glEnd();
  }
}

//----------------------------------------------------------------
// the_warped_image_dl_elem_t::update_bbox
// 
void
the_warped_image_dl_elem_t::update_bbox(the_bbox_t & bbox) const
{
  bbox += bbox_;
}
