/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : PreferencesDialog.h
// Author       : Pavel Aleksandrovich Koshevoy
// Created      : Mon Sep 25 21:58:00 MDT 2006
// Copyright    : (C) 2006-2008 University of Utah
// License      : GPLv2
// Description  : UI for the user preferences.

#ifndef PREFERENCES_DIALOG_H_
#define PREFERENCES_DIALOG_H_

// local includes:
#include "ui_PreferencesDialog.h"

// Qt includes:
#include <QDialog>

// forward declarations:
extern bool prefsAllowLinearFiltering();
extern bool prefsAllowTransparency();
extern bool prefsShouldFitToImage();
extern unsigned int prefsSlideShowDelay();


//----------------------------------------------------------------
// PreferencesDialog
// 
class PreferencesDialog : public QDialog,
			  public Ui::PreferencesDialog
{
  Q_OBJECT
  
public:
  PreferencesDialog(QWidget * parent = 0, Qt::WFlags f = 0);
  
  // helper:
  void loadSettings();
  
  // virtual:
  int exec();
  
  // accessors:
  inline const bool & allowLinearFiltering() const
  { return allow_linear_filtering_; }
  
  inline bool allowTransparency() const
  { return allow_transparency_; }

  inline bool shouldFitToImage() const
  { return should_fit_to_image_; }
  
  inline unsigned int slideShowDelay()
  { return slide_show_delay_; }
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  
  bool allow_linear_filtering_;
  bool allow_transparency_;
  bool should_fit_to_image_;
  unsigned int slide_show_delay_;
};


#endif // PREFERENCES_DIALOG_H_
