/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : the_iv_proc.cxx
// Author       : Pavel Aleksandrovich Koshevoy
// Created      : Sun Sep 24 16:41:00 MDT 2006
// Copyright    : (C) 2006-2008 University of Utah
// License      : GPLv2
// Description  : This procedure manages the displayed image.

// local includes:
#include "the_iv_proc.hxx"
#include "PreferencesDialog.h"

// thelib includes:
#include <doc/the_graph.hxx>
#include <opengl/the_view.hxx>
#include <ui/the_document_ui.hxx>
#include <ui/the_trail.hxx>


//----------------------------------------------------------------
// the_iv_proc_t::the_iv_proc_t
// 
the_iv_proc_t::the_iv_proc_t():
  the_procedure_t(),
  dl_(new the_disp_list_t()),
  w_(0),
  h_(0)
{}

//----------------------------------------------------------------
// the_iv_proc_t::regenerate
// 
bool
the_iv_proc_t::regenerate()
{
  regenerated_ = true;
  return regenerated_;
}

//----------------------------------------------------------------
// the_iv_proc_t::draw
// 
void
the_iv_proc_t::draw(const the_view_t & view) const
{
  view.view_mgr().reset_opengl_viewing();
  view.view_mgr().setup_opengl_3d_viewing();
  
  glPushAttrib(GL_ENABLE_BIT);
  {
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_ALPHA_TEST);
    
    if (prefsAllowTransparency())
    {
      glEnable(GL_BLEND);
    }
    else
    {
      glDisable(GL_BLEND);
    }
    
    dl_->draw();
  }
  glPopAttrib();
}

//----------------------------------------------------------------
// the_iv_proc_t::calc_bbox
// 
void
the_iv_proc_t::calc_bbox(const the_view_t &, the_bbox_t & bbox) const
{
  bbox += dl_->bbox();
}

//----------------------------------------------------------------
// the_iv_proc_t::save
// 
bool
the_iv_proc_t::save(std::ostream &) const
{
  return false;
}

//----------------------------------------------------------------
// the_iv_proc_t::load
// 
bool
the_iv_proc_t::load(std::istream &)
{
  return false;
}
