/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : the_iv_proc.hxx
// Author       : Pavel Aleksandrovich Koshevoy
// Created      : Sun Sep 24 16:40:00 MDT 2006
// Copyright    : (C) 2006-2008 University of Utah
// License      : GPLv2
// Description  : This procedure manages the displayed image.

#ifndef THE_IV_PROC_HXX_
#define THE_IV_PROC_HXX_

// Boost includes:
#include <boost/shared_ptr.hpp>

// thelib includes:
#include <utils/the_text.hxx>
#include <opengl/the_disp_list.hxx>
#include <doc/the_procedure.hxx>

// forward declarations:
class the_bbox_t;
class the_view_t;
class the_view_volume_t;
class the_image_t;
class the_xform_t;
class the_mosaic_t;


//----------------------------------------------------------------
// the_iv_proc_t
// 
class the_iv_proc_t : public the_procedure_t
{
public:
  the_iv_proc_t();
  
  // virtual:
  the_primitive_t * clone() const
  { return new the_iv_proc_t(*this); }
  
  // virtual:
  const char * name() const
  { return "the_iv_proc_t"; }
  
  // virtual:
  bool regenerate();
  
  // virtual:
  void draw(const the_view_t & view) const;
  
  // virtual: calculate the model bounding box:
  void calc_bbox(const the_view_t & view, the_bbox_t & bbox) const;
  
  // virtual: file io:
  bool save(std::ostream & stream) const;
  bool load(std::istream & stream);
  
  // image display list:
  boost::shared_ptr<the_disp_list_t> dl_;
  
  // image width and height:
  unsigned int w_;
  unsigned int h_;
};


#endif // THE_IV_PROC_HXX_
