/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : the_iv_proc_ui.hxx
// Author       : Pavel Aleksandrovich Koshevoy
// Created      : Sun Sep 24 16:48:00 MDT 2006
// Copyright    : (C) 2006-2008 University of Utah
// License      : GPLv2
// Description  : UI for the image viewer procedure.

#ifndef THE_IV_PROC_UI_HXX_
#define THE_IV_PROC_UI_HXX_

// local includes:
#include "the_iv_proc.hxx"

// the includes:
#include <ui/the_procedure_ui.hxx>
#include <eh/the_document_eh.hxx>
#include <doc/the_document.hxx>
#include <doc/the_registry.hxx>

// Qt includes:
#include <QObject>

// forward declarations:
class the_main_window_t;
class the_transaction_t;
class QFileDialog;


//----------------------------------------------------------------
// the_iv_proc_eh_t
// 
class the_iv_proc_eh_t : public the_document_eh_t<the_document_t>
{
public:
  the_iv_proc_eh_t(the_document_ui_t * doc_ui):
    the_document_eh_t<the_document_t>(doc_ui)
  {}
  
  // virtual:
  const char * name() const
  { return "the_iv_proc_eh_t"; }
  
  // virtual:
  bool is_modifier_active_locally(the_eh_modifier_t mod) const;
  
protected:
  // virtual:
  bool processor(the_input_device_eh_t::the_event_type_t);
};

//----------------------------------------------------------------
// the_iv_proc_ui_t
// 
class the_iv_proc_ui_t : public QObject,
			 public the_procedure_ui_t
{
  Q_OBJECT
public:
  the_iv_proc_ui_t(the_main_window_t * doc_ui);
  ~the_iv_proc_ui_t();
  
  // helper functions:
  the_iv_proc_t * proc();
  
  // virtual: start/stop the ui:
  void install();
  void uninstall();
  
  // virtual: ui update mechanism:
  void sync_ui() {}
  
  // helper:
  void file_load(const QString & filepath);
  
public slots:
  // the image reader thread:
  void transactionFinished(the_transaction_t *);
  
  // the file menu:
  void file_load();
  
private:
  // the event handler that will update the procedure:
  the_iv_proc_eh_t eh_;
  
  // helper dialog:
  QFileDialog * file_dialog_;
};


#endif // THE_IV_PROC_UI_HXX_
