/*  
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

// File         : the_main_window.hxx
// Author       : Pavel Aleksandrovich Koshevoy
// Created      : Sun Sep 24 16:41:00 MDT 2006
// Copyright    : (C) 2006-2088 University of Utah
// License      : GPLv2
// Description  : main application window widget

#ifndef THE_MAIN_WINDOW_HXX_
#define THE_MAIN_WINDOW_HXX_

// system includes:
#include <list>

// Qt includes:
#include <QtGui>
#include <QGridLayout>
#include <QMainWindow>
#include <QDirModel>
#include <QTimer>

// local includes:
#include "ui_MainWindow.h"
#include "ui_AboutDialog.h"
#include "PreferencesDialog.h"

// thelib includes:
#include <doc/the_document.hxx>
#include <ui/the_document_ui.hxx>
#include <Qt/the_qt_view.hxx>

// forward declarations:
class the_iv_proc_ui_t;
class the_transaction_t;


//----------------------------------------------------------------
// FileSysModel
// 
class FileSysModel : public QDirModel
{
public:
  void trigger_update(const QModelIndex & index);
};


//----------------------------------------------------------------
// KeyPressObserver
// 
class KeyPressObserver : public QObject
{
  Q_OBJECT
public:
  KeyPressObserver(QObject * parent = NULL);
  
protected:
  // virtual:
  bool eventFilter(QObject * obj, QEvent * event);
};


//----------------------------------------------------------------
// the_main_window_t
// 
class the_main_window_t : public QMainWindow,
			  public Ui::MainWindow,
			  public the_document_ui_t
{
  Q_OBJECT
  
public:
  the_main_window_t();
  ~the_main_window_t();
  
public slots:
  // thumbnail provider thread:
  void thumbnailReady(the_transaction_t *);
  
  // the file menu:
  void file_exit();
  
  // the edit menu:
  void edit_preferences();
  
  // helper:
  void go_here(const QString & path);
  
  // the go menu:
  void go_up();
  void go_back();
  void go_forward();
  void go_to_address();
  void go_to_selected();
  
  // the slide show:
  void slot_prev();
  void slot_next();
  void slot_play();
  void slot_stop();
  
  // the view menu:
  void orientation_restore();
  void full_screen(bool on);
  
  // the help menu:
  void helpAbout();
  
  // the iconList:
  void enterDirectory(QModelIndex);
  
  // helper:
  void fitToImage();
  
public:
  // virtual: ui update mechanism:
  void sync_ui();
  
  // virtual:
  void kill_ui();
  
  // shortcut:
  inline the_document_t * document() const
  { return doc_so().document(); }
  
  // collect the visible views:
  void get_visible_views(std::vector<the_view_t *> & views) const;
  
  // if in single view mode, return the view:
  the_view_t * single_view() const;
  
  // accessor to the iv procedure ui:
  inline the_iv_proc_ui_t * iv_proc_ui() const
  { return iv_proc_ui_; }
  
  // preference accessor:
  inline PreferencesDialog & preferences()
  { return preferences_ui_; }
  
  // check whether the application is in full screen mode:
  bool isFullScreen() const;
  
  // toggle the slideshow:
  void toggleSlideShow();
  
protected:
  // virtual:
  void closeEvent(QCloseEvent * e);
  void dragEnterEvent(QDragEnterEvent * e);
  void dropEvent(QDropEvent * e);
  
  // helper:
  QString enter_dir(const QModelIndex & root);
  
  // the file system model:
  FileSysModel * filesys_;
  
  // the procedure ui:
  the_iv_proc_ui_t * iv_proc_ui_;
  
  // the preferences ui:
  PreferencesDialog preferences_ui_;
  
  // navigation history:
  std::list<QString> history_[2];
  
  // the slideshow timer:
  QTimer timer_;
};


//----------------------------------------------------------------
// AboutDialog
// 
class AboutDialog : public QDialog,
		    public Ui::AboutDialog
{
  Q_OBJECT
  
public:
  AboutDialog(QWidget * parent = 0, Qt::WFlags f = 0):
    QDialog(parent, f),
    Ui::AboutDialog()
  {
    Ui::AboutDialog::setupUi(this);
  }
};


#endif // THE_MAIN_WINDOW_HXX_
