#include "itkImage.h"
#include "../track/util.h"
#include "itkImageRandomNonRepeatingConstIteratorWithIndex.h"
#include "itkStatisticsImageFilter.h"


const     unsigned int    Dimension = 2;
typedef itk::Image< float, Dimension >  ImageType;
typedef itk::ImageRandomNonRepeatingConstIteratorWithIndex < ImageType >
                       ImageRandomRegionIterator;
typedef itk::StatisticsImageFilter< ImageType > StatFilterType;


int main(int argc, char* argv[] )
{
  if( argc < 3 )
    {
    std::cerr << "Missing Parameters " << std::endl;
    std::cerr << "Usage: " << argv[0];
    std::cerr << " inputImage  outputImage"  << std::endl;
    return 1;
    }


  //
  //  read image
  //
  ImageType::Pointer image = ImageType::New();
  image = load<ImageType>(argv[1]);
  image->Update();
  int sx = image->GetLargestPossibleRegion().GetSize()[0];
  int sy = image->GetLargestPossibleRegion().GetSize()[1];

  //
  //  make output image
  //
  ImageType::Pointer new_image = new2d< ImageType >( sx, sy );
  new_image->FillBuffer( 255 );

  //
  //  get max number of black pixels
  //
  StatFilterType::Pointer statFilter_img = StatFilterType::New();
  statFilter_img->SetInput( image );
  statFilter_img->Update();
  double sum = statFilter_img->GetSum();
  int num_white = (int) sum / 255;
  int num_black = abs( num_white - (sx*sy) );
  cout << "max black pixels: " << num_black << endl ;
  cout << "max white pixels: " << num_white << endl ;

  //
  // get non membrane training examples
  //
  int n = 0;
  ImageRandomRegionIterator rand_iter
     ( image, image->GetLargestPossibleRegion() );
  rand_iter.SetNumberOfSamples( num_black ); 
  //train_rand_iter.ReinitializeSeed( time(0) );
  for( rand_iter.GoToBegin(); !rand_iter.IsAtEnd(); ++rand_iter )
  {
     ImageType::IndexType ind = rand_iter.GetIndex();

     if( rand_iter.Get() == 0 )  // if it's a non-membrane pixel
     {
        new_image->SetPixel( ind, 0 );
        n++; 
     }
     if( n > num_white * 2 )
        break;
  }
  new_image->Update();

  cout << n << " black pixels added" << endl ;


  //
  //  save
  //
  save<ImageType>( new_image, argv[2] );


  return 0;
}
