/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: ConnectedComponentImageFilter.cxx,v $
  Language:  C++
  Date:      $Date: 2008/10/23 21:43:31 $
  Version:   $Revision: 1.7 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif
#include "itkConnectedComponentImageFilter.h"
#include "itkScalarConnectedComponentImageFilter.h"
#include "itkRelabelComponentImageFilter.h"
#include "itkBinaryThresholdImageFilter.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkImageRegionIterator.h"
#include "itkSimpleFilterWatcher.h"
#include "itkImageRegionIterator.h"
#include "itkInvertIntensityImageFilter.h"
#include "vnl/vnl_sample.h"
#include "math.h"
#include "../track/util.h"

using namespace std;

//typedef itk::Image<float, 2> InternalImageType;
//typedef   unsigned short  InternalPixelType;
typedef   float  InternalPixelType;
const     unsigned int    Dimension = 2;

typedef itk::Image< InternalPixelType, Dimension >  InternalImageType;
typedef itk::Image<unsigned short,Dimension> OutputImageType;

typedef itk::RGBPixel<unsigned char>   RGBPixelType;
typedef itk::Image<RGBPixelType, Dimension>    RGBImageType;

typedef itk::ImageFileReader< InternalImageType > ReaderType;
typedef itk::ImageFileWriter<  RGBImageType  > WriterType;

typedef itk::BinaryThresholdImageFilter< InternalImageType, InternalImageType > 
                 ThresholdFilterType;
typedef itk::ConnectedComponentImageFilter< InternalImageType, OutputImageType > 
                 ConnectedComponentFilterType;
typedef itk::RelabelComponentImageFilter< OutputImageType, OutputImageType > 
                 RelabelType;
typedef itk::InvertIntensityImageFilter< InternalImageType, InternalImageType >
                           InvertIntensityImageFilter;


double calcDistance(float x, float y, InternalImageType::IndexType I)
{
   double d = 0;
   double x2 = I[0], y2 = I[1];
   
   d = sqrt( pow((x2-x), 2) + pow((y2-y),2) );

   return d;

}

vector<long int> findBlobs(
      std::map<long int, std::vector<InternalImageType::IndexType> > CCM,
      double min_dist)
{
   //  loop over each connected component in CCM
   vector<long int> labeledBlobs;
   std::map<long int, std::vector<InternalImageType::IndexType> >::
               const_iterator iter;
   for(iter = CCM.begin(); iter != CCM.end(); iter++)
   {
      //cout << iter->first << " " << iter->second.size() << endl ;
      vector<InternalImageType::IndexType> V = iter->second;

      //
      // get a bounding box around the component
      //
      double littlex = INT_MAX, littley = INT_MAX,
             bigx = 0, bigy = 0;
      for(unsigned int i = 0; i < V.size(); i++)
      {
         //cout << "pt: " << V[i] << endl ;
         if(V[i][0] < littlex) littlex = V[i][0];
         if(V[i][0] > bigx) bigx = V[i][0];
         if(V[i][1] < littley) littley = V[i][1];
         if(V[i][1] > bigy) bigy = V[i][1];
      }
      //cout << "bb: " << littlex << ", " << littley << " - "
      //   << bigx << ", " << bigy << endl ;

      // calculate the ratio of componet pixels to the size of the
      // bounding box
      double xsize = bigx - littlex;
      double ysize = bigy - littley;
      double ratio = ((double)iter->second.size())/(xsize*ysize);
      if (ratio > min_dist)//.7)
      {
         cout << "to remove " << iter->first << ", " << ratio << endl; 
         //   << iter->second.size() << " " << xsize*ysize << endl ;
         labeledBlobs.push_back(iter->first);
      }

      
      // calculate the center of mass and remove blobs with a certain
      // width
      /*
      double meanx = 0, meany = 0;
      for(unsigned int i = 0; i < V.size(); i++)
      {
         meanx += V[i][0];
         meany += V[i][1];
      }
      double x = meanx / (double)V.size();
      double y = meany / (double)V.size();
      // calculate the largest distance from each point to the center of
      // mass
      double distance = 0;
      double max_distance = 0;
      for(unsigned int i = 0; i < V.size(); i++)
      {
         //cout << x << ", " << y << " " << V[i] << endl ;
         distance = calcDistance(x,y,V[i]); 
         if(distance > max_distance)
            max_distance = distance;
      }

      if(max_distance < min_dist)
      {
         labeledBlobs.push_back(iter->first);
         cout << "remove " << iter->first << endl ;
      }
      cout << "max_distance: " << max_distance << endl ;
      */
   }

   return labeledBlobs;

}


int main(int argc, char* argv[] )
{
  if( argc < 6 )
    {
    std::cerr << "Missing Parameters " << std::endl;
    std::cerr << "Usage: " << argv[0];
    std::cerr << " inputImage labelImage outputImage "
              << "minimum_object_size min_distance" << std::endl;
    return 1;
    }


  
  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();
  

  reader->SetFileName( argv[1] );
  reader->Update();
  InternalImageType::Pointer img = reader->GetOutput();


  //
  //  save a copy of the image
  //
  InternalImageType::Pointer floatImage = new2d< InternalImageType >
     ( img->GetLargestPossibleRegion().GetSize()[0],
       img->GetLargestPossibleRegion().GetSize()[1] );
  /*
  InternalImageType::Pointer floatImage = InternalImageType::New();
  floatImage = copy< InternalImageType >( img );
  floatImage->Update();
  */
  //save< InternalImageType >( floatImage, "floatimage.mha" );

  /*
  InternalPixelType threshold_low, threshold_hi;
  threshold_low = 0;//atoi( argv[3]);
  threshold_hi = .9;//atoi( argv[4]);

  ThresholdFilterType::Pointer threshold = ThresholdFilterType::New();
  threshold->SetInput ( img );
  threshold->SetInsideValue(itk::NumericTraits<InternalPixelType>::One);
  threshold->SetOutsideValue(itk::NumericTraits<InternalPixelType>::Zero);
  threshold->SetLowerThreshold(threshold_low);
  threshold->SetUpperThreshold(threshold_hi);
  threshold->Update();
  save< InternalImageType >( threshold->GetOutput(), "thresh.mha" );
  */
  
  ConnectedComponentFilterType::Pointer ccfilter 
     = ConnectedComponentFilterType::New();
  //ccfilter->SetInput ( threshold->GetOutput() );//reader->GetOutput());
  ccfilter->SetInput ( reader->GetOutput());
  //itk::SimpleFilterWatcher watcher(ccfilter);
  //watcher.QuietOn();

  // Set/Get whether the connected components are defined strictly by
  // face connectivity or by face+edge+vertex connectivity. Default is
  // FullyConnectedOff. For objects that are 1 pixel wide, use
  // FullyConnectedOn.
  /*
  if (argc > 5)
    {
    int fullyConnected = atoi( argv[5] );
    filter->SetFullyConnected( fullyConnected );
    }
  */

  RelabelType::Pointer relabel = RelabelType::New();
  relabel->SetInput( ccfilter->GetOutput() );
  relabel->Update();
  save< OutputImageType >( ccfilter->GetOutput(), argv[2]);
  

  if (argc > 3)
  {
     int minSize = atoi( argv[4] );
     relabel->SetMinimumObjectSize( minSize );
     std::cerr << "minSize: " << minSize << std::endl;
  }
  
  try
  {
     relabel->Update();
  }
  catch( itk::ExceptionObject & excep )
  {
     std::cerr << "Relabel: exception caught !" << std::endl;
     std::cerr << excep << std::endl;
  }

  // Remap the labels to viewable colors
  RGBImageType::Pointer colored = RGBImageType::New();
  colored->SetRegions( ccfilter->GetOutput()->GetBufferedRegion() );
  colored->Allocate();

  unsigned short numObjects = relabel->GetNumberOfObjects();
 
  //
  //  Make the colormap
  //
  std::vector<RGBPixelType> colormap;
  RGBPixelType px;
  colormap.resize( numObjects+1 );
  vnl_sample_reseed( 1031571 );
  for (unsigned short i=0; i < colormap.size(); ++i)
  {
     px.SetRed(
           static_cast<unsigned char>(255*vnl_sample_uniform( 0.3333, 1.0 ) ));
     px.SetGreen(
           static_cast<unsigned char>(255*vnl_sample_uniform( 0.3333, 1.0 ) ));
     px.SetBlue(
           static_cast<unsigned char>(255*vnl_sample_uniform( 0.3333, 1.0 ) ));

     colormap[i] = px;
  }

 
  //
  //  loop over the image and relabel the components, and remove
  //  components from the original image not included
  //

  itk::ImageRegionIterator<OutputImageType>
    it(relabel->GetOutput(), relabel->GetOutput()->GetBufferedRegion());
  itk::ImageRegionIterator<RGBImageType> cit(colored,
                                             colored->GetBufferedRegion());
 
  InternalImageType::IndexType index;
  std::map<long int, std::vector<InternalImageType::IndexType> > connectedComponentMap;
  std::cout << "relabeling" << std::endl ;
  while( !it.IsAtEnd() )
  {
     //
     //  if *it equals 0, it's part of the background, and the connected
     //  component of min size have been removed
     if (it.Get() == 0)
     {
        cit.Set(RGBPixelType(itk::NumericTraits< unsigned char >::Zero ));
        //std5::cout << cit.GetIndex() << std::endl;
        index[0] = cit.GetIndex()[0];
        index[1] = cit.GetIndex()[1];
        floatImage->SetPixel(index, 255);
        //fit.Set(55);
     }
     else
     {
        cit.Set( colormap[it.Get()] );
        //std::cout << cit.GetIndex() << std::endl;
        long int label = it.Get();
        if(connectedComponentMap.find(label) != connectedComponentMap.end())
        {
           InternalImageType::IndexType new_index;
           new_index[0] = cit.GetIndex()[0];
           new_index[1] = cit.GetIndex()[1];
           connectedComponentMap[label].push_back(new_index);
        }
        else
        {
           InternalImageType::IndexType new_index;
           new_index[0] = cit.GetIndex()[0];
           new_index[1] = cit.GetIndex()[1];
           std::vector<InternalImageType::IndexType> indexList;
           indexList.push_back(new_index);
           connectedComponentMap[label] = indexList;
        }
     }
     ++it;
     ++cit;
     //++fit;
  }

  std::cout << "done relabeling" << std::endl ;
  save< RGBImageType >( colored, "colored.png" );
  save< InternalImageType >( floatImage, "colorseg.mha" );


  //
  //  fine blobs
  //
  cout << "finding blobs" << endl ;
  vector<long int> labels_to_remove = 
               findBlobs(connectedComponentMap, atof(argv[5]));

  //
  // removing blobs
  //
  cout << "removing " << labels_to_remove.size() << " blobs" << endl ;
  it.GoToBegin();
  while( !it.IsAtEnd() )
  {
     long int label = it.Get();
     if(label != 0)  // skip the background component
     {
        //cout << "label: " << label << endl;

        if(find(labels_to_remove.begin(), labels_to_remove.end(), label) != 
              labels_to_remove.end())
        {
           index[0] = it.GetIndex()[0];
           index[1] = it.GetIndex()[1];
           //cout << index << endl ;
           floatImage->SetPixel(index, 255);
        }
     }
     ++it;
  }
  floatImage->Update();

  // invert the image
  InvertIntensityImageFilter::Pointer invert = InvertIntensityImageFilter::New();
  invert->SetInput( floatImage );
  invert->SetMaximum( 255 );
  invert->Update();
  floatImage = invert->GetOutput();

cout << "done " << endl ;
  //
  //  save the image
  //
  save< InternalImageType >( floatImage, argv[3] );
 
  //
  //  Write out the data
  //
  try
  {
     writer->SetInput (colored);
     writer->SetFileName( "labeled.tif" );//mha" );
     writer->Update();
     std::cout << "labeled image writen" << std::endl;
  }
  catch( itk::ExceptionObject & excep )
  {
     std::cerr << "Exception caught !" << std::endl;
     std::cerr << excep << std::endl;
  }
  

  

  return 0;
}
