/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: VesselnessMeasureImageFilter.cxx,v $
  Language:  C++
  Date:      $Date: 2005/08/27 01:46:06 $
  Version:   $Revision: 1.2 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#ifdef __BORLANDC__
#define ITK_LEAN_AND_MEAN
#endif

// The example takes an image (say MRA image), computes the vesselness measure
// of the image using the HessianRecursiveGaussianImageFilter and the
// Hessian3DToVesselnessMeasureImageFilter. The goal is to detect bright tubular
// structures in the image.

#include "itkHessian3DToVesselnessMeasureImageFilter.h"
#include "itkHessianRecursiveGaussianImageFilter.h"
#include "itkSymmetricSecondRankTensor.h"
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"

int main( int argc, char *argv[] )
{
  if( argc < 3 )
    {
    std::cerr << "Usage: inputImage outputImage [sigma] " << std::endl;
    }
  
  const     unsigned int        Dimension       = 2;
  typedef   float               InputPixelType;
  
  typedef   itk::Image< InputPixelType, Dimension >   InputImageType;
  
  typedef   itk::HessianRecursiveGaussianImageFilter< 
                            InputImageType >              HessianFilterType;
  typedef   itk::ImageFileReader< InputImageType >  ReaderType;
  typedef   itk::ImageFileWriter< InputImageType > WriterType;
  
  HessianFilterType::Pointer hessianFilter = HessianFilterType::New();


  ReaderType::Pointer   reader = ReaderType::New();
  reader->SetFileName( argv[1] );
  hessianFilter->SetInput( reader->GetOutput() );


  if( argc >= 4 )
  {
     hessianFilter->SetSigma( static_cast< double >(atof(argv[3])) );
  }

  WriterType::Pointer   writer = WriterType::New();
  writer->SetInput( hessianFilter->GetOutput() );
  writer->SetFileName( argv[2] );
  writer->Update();
  return EXIT_SUCCESS;
}

    
