#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#ifdef __BORLANDC__
#define ITK_LEAN_AND_MEAN
#endif

#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkIntensityWindowingImageFilter.h"
#include <itkStatisticsImageFilter.h>
#include "itkImage.h"

using namespace std;

int main( int argc, char ** argv )
{
  // Verify the number of parameters in the command line
  if( argc < 3 )
    {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << " inputImageFile  outputImageFile " << std::endl;
    return EXIT_FAILURE;
    }

  typedef float               InputPixelType;
  //typedef unsigned char       InputPixelType;
  //typedef unsigned short               InputPixelType;
  typedef float               OutputPixelType;
  //typedef unsigned char       OutputPixelType;
  //const   unsigned int        Dimension = 3;
  const   unsigned int        Dimension = 2;

  typedef itk::Image< InputPixelType,  Dimension >    InputImageType;
  typedef itk::Image< OutputPixelType, Dimension >    OutputImageType;
  typedef itk::ImageFileReader< InputImageType  >  ReaderType;
  typedef itk::ImageFileWriter< OutputImageType >  WriterType;
/*
  typedef itk::RescaleIntensityImageFilter< 
                                  InputImageType, 
                                  OutputImageType >    FilterType;
  FilterType::Pointer filter = FilterType::New();
  filter->SetOutputMinimum(   0 );
  filter->SetOutputMaximum( 255 );
  filter->SetInput( reader->GetOutput() );
*/
  WriterType::Pointer writer = WriterType::New();

  const char * inputFilename  = argv[1];
  const char * outputFilename = argv[2];

  ReaderType::Pointer reader = ReaderType::New();
  reader->SetFileName( inputFilename  );
  writer->SetFileName( outputFilename );

  typedef itk::StatisticsImageFilter< OutputImageType > StatFilterType;
  StatFilterType::Pointer statFilter = StatFilterType::New();
  statFilter->SetInput( reader->GetOutput() );
  statFilter->Update();
  float var = statFilter->GetVariance();
  float mean = statFilter->GetMean();
  cout << "var: " << var << ", mean: " << mean << endl;

  typedef itk::IntensityWindowingImageFilter<
                                   OutputImageType,
                                   OutputImageType >
                                   IntensityFilterType;

  IntensityFilterType::Pointer intensityWindowing =
     IntensityFilterType::New();

  intensityWindowing->SetWindowMinimum( mean - (3*sqrt(var)) ); //0 ) ; 
  intensityWindowing->SetWindowMaximum( mean + (3*sqrt(var)) );//80 );

  intensityWindowing->SetOutputMinimum(   0.0 );
  intensityWindowing->SetOutputMaximum( 255.0 ); // floats but in the range of chars.

  intensityWindowing->SetInput( reader->GetOutput() );
  //intensityWindowing->SetInput( filter->GetOutput() );


  writer->SetInput( intensityWindowing->GetOutput() );



  try 
    { 
    writer->Update(); 
    } 
  catch( itk::ExceptionObject & err ) 
    { 
    std::cerr << "ExceptionObject caught !" << std::endl; 
    std::cerr << err << std::endl; 
    return EXIT_FAILURE;
    } 
  // Software Guide : EndCodeSnippet


  return EXIT_SUCCESS;
}




