#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#ifdef __BORLANDC__
#define ITK_LEAN_AND_MEAN
#endif

#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkRescaleIntensityImageFilter.h"
#include "itkIntensityWindowingImageFilter.h"
#include "itkImage.h"


int main( int argc, char ** argv )
{
  // Verify the number of parameters in the command line
  if( argc < 3 )
    {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << " inputImageFile  outputImageFile " << std::endl;
    return EXIT_FAILURE;
    }

  typedef float               InputPixelType;
  //typedef unsigned char       InputPixelType;
  //typedef unsigned short               InputPixelType;
  //typedef short               InputPixelType;
  //typedef unsigned int             InputPixelType;

  //typedef float               OutputPixelType;
  typedef unsigned char       OutputPixelType;
  const   unsigned int        Dimension = 3;
  //const   unsigned int        Dimension = 2;

  typedef itk::Image< InputPixelType,  Dimension >    InputImageType;
  typedef itk::Image< OutputPixelType, Dimension >    OutputImageType;
  typedef itk::ImageFileReader< InputImageType  >  ReaderType;
  typedef itk::ImageFileWriter< OutputImageType >  WriterType;

  typedef itk::RescaleIntensityImageFilter< 
                                  InputImageType, 
                                  OutputImageType >    FilterType;
  FilterType::Pointer filter = FilterType::New();
  filter->SetOutputMinimum(   0 );
  filter->SetOutputMaximum( 255 );
  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();
  filter->SetInput( reader->GetOutput() );

  /*
  typedef itk::IntensityWindowingImageFilter<
                                   OutputImageType,
                                   OutputImageType >
                                   IntensityFilterType;

  IntensityFilterType::Pointer intensityWindowing =
     IntensityFilterType::New();

  intensityWindowing->SetWindowMinimum( 0 ) ; 
  intensityWindowing->SetWindowMaximum( 80 );

  intensityWindowing->SetOutputMinimum(   0.0 );
  intensityWindowing->SetOutputMaximum( 255.0 ); // floats but in the range of chars.

  intensityWindowing->SetInput( filter->GetOutput() );
  //intensityWindowing->SetInput( reader->GetOutput() );
  */



  writer->SetInput( filter->GetOutput() );
  //writer->SetInput( reader->GetOutput(reader->GetOutput()) );
  //writer->SetInput( intensityWindowing->GetOutput() );

  const char * inputFilename  = argv[1];
  const char * outputFilename = argv[2];


  reader->SetFileName( inputFilename  );
  writer->SetFileName( outputFilename );

  try 
    { 
    writer->Update(); 
    } 
  catch( itk::ExceptionObject & err ) 
    { 
    std::cerr << "ExceptionObject caught !" << std::endl; 
    std::cerr << err << std::endl; 
    return EXIT_FAILURE;
    } 
  // Software Guide : EndCodeSnippet


  return EXIT_SUCCESS;
}




