#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#ifdef __BORLANDC__
#define ITK_LEAN_AND_MEAN
#endif

// \item First, the user can define a single threshold.  Any pixels with values
// below this threshold will be replaced by a user defined value, called here the
// \code{OutsideValue}.  Pixels with values above the threshold remain
// unchanged. This type of thresholding is illustrated in
// Figure~\ref{fig:ThresholdTransferFunctionBelow}.
//
// \item Second, the user can define a particular threshold such that all the
// pixels with values above the threshold will be replaced by the
// \code{OutsideValue}.  Pixels with values below the threshold remain
// unchanged. This is illustrated in Figure~\ref{fig:ThresholdTransferFunctionAbove}.
//
// \item Third, the user can provide two thresholds. All the pixels with
// intensity values inside the range defined by the two thresholds will remain
// unchanged. Pixels with values outside this range will be assigned to the

#include "itkThresholdImageFilter.h"
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"

int main( int argc, char * argv[] )
{

  if( argc < 2 )
    {
    std::cerr << "Usage: " << argv[0] << " inputImageFile ";
    std::cerr << " outputImageFile" << std::endl;  
    return EXIT_FAILURE;
    }
  
  typedef  float  PixelType;
  //typedef  unsigned char  PixelType;
  typedef itk::Image< PixelType,  2 >   ImageType;
  typedef itk::ThresholdImageFilter< ImageType >  FilterType;
  typedef itk::ImageFileReader< ImageType >  ReaderType;
  typedef itk::ImageFileWriter< ImageType >  WriterType;

  ReaderType::Pointer reader = ReaderType::New();
  FilterType::Pointer filter = FilterType::New();

  WriterType::Pointer writer = WriterType::New();
  writer->SetInput( filter->GetOutput() );
  reader->SetFileName( argv[1] );



  filter->SetInput( reader->GetOutput() );
  //  The method \code{SetOutsideValue()} defines the intensity value to be
  //  assigned to those pixels whose intensities are outside the range defined
  //  by the lower and upper thresholds. 
  filter->SetOutsideValue( 0 );

  filter->ThresholdBelow( 30 );
  //  The method \code{ThresholdBelow()} defines the intensity value below
  //  which pixels of the input image will be changed to the \code{OutsideValue}.

  filter->Update();


  writer->SetFileName( argv[2] );
  writer->Update();

/*
  //  Software Guide : BeginLatex
  //  
  //  The output of this example is shown in
  //  Figure~\ref{fig:ThresholdTransferFunctionBelow}.  The second operating mode of
  //  the filter is now enabled by calling the method
  //  \code{ThresholdAbove()}.
  //
  //  Software Guide : EndLatex 

  // Software Guide : BeginCodeSnippet
  filter->ThresholdAbove( 180 );
  filter->Update();
  // Software Guide : EndCodeSnippet

  writer->SetFileName( argv[3] );
  writer->Update();


  filter->ThresholdOutside( 170,190 );
  //  Updating the filter with this new setting produces the output shown in
  //  Figure~\ref{fig:ThresholdTransferFunctionAbove}.  The third operating
  //  mode of the filter is enabled by calling \code{ThresholdOutside()}.
  filter->Update();


  writer->SetFileName( argv[4] );
  writer->Update();

*/
  return EXIT_SUCCESS;
}

