#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif

#ifdef __BORLANDC__
#define ITK_LEAN_AND_MEAN
#endif


#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkBinaryErodeImageFilter.h"
#include "itkBinaryDilateImageFilter.h"
#include "itkBinaryBallStructuringElement.h" 
#include "itkBinaryThresholdImageFilter.h"

const unsigned int Dimension = 2;

typedef float   InputPixelType;
typedef float   OutputPixelType;
typedef itk::Image< InputPixelType,  Dimension >   InputImageType;
typedef itk::Image< OutputPixelType, Dimension >   OutputImageType;
typedef itk::ImageFileReader< InputImageType  >  ReaderType;
typedef itk::ImageFileWriter< OutputImageType >  WriterType;
typedef itk::BinaryThresholdImageFilter< InputImageType, InputImageType >  ThresholdFilterType;

typedef itk::BinaryBallStructuringElement
     < InputPixelType, Dimension  > StructuringElementType;
typedef itk::BinaryErodeImageFilter
     < InputImageType, OutputImageType, StructuringElementType >  
            ErodeFilterType;
typedef itk::BinaryDilateImageFilter
      < InputImageType, OutputImageType, StructuringElementType >  
             DilateFilterType;

int main( int argc, char * argv[] )
{
  if( argc < 5 )
    {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  ";
    std::cerr << " outputImageFileDilation  outputImageFileErosion ";
    std::cerr << " kernelSize";
    std::cerr << std::endl;
    //std::cerr << " lowerThreshold upperThreshold " << std::endl;
    return EXIT_FAILURE;
    }


  // Creation of Reader and Writer filters
  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writerDilation = WriterType::New();
  WriterType::Pointer writerErosion  = WriterType::New();


  DilateFilterType::Pointer binaryDilate = DilateFilterType::New();
  DilateFilterType::Pointer binaryDilate1 = DilateFilterType::New();
  ErodeFilterType::Pointer  binaryErode  = ErodeFilterType::New();
  StructuringElementType  structuringElement;

  //structuringElement.SetRadius( 1 );  // 3x3 structuring element
  structuringElement.SetRadius( atoi(argv[4]) );
  structuringElement.CreateStructuringElement();

  binaryErode->SetKernel(  structuringElement );
  binaryDilate->SetKernel( structuringElement );
  binaryDilate1->SetKernel( structuringElement );


  reader->SetFileName( argv[1] );
 
  

  InputPixelType background =   0;
  InputPixelType foreground = 255;
  //ThresholdFilterType::Pointer thresholder = ThresholdFilterType::New();
  //const InputPixelType lowerThreshold = atoi( argv[4] );
  //const InputPixelType upperThreshold = atoi( argv[5] );
  //thresholder->SetInput( reader->GetOutput() );
  //thresholder->SetOutsideValue( background );
  //thresholder->SetInsideValue(  foreground );
  //thresholder->SetLowerThreshold( lowerThreshold );
  //thresholder->SetUpperThreshold( upperThreshold );

  binaryDilate->SetInput( reader->GetOutput() );
  //binaryDilate->SetDilateValue( foreground );
  binaryDilate->SetDilateValue( background );
  binaryDilate->Update();
  //InputImageType::Pointer dilated_image = binaryDilate->GetOutput();
  writerDilation->SetFileName( argv[2] );
  writerDilation->SetInput( binaryDilate->GetOutput() );
  writerDilation->Update();

  //binaryDilate->SetInput( reader->GetOutput() );
  binaryDilate1->SetInput( binaryDilate->GetOutput() );
  binaryDilate1->SetDilateValue( foreground );
  binaryDilate1->Update();
  writerDilation->SetFileName( argv[3] );
  writerDilation->SetInput( binaryDilate1->GetOutput() );
  writerDilation->Update();

  /*
  erode seems to be broken?  I can't make it work!
  //binaryErode->SetInput( reader->GetOutput() );
  binaryErode->SetInput( binaryDilate->GetOutput() );
  //binaryErode->SetErodeValue( background );
  binaryErode->SetErodeValue( foreground );
  binaryErode->Update();
  writerErosion->SetFileName(  argv[3] );
  writerErosion->SetInput( binaryErode->GetOutput() );
  writerErosion->Update();
  */


  return EXIT_SUCCESS;
}

