/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: RemoveBlobs.cxx,v $
  Language:  C++
  Date:      $Date: 2007/08/05 22:06:25 $
  Version:   $Revision: 1.1 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkImageRegionIterator.h"
#include "vnl/vnl_sample.h"

int main(int argc, char* argv[] )
{
  if( argc < 3 )
    {
    std::cerr << "Missing Parameters " << std::endl;
    std::cerr << "Usage: " << argv[0];
    std::cerr << " inputImage  outputImage [minimum_object_size]" << std::endl;
    return 1;
    }

  const     unsigned int    Dimension = 2;
  
  typedef itk::Image< float, Dimension >  ImageType;
  typedef itk::ImageFileReader< ImageType > ReaderType;
  typedef itk::ImageFileWriter< ImageType > WriterType;

  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();
  
  reader->SetFileName( argv[1] );
 
  ImageType::Pointer image = ImageType::New();
  ImageType::Pointer image_copy = ImageType::New();
  image = reader->GetOutput(); image->Update();
  image_copy = reader->GetOutput(); image_copy->Update();

  ImageType::RegionType::SizeType sz =
     image->GetLargestPossibleRegion().GetSize();

  int buf = atoi(argv[3]);
  ImageType::IndexType pixel;

//  std::cout << "indexing..." << sz << "  buf: " << buf << std::endl ;

  // calculate the ratio of black pixels to white pixels
  for(int y = buf; y < sz[1]-buf; y++)
  {
     for(int x = buf; x < sz[0]-buf; x++)
     {
//        std::cout << "x,y: " << x << "," << y << std::endl;
        //
        //  loop over a small box around this pixel
        //
        float black_pixels = 0.01;
        float white_pixels = 0.01;
        for(int i = y; i < y+buf; i++)
        {
           for(int j = x; j < x+buf; j++)
           {
 //             std::cout << "i,j: " << i << "," << j << std::endl;
              pixel[0] = j; pixel[1] = i;
//              std::cout << pixel ;//<< " " << val << std::endl ;
              float val = image->GetPixel(pixel);
//              std::cout << " " << val << std::endl;
              if( val == 0 )
                 black_pixels++;
              else
                 white_pixels++;
           }
        }

//        std::cout << "ratio: " << black_pixels << "/" << white_pixels
//           << " " << black_pixels / white_pixels << std::endl ;
        if (black_pixels / white_pixels > 4)
        {
           for(int i = y; i < y+buf; i++)
           {
              for(int j = x; j < x+buf; j++)
              {
                 pixel[0] = j; pixel[1] = i;
                 float val = image->GetPixel(pixel);
                 if( val == 0 )
                    image_copy->SetPixel(pixel, 100);
              }
           }
        }
     }
  }
  image_copy->Update();
  
  
  typedef itk::ImageFileWriter<  ImageType  > WriterType;
  WriterType::Pointer floatWriter = WriterType::New();
  try
    {
    floatWriter->SetInput (image_copy);
    floatWriter->SetFileName( argv[2] );
    floatWriter->Update();
    std::cout << argv[2] << " writen" << std::endl;
    }
  catch( itk::ExceptionObject & excep )
    {
    std::cerr << "Exception caught !" << std::endl;
    std::cerr << excep << std::endl;
    }
  

  return 0;
}
