#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkImageRegionIterator.h"
#include "itkRegionOfInterestImageFilter.h"
#include "itkExtractImageFilter.h"
#include "itkMultiplyImageFilter.h"
#include "itkDivideImageFilter.h"
#include <itkStatisticsImageFilter.h>
#include "vnl/vnl_sample.h"
#include "../track/util.h"

const     unsigned int    Dimension = 2;
typedef itk::Image< float, Dimension >  ImageType;
typedef itk::ImageRegionIterator< ImageType > ImageIterator;
typedef itk::RegionOfInterestImageFilter
   <ImageType, ImageType> ExtractRegionFilterType;

typedef itk::MultiplyImageFilter
              < ImageType, ImageType, ImageType > 
              MultiplyImageFilter;
typedef itk::DivideImageFilter
              < ImageType, ImageType, ImageType > 
              DivideImageFilter;
typedef itk::StatisticsImageFilter< ImageType > StatFilterType;



int main(int argc, char* argv[] )
{
  if( argc < 3 )
    {
    std::cerr << "Missing Parameters " << std::endl;
    std::cerr << "Usage: " << argv[0];
    std::cerr << " inputImage  outputImage [mask size]" << std::endl;
    return 1;
    }

  char* in = argv[1];
  char* out_image = argv[2];
  double mask_size = atof(argv[3]);
  double r = mask_size;
  ImageType::Pointer in_image = load< ImageType >( in );
  int sizex =  in_image->GetLargestPossibleRegion().GetSize()[0];
  int sizey =  in_image->GetLargestPossibleRegion().GetSize()[1];

  ImageIterator img_iter( in_image, in_image->GetLargestPossibleRegion());
  img_iter.GoToBegin();
  while( !img_iter.IsAtEnd() )
  {
     ImageType::IndexType ind = img_iter.GetIndex();
     int x = ind[0];
     int y = ind[1];
     if( (x > r) && (y > r) && (x+r < sizex) && (y+r < sizey) )
     {
        //cout << ind << " " ;
        ImageType::IndexType start;
        start[0] = (long int)x - r;
        start[1] = (long int)y - r;
        ImageType::SizeType size;
        size[0] = (int)2*r-1;
        size[1] = (int)2*r-1;
        //cout << "region image size: " << size[0] << ", " << size[1] << endl ;
        ImageType::RegionType region;
        region.SetSize(size);
        region.SetIndex(start);
        ExtractRegionFilterType::Pointer regionFilter =
           ExtractRegionFilterType::New();
        regionFilter->SetRegionOfInterest( region );
        regionFilter->SetInput( in_image );
        ImageType::Pointer region_image = regionFilter->GetOutput();

        StatFilterType::Pointer statFilter = StatFilterType::New();
        statFilter->SetInput( region_image );
        statFilter->Update();
        float mean = statFilter->GetMean();

        // get this pixel
        float pixel_val = img_iter.Get();

        if( mean < pixel_val )
           new_iter.Set( mean );

     }
     else
        new_iter.Set( 1 );

     ++img_iter;
     ++new_iter;
  }


  save< ImageType >( new_image, out_image );

  return 0;
}

