
#include "itkImage.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkHistogramMatchingImageFilter.h"

#include "itkSigmoidImageFilter.h"


int main( int argc, char * argv[] )
{
  if( argc < 8 )
    {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile   outputImageFile";
    std::cerr << " histogram_match_outputImageFile";
    std::cerr << " OutputMin OutputMax SigmoidAlpha SigmoidBeta" << std::endl;
    return EXIT_FAILURE;
    }

  typedef   float  InputPixelType;
  typedef   float  OutputPixelType;

  typedef itk::Image< InputPixelType,  2 >   InputImageType;
  typedef itk::Image< OutputPixelType, 2 >   OutputImageType;
  // Software Guide : EndCodeSnippet

  typedef itk::ImageFileReader< InputImageType  >  ReaderType;
  typedef itk::ImageFileWriter< OutputImageType >  WriterType;

  ReaderType::Pointer reader = ReaderType::New();
  WriterType::Pointer writer = WriterType::New();
  WriterType::Pointer writer2 = WriterType::New();

  reader->SetFileName( argv[1] );
  writer->SetFileName( argv[2] );
  writer2->SetFileName( argv[3] );


  typedef itk::SigmoidImageFilter<
               InputImageType, OutputImageType >  SigmoidFilterType;
  SigmoidFilterType::Pointer sigmoidFilter = SigmoidFilterType::New();

  const OutputPixelType outputMinimum = atoi( argv[4] );
  const OutputPixelType outputMaximum = atoi( argv[5] );

  // Software Guide : BeginCodeSnippet
  sigmoidFilter->SetOutputMinimum(   outputMinimum  );
  sigmoidFilter->SetOutputMaximum(   outputMaximum  );

  const double  alpha = atof( argv[6] );
  const double  beta  = atof( argv[7] );

  // Software Guide : BeginCodeSnippet
  sigmoidFilter->SetAlpha(  alpha  );
  sigmoidFilter->SetBeta(   beta   );
  sigmoidFilter->SetInput( reader->GetOutput() );
  writer->SetInput( sigmoidFilter->GetOutput() );
  writer->Update();

  typedef itk::HistogramMatchingImageFilter
     < OutputImageType, OutputImageType > HistogramMatchingImageFilter;
  HistogramMatchingImageFilter::Pointer match_filter =
     HistogramMatchingImageFilter::New();
  match_filter->SetReferenceImage( reader->GetOutput() );
  match_filter->SetInput( sigmoidFilter->GetOutput() );
  writer2->SetInput( match_filter->GetOutput() );
  writer2->Update();
  
  

  return EXIT_SUCCESS;
}

