/*=========================================================================

  Program:   Insight Segmentation & Registration Toolkit
  Module:    $RCSfile: SubtractImages.cxx,v $
  Language:  C++
  Date:      $Date: 2008/08/13 16:24:29 $
  Version:   $Revision: 1.2 $

  Copyright (c) Insight Software Consortium. All rights reserved.
  See ITKCopyright.txt or http://www.itk.org/HTML/Copyright.htm for details.

     This software is distributed WITHOUT ANY WARRANTY; without even 
     the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
     PURPOSE.  See the above copyright notices for more information.

=========================================================================*/
#if defined(_MSC_VER)
#pragma warning ( disable : 4786 )
#endif
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkImageRegionIterator.h"
#include "itkSubtractImageFilter.h"
#include "itkImageRegionIterator.h"
#include "vnl/vnl_sample.h"
#include "../track/util.h"


const     unsigned int    Dimension = 2;
typedef itk::Image< float, Dimension >  ImageType;
typedef itk::SubtractImageFilter
           < ImageType, ImageType, ImageType > SubtractImageFilter;
typedef itk::ImageRegionIterator< ImageType > ImageRegionIterator;


int main(int argc, char* argv[] )
{
  if( argc < 3 )
    {
    std::cerr << "Missing Parameters " << std::endl;
    std::cerr << "Usage: " << argv[0];
    std::cerr << " inputImage1 inputImage2 outputImage"  << std::endl;
    return 1;
    }


  //
  //  read image
  //
  ImageType::Pointer image1 = ImageType::New();
  image1 = load<ImageType>(argv[1]);
  image1->Update();
  ImageType::Pointer image2 = ImageType::New();
  image2 = load<ImageType>(argv[2]);
  image2->Update();
  ImageType::Pointer image3 = new2d< ImageType >
     ( image1->GetLargestPossibleRegion().GetSize()[0],
       image1->GetLargestPossibleRegion().GetSize()[1] );

  SubtractImageFilter::Pointer sub_image = SubtractImageFilter::New();
  sub_image->SetInput1( image1 );
  sub_image->SetInput2( image2 );
  sub_image->Update();
  image3 = sub_image->GetOutput();

  //
  //  use largest of either value... if 
  //
  /*
  ImageRegionIterator img1_iter( image1, image1->GetLargestPossibleRegion() ); 
  ImageRegionIterator img2_iter( image2, image2->GetLargestPossibleRegion() ); 
  ImageRegionIterator img3_iter( image3, image3->GetLargestPossibleRegion() ); 
  img1_iter.GoToBegin();
  img2_iter.GoToBegin();
  img3_iter.GoToBegin();
  while( !img1_iter.IsAtEnd() )
  {
     if( img1_iter.Get() > img2_iter.Get() )
        img3_iter.Set( img1_iter.Get() );
     else
        img3_iter.Set( 0 );//img2_iter.Get() - img1_iter.Get() );
     ++img1_iter;
     ++img2_iter;
     ++img3_iter;
  }
  */

  //
  //  save
  //
  save<ImageType>(image3, argv[3]);


  return 0;
}
