#include "../track/util.h"
#include "itkImage.h"
#include "itkBinaryThresholdImageFilter.h"
#include "itkBinaryThinningImageFilter.h"
#include "itkBinaryPruningImageFilter.h"

const unsigned int Dimension = 2;

typedef float PixelType;
typedef itk::Image< PixelType, Dimension > ImageType;
typedef itk::BinaryThinningImageFilter< ImageType, ImageType >
                 BinaryThinningImageFilter;
typedef itk::BinaryPruningImageFilter< ImageType, ImageType >
                 BinaryPruningImageFilter;

int main( int argc, char * argv[] )
{
  if( argc < 3 )
    {
    std::cerr << "Usage: " << std::endl;
    std::cerr << argv[0] << "  inputImageFile  ";
    std::cerr << " outputImageFile ";
    std::cerr << std::endl;
    return EXIT_FAILURE;
    }

  // read in the image
  ImageType::Pointer in_image = load< ImageType >( argv[1] );

  // invert it, so membranes are white, not black
  float min, max;
  image_min_max<ImageType>(in_image, min, max);
  remap_min_max_inplace<ImageType>(in_image, max, min);
  in_image = rescale< ImageType >( in_image, 0, 1 );
  //save< ImageType >( in_image, "invert.mha" );

  /*
  BinaryPruningImageFilter::Pointer prune =
     BinaryPruningImageFilter::New();
  prune->SetInput( in_image );
  prune->Update();
  in_image = prune->GetOutput();
  */

  // run the filter
  BinaryThinningImageFilter::Pointer thin =
     BinaryThinningImageFilter::New();
  thin->SetInput( in_image );
  thin->Update();
  in_image = thin->GetOutput();  // 0 and 1 output

  // invert it again
  remap_min_max_inplace<ImageType>(in_image, 1, 0);

  save< ImageType >( in_image, argv[2] );

  return EXIT_SUCCESS;
}

