#include <iostream>
#include "itkVectorGradientAnisotropicDiffusionImageFilter.h"
#include "itkGradientAnisotropicDiffusionImageFilter.h"
#include "itkVectorGradientMagnitudeImageFilter.h"
#include "itkWatershedImageFilter.h"
#include "itkImageFileReader.h"
#include "itkImageFileWriter.h"
#include "itkVectorCastImageFilter.h"
#include "itkCastImageFilter.h"
#include "itkUnaryFunctorImageFilter.h"
#include "itkScalarToRGBPixelFunctor.h"
#include "util.h"

//
//  annoying typedefs
//
typedef itk::RGBPixel<unsigned char>   RGBPixelType;
typedef itk::Image<RGBPixelType, 2>    RGBImageType;
typedef itk::Vector<float, 3>          VectorPixelType;
typedef itk::Image<VectorPixelType, 2> VectorImageType;
typedef itk::Image<unsigned long, 2>   LabeledImageType;
typedef itk::Image<float, 2>           ScalarImageType;

//typedef itk::ImageFileReader<RGBImageType> FileReaderType;
typedef itk::ImageFileReader<ScalarImageType> FileReaderType;
typedef itk::VectorCastImageFilter<RGBImageType, VectorImageType> VectorCastFilterType;
typedef itk::CastImageFilter<LabeledImageType, ScalarImageType> CastFilter;

typedef itk::VectorGradientAnisotropicDiffusionImageFilter<VectorImageType,
        VectorImageType>  VectorDiffusionFilterType;
typedef itk::GradientAnisotropicDiffusionImageFilter
       <ScalarImageType, ScalarImageType>  DiffusionFilterType;
typedef itk::VectorGradientMagnitudeImageFilter<VectorImageType> GradientMagnitudeFilterType; 
typedef itk::WatershedImageFilter<ScalarImageType> WatershedFilterType;
typedef itk::ImageFileWriter<RGBImageType> FileWriterType;
typedef itk::Functor::ScalarToRGBPixelFunctor<unsigned long> ColorMapFunctorType;
typedef itk::UnaryFunctorImageFilter<LabeledImageType,
        RGBImageType, ColorMapFunctorType> ColorMapFilterType;



int main( int argc, char *argv[] )
{
  if (argc < 8 )
    {
    std::cerr << "Missing Parameters " << std::endl;
    std::cerr << "Usage: " << argv[0];
    std::cerr << " inputImage outputImage.png outputImage.mha ";
    std::cerr << "conductanceTerm diffusionIterations ";
    std::cerr << "lowerThreshold outputScaleLevel gradientMode " << std::endl;
    return 1;
    }

  //
  //  Read in the file
  //
  FileReaderType::Pointer reader = FileReaderType::New();
  reader->SetFileName(argv[1]);
 
  //
  //  make the caster filter
  //
  //VectorCastFilterType::Pointer caster = VectorCastFilterType::New();
  
  //
  //  diffusion filter
  //
  DiffusionFilterType::Pointer diffusion = DiffusionFilterType::New();
  diffusion->SetNumberOfIterations( atoi(argv[5]) );
  diffusion->SetConductanceParameter( atof(argv[4]) );
  diffusion->SetTimeStep(0.125);

  //
  //  gradient magnitude filter
  //
  GradientMagnitudeFilterType::Pointer 
    gradient = GradientMagnitudeFilterType::New();
  gradient->SetUsePrincipleComponents(atoi(argv[7]));

  //
  //  watershed filter
  //
  WatershedFilterType::Pointer watershed = WatershedFilterType::New();
  watershed->SetLevel( atof(argv[7]) );
  watershed->SetThreshold( atof(argv[6]) );

  //
  //  color mapper
  //
  ColorMapFilterType::Pointer colormapper = ColorMapFilterType::New();
  // Software Guide : EndCodeSnippet


  //
  //  execute the pipeline
  //
  /*
  caster->SetInput(reader->GetOutput());
  diffusion->SetInput(caster->GetOutput());
  gradient->SetInput(diffusion->GetOutput());
  */
  //diffusion->SetInput(reader->GetOutput());
  //diffusion->Update();
  //save<ScalarImageType>(diffusion->GetOutput(), "diff.mha");
  //watershed->SetInput(diffusion->GetOutput());
  watershed->SetInput(reader->GetOutput());
  colormapper->SetInput(watershed->GetOutput());
  // Software Guide : EndCodeSnippet

  //
  //  write the output
  //
  CastFilter::Pointer cast_image = CastFilter::New();
  cast_image->SetInput( watershed->GetOutput() );
  cast_image->Update();
  save< ScalarImageType >( cast_image->GetOutput(), argv[3] );

  save< RGBImageType >( colormapper->GetOutput(), argv[2] );
    
  return 0;
}

