// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : Fl_Knot_Vector.H
// Author       : Pavel A. Koshevoy
// Created      : 2004/11/24 16:23
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : FLTK widget for editing a bspline curve knot vector.

#ifndef Fl_Knot_Vector_H
#define Fl_Knot_Vector_H

// system includes:
#include <vector>
#include <list>

// FLTK includes:
#include <FL/Fl_Widget.H>

// local includes:
#include "math/v3x1p3x1.hxx"


//----------------------------------------------------------------
// duck_t
// 
class duck_t
{
public:
  duck_t(int X = -1,
	 int Y = -1,
	 unsigned int I = UINT_MAX):
    x(X),
    y(Y),
    i(I)
  {}
  
  inline bool operator == (const duck_t & d) const
  { return i == d.i && x == d.x && y == d.y; }
  
  int x;
  int y;
  unsigned int i;
};

//----------------------------------------------------------------
// Fl_Knot_Vector
// 
class FL_EXPORT Fl_Knot_Vector : public Fl_Widget
{
public:
  Fl_Knot_Vector(int x, int y, int w, int h, const char * label = 0);
  
  // virtual:
  int handle(int event);
  
  // accessors:
  inline const std::vector<float> & value() const
  { return knots_; }
  
  inline void value(const std::vector<float> & knots)
  {
    knots_ = knots;
    redraw();
  }
  
protected:
  // virtual:
  void draw();
  
private:
  inline void set_anchor()
  { anchor_ = ducks_; }
  
  std::vector<float> knots_;
  
  std::list<unsigned int> multiplicity_;
  std::list<float> parameter_;
  std::list<duck_t> ducks_;
  std::list<duck_t> anchor_;
  
  const duck_t * active_duck_;
};


#endif // Fl_Knot_Vector_H
