// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_qt_view.hxx
// Author       : Pavel A. Koshevoy
// Created      : Sun Jun 23 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : A Qt based port of the OpenGL view widget.

#ifndef THE_QT_VIEW_HXX_
#define THE_QT_VIEW_HXX_

// local includes:
#include "opengl/the_view.hxx"

// QT includes:
#include <QtOpenGL>
#include <QGLWidget>
#include <QEvent>
#include <QMouseEvent>
#include <QTabletEvent>
#include <QKeyEvent>


//----------------------------------------------------------------
// the_qt_view_t
// 
class the_qt_view_t : public QGLWidget,
		      public the_view_t	  
{
  Q_OBJECT
  
public:
  // Constructor for the main view:
  the_qt_view_t(QWidget * parent,
		const char * name,
		QGLWidget * shared = NULL,
		const the_view_mgr_orientation_t & o = THE_ISOMETRIC_VIEW_E);
  
  // virtual:
  bool is_hidden()
  { return QGLWidget::isHidden(); }
  
  // virtual:
  void set_focus()
  { QGLWidget::setFocus(); }
  
  // virtual:
  void refresh()
  {
    QGLWidget::updateGL();
    QGLWidget::doneCurrent();
  }
  
  // virtual:
  bool gl_context_is_valid() const
  { return QGLWidget::isValid(); }
  
  // virtual:
  void gl_make_current()
  { QGLWidget::makeCurrent(); }
  
  // virtual:
  void gl_done_current()
  { QGLWidget::doneCurrent(); }
  
  // virtual:
  void change_cursor(const the_cursor_id_t & cursor_id);
  
protected:
  // virtual: QT/OpenGL stuff:
  void initializeGL();
  void resizeGL(int width, int height);
  void paintGL();
  
  
  // virtual: Mouse/Keyboard/View events will be passed on to another handler:
  void showEvent(QShowEvent * e);
  
  void mousePressEvent(QMouseEvent * e);
  void mouseReleaseEvent(QMouseEvent * e);
  void mouseDoubleClickEvent(QMouseEvent * e);
  void mouseMoveEvent(QMouseEvent * e);
  void wheelEvent(QWheelEvent * e);
  
  void keyPressEvent(QKeyEvent * e);
  void keyReleaseEvent(QKeyEvent * e);
  
  void tabletEvent(QTabletEvent * e);
  
protected:
  // disable default constructor:
  the_qt_view_t();
};


#endif // THE_QT_VIEW_HXX_
