// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_graph.hxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Jul  1 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Wrapper class for a dependency-sorted list of primitives:

#ifndef THE_GRAPH_HXX_
#define THE_GRAPH_HXX_

// system includes:
#include <list>

// local includes:
#include "utils/the_unique_list.hxx"
#include "utils/the_indentation.hxx"

// forward declarations:
class the_registry_t;
class the_pick_list_t;


//----------------------------------------------------------------
// the_graph_t
// 
class the_graph_t : public std::list<unsigned int>
{
public:
  the_graph_t();
  
  the_graph_t(const the_registry_t * registry);
  
  the_graph_t(const the_registry_t * registry,
	      const unsigned int & root);
  
  the_graph_t(const the_registry_t * registry,
	      const std::list<unsigned int> & roots);
  
#ifndef NOUI
  the_graph_t(const the_registry_t * registry,
	      const the_pick_list_t & roots);
#endif // NOUI
  
  inline void set_context(const the_registry_t * registry)
  {
    registry_ = registry;
    dependency_sort();
  }
  
  void set_graph(const the_registry_t * registry,
		 const the::unique_list<unsigned int> & graph);
  
  void set_roots(const the_registry_t * registry,
		 const std::list<unsigned int> & roots);
  
  void dependency_sort();
  
  // regenerate the primitives according to their pecking order
  // (supporter before dependent) if any of the primitives fail
  // to regenerate return false and don't bother regenerating the
  // remaining primitives:
  bool regenerate() const;
  
  // For debugging, dumps this model primitive id dispatcher:
  void dump(ostream & strm, unsigned int indent = 0) const;
  
private:
  // this will be called after sorting the graph:
  inline void remove_duplicates()
  { if (!empty()) unique(); }
  
  // the registry that owns the primitives stored in this graph:
  const the_registry_t * registry_;
};


#endif // THE_GRAPH_HXX_
