// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_reference.hxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Jul  1 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : A reference object used to establish graph dependencies
//                between document primitives.

#ifndef THE_REFERENCE_HXX_
#define THE_REFERENCE_HXX_

// local includes:
#include "doc/the_registry.hxx"
#include "math/v3x1p3x1.hxx"

#ifndef NOUI
#include "opengl/the_point_symbols.hxx"
#endif // NOUI

// forward declarations:
class the_view_mgr_t;


//----------------------------------------------------------------
// the_reference_t
// 
// Base class for the reference types:
// 
class the_reference_t
{
public:
  the_reference_t(const unsigned int & id);
  virtual ~the_reference_t() {}
  
  // a method for cloning references (potential memory leak):
  virtual the_reference_t * clone() const = 0;
  
  // a human-readable name, should be provided by each instantiable primitive:
  virtual const char * name() const = 0;
  
  // accessor to the reference primitive:
  template <class T>
  T * references(the_registry_t * registry) const
  { return registry->template elem<T>(id_); }
  
  // calculate the 3D value of this reference:
  virtual bool eval(the_registry_t * registry, p3x1_t & wcs_pt) const = 0;
  
  // if possible, adjust this reference:
  virtual bool move(the_registry_t * registry,
		    const the_view_mgr_t & view_mgr,
		    const p3x1_t & wcs_pt) = 0;
  
  // equality test:
  virtual bool equal(const the_reference_t * ref) const
  { return id_ == ref->id_; }
  
  // equality test operator:
  inline bool operator == (const the_reference_t & ref) const
  { return equal(&ref); }
  
  // reference model primitive id accessor:
  inline const unsigned int & id() const
  { return id_; }
  
#ifndef NOUI
  // symbol id used to display this reference:
  virtual the_point_symbol_id_t symbol() const = 0;
#endif // NOUI
  
  // file io:
  virtual bool save(std::ostream & stream) const;
  virtual bool load(std::istream & stream);
  
  // For debugging, dumps the id:
  virtual void dump(ostream & strm, unsigned int indent = 0) const;
  
protected:
  the_reference_t();
  
  // the primitive that is being referenced:
  unsigned int id_;
};


#endif // THE_REFERENCE_HXX_
