// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_input_device_event.cxx
// Author       : Pavel A. Koshevoy
// Created      : 2001/06/24 16:47:24
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : mouse, keyboard and tablet event class implementation.

// local includes:
#include "eh/the_input_device_event.hxx"
#include "ui/the_trail.hxx"
#include "utils/the_indentation.hxx"


//----------------------------------------------------------------
// the_mouse_btn_t::tran_down_
// 
unsigned int
the_mouse_event_t::tran_down_ = 0;

//----------------------------------------------------------------
// the_mouse_event_t::tran_up_
// 
unsigned int
the_mouse_event_t::tran_up_ = 0;

//----------------------------------------------------------------
// the_mouse_event_t::the_mouse_event_t
// 
the_mouse_event_t::the_mouse_event_t(the_view_t * widget,
				     unsigned int btns,
				     unsigned int tran,
				     unsigned int mods,
				     bool double_click,
				     bool moving,
				     const p2x1_t & scs_pt):
  the_input_device_event_t(widget),
  btns_(btns),
  tran_(tran),
  mods_(mods),
  double_click_(double_click),
  moving_(moving),
  scs_pt_(scs_pt)
{}

//----------------------------------------------------------------
// the_mouse_event_t::setup_transition_detectors
// 
void
the_mouse_event_t::setup_transition_detectors(unsigned int tran_down,
					      unsigned int tran_up)
{
  tran_down_ = tran_down;
  tran_up_ = tran_up;
}


//----------------------------------------------------------------
// the_wheel_event_t::the_wheel_event_t
// 
the_wheel_event_t::the_wheel_event_t(the_view_t * widget,
				     unsigned int btns,
				     unsigned int tran,
				     unsigned int mods,
				     const p2x1_t & scs_pt,
				     double degrees_rotated,
				     bool vertical):
  the_input_device_event_t(widget),
  btns_(btns),
  tran_(tran),
  mods_(mods),
  scs_pt_(scs_pt),
  degrees_rotated_(degrees_rotated),
  vertical_(vertical)
{}


//----------------------------------------------------------------
// the_keybd_event_t::tran_down_
// 
unsigned int
the_keybd_event_t::tran_down_ = ~0;

//----------------------------------------------------------------
// the_keybd_event_t::tran_up_
// 
unsigned int
the_keybd_event_t::tran_up_ = ~0;

//----------------------------------------------------------------
// the_keybd_event_t::setup_transition_detectors
// 
void
the_keybd_event_t::setup_transition_detectors(int tran_down,
					      int tran_up)
{
  tran_down_ = tran_down;
  tran_up_ = tran_up;
}

//----------------------------------------------------------------
// the_keybd_event_t::the_keybd_event_t
// 
the_keybd_event_t::the_keybd_event_t(the_view_t * widget,
				     unsigned int key,
				     unsigned int tran,
				     unsigned int mods,
				     bool autorepeat):
  the_input_device_event_t(widget),
  key_(key),
  tran_(tran),
  mods_(mods),
  autorepeat_(autorepeat)
{}

//----------------------------------------------------------------
// operator <<
// 
ostream &
operator << (ostream & sout, const the_keybd_event_t & ke)
{
  sout << "the_keybd_event_t: " << endl;
  
  if (ke.key() < 128)
  {
    sout << "key_    = " << ke.key() << " (" << char(ke.key()) << ')' << endl;
  }
  else
  {
    sout << "key_    = " << ke.key() << endl;
  }
  
  sout << "tran_   = " << ke.tran() << endl
       << "mods_   = " << ke.mods() << endl
       << "widget_ = " << ke.widget() << endl
       << "autorepeat_ = " << ke.autorepeat() << endl;
  
  return sout;
}


//----------------------------------------------------------------
// the_wacom_event_t::the_wacom_event_t
// 
the_wacom_event_t::the_wacom_event_t(the_view_t * widget,
				     the_tablet_tool_t tool,
				     unsigned long int tool_id,
				     const p2x1_t & scs_pt,
				     const p2x1_t & tilt,
				     float pressure,
				     float tangential_pressure,
				     float rotation,
				     float z_position):
  the_input_device_event_t(widget),
  tool_(tool),
  tool_id_(tool_id),
  scs_pt_(scs_pt),
  tilt_(tilt),
  pressure_(pressure),
  tangential_pressure_(tangential_pressure),
  rotation_(rotation),
  z_position_(z_position)
{}

//----------------------------------------------------------------
// the_wacom_event_t::dump
// 
void
the_wacom_event_t::dump(ostream & sout, unsigned int indent) const
{
  const char * tool_str = NULL;
  switch (tool_)
  {
    case THE_TABLET_PEN_E:
      tool_str = "Pen";
      break;
      
    case THE_TABLET_ERASER_E:
      tool_str = "Eraser";
      break;
      
    case THE_TABLET_CURSOR_E:
      tool_str = "Cursor";
      break;
      
    default:
      tool_str = "unknown tablet tool";
      break;
  }
  
  sout << INDSCP << "the_wacom_event_t(" << (void *)this << ")" << endl
       << INDSCP << "{" << endl
       << INDSTR << "tool                = " << tool_str << endl
       << INDSTR << "tool_id             = " << int(tool_id_) << endl
       << INDSTR << "scs_pt              = " << scs_pt_ << endl
       << INDSTR << "tilt                = " << tilt_ << endl
       << INDSTR << "pressure            = " << pressure_ << endl
       << INDSTR << "tangential pressure = " << tangential_pressure_ << endl
       << INDSTR << "rotation            = " << rotation_ << endl
       << INDSTR << "Z position          = " << z_position_ << endl
       << INDSCP << "}" << endl << endl;
}

//----------------------------------------------------------------
// operator <<
// 
ostream &
operator << (ostream & sout, const the_wacom_event_t & we)
{
  we.dump(sout);
  return sout;
}
