#!/bin/csh

if ( ! ${?CPU} ) then
    setenv CPU `uname -p`
endif
if ( ${CPU} == "unknown" ) then
    set OS = `uname -o`
    if ( ${OS} == "Cygwin" ) then
	setenv CPU w32
    endif
endif

# use the intel compiler, when available:
if ( ${CPU} == "ia64" && -e /opt/intel/cc/9.0/bin/iccvars.csh ) then
    setenv CC icc
    setenv CXX icpc
    source /opt/intel/cc/9.0/bin/iccvars.csh
endif

# initialize the search paths as necessary:
if ( ! ${?PATH} ) then 
    setenv PATH "" 
endif
if ( ! ${?MANPATH} ) then 
    setenv MANPATH "" 
endif
if ( ! ${?CPATH} ) then 
    setenv CPATH ""
endif
if ( ! ${?LIBRARY_PATH} ) then 
    setenv LIBRARY_PATH ""
endif
if ( ! ${?LD_LIBRARY_PATH} ) then 
    setenv LD_LIBRARY_PATH ""
endif

# add ITK to the search paths:
if ( ${?ITK_SOURCE_DIR} ) then
    setenv CPATH ${ITK_SOURCE_DIR}:${CPATH}
endif

if ( ${?ITK_BINARY_DIR} ) then
    setenv ITK_DIR ${ITK_BINARY_DIR}
    setenv LIBRARY_PATH ${ITK_BINARY_DIR}/bin:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${ITK_BINARY_DIR}/bin:${LD_LIBRARY_PATH}
endif

# add Qt to the paths:
if ( ${?QT4_DIR} ) then
    setenv QTDIR ${QT4_DIR}
    setenv PATH ${QT4_DIR}/bin:${PATH}
    setenv CPATH ${QT4_DIR}/include:${CPATH}:/usr/sci/crcnsdata/boost_1_33_1
    setenv MANPATH ${QT4_DIR}/doc/man:${MANPATH}
    setenv LIBRARY_PATH ${QT4_DIR}/lib:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${QT4_DIR}/lib:${LD_LIBRARY_PATH}
endif

# add GLEW to the search paths:
if ( ${?GLEW_DIR} ) then
    setenv PATH ${GLEW_DIR}/bin:${PATH}
    setenv CPATH ${GLEW_DIR}/include:${CPATH}
    setenv MANPATH ${GLEW_DIR}/share/man:${MANPATH}
    setenv LIBRARY_PATH ${GLEW_DIR}/lib:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${GLEW_DIR}/lib:${LD_LIBRARY_PATH}
endif

# add Cg to the search paths:
if ( ${?CG_DIR} ) then
    setenv PATH ${CG_DIR}/bin:${PATH}
    setenv CPATH ${CG_DIR}/include:${CPATH}
    setenv MANPATH ${CG_DIR}/share/man:${MANPATH}
    if ( -e ${CG_DIR}/lib64 ) then
	setenv CG_LIB_DIR ${CG_DIR}/lib64
    else
	setenv CG_LIB_DIR ${CG_DIR}/lib
    endif
    setenv LIBRARY_PATH ${CG_LIB_DIR}:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${CG_LIB_DIR}:${LD_LIBRARY_PATH}
endif

# add FFTW to the search paths:
if ( ${?FFTW_DIR} ) then
    setenv PATH ${FFTW_DIR}/bin:${PATH}
    setenv CPATH ${FFTW_DIR}/include:${CPATH}
    setenv MANPATH ${FFTW_DIR}/share/man:${MANPATH}
    setenv LIBRARY_PATH ${FFTW_DIR}/lib:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${FFTW_DIR}/lib:${LD_LIBRARY_PATH}
endif

# add BOOST to the search paths:
if ( ${?BOOST_ROOT} ) then
    setenv CPATH ${BOOST_ROOT}/include:${CPATH}
    setenv LIBRARY_PATH ${BOOST_ROOT}/lib:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${BOOST_ROOT}/lib:${LD_LIBRARY_PATH}
endif

# add THE libraries to the search paths:
if ( ${?THE_SRC_DIR} ) then
    setenv CPATH ${THE_SRC_DIR}:${CPATH}
endif

if ( ${?THE_BIN_DIR} ) then
    setenv LIBRARY_PATH ${THE_BIN_DIR}:${LIBRARY_PATH}
    setenv LD_LIBRARY_PATH ${THE_BIN_DIR}:${LD_LIBRARY_PATH}
endif
