// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_tensurf.hxx
// Author       : Pavel A. Koshevoy
// Created      : Tue Dec 28 13:36:00 MDT 2004
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : A tensor product surface.

#ifndef THE_TENSURF_HXX_
#define THE_TENSURF_HXX_

// local includes:
#include "doc/the_primitive.hxx"
#include "math/v3x1p3x1.hxx"
#include "geom/the_bspline.hxx"
#include "opengl/the_vertex.hxx"
#include "opengl/the_disp_list.hxx"

// system includes:
#include <vector>

// forward declarations:
class the_grid_t;


//----------------------------------------------------------------
// the_tensurf_t
// 
// A tensor b-spline surface defined by two uniform knot vectors
// and supported by a quadrilateral grid.
// 
class the_tensurf_t : public the_primitive_t
{
public:
  // virtual:
  the_primitive_t * clone() const
  { return new the_tensurf_t(*this); }
  
  // virtual:
  const char * name() const
  { return "the_tensurf_t"; }
  
  // virtual:
  bool regenerate();
  
  // virtual: display a piecewise linear approximation of the curve:
  the_dl_elem_t * dl_elem() const;
  
  // the tensor surface is supported by a grid:
  the_grid_t * grid() const;
  
  // virtual: For debugging, dumps all segments
  void dump(ostream & strm, unsigned int indent = 0) const;
  
private:
  the_knot_vector_t knot_vector_u_;
  the_knot_vector_t knot_vector_v_;
  
  std::vector<std::vector<the_vertex_t> > tri_mesh_;
};


#endif // THE_TENSURF_HXX_
