// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : image_tile.hxx
// Author       : Pavel A. Koshevoy
// Created      : Sun Sep 3 18:01:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : a textured image tile convenience class

#ifndef IMAGE_TILE_HXX_
#define IMAGE_TILE_HXX_

// system includes:
#include <iostream>

// Boost includes:
#include <boost/shared_ptr.hpp>

// local includes:
#include "opengl/OpenGLCapabilities.h"
#include "utils/the_indentation.hxx"
#include "math/v3x1p3x1.hxx"

// namespace access:
using std::ostream;
using std::endl;

// forward declarations:
class texture_base_t;
class the_bbox_t;


//----------------------------------------------------------------
// image_tile_t
//
// a texture mapped OpenGL quadrilateral.
//
class image_tile_t
{
public:
  image_tile_t();
  ~image_tile_t();
  
  // OpenGL helper:
  void draw(const GLuint & texture_id,
	    const GLint & mag_filter = GL_NEAREST,
	    const GLint & min_filter = GL_NEAREST) const;
  
  // helper:
  void update_bbox(the_bbox_t & bbox) const;
  
  //----------------------------------------------------------------
  // quad_t
  // 
  class quad_t
  {
  public:
    inline void dump(ostream & strm, unsigned int indent = 0) const
    {
      strm << INDSCP << "quad_t(" << (void *)this << ")" << endl
	   << INDSCP << "{" << endl
	   << INDSTR << "x_ = " << x_ << endl
	   << INDSTR << "y_ = " << y_ << endl
	   << INDSTR << "w_ = " << w_ << endl
	   << INDSTR << "h_ = " << h_ << endl
	   << INDSCP << "}" << endl;
    }
    
    GLint x_;
    GLint y_;
    GLsizei w_;
    GLsizei h_;
  };
  
  // quad coordinates:
  p3x1_t corner_[4];
  
  // texture coordinates:
  GLfloat s0_;
  GLfloat s1_;
  GLfloat t0_;
  GLfloat t1_;
  
  // the texture:
  boost::shared_ptr<texture_base_t> texture_;
};


//----------------------------------------------------------------
// operator <<
// 
inline ostream &
operator << (ostream & strm, const image_tile_t::quad_t & quad)
{
  quad.dump(strm);
  return strm;
}


#endif // IMAGE_TILE_HXX_
