// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : texture_data.cxx
// Author       : Pavel A. Koshevoy
// Created      : Sun Sep 3 18:27:00 MDT 2006
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : a texture data helper class

// local includes:
#include "image/texture_data.hxx"
#include "utils/the_exception.hxx"

// system includes:
#include <sstream>

//----------------------------------------------------------------
// _1MB
// 
static const size_t _1MB = 1024 * 1024;

//----------------------------------------------------------------
// DEBUG_TEXTURE_DATA
// 
// #define DEBUG_TEXTURE_DATA

#ifdef DEBUG_TEXTURE_DATA
#include "utils/the_indentation.hxx"
#include <iostream>
#include <QMutex>

static QMutex mutex;
using std::cerr;
using std::endl;

static size_t total = 0;
#endif // DEBUG_TEXTURE_DATA

//----------------------------------------------------------------
// texture_data_t::texture_data_t
// 
texture_data_t::texture_data_t(size_t bytes)
{
  data_ = malloc(bytes);
  size_ = bytes;
  
  if (data_ == NULL)
  {
    std::ostringstream os;
    os << "could not allocate the texture data buffer of "
       << (size_ / _1MB) << "MB" << std::endl;
    the_exception_t e(os.str().c_str());
    throw e;
  }
  
#ifdef DEBUG_TEXTURE_DATA
  QMutexLocker locker(&mutex);
  total += size_;
  cerr << this << " -- malloced " << size_ / _1MB << "MB"
       << ", total allocated: " << total / _1MB << "MB" << endl;
#endif // DEBUG_TEXTURE_DATA
}

//----------------------------------------------------------------
// texture_data_t::~texture_data_t
// 
texture_data_t::~texture_data_t()
{
  free(data_);
  
#ifdef DEBUG_TEXTURE_DATA
  QMutexLocker locker(&mutex);
  total -= size_;
  cerr << this << " -- released " << size_ / _1MB << "MB"
       << ", total allocated: " << total / _1MB << "MB" << endl;
  // put_breakpoint_here();
#endif // DEBUG_TEXTURE_DATA
}
