// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_color_blend.cxx
// Author       : Pavel A. Koshevoy
// Created      : Fri Nov  8 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Linear interpolation between 2 colors.

// local includes:
#include "math/the_color_blend.hxx"


//----------------------------------------------------------------
// the_color_blend_t::eval
// 
const the_color_t
the_color_blend_t::eval(float t) const
{
  unsigned int color_a_index = (unsigned int)((float)(color_.size() - 1) * t);
  unsigned int color_b_index = color_a_index + 1;
  
  float color_param = t;
  if (color_.size() > 1)
  {
    color_param = (((float)(color_.size() - 1) * t -
		    (float)(color_a_index)) /
		   ((float)(color_b_index) -
		    (float)(color_a_index)));
  }
  
  if (color_b_index == color_.size()) color_b_index--;
  
  const the_color_t & color_a = color_[color_a_index];
  const the_color_t & color_b = color_[color_b_index];
  
  return the_color_t((1.0 - color_param) * color_a +
		     color_param * color_b);
}
