// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_color_blend.hxx
// Author       : Pavel A. Koshevoy
// Created      : Fri Nov  8 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Linear interpolation between 2 colors.

#ifndef THE_COLOR_BLEND_HXX_
#define THE_COLOR_BLEND_HXX_

// system includes:
#include <vector>

// local includes:
#include "math/the_color.hxx"


//----------------------------------------------------------------
// the_color_blend_t
// 
// uniformly spaced color blend:
// 
class the_color_blend_t
{
public:
  the_color_blend_t() {}
  the_color_blend_t(unsigned int num_colors): color_(num_colors) {}
  ~the_color_blend_t() {}
  
  // find between which two colors the parameter "t" falls,
  // blend between them and return the result:
  const the_color_t eval(float t) const;
  
  // short-hand for "eval(..)":
  inline const the_color_t operator() (float t) const
  { return eval(t); }
  
  // low level accessors:
  inline const std::vector<the_color_t> & color() const
  { return color_; }
  
  inline std::vector<the_color_t> & color()
  { return color_; }
  
private:
  // an array of colors:
  std::vector<the_color_t> color_;
};


#endif // THE_COLOR_BLEND_HXX_
