// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_plane.hxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Jul  1 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : A 3D plane class.

#ifndef THE_PLANE_HXX_
#define THE_PLANE_HXX_

// local includes:
#include "math/the_coord_sys.hxx"

// forward declarations:
class the_ray_t;


//----------------------------------------------------------------
// the_plane_t
// 
class the_plane_t
{
public:
  the_plane_t(const the_coord_sys_t & cs): cs_(cs) {}
  
  // equality test operators:
  inline bool operator == (const the_plane_t & p) const
  { return (cs_ == p.cs_); }
  
  inline bool operator != (const the_plane_t & p) const
  { return !(*this == p); }
  
  // intersect this plane with a ray, return true if successful,
  // store the result as parameter t along ray:
  bool intersect(const the_ray_t & r, float & param) const;
  
  // intersect this plane with a ray and return the point of intersection:
  const p3x1_t operator * (const the_ray_t & r) const;
  
  // project a point onto this plane:
  const p2x1_t operator * (const p3x1_t & wcs_pt) const;
  
  // express a local (u, v) point in the world coordinate system:
  const p3x1_t operator * (const p2x1_t & lcs_pt) const;
  
  // find the normal vector from the plane to a given point:
  const v3x1_t operator - (const p3x1_t & wcs_pt) const;
  
  // accessor to the local coordinate system:
  inline const the_coord_sys_t & cs() const
  { return cs_; }
  
  // calculate the distance from a point to the plane along the plane normal:
  inline float dist(const p3x1_t & pt) const
  { return cs_.z_axis() * (pt - cs_.origin()); }
  
  // For debugging:
  void dump(ostream & strm, unsigned int indent = 0) const;
  
private:
  // Coordinate system of this plane. Z is the normal, O is a point in plane:
  the_coord_sys_t cs_;
};

extern ostream &
operator << (ostream & stream, const the_plane_t & plane);


#endif // THE_PLANE_HXX_
