// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_appearance.hxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Jul  1 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : The appearance class in combination with the palette class
//                provides basic theme/skin support for the OpenGL view widget.

#ifndef THE_APPEARANCE_HXX_
#define THE_APPEARANCE_HXX_

// local includes:
#include "opengl/the_palette.hxx"
#include "opengl/the_ep_grid.hxx"

// forward declarations:
class the_view_t;


//----------------------------------------------------------------
// the_appearance_t
//
// base class for various view appearances (original, ampad, etc..)
class the_appearance_t
{
public:
  the_appearance_t(the_palette_id_t id): palette_(id) {}
  virtual ~the_appearance_t() {}
  
  // palette accessor:
  inline const the_palette_t & palette() const
  { return palette_; }
  
  virtual void draw_background(the_view_t & view) const;
  
  virtual void draw_edit_plane(the_view_t & view) const = 0;
  
  virtual void draw_coordinate_system(the_view_t & view) const;
  
  virtual void draw_view_label(the_view_t & view) const;
  
protected:
  // color palette specific to the appearance:
  the_palette_t palette_;
};


//----------------------------------------------------------------
// the_original_appearance_t
// 
class the_original_appearance_t : public the_appearance_t
{
public:
  the_original_appearance_t(): the_appearance_t(THE_ORIGINAL_PALETTE_E) {}
  
  // virtual:
  void draw_edit_plane(the_view_t & view) const;
};


//----------------------------------------------------------------
// the_ampad_appearance_t
// 
class the_ampad_appearance_t : public the_appearance_t
{
public:
  the_ampad_appearance_t(): the_appearance_t(THE_AMPAD_PALETTE_E) {}
  
  // virtual:
  void draw_edit_plane(the_view_t & view) const;
};


//----------------------------------------------------------------
// the_generic_appearance_t
// 
class the_generic_appearance_t : public the_appearance_t
{
public:
  the_generic_appearance_t(the_palette_id_t id = THE_NORCOM_PALETTE_E):
    the_appearance_t(id) {}
  
  // virtual:
  void draw_edit_plane(the_view_t & view) const;
};


//----------------------------------------------------------------
// THE_APPEARANCE
// 
extern const the_appearance_t & THE_APPEARANCE;


#endif // THE_APPEARANCE_HXX_
