// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_font.cxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Jul  1 21:53:36 MDT 2002
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : An OpenGL bitmap font class.

// local includes:
#include "opengl/the_font.hxx"
#include "opengl/OpenGLCapabilities.h"
#include "math/the_color.hxx"


//----------------------------------------------------------------
// the_font_t::print
// 
void
the_font_t::print(const the_text_t & str,
		  const p3x1_t & pos,
		  const the_color_t & color) const
{
  glColor4fv(color.rgba());
  glRasterPos3fv(pos.data());
  
  const unsigned int str_len = str.size();
  if (dl_offset_ != 0)
  {
    the_scoped_gl_attrib_t push_attr(GL_LIST_BIT);
    {
      glListBase(dl_offset_);
      glCallLists(str_len, GL_UNSIGNED_BYTE, (GLubyte *)(str.text()));
    }
  }
  else
  {
    for (unsigned int i = 0; i < str_len; i++)
    {
      draw_bitmap(int(str.operator[](i)));
    }
  }
}

//----------------------------------------------------------------
// the_font_t::print
// 
void
the_font_t::print(const the_text_t & str,
		  const p3x1_t & pos,
		  const the_color_t & font_color,
		  const the_color_t & mask_color) const
{
  print_mask(str, pos, mask_color);
  print(str, pos, font_color);
}

//----------------------------------------------------------------
// the_font_t::print_mask
// 
void
the_font_t::print_mask(const the_text_t & str,
		       const p3x1_t & pos,
		       const the_color_t & color) const
{
  glColor4fv(color.rgba());
  glRasterPos3fv(pos.data());
  
  const unsigned int str_len = str.size();
  for (unsigned int i = 0; i < str_len; i++)
  {
    draw_bitmap_mask(int(str.operator[](i)));
  }
}
