// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_view_mgr_orientation.hxx
// Author       : Pavel A. Koshevoy
// Created      : Sun May 30 19:55:00 MDT 2004
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : Predefined view orientations.

#ifndef THE_VIEW_MGR_ORIENTATION_HXX_
#define THE_VIEW_MGR_ORIENTATION_HXX_

// local includes:
#include "math/v3x1p3x1.hxx"

// system includes:
#include <iostream>

// namespace access:
using std::ostream;


//----------------------------------------------------------------
// the_view_mgr_orientation_t
//
// A list of predefined view point orientations:
typedef enum
{
  THE_ISOMETRIC_VIEW_E,
  THE_TOP_VIEW_E,
  THE_BOTTOM_VIEW_E,
  THE_LEFT_VIEW_E,
  THE_RIGHT_VIEW_E,
  THE_FRONT_VIEW_E,
  THE_BACK_VIEW_E,
  THE_XY_VIEW_E
} the_view_mgr_orientation_t;

//----------------------------------------------------------------
// the_view_orientation_text
// 
extern const char *
the_view_orientation_text(const the_view_mgr_orientation_t & orientation);

//----------------------------------------------------------------
// operator <<
// 
inline ostream &
operator << (ostream & s, const the_view_mgr_orientation_t & orientation)
{
  return s << the_view_orientation_text(orientation);
}

// global orientation vectors (look from and up):
extern const v3x1_t THE_ORIENTATION_LF[];
extern const v3x1_t THE_ORIENTATION_UP[];


#endif // THE_VIEW_MGR_ORIENTATION_HXX_
