// -*- Mode: c++; tab-width: 8; c-basic-offset: 2; indent-tabs-mode: t -*-
// NOTE: the first line of this file sets up source code indentation rules
// for Emacs; it is also a hint to anyone modifying this file.

/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// File         : the_procedure_ui.hxx
// Author       : Pavel A. Koshevoy
// Created      : Mon Apr 10 12:00:00 MDT 2004
// Copyright    : (C) 2004-2008 University of Utah
// License      : GPLv2
// Description  : The base class for a document procedure
//                (a tool) user interface.

#ifndef THE_PROCEDURE_UI_HXX_
#define THE_PROCEDURE_UI_HXX_

// local includes:
#include "ui/the_document_ui.hxx"
#include "doc/the_document_so.hxx"
#include "doc/the_document.hxx"


//----------------------------------------------------------------
// the_procedure_ui_t
// 
// The procedure user interface abstract class.
// 
class the_procedure_ui_t
{
  friend class the_document_ui_t;
  
public:
  the_procedure_ui_t(the_document_ui_t & doc_ui):
    doc_ui_(doc_ui),
    installed_(false)
  {}
  
  virtual ~the_procedure_ui_t()
  {}
  
  // accessor to the document UI object:
  inline the_document_ui_t & doc_ui()
  { return doc_ui_; }
  
  // accessor to the document object:
  inline the_document_t * document() const
  { return doc_ui_.doc_so().document(); }
  
  // start/stop the ui:
  virtual void install()
  { doc_ui().proc_ui_installed(this); }
  
  virtual void uninstall()
  { doc_ui().proc_ui_uninstalled(this); }
  
  // check whether the UI is running:
  inline const bool & installed() const
  { return installed_; }
  
  // ui update mechanism:
  virtual void sync_ui() = 0;
  
protected:
  // disable copy constructor and assignment operator:
  the_procedure_ui_t(const the_procedure_ui_t & ui);
  the_procedure_ui_t & operator = (const the_procedure_ui_t & ui);
  
  // helper functions:
  inline the_document_so_t & doc_so()
  { return doc_ui_.doc_so(); }
  
  // the procedure state object:
  the_document_ui_t & doc_ui_;
  
  // a flag indicating whether the UI is installed or not:
  bool installed_;
};


#endif // THE_PROCEDURE_UI_HXX_
